@@ -39,8 +39,8 @@ void WayAddNodeCommand::undo()
     if (theLayer && oldLayer && (theLayer != oldLayer)) {
         theLayer->remove(theRoad);
         oldLayer->add(theRoad);
-        decDirtyLevel(oldLayer);
     }
+    decDirtyLevel(oldLayer, theRoad);
 }
 
 void WayAddNodeCommand::redo()
@@ -49,9 +49,9 @@ void WayAddNodeCommand::redo()
     theRoad->add(theTrackPoint, Position);
     if (theLayer && oldLayer && (theLayer != oldLayer)) {
         oldLayer->remove(theRoad);
-        incDirtyLevel(oldLayer);
         theLayer->add(theRoad);
     }
+    incDirtyLevel(oldLayer, theRoad);
     Command::redo();
 }
 
@@ -63,7 +63,7 @@ bool WayAddNodeCommand::buildDirtyList(DirtyList& theList)
         return theList.noop(theRoad);
     if (!theRoad->layer())
         return theList.update(theRoad);
-    if (theRoad->layer()->isUploadable() && theTrackPoint->layer()->isUploadable())
+    if (theRoad->isUploadable() && theTrackPoint->isUploadable())
         return theList.update(theRoad);
 
     return theList.noop(theRoad);
@@ -102,8 +102,11 @@ WayAddNodeCommand * WayAddNodeCommand::fromXML(Document * d, QDomElement e)
         a->oldLayer = d->getLayer(e.attribute("oldlayer"));
     else
         a->oldLayer = NULL;
-    a->theRoad = Feature::getWayOrCreatePlaceHolder(d, a->theLayer, e.attribute("road"));
-    a->theTrackPoint = Feature::getTrackPointOrCreatePlaceHolder(d, a->theLayer, e.attribute("trackpoint"));
+    if (!a->theLayer)
+        return NULL;
+
+    a->theRoad = Feature::getWayOrCreatePlaceHolder(d, a->theLayer, IFeature::FId(IFeature::LineString, e.attribute("road").toLongLong()));
+    a->theTrackPoint = Feature::getTrackPointOrCreatePlaceHolder(d, a->theLayer, IFeature::FId(IFeature::Point, e.attribute("trackpoint").toLongLong()));
     a->Position = e.attribute("pos").toUInt();
 
     Command::fromXML(d, e, a);
@@ -152,8 +155,8 @@ void WayRemoveNodeCommand::undo()
     if (theLayer && oldLayer && (theLayer != oldLayer)) {
         theLayer->remove(theRoad);
         oldLayer->add(theRoad);
-        decDirtyLevel(oldLayer);
     }
+    decDirtyLevel(oldLayer, theRoad);
 }
 
 void WayRemoveNodeCommand::redo()
@@ -167,9 +170,9 @@ void WayRemoveNodeCommand::redo()
 
     if (theLayer && oldLayer && (theLayer != oldLayer)) {
         oldLayer->remove(theRoad);
-        incDirtyLevel(oldLayer);
         theLayer->add(theRoad);
     }
+    incDirtyLevel(oldLayer, theRoad);
     Command::redo();
 }
 
@@ -181,7 +184,7 @@ bool WayRemoveNodeCommand::buildDirtyList(DirtyList& theList)
         return theList.noop(theRoad);
     if (!theRoad->layer())
         return theList.update(theRoad);
-    if (theRoad->layer()->isUploadable() && (theNode->layer()->isUploadable() || theNode->hasOSMId()))
+    if (theRoad->isUploadable() && (theNode->isUploadable() || theNode->hasOSMId()))
         return theList.update(theRoad);
 
     return theList.noop(theRoad);
@@ -222,9 +225,12 @@ WayRemoveNodeCommand * WayRemoveNodeCommand::fromXML(Document * d, QDomElement e
         a->oldLayer = d->getLayer(e.attribute("oldlayer"));
     else
         a->oldLayer = NULL;
+    if (!a->theLayer)
+        return NULL;
+
     a->wasClosed = (e.hasAttribute("closed") && e.attribute("closed") == "true");
-    a->theRoad = Feature::getWayOrCreatePlaceHolder(d, a->theLayer, e.attribute("road"));
-    a->theNode = Feature::getTrackPointOrCreatePlaceHolder(d, a->theLayer, e.attribute("trackpoint"));
+    a->theRoad = Feature::getWayOrCreatePlaceHolder(d, a->theLayer, IFeature::FId(IFeature::LineString, e.attribute("road").toLongLong()));
+    a->theNode = Feature::getTrackPointOrCreatePlaceHolder(d, a->theLayer, IFeature::FId(IFeature::Point, e.attribute("trackpoint").toLongLong()));
     a->Idx = e.attribute("index").toUInt();
 
     Command::fromXML(d, e, a);
