@@ -19,6 +19,9 @@
 
 #include "Features.h"
 
+#include "Utils/SelectionDialog.h"
+#include "Utils/TagSelector.h"
+
 #include <QAction>
 #include <QTimer>
 #include <QMenu>
@@ -26,13 +29,20 @@
 #define MAX_FEATS 100
 
 FeaturesDock::FeaturesDock(MainWindow* aParent)
-    : MDockAncestor(aParent), Main(aParent), curFeatType(Feature::Relations)
+    : MDockAncestor(aParent),
+    Main(aParent),
+    curFeatType(IFeature::OsmRelation),
+    findMode(false)
 {
 //    setMinimumSize(220,100);
     setObjectName("FeaturesDock");
 
     ui.setupUi(getWidget());
 
+#ifdef Q_OS_MAC
+    ui.cbWithin->setMinimumWidth(30);
+#endif
+
     ui.cbWithin->setChecked(M_PREFS->getFeaturesWithin());
     ui.FeaturesList->sortItems();
 
@@ -48,6 +58,8 @@ FeaturesDock::FeaturesDock(MainWindow* aParent)
     connect(ui.FeaturesList, SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(on_FeaturesList_customContextMenuRequested(const QPoint &)));
 
     connect(ui.cbWithin, SIGNAL(stateChanged(int)), this, SLOT(on_rbWithin_stateChanged(int)));
+    connect(ui.btFind, SIGNAL(clicked(bool)), SLOT(on_btFind_clicked(bool)));
+    connect(ui.btReset, SIGNAL(clicked(bool)), SLOT(on_btReset_clicked(bool)));
 
     centerAction = new QAction(NULL, this);
     connect(centerAction, SIGNAL(triggered()), this, SLOT(on_centerAction_triggered()));
@@ -60,13 +72,13 @@ FeaturesDock::FeaturesDock(MainWindow* aParent)
 
     int t;
     t = ui.tabBar->addTab(NULL);
-    ui.tabBar->setTabData(t, Feature::Relations);
+    ui.tabBar->setTabData(t, IFeature::OsmRelation);
     t = ui.tabBar->addTab(NULL);
-    ui.tabBar->setTabData(t, Feature::Ways);
+    ui.tabBar->setTabData(t, IFeature::LineString);
     t = ui.tabBar->addTab(NULL);
-    ui.tabBar->setTabData(t, Feature::Nodes);
+    ui.tabBar->setTabData(t, IFeature::Point);
     t = ui.tabBar->addTab(NULL);
-    ui.tabBar->setTabData(t, Feature::All);
+    ui.tabBar->setTabData(t, IFeature::All);
     ui.tabBar->setElideMode(Qt::ElideRight);
     ui.tabBar->setUsesScrollButtons(true);
     retranslateTabBar();
@@ -238,6 +250,37 @@ void FeaturesDock::on_addSelectAction_triggered()
     Main->view()->blockSignals(false);
 }
 
+void FeaturesDock::on_btFind_clicked(bool)
+{
+    SelectionDialog* dlg = new SelectionDialog(Main);
+    if (!dlg->exec())
+        return;
+
+    TagSelector* tsel = TagSelector::parse(dlg->edTagQuery->text());
+    if (!tsel)
+        return;
+
+    Found.clear();
+    int added = 0;
+    for (VisibleFeatureIterator i(Main->document()); !i.isEnd() && (!dlg->sbMaxResult->value() || added < dlg->sbMaxResult->value()); ++i) {
+        if (tsel->matches(i.get(), Main->view()->pixelPerM())) {
+            Found << i.get();
+        }
+    }
+
+    findMode = true;
+    ui.tabBar->blockSignals(true);
+    ui.tabBar->setCurrentIndex(3);
+    ui.tabBar->blockSignals(false);
+    tabChanged(3);
+}
+
+void FeaturesDock::on_btReset_clicked(bool)
+{
+    findMode = false;
+    updateList();
+}
+
 void FeaturesDock::changeEvent(QEvent * event)
 {
     if (event->type() == QEvent::LanguageChange)
@@ -247,7 +290,7 @@ void FeaturesDock::changeEvent(QEvent * event)
 
 void FeaturesDock::tabChanged(int idx)
 {
-    curFeatType = (Feature::FeatureType)ui.tabBar->tabData(idx).toInt();
+    curFeatType = (IFeature::FeatureType)ui.tabBar->tabData(idx).toInt();
     ui.FeaturesList->clear();
     Highlighted.clear();
 
@@ -269,29 +312,25 @@ void FeaturesDock::addItem(MapFeaturePtr F)
     if (Highlighted.contains(F))
         return;
 
-    if (curFeatType == Feature::Relations || curFeatType == Feature::All)
+    if (curFeatType == IFeature::OsmRelation || curFeatType == Feature::All)
     {
         if (Relation* L = CAST_RELATION(F)) {
             QListWidgetItem* anItem = new QListWidgetItem(L->description(), ui.FeaturesList);
             anItem->setData(Qt::UserRole, QVariant::fromValue(F));
         }
     }
-    if (curFeatType == Feature::Ways || curFeatType == Feature::All)
+    if (curFeatType == IFeature::LineString || curFeatType == IFeature::Polygon || curFeatType == Feature::All)
     {
         if (Way* R = CAST_WAY(F)) {
             QListWidgetItem* anItem = new QListWidgetItem(R->description(), ui.FeaturesList);
             anItem->setData(Qt::UserRole, QVariant::fromValue(F));
         }
     }
-    if (curFeatType == Feature::Nodes || curFeatType == Feature::All)
+    if (curFeatType == IFeature::Point || curFeatType == Feature::All)
     {
         if (Node* N = CAST_NODE(F)) {
-        for (int i=0; i<N->tagSize(); ++i)
-            if ((N->tagKey(i) != "created_by") && (N->tagKey(i) != "ele")) {
-                QListWidgetItem* anItem = new QListWidgetItem(N->description(), ui.FeaturesList);
-                anItem->setData(Qt::UserRole, QVariant::fromValue(F));
-                break;
-            }
+            QListWidgetItem* anItem = new QListWidgetItem(N->description(), ui.FeaturesList);
+            anItem->setData(Qt::UserRole, QVariant::fromValue(F));
         }
     }
 }
@@ -300,6 +339,7 @@ void FeaturesDock::invalidate()
 {
     ui.FeaturesList->clear();
     Highlighted.clear();
+    Found.clear();
 }
 
 void FeaturesDock::clearItems()
@@ -328,24 +368,36 @@ void FeaturesDock::updateList()
     if (!isVisible() || !Main->document())
         return;
 
-    for (int j=0; j<Main->document()->layerSize(); ++j) {
-        if (!Main->document()->getLayer(j)->size()
-            || !Main->document()->getLayer(j)->isVisible()
-            || Main->document()->getLayer(j)->isIndexingBlocked()
-            )
-            continue;
-
-        if (dynamic_cast<ImageMapLayer*>(Main->document()->getLayer(j)))
-            continue;
-
-        QList < MapFeaturePtr > ret = Main->document()->getLayer(j)->indexFind(theViewport);
-        foreach (MapFeaturePtr F, ret) {
+    if (findMode) {
+        foreach (MapFeaturePtr F, Found) {
             if (ui.cbWithin->isChecked()) {
                 if (ggl::within(F->boundingBox(), Main->view()->viewport()))
                     addItem(F);
             } else
                 addItem(F);
         }
+    } else {
+        for (int j=0; j<Main->document()->layerSize(); ++j) {
+            if (!Main->document()->getLayer(j)->size()
+                || Main->document()->getLayer(j)->isIndexingBlocked()
+                )
+                continue;
+
+            if (dynamic_cast<ImageMapLayer*>(Main->document()->getLayer(j)))
+                continue;
+
+            QList < MapFeaturePtr > ret = Main->document()->getLayer(j)->indexFind(theViewport);
+            foreach (MapFeaturePtr F, ret) {
+                if (F->isHidden())
+                    continue;
+
+                if (ui.cbWithin->isChecked()) {
+                    if (ggl::within(F->boundingBox(), Main->view()->viewport()))
+                        addItem(F);
+                } else
+                    addItem(F);
+            }
+        }
     }
     ui.FeaturesList->sortItems();
 
