@@ -49,6 +49,9 @@ public slots:
     void on_downloadAction_triggered();
     void on_addSelectAction_triggered();
 
+    void on_btFind_clicked(bool);
+    void on_btReset_clicked(bool);
+
     void on_Viewport_changed();
 
     void tabChanged(int idx);
@@ -57,6 +60,7 @@ public slots:
 
 private:
     QList<Feature*> Highlighted;
+    QList<Feature*> Found;
 
     MainWindow* Main;
     Ui::FeaturesDockWidget ui;
@@ -67,11 +71,13 @@ private:
     QAction* deleteAction;
 
     CoordBox theViewport;
-    Feature::FeatureType curFeatType;
+    IFeature::FeatureType curFeatType;
 
     void clearItems();
     void addItem(MapFeaturePtr F);
 
+    bool findMode;
+
 public:
     void changeEvent(QEvent*);
     void retranslateUi();
