@@ -41,7 +41,7 @@ bool GeoImageDock::getWalkingPapersDetails(const QUrl& reqUrl, double &lat, doub
             &q, SLOT(quit()));
     QNetworkReply *reply = manager.get(QNetworkRequest(reqUrl));
 
-    tT.start(10000); // 10s timeout
+    tT.start(M_PREFS->getNetworkTimeout());
     q.exec();
     if(tT.isActive()) {
         // download complete
@@ -87,23 +87,30 @@ GeoImageDock::GeoImageDock(MainWindow *aMain)
     setWidget(Image);
     setObjectName("geoImageDock");
 
+    setFocusPolicy(Qt::StrongFocus);
     setContextMenuPolicy(Qt::ActionsContextMenu);
 
     centerAction = new QAction(tr("Center map"), this);
     remImagesAction = new QAction(tr("Remove Images"), this);
     toClipboardAction = new QAction(tr("Copy filename to clipboard"), this);
-    nextImageAction = new QAction(tr("Select next image"), Main); // make it available from everywhere
+    nextImageAction = new QAction(tr("Select next image"), this);
     nextImageAction->setShortcut(tr("PgDown"));
-    previousImageAction = new QAction(tr("Select previous image"), Main); // make it available from everywhere
+    nextImageAction->setShortcutContext(Qt::WidgetWithChildrenShortcut);
+    previousImageAction = new QAction(tr("Select previous image"), this);
     previousImageAction->setShortcut(tr("PgUp"));
+    previousImageAction->setShortcutContext(Qt::WidgetWithChildrenShortcut);
     saveImageAction = new QAction(tr("Save geotagged image..."), this);
 
+    QAction* sep = new QAction(this);
+    sep->setSeparator(true);
+
     addAction(centerAction);
-    addAction(remImagesAction);
     addAction(nextImageAction);
     addAction(previousImageAction);
     addAction(toClipboardAction);
     addAction(saveImageAction);
+    addAction(sep);
+    addAction(remImagesAction);
 
     connect(centerAction, SIGNAL(triggered()), this, SLOT(centerMap()));
     connect(remImagesAction, SIGNAL(triggered()), this, SLOT(removeImages()));
@@ -182,6 +189,11 @@ void GeoImageDock::setImage(int ImageId)
                 if (usedTrackPoints.size()) {
                     if (++lookImage >= usedTrackPoints.size())
                         lookImage = 0;
+                    if (lookImage == ImageId) {
+                        Image->setImage("");
+                        curImage = -1;
+                        return;
+                    }
                 } else
                     break;
             } else {
@@ -353,8 +365,8 @@ void GeoImageDock::loadImage(QString file, Coord pos)
     }
 
     if (theLayer->isReadonly()) { // nodes from readonly layers can not be selected and therefore associated images can not be displayed
-        if (QMessageBox::question(this, tr("Layer is readonly"),
-         tr("The used layer is not writeable. Should it be made writeable?\nIf not, you can't load images that belongs to it."),
+        if (QMessageBox::question(this, tr("Layer is read-only"),
+         tr("The used layer is not writeable. Should it be made writeable?\nIf not, you can't load images that belong to it."),
          QMessageBox::Yes | QMessageBox::Cancel, QMessageBox::Yes) == QMessageBox::Yes)
             theLayer->getWidget()->setLayerReadonly(false); // this makes/updates both the widget and the layer with readonly = false
         else
@@ -369,8 +381,11 @@ void GeoImageDock::loadImage(QString file, Coord pos)
             break;
         else
             Pt = 0;
-    if (!Pt)
+    if (!Pt) {
         Pt = new Node(pos);
+        theLayer->add(Pt);
+        theLayer->indexAdd(Pt->boundingBox(), Pt);
+    }
 
     QDateTime time = QFileInfo(file).created();
 
@@ -378,11 +393,6 @@ void GeoImageDock::loadImage(QString file, Coord pos)
     Pt->setTag("_picture_", "GeoTagged");
     Pt->setPhoto(QPixmap(file));
     addUsedTrackpoint(NodeData(Pt, file, time, i == theLayer->size()));
-    if (i == theLayer->size()) {
-        theLayer->add(Pt);
-        theLayer->indexAdd(Pt->boundingBox(), Pt);
-    }
-
 }
 
 void GeoImageDock::loadImages(QStringList fileNames)
@@ -458,8 +468,8 @@ void GeoImageDock::loadImages(QStringList fileNames)
     }
 
     if (theLayer->isReadonly()) { // nodes from readonly layers can not be selected and therefore associated images can not be displayed
-        if (QMessageBox::question(this, tr("Layer is readonly"),
-         tr("The used layer is not writeable. Should it be made writeable?\nIf not, you can't load images that belongs to it."),
+        if (QMessageBox::question(this, tr("Layer is read-only"),
+         tr("The used layer is not writeable. Should it be made writeable?\nIf not, you can't load images that belong to it."),
          QMessageBox::Yes | QMessageBox::Cancel, QMessageBox::Yes) == QMessageBox::Yes)
             theLayer->getWidget()->setLayerReadonly(false); // this makes/updates both the widget and the layer with readonly = false
         else
@@ -618,23 +628,23 @@ void GeoImageDock::loadImages(QStringList fileNames)
             Coord newPos(angToCoord(lat), angToCoord(lon));
             Node *Pt = 0;
             int i = 0;
-            for (; i<theLayer->size(); ++i) // use existing TrackPoint if there is one in small distance
+            for (; i<theLayer->size(); ++i) { // use existing TrackPoint if there is one in small distance
                 if ((Pt = CAST_NODE(theLayer->get(i))) &&
                  Pt->position().distanceFrom(newPos) <= .002)
                     break;
                 else
                     Pt = 0;
-            if (!Pt)
+            }
+            if (!Pt) {
                 Pt = new Node(newPos);
+                theLayer->add(Pt);
+                theLayer->indexAdd(Pt->boundingBox(), Pt);
+            }
 
             //Pt->setTag("_waypoint_", "true");
             Pt->setTag("_picture_", "GeoTagged");
             Pt->setPhoto(QPixmap(file));
             addUsedTrackpoint(NodeData(Pt, file, time, i == theLayer->size()));
-            if (i == theLayer->size()) {
-                theLayer->add(Pt);
-                theLayer->indexAdd(Pt->boundingBox(), Pt);
-            }
         } else if (!time.isNull() && res == 2) {
 
             if (offset == -1) { // ask the user to specify an offset for the images
@@ -771,7 +781,17 @@ void GeoImageDock::saveImage()
 
     QFileInfo fi(usedTrackPoints.at(index).filename);
     QString fn = fi.absoluteFilePath() + "/" + fi.completeBaseName() + ".jpg";
-    fn = QFileDialog::getSaveFileName(0, "Specify output filename", fn, tr("JPEG Images (*.jpg)"));
+
+    QFileDialog dlg(0, tr("Specify output filename"), fn, tr("JPEG Images (*.jpg)"));
+    dlg.setFileMode(QFileDialog::AnyFile);
+    dlg.setDefaultSuffix("jpg");
+    dlg.setAcceptMode(QFileDialog::AcceptSave);
+
+    if (dlg.exec()) {
+        if (dlg.selectedFiles().size())
+            fn = dlg.selectedFiles()[0];
+    }
+//    fn = QFileDialog::getSaveFileName(0, "Specify output filename", fn, tr("JPEG Images (*.jpg)"));
     qDebug() << fn;
     if (!fn.isEmpty()) {
         Exiv2::Image::AutoPtr imageIn, imageOut;
