@@ -3,6 +3,7 @@
 #include "MainWindow.h"
 #include "ui_MainWindow.h"
 #include "MapView.h"
+#include "Interaction.h"
 #include "TagModel.h"
 #include "Utils/EditCompleterDelegate.h"
 #include "Utils/ShortcutOverrideFilter.h"
@@ -50,7 +51,6 @@ PropertiesDock::PropertiesDock(MainWindow* aParent)
     shortcutFilter->addOverride(Qt::Key_Right);
     shortcutFilter->addOverride(Qt::Key_F2);
     shortcutFilter->addOverride(Qt::Key_Delete);
-    shortcutFilter->addOverride(Qt::Key_Escape);
 
     centerAction = new QAction(NULL, this);
     connect(centerAction, SIGNAL(triggered()), this, SLOT(on_centerAction_triggered()));
@@ -91,18 +91,11 @@ static bool isChildOfSingleRoad(Feature *mapFeature)
 
 static bool isChildOfSingleRelation(Feature *mapFeature)
 {
-    int parents = mapFeature->sizeParents();
-
-    if (parents == 0)
-        return false;
-
     int parentRelations = 0;
-
-    int i;
-    for (i=0; i<parents; i++)
+    for (int i=0; i<mapFeature->sizeParents(); i++)
     {
-        Feature * parent = mapFeature->getParent(i);
-        if (parent->isDeleted()) continue;
+        Feature * parent = CAST_FEATURE(mapFeature->getParent(i));
+        if (!parent || parent->isDeleted()) continue;
 
         bool isParentRelation = dynamic_cast<Relation*>(parent) != 0;
         if (isParentRelation)
@@ -116,18 +109,10 @@ static bool isChildOfSingleRelation(Feature *mapFeature)
 
 static bool isChildOfRelation(Feature *mapFeature)
 {
-    int parents = mapFeature->sizeParents();
-
-    if (parents == 0)
-        return false;
-
-    int i;
-    for (i=0; i<parents; i++)
+    for (int i=0; i<mapFeature->sizeParents(); i++)
     {
-        Feature * parent = mapFeature->getParent(i);
-        if (parent->isDeleted()) continue;
-
-        if (dynamic_cast<Relation*>(parent))
+        Relation * rel = CAST_RELATION(mapFeature->getParent(i));
+        if (rel && !rel->isDeleted())
             return true;
     }
 
@@ -160,7 +145,7 @@ void PropertiesDock::checkMenuStatus()
         IsParentRelation = isChildOfSingleRelation(Selection[0]);
         IsParentArea = isChildOfArea(Selection[0]);
         IsOpenStreetBug = isChildOfArea(Selection[0]);
-        IsOpenStreetBug = IsPoint && Selection[0]->id().startsWith("osb_");
+        IsOpenStreetBug = IsPoint && (Selection[0]->id().type & IFeature::Special);
     }
     for (int i=0; i<Selection.size(); ++i)
     {
@@ -182,23 +167,30 @@ void PropertiesDock::checkMenuStatus()
         if (isChildOfRelation(Selection[i]))
             ++NumRelationChild;
 
-        if (!Selection[i]->isDirty() && !Selection[i]->isSpecial())
+        if (!Selection[i]->isUploadable() && !Selection[i]->isSpecial())
             ++NumCommitableFeature;
     }
     Main->ui->createRelationAction->setEnabled(Selection.size());
     Main->ui->editRemoveAction->setEnabled(Selection.size());
     Main->ui->editMoveAction->setEnabled(true);
-    Main->ui->editReverseAction->setEnabled(IsRoad);
-    Main->ui->roadAddStreetNumbersAction->setEnabled(NumRoads > 1);
+    Main->ui->editReverseAction->setEnabled(IsRoad || NumAreas + NumRoads > 0);
+    Main->ui->roadAddStreetNumbersAction->setEnabled(NumRoads >= 1);
     Main->ui->roadJoinAction->setEnabled(NumRoads >= 1 && canJoinRoads(this));
     Main->ui->roadCreateJunctionAction->setEnabled(NumRoads > 1 && canCreateJunction(this));
     Main->ui->roadSplitAction->setEnabled((IsParentRoadInner && !IsParentArea) || (NumRoads && NumPoints) || (NumAreas && NumPoints > 1));
     Main->ui->roadBreakAction->setEnabled(IsParentRoadInner || ((NumRoads == 1 || NumAreas == 1) && NumPoints) || (NumRoads > 1 && canBreakRoads(this)));
     Main->ui->roadSimplifyAction->setEnabled(IsRoad || NumRoads > 0 || NumAreas > 0);
+    Main->ui->roadSubdivideAction->setEnabled((NumRoads + NumAreas) == 1 && (!NumPoints || NumPoints == 2) && canSubdivideRoad(this));
+    Main->ui->roadAxisAlignAction->setEnabled((NumRoads + NumAreas) > 0 || (NumRelation > 0 && canAxisAlignRoads(this)));
+    Main->ui->areaJoinAction->setEnabled(NumAreas > 1);
+    Main->ui->areaSplitAction->setEnabled(NumAreas == 1 && NumPoints == 2 && canSplitArea(this));
+    Main->ui->areaTerraceAction->setEnabled(NumAreas == 1 && NumRoads == 0 && canTerraceArea(this));
+    Main->ui->featureSelectChildrenAction->setEnabled(Selection.size());
     Main->ui->featureDeleteAction->setEnabled((IsPoint || IsRoad || IsRelation) && !Selection[0]->isDirty());
     Main->ui->featureCommitAction->setEnabled(NumCommitableFeature);
     Main->ui->nodeMergeAction->setEnabled(NumPoints > 1);
     Main->ui->nodeAlignAction->setEnabled(NumPoints > 2);
+    Main->ui->nodeSpreadAction->setEnabled(NumPoints > 2);
     Main->ui->nodeDetachAction->setEnabled(NumPoints && canDetachNodes(this));
     Main->ui->relationAddMemberAction->setEnabled(NumRelation && Selection.size() > 1);
     Main->ui->relationRemoveMemberAction->setEnabled((NumRelation && Selection.size() > 1 && NumRelationChild) || IsParentRelation);
@@ -391,9 +383,9 @@ void PropertiesDock::switchUi()
         if (FullSelection[0]->isVirtual())
             switchToNoUi();
         else if (CAST_NODE(FullSelection[0]))
-            switchToTrackPointUi();
+            switchToNodeUi();
         else if (CAST_WAY(FullSelection[0]))
-            switchToRoadUi();
+            switchToWayUi();
         else if (CAST_RELATION(FullSelection[0]))
             switchToRelationUi();
         else
@@ -424,7 +416,7 @@ void PropertiesDock::switchToMultiUi()
     setWindowTitle(tr("Properties - Multiple elements"));
 }
 
-void PropertiesDock::switchToTrackPointUi()
+void PropertiesDock::switchToNodeUi()
 {
     NowShowing = TrackPointUiShowing;
     QWidget* NewUi = new QWidget(this);
@@ -437,10 +429,11 @@ void PropertiesDock::switchToTrackPointUi()
     connect(TrackPointUi.Longitude,SIGNAL(editingFinished()),this, SLOT(on_TrackPointLon_editingFinished()));
     connect(TrackPointUi.Latitude,SIGNAL(editingFinished()),this, SLOT(on_TrackPointLat_editingFinished()));
     connect(TrackPointUi.RemoveTagButton,SIGNAL(clicked()),this, SLOT(on_RemoveTagButton_clicked()));
+    connect(TrackPointUi.SourceTagButton,SIGNAL(clicked()),this, SLOT(on_SourceTagButton_clicked()));
     setWindowTitle(tr("Properties - Node"));
 }
 
-void PropertiesDock::switchToRoadUi()
+void PropertiesDock::switchToWayUi()
 {
     NowShowing = RoadUiShowing;
     QWidget* NewUi = new QWidget(this);
@@ -451,6 +444,7 @@ void PropertiesDock::switchToRoadUi()
         CurrentUi->deleteLater();
     CurrentUi = NewUi;
     connect(RoadUi.RemoveTagButton,SIGNAL(clicked()),this, SLOT(on_RemoveTagButton_clicked()));
+    connect(RoadUi.SourceTagButton,SIGNAL(clicked()),this, SLOT(on_SourceTagButton_clicked()));
     setWindowTitle(tr("Properties - Road"));
 }
 
@@ -469,6 +463,7 @@ void PropertiesDock::switchToRelationUi()
     connect(RelationUi.MembersView, SIGNAL(clicked(QModelIndex)), this, SLOT(on_Member_clicked(QModelIndex)));
     connect(RelationUi.RemoveMemberButton,SIGNAL(clicked()),this, SLOT(on_RemoveMemberButton_clicked()));
     connect(RelationUi.RemoveTagButton,SIGNAL(clicked()),this, SLOT(on_RemoveTagButton_clicked()));
+    connect(RelationUi.SourceTagButton,SIGNAL(clicked()),this, SLOT(on_SourceTagButton_clicked()));
     connect(RelationUi.btMemberUp, SIGNAL(clicked()), SLOT(on_btMemberUp_clicked()));
     connect(RelationUi.btMemberDown, SIGNAL(clicked()), SLOT(on_btMemberDown_clicked()));
     setWindowTitle(tr("Properties - Relation"));
@@ -507,9 +502,9 @@ void PropertiesDock::resetValues()
 
         if ((Pt) && (NowShowing == TrackPointUiShowing))
         {
-            TrackPointUi.Id->setText(Pt->id());
-            TrackPointUi.Latitude->setText(QString::number(coordToAng(Pt->position().lat()),'g',8));
-            TrackPointUi.Longitude->setText(QString::number(coordToAng(Pt->position().lon()),'g',8));
+            TrackPointUi.Id->setText(QString::number(Pt->id().numId));
+            TrackPointUi.Latitude->setText(COORD2STRING(coordToAng(Pt->position().lat())));
+            TrackPointUi.Longitude->setText(COORD2STRING(coordToAng(Pt->position().lon())));
             TrackPointUi.TagView->setModel(theModel);
             TrackPointUi.TagView->setItemDelegate(delegate);
 
@@ -522,7 +517,7 @@ void PropertiesDock::resetValues()
                 }
             }
             if (theTemplates) {
-                w = theTemplates->getWidget(Pt);
+                w = theTemplates->getWidget(Pt, Main->view());
                 w->installEventFilter(shortcutFilter);
                 TrackPointUi.variableLayout->addWidget(w);
             }
@@ -535,7 +530,7 @@ void PropertiesDock::resetValues()
         }
         else if ((R) && (NowShowing == RoadUiShowing))
         {
-            RoadUi.Id->setText(R->id());
+            RoadUi.Id->setText(QString::number(R->id().numId));
             //RoadUi.Name->setText(R->tagValue("name",""));
             RoadUi.TagView->setModel(theModel);
             RoadUi.TagView->setItemDelegate(delegate);
@@ -549,7 +544,7 @@ void PropertiesDock::resetValues()
                 }
             }
             if (theTemplates) {
-                w = theTemplates->getWidget(R);
+                w = theTemplates->getWidget(R, Main->view());
                 w->installEventFilter(shortcutFilter);
                 RoadUi.variableLayout->addWidget(w);
             }
@@ -571,7 +566,7 @@ void PropertiesDock::resetValues()
                 }
             }
             if (theTemplates) {
-                w = theTemplates->getWidget(L);
+                w = theTemplates->getWidget(L, Main->view());
                 w->installEventFilter(shortcutFilter);
                 RelationUi.variableLayout->addWidget(w);
             }
@@ -601,7 +596,7 @@ void PropertiesDock::resetValues()
     /* first column is the width of the default text (Edit this to add...)  */
     /* And the rest of the space is assigned to the second column           */
     if (CurrentTagView) {
-        if (M_PREFS->getTagListFirstColumnWidth())
+        if (M_PREFS->getTagListFirstColumnWidth() && M_PREFS->getTagListFirstColumnWidth() > 0 && M_PREFS->getTagListFirstColumnWidth() < CurrentTagView->width())
             CurrentTagView->setColumnWidth(
                 0, M_PREFS->getTagListFirstColumnWidth()
             );
@@ -696,15 +691,15 @@ void PropertiesDock::on_RemoveTagButton_clicked()
         if (Content.isValid())
         {
             QString KeyName = Content.toString();
-            L = new CommandList(MainWindow::tr("Clear Tag '%1' on %2").arg(KeyName).arg(Selection[0]->id()), Selection[0]);
+            L = new CommandList(MainWindow::tr("Clear Tag '%1' on %2").arg(KeyName).arg(Selection[0]->id().numId), Selection[0]);
             for (int i=0; i<Selection.size(); ++i)
-                if (Selection[i]->findKey(KeyName) < Selection[i]->tagSize())
+                if (Selection[i]->findKey(KeyName) != -1)
                     L->add(new ClearTagCommand(Selection[i],KeyName,Main->document()->getDirtyOrOriginLayer(Selection[i]->layer())));
         }
     }
     else
     {
-        L = new CommandList(MainWindow::tr("Clear %1 tags on %2").arg(indexes.count()).arg(Selection[0]->id()), Selection[0]);
+        L = new CommandList(MainWindow::tr("Clear %1 tags on %2").arg(indexes.count()).arg(Selection[0]->id().numId), Selection[0]);
         while (!indexes.isEmpty()) {
             QModelIndex index = indexes.takeLast();
             QModelIndex idx = index.sibling(index.row(),0);
@@ -713,7 +708,7 @@ void PropertiesDock::on_RemoveTagButton_clicked()
             {
                 QString KeyName = Content.toString();
                 for (int i=0; i<Selection.size(); ++i)
-                    if (Selection[i]->findKey(KeyName) < Selection[i]->tagSize())
+                    if (Selection[i]->findKey(KeyName) != -1)
                         L->add(new ClearTagCommand(Selection[i],KeyName,Main->document()->getDirtyOrOriginLayer(Selection[i]->layer())));
             }
         }
@@ -727,6 +722,22 @@ void PropertiesDock::on_RemoveTagButton_clicked()
     }
 }
 
+void PropertiesDock::on_SourceTagButton_clicked()
+{
+    QStringList sl = Main->document()->getCurrentSourceTags();
+    if (!sl.size())
+        return;
+
+    QString src = Selection[0]->tagValue("source", "");
+    if (!src.isEmpty())
+        sl.prepend(src);
+
+    CommandList* L = new CommandList(MainWindow::tr("Set \"source\" tag on %1").arg(Selection[0]->id().numId), Selection[0]);
+    L->add(new SetTagCommand(Selection[0], "source", sl.join(";")));
+    Main->document()->addHistory(L);
+    Main->invalidateView();
+}
+
 void PropertiesDock::on_RemoveMemberButton_clicked()
 {
     if (CurrentMembersView)
@@ -821,7 +832,7 @@ void PropertiesDock::on_btMemberUp_clicked()
 {
     Relation* R = dynamic_cast<Relation*>(Selection[0]);
     if (R) {
-        CommandList* theList = new CommandList(MainWindow::tr("Reorder members in relation %1").arg(R->id()), R);
+        CommandList* theList = new CommandList(MainWindow::tr("Reorder members in relation %1").arg(R->id().numId), R);
 
         QModelIndex index;
         foreach(index, CurrentMembersView->selectionModel()->selectedIndexes())
@@ -866,7 +877,7 @@ void PropertiesDock::on_btMemberDown_clicked()
 {
     Relation* R = dynamic_cast<Relation*>(Selection[0]);
     if (R) {
-        CommandList* theList = new CommandList(MainWindow::tr("Reorder members in relation %1").arg(R->id()), R);
+        CommandList* theList = new CommandList(MainWindow::tr("Reorder members in relation %1").arg(R->id().numId), R);
 
         QModelIndex index;
         foreach(index, CurrentMembersView->selectionModel()->selectedIndexes())
@@ -1016,7 +1027,7 @@ bool PropertiesDock::loadTemplates(const QString& filename)
         File.setFileName(M_PREFS->getDefaultTemplate());
 
     if (!File.open(QIODevice::ReadOnly)) {
-        QMessageBox::warning(Main,"Template read error", "Error reading template file");
+        qDebug() << tr("Error reading template file");
         return false;
     }
 
