@@ -1,21 +1,32 @@
+#include "Global.h"
+
 #include "Command.h"
 
 #include "Feature.h"
 #include "Document.h"
+#include "OsbLayer.h"
 #include "ImageMapLayer.h"
 
 #include "ImportNMEA.h"
 #include "ImportExportOsmBin.h"
 #include "ImportExportKML.h"
-#include "ImportExportSHP.h"
 #include "ImportExportCSV.h"
 #include "ImportExportOSC.h"
+#include "ImportExportGdal.h"
 
 #include "LayerWidget.h"
 
+#include "Utils/TagSelector.h"
+#include "PaintStyle/IPaintStyle.h"
+#include "PaintStyle/FeaturePainter.h"
+
+#include "LayerIterator.h"
+#include "IMapAdapter.h"
+
 #include <QtCore/QString>
 #include <QMultiMap>
 #include <QProgressDialog>
+#include <QClipboard>
 
 #include <QMap>
 #include <QList>
@@ -29,7 +40,14 @@ class MapDocumentPrivate
 {
 public:
     MapDocumentPrivate()
-    : History(new CommandHistory()), dirtyLayer(0), uploadedLayer(0)/*, trashLayer(0)*/, theDock(0), lastDownloadLayer(0)
+        : History(new CommandHistory())
+        , dirtyLayer(0)
+        , uploadedLayer(0)
+        /*, trashLayer(0)*/
+        , theDock(0)
+        , lastDownloadLayer(0)
+        , tagFilter(0), FilterRevision(0)
+        , layerNum(0)
     {
     };
     ~MapDocumentPrivate()
@@ -43,35 +61,52 @@ public:
             delete Layers[i];
         }
     }
-    CommandHistory*				History;
-    QList<Layer*>		Layers;
-    DirtyLayer*				dirtyLayer;
-    UploadedLayer*			uploadedLayer;
-    LayerDock*					theDock;
-    Layer*					lastDownloadLayer;
+    CommandHistory*	History;
+    QList<Layer*> Layers;
+    DirtyLayer*	dirtyLayer;
+    UploadedLayer* uploadedLayer;
+    LayerDock*	theDock;
+    Layer*	lastDownloadLayer;
+    QDateTime lastDownloadTimestamp;
+    QHash<Layer*, CoordBox>	downloadBoxes;
 
-    QHash<Layer*, CoordBox>		downloadBoxes;
+    TagSelector* tagFilter;
+    int FilterRevision;
+    QString title;
+    int layerNum;
+    mutable QString Id;
 
-    QHash< QString, QSet<QString> * >		tagList;
+    QList<FeaturePainter> theFeaturePainters;
 
 };
 
 Document::Document()
     : p(new MapDocumentPrivate)
 {
-    if (!(M_PREFS->apiVersionNum() > 0.5))
-        addToTagList("created_by", QString("Merkaartor v%1%2").arg(STRINGIFY(VERSION)).arg(STRINGIFY(REVISION)));
+    setFilterType(M_PREFS->getCurrentFilter());
+    p->title = tr("untitled");
+
+    for (int i=0; i<M_STYLE->painterSize(); ++i) {
+        p->theFeaturePainters.append(FeaturePainter(*M_STYLE->getPainter(i)));
+    }
 }
 
 Document::Document(LayerDock* aDock)
-: p(new MapDocumentPrivate)
+    : p(new MapDocumentPrivate)
 {
     p->theDock = aDock;
+    setFilterType(M_PREFS->getCurrentFilter());
+    p->title = tr("untitled");
+
+    for (int i=0; i<M_STYLE->painterSize(); ++i) {
+        p->theFeaturePainters.append(FeaturePainter(*M_STYLE->getPainter(i)));
+    }
 }
 
 Document::Document(const Document&, LayerDock*)
 : p(0)
 {
+    p->title = tr("untitled");
 }
 
 Document::~Document()
@@ -79,26 +114,69 @@ Document::~Document()
     delete p;
 }
 
-void Document::addDefaultLayers()
+const QString& Document::id() const
 {
-    ImageMapLayer*l = addImageLayer();
-    l->setMapAdapter(M_PREFS->getBackgroundPlugin(), M_PREFS->getSelectedServer());
-    if (M_PREFS->getBackgroundPlugin() != NONE_ADAPTER_UUID) {
-        l->setVisible(M_PREFS->getBgVisible());
-        // Sync the menu entry label & visible checkbox to the layer
-        QMenu *lMenu = l->getWidget()->getAssociatedMenu();
-        lMenu->menuAction()->menu()->actions().at(0)->setChecked(M_PREFS->getBgVisible());
-        lMenu->setTitle(l->name());
-    }
-
-    p->dirtyLayer = new DirtyLayer(tr("Dirty layer"));
-    add(p->dirtyLayer);
-
-    p->uploadedLayer = new UploadedLayer(tr("Uploaded layer"));
-    add(p->uploadedLayer);
+    if (p->Id.isEmpty())
+        p->Id = QUuid::createUuid().toString();
+    return p->Id;
 }
 
-bool Document::toXML(QDomElement xParent, QProgressDialog * progress)
+void Document::setPainters(QList<Painter> aPainters)
+{
+    p->theFeaturePainters.clear();
+    for (int i=0; i<aPainters.size(); ++i) {
+        FeaturePainter fp(aPainters[i]);
+        p->theFeaturePainters.append(fp);
+    }
+}
+
+int Document::getPaintersSize()
+{
+    return p->theFeaturePainters.size();
+}
+
+const Painter* Document::getPainter(int i)
+{
+    return &p->theFeaturePainters[i];
+}
+
+void Document::addDefaultLayers()
+{
+    /*ImageMapLayer*l = */addImageLayer();
+//    l->setMapAdapter(M_PREFS->getBackgroundPlugin(), M_PREFS->getSelectedServer());
+//    if (M_PREFS->getBackgroundPlugin() != NONE_ADAPTER_UUID) {
+//        l->setVisible(M_PREFS->getBgVisible());
+//        // Sync the menu entry label & visible checkbox to the layer
+//        QMenu *lMenu = l->getWidget()->getAssociatedMenu();
+//        lMenu->menuAction()->menu()->actions().at(0)->setChecked(M_PREFS->getBgVisible());
+//        lMenu->setTitle(l->name());
+//    }
+
+    if (g_Merk_Frisius) {
+        DrawingLayer* aLayer = addDrawingLayer();
+        setLastDownloadLayer(aLayer);
+    } else {
+        p->dirtyLayer = new DirtyLayer(tr("Dirty layer"));
+        add(p->dirtyLayer);
+
+        p->uploadedLayer = new UploadedLayer(tr("Uploaded layer"));
+        add(p->uploadedLayer);
+    }
+
+    addFilterLayers();
+}
+
+void Document::addFilterLayers()
+{
+    foreach (FilterItem it, *M_PREFS->getFiltersList()->getFilters()) {
+        if (it.deleted)
+            continue;
+        FilterLayer* f = new FilterLayer(it.id.toString(), it.name, it.filter);
+        addFilterLayer(f);
+    }
+}
+
+bool Document::toXML(QDomElement xParent, bool asTemplate, QProgressDialog * progress)
 {
     bool OK = true;
 
@@ -109,55 +187,72 @@ bool Document::toXML(QDomElement xParent, QProgressDialog * progress)
     mapDoc = xParent.ownerDocument().createElement("MapDocument");
     xParent.appendChild(mapDoc);
 
-    if (p->lastDownloadLayer)
+    mapDoc.setAttribute("xml:id", id());
+    if (!asTemplate)
+        mapDoc.setAttribute("layernum", p->layerNum);
+    if (p->lastDownloadLayer) {
         mapDoc.setAttribute("lastdownloadlayer", p->lastDownloadLayer->id());
-
-    for (int i=0; i<p->Layers.size(); ++i) {
-        progress->setMaximum(progress->maximum() + p->Layers[i]->size());
+        mapDoc.setAttribute("lastdownloadtimestamp", p->lastDownloadTimestamp.toUTC().toString(Qt::ISODate)+"Z");
     }
 
     for (int i=0; i<p->Layers.size(); ++i) {
-        p->Layers[i]->toXML(mapDoc, progress);
+        progress->setMaximum(progress->maximum() + p->Layers[i]->getDisplaySize());
     }
 
-    OK = history().toXML(mapDoc, progress);
+    for (int i=0; i<p->Layers.size(); ++i) {
+        if (p->Layers[i]->isEnabled()) {
+            if (asTemplate && p->Layers[i]->classType() == Layer::DrawingLayerType)
+                continue;
+            p->Layers[i]->toXML(mapDoc, asTemplate, progress);
+        }
+    }
+
+    if (!asTemplate) {
+        OK = history().toXML(mapDoc, progress);
+    }
 
     return OK;
 }
 
-Document* Document::fromXML(const QDomElement e, double version, LayerDock* aDock, QProgressDialog * progress)
+Document* Document::fromXML(QString title, const QDomElement e, double version, LayerDock* aDock, QProgressDialog * progress)
 {
     Document* NewDoc = new Document(aDock);
+    NewDoc->p->title = title;
 
     CommandHistory* h = 0;
 
+    if (e.hasAttribute("xml:id"))
+        NewDoc->p->Id = e.attribute("xml:id");
+    if (e.hasAttribute("layernum"))
+        NewDoc->p->layerNum = e.attribute("layernum").toInt();
+    else
+        NewDoc->p->layerNum = 1;
+    if (e.hasAttribute("lastdownloadlayer")) {
+        NewDoc->p->lastDownloadTimestamp = QDateTime::fromString(e.attribute("lastdownloadtimestamp").left(19), Qt::ISODate);
+        NewDoc->p->lastDownloadTimestamp.setTimeSpec(Qt::UTC);
+    }
+
     QDomElement c = e.firstChildElement();
     while(!c.isNull()) {
         if (c.tagName() == "ImageMapLayer") {
             /*ImageMapLayer* l =*/ ImageMapLayer::fromXML(NewDoc, c, progress);
-        } else
-        if (c.tagName() == "DeletedMapLayer") {
+        } else if (c.tagName() == "DeletedMapLayer") {
             /*DeletedMapLayer* l =*/ DeletedLayer::fromXML(NewDoc, c, progress);
-        } else
-        if (c.tagName() == "DirtyLayer" || c.tagName() == "DirtyMapLayer") {
+        } else if (c.tagName() == "DirtyLayer" || c.tagName() == "DirtyMapLayer") {
             /*DirtyMapLayer* l =*/ DirtyLayer::fromXML(NewDoc, c, progress);
-        } else
-        if (c.tagName() == "UploadedLayer" || c.tagName() == "UploadedMapLayer") {
+        } else if (c.tagName() == "UploadedLayer" || c.tagName() == "UploadedMapLayer") {
             /*UploadedMapLayer* l =*/ UploadedLayer::fromXML(NewDoc, c, progress);
-        } else
-        if (c.tagName() == "DrawingLayer" || c.tagName() == "DrawingMapLayer") {
+        } else if (c.tagName() == "DrawingLayer" || c.tagName() == "DrawingMapLayer") {
             /*DrawingMapLayer* l =*/ DrawingLayer::fromXML(NewDoc, c, progress);
-        } else
-        if (c.tagName() == "TrackLayer" || c.tagName() == "TrackMapLayer") {
+        } else if (c.tagName() == "TrackLayer" || c.tagName() == "TrackMapLayer") {
             /*TrackMapLayer* l =*/ TrackLayer::fromXML(NewDoc, c, progress);
-        } else
-        if (c.tagName() == "ExtractedLayer") {
+        } else if (c.tagName() == "ExtractedLayer") {
             /*DrawingMapLayer* l =*/ DrawingLayer::fromXML(NewDoc, c, progress);
-        } else
-        if (c.tagName() == "OsbLayer" || c.tagName() == "OsbMapLayer") {
+        } else if (c.tagName() == "OsbLayer" || c.tagName() == "OsbMapLayer") {
             /*OsbMapLayer* l =*/ OsbLayer::fromXML(NewDoc, c, progress);
-        } else
-        if (c.tagName() == "CommandHistory") {
+        } else if (c.tagName() == "FilterLayer") {
+            /*OsbMapLayer* l =*/ FilterLayer::fromXML(NewDoc, c, progress);
+        } else if (c.tagName() == "CommandHistory") {
             if (version > 1.0)
                 h = CommandHistory::fromXML(NewDoc, c, progress);
         }
@@ -168,15 +263,17 @@ Document* Document::fromXML(const QDomElement e, double version, LayerDock* aDoc
         c = c.nextSiblingElement();
     }
 
+    if (e.hasAttribute("lastdownloadlayer")) {
+        NewDoc->p->lastDownloadLayer = NewDoc->getLayer(e.attribute("lastdownloadlayer"));
+    }
+
+
     if (progress->wasCanceled()) {
         delete NewDoc;
         NewDoc = NULL;
     }
 
     if (NewDoc) {
-        if (e.hasAttribute("lastdownloadlayer"))
-            NewDoc->setLastDownloadLayer(NewDoc->getLayer(e.attribute("lastdownloadlayer")));
-
         if (h)
             NewDoc->setHistory(h);
 
@@ -270,55 +367,28 @@ ImageMapLayer* Document::addImageLayer(ImageMapLayer* aLayer)
     return theLayer;
 }
 
-void Document::addToTagList(QString k, QString v)
+DrawingLayer* Document::addDrawingLayer(DrawingLayer *aLayer)
 {
-#ifndef _MOBILE
-    bool ok;
-    v.toDouble(&ok);
-    if (ok)
-        return;
-    if (p->tagList.contains(k)) {
-        if (!p->tagList.value(k)->contains(v)) {
-            //static_cast< QSet<QString> * >(p->tagList.value(k))->insert(v);
-            p->tagList.value(k)->insert(v);
-        }
-    } else {
-        QSet<QString> *values = new QSet<QString>;
-        values->insert(v);
-        p->tagList.insert(k, values);
+    DrawingLayer* theLayer = aLayer;
+    if (!theLayer)
+        theLayer = new DrawingLayer(tr("Drawing layer #%1").arg(p->layerNum++));
+    add(theLayer);
+    return theLayer;
+}
+
+FilterLayer* Document::addFilterLayer(FilterLayer *aLayer)
+{
+    FilterLayer* theLayer = aLayer;
+    if (!theLayer)
+        theLayer = new FilterLayer(QUuid::createUuid(), tr("Filter layer #%1").arg(++p->layerNum), "false");
+    add(theLayer);
+
+    FeatureIterator it(this);
+    for(;!it.isEnd(); ++it) {
+        it.get()->updateFilters();
     }
-#endif
-}
 
-QList<QString> Document::getTagKeys()
-{
-    return p->tagList.keys();
-}
-
-QList<QString> Document::getTagValues()
-{
-    return getTagValueList("*");
-}
-
-QStringList Document::getTagList()
-{
-    return p->tagList.uniqueKeys();
-}
-
-QStringList Document::getTagValueList(QString k)
-{
-    if (k == "*") {
-        QSet<QString> allValues;
-        QSet<QString> *tagValues;
-        foreach (tagValues, p->tagList) {
-            allValues += *tagValues;
-        }
-        return allValues.toList();
-    } else if (p->tagList.contains(k)) {
-        return p->tagList.value(k)->toList();
-    } else {
-        return QStringList();
-    }
+    return theLayer;
 }
 
 void Document::remove(Layer* aLayer)
@@ -393,23 +463,15 @@ QList<Feature*> Document::getFeatures(Layer::LayerType layerType)
     return theFeatures;
 }
 
-Feature* Document::getFeature(const QString& id, bool exact)
+Feature* Document::getFeature(const IFeature::FId& id)
 {
     for (int i=0; i<p->Layers.size(); ++i)
     {
         Feature* F = p->Layers[i]->get(id);
         if (F)
             return F;
-        if (!exact) {
-            if ((F = p->Layers[i]->get("node_"+id)))
-                return F;
-            if ((F = p->Layers[i]->get("way_"+id)))
-                return F;
-            if ((F = p->Layers[i]->get("rel_"+id)))
-                return F;
-        }
     }
-    return 0;
+    return NULL;
 }
 
 void Document::setDirtyLayer(DirtyLayer* aLayer)
@@ -417,22 +479,68 @@ void Document::setDirtyLayer(DirtyLayer* aLayer)
     p->dirtyLayer = aLayer;
 }
 
-Layer* Document::getDirtyOrOriginLayer(Layer* aLayer) const
+Layer* Document::getDirtyLayer()
 {
-    if (!aLayer || aLayer->isUploadable())
-        return p->dirtyLayer;
-    else
-        return aLayer;
+    if (!p->dirtyLayer) {
+        p->dirtyLayer = new DirtyLayer(tr("Dirty layer"));
+        add(p->dirtyLayer);
+    }
+    return p->dirtyLayer;
 }
 
-Layer* Document::getDirtyOrOriginLayer(Feature* F) const
+Layer* Document::getDirtyOrOriginLayer(Layer* aLayer)
 {
+    if (g_Merk_Frisius) {
+        if (aLayer)
+            return aLayer;
+        else if (p->lastDownloadLayer)
+            return p->lastDownloadLayer;
+        else {
+            DrawingLayer* firstDrLayer = NULL;
+            for (int i=0; i<layerSize(); ++i) {
+                if (!getLayer(i)->isEnabled())
+                    continue;
+                if (getLayer(i)->classType() == Layer::DrawingLayerType) {
+                    if (!firstDrLayer)
+                        firstDrLayer = dynamic_cast<DrawingLayer*>(getLayer(i));
+                    if (getLayer(i)->isSelected())
+                        return (Layer*)getLayer(i);
+                }
+            }
+            if (firstDrLayer)
+                return firstDrLayer;
+            else
+                return addDrawingLayer();
+        }
+    } else {
+        if (!aLayer || (aLayer && !aLayer->isUploadable()))
+            return p->dirtyLayer;
+        else
+            return aLayer;
+    }
+}
+
+Layer* Document::getDirtyOrOriginLayer(Feature* F)
+{
+    if (g_Merk_Frisius) {
+        return getDirtyOrOriginLayer(F->layer());
+    }
+
     if (!F || !F->layer() || F->layer()->isUploadable())
         return p->dirtyLayer;
     else
         return F->layer();
 }
 
+int Document::getDirtySize() const
+{
+    int dirtyObjects = 0;
+    for (int i=0; i<layerSize(); ++i) {
+        dirtyObjects += getLayer(i)->getDirtySize();
+    }
+    return dirtyObjects;
+}
+
 void Document::setUploadedLayer(UploadedLayer* aLayer)
 {
     p->uploadedLayer = aLayer;
@@ -509,7 +617,7 @@ QString Document::exportOSM(QMainWindow* main, const CoordBox& aCoordBox, bool r
     QDomElement o = theXmlDoc.createElement("osm");
     theXmlDoc.appendChild(o);
     o.setAttribute("version", "0.6");
-    o.setAttribute("generator", QString("Merkaartor %1").arg(STRINGIFY(VERSION)));
+    o.setAttribute("generator", QString("%1 %2").arg(qApp->applicationName()).arg(STRINGIFY(VERSION)));
 
     QDomElement bb = theXmlDoc.createElement("bound");
     o.appendChild(bb);
@@ -524,10 +632,10 @@ QString Document::exportOSM(QMainWindow* main, const CoordBox& aCoordBox, bool r
         QDomElement bnds = theXmlDoc.createElement("bounds");
         o.appendChild(bnds);
 
-        bnds.setAttribute("minlat", QString::number(coordToAng(aCoordBox.bottomLeft().lat()),'f',6));
-        bnds.setAttribute("minlon", QString::number(coordToAng(aCoordBox.bottomLeft().lon()),'f',6));
-        bnds.setAttribute("maxlat", QString::number(coordToAng(aCoordBox.topRight().lat()),'f',6));
-        bnds.setAttribute("maxlon", QString::number(coordToAng(aCoordBox.topRight().lon()),'f',6));
+        bnds.setAttribute("minlat", COORD2STRING(coordToAng(aCoordBox.bottomLeft().lat())));
+        bnds.setAttribute("minlon", COORD2STRING(coordToAng(aCoordBox.bottomLeft().lon())));
+        bnds.setAttribute("maxlat", COORD2STRING(coordToAng(aCoordBox.topRight().lat())));
+        bnds.setAttribute("maxlon", COORD2STRING(coordToAng(aCoordBox.topRight().lon())));
     }
 
     for (int i=0; i < exportedFeatures.size(); i++) {
@@ -537,9 +645,9 @@ QString Document::exportOSM(QMainWindow* main, const CoordBox& aCoordBox, bool r
     return theXmlDoc.toString(2);
 }
 
-QString Document::exportOSM(QMainWindow* main, QList<Feature*> aFeatures)
+QString Document::exportOSM(QMainWindow* main, QList<Feature*> aFeatures, bool forCopyPaste)
 {
-    QList<Feature*> exportedFeatures = exportCoreOSM(aFeatures);
+    QList<Feature*> exportedFeatures = exportCoreOSM(aFeatures, forCopyPaste);
     CoordBox aCoordBox;
 
     IProgressWindow* aProgressWindow = dynamic_cast<IProgressWindow*>(main);
@@ -569,13 +677,13 @@ QString Document::exportOSM(QMainWindow* main, QList<Feature*> aFeatures)
     QDomElement o = theXmlDoc.createElement("osm");
     theXmlDoc.appendChild(o);
     o.setAttribute("version", "0.6");
-    o.setAttribute("generator", QString("Merkaartor %1").arg(STRINGIFY(VERSION)));
+    o.setAttribute("generator", QString("%1 %2").arg(qApp->applicationName()).arg(STRINGIFY(VERSION)));
 
     if (exportedFeatures.size()) {
-        aCoordBox = exportedFeatures[0]->boundingBox();
+        aCoordBox = exportedFeatures[0]->boundingBox(true);
         exportedFeatures[0]->toXML(o, dlg);
         for (int i=1; i < exportedFeatures.size(); i++) {
-            aCoordBox.merge(exportedFeatures[i]->boundingBox());
+            aCoordBox.merge(exportedFeatures[i]->boundingBox(true));
             exportedFeatures[i]->toXML(o, dlg);
         }
     }
@@ -592,7 +700,7 @@ QString Document::exportOSM(QMainWindow* main, QList<Feature*> aFeatures)
     return theXmlDoc.toString(2);
 }
 
-QList<Feature*> Document::exportCoreOSM(QList<Feature*> aFeatures)
+QList<Feature*> Document::exportCoreOSM(QList<Feature*> aFeatures, bool forCopyPaste)
 {
     QList<Feature*> exportedFeatures;
     QList<Feature*>::Iterator i;
@@ -614,16 +722,23 @@ QList<Feature*> Document::exportCoreOSM(QList<Feature*> aFeatures)
             } else {
                 //FIXME Not working for relation (not made of point?)
                 if (Relation* G = dynamic_cast<Relation*>(*i)) {
-                    for (int j=0; j < G->size(); j++) {
-                        if (Way* R = dynamic_cast<Way*>(G->get(j))) {
-                            for (int k=0; k < R->size(); k++) {
-                                if (Node* P = dynamic_cast<Node*>(R->get(k))) {
-                                    if (!exportedFeatures.contains(P))
-                                        exportedFeatures.append(P);
+                    if (!forCopyPaste) {
+                        for (int j=0; j < G->size(); j++) {
+                            if (Way* R = CAST_WAY(G->get(j))) {
+                                for (int k=0; k < R->size(); k++) {
+                                    if (Node* P = dynamic_cast<Node*>(R->get(k))) {
+                                        if (!exportedFeatures.contains(P))
+                                            exportedFeatures.append(P);
+                                    }
                                 }
+                                if (!exportedFeatures.contains(R))
+                                    exportedFeatures.append(R);
+                            } else
+                            if (Node* P = CAST_NODE(G->get(j))) {
+                                if (!exportedFeatures.contains(P))
+                                    exportedFeatures.append(P);
                             }
-                        if (!exportedFeatures.contains(R))
-                            exportedFeatures.append(R);
+
                         }
                     }
                     if (!exportedFeatures.contains(G))
@@ -649,7 +764,7 @@ bool Document::importNMEA(const QString& filename, TrackLayer* NewLayer)
         return false;
 }
 
-bool Document::importOSC(const QString& filename, DirtyLayer* NewLayer)
+bool Document::importOSC(const QString& filename, DrawingLayer* NewLayer)
 {
     ImportExportOSC imp(this);
     if (!imp.loadFile(filename))
@@ -675,18 +790,18 @@ bool Document::importKML(const QString& filename, TrackLayer* NewLayer)
         return false;
 }
 
-bool Document::importSHP(const QString& filename, DrawingLayer* NewLayer)
+bool Document::importGDAL(const QString& filename, DrawingLayer* NewLayer)
 {
     Q_UNUSED(filename)
     Q_UNUSED(NewLayer)
 
 #ifdef USE_GDAL
-    ImportExportSHP imp(this);
+    ImportExportGdal imp(this);
     if (!imp.loadFile(filename))
         return false;
-    imp.import(NewLayer);
+    bool ret = imp.import(NewLayer);
 
-    if (NewLayer->size())
+    if (ret && NewLayer->size())
         return true;
     else
         return false;
@@ -739,7 +854,29 @@ const QList<CoordBox> Document::getDownloadBoxes() const
     return p->downloadBoxes.values();
 }
 
-Layer * Document::getLastDownloadLayer()
+const QList<CoordBox> Document::getDownloadBoxes(Layer* l) const
+{
+    return p->downloadBoxes.values(l);
+}
+
+bool Document::isDownloadedSafe(const CoordBox& bb) const
+{
+    QHashIterator<Layer*, CoordBox>it(p->downloadBoxes);
+    while(it.hasNext()) {
+        it.next();
+        if (it.value().intersects(bb))
+            return true;
+    }
+
+    return false;
+}
+
+QDateTime Document::getLastDownloadLayerTime() const
+{
+    return p->lastDownloadTimestamp;
+}
+
+Layer * Document::getLastDownloadLayer() const
 {
     return p->lastDownloadLayer;
 }
@@ -747,6 +884,7 @@ Layer * Document::getLastDownloadLayer()
 void Document::setLastDownloadLayer(Layer * aLayer)
 {
     p->lastDownloadLayer = aLayer;
+    p->lastDownloadTimestamp = QDateTime::currentDateTime();
 }
 
 void Document::on_imageRequested(ImageMapLayer* anImageLayer)
@@ -768,7 +906,7 @@ QPair<bool,CoordBox> Document::boundingBox()
 {
     int First;
     for (First = 0; First < layerSize(); ++First)
-        if (getLayer(First)->size())
+        if (getLayer(First)->size() && !getLayer(First)->boundingBox().isNull())
             break;
     if (First == layerSize())
         return qMakePair(false,CoordBox(Coord(0,0),Coord(0,0)));
@@ -776,15 +914,154 @@ QPair<bool,CoordBox> Document::boundingBox()
     CoordBox BBox = aLayer->boundingBox();
     for (int i=First+1; i<layerSize(); ++i)
         aLayer = getLayer(i);
-        if (aLayer->size())
+        if (aLayer->size() && !aLayer->boundingBox().isNull())
             BBox.merge(aLayer->boundingBox());
     return qMakePair(true,BBox);
 }
 
-bool Document::hasUnsavedChanges()
+bool Document::setFilterType(FilterType aFilter)
 {
-//	return aDoc.history().index();
-    return (getDirtyOrOriginLayer()->getDirtySize() > 0);
+    p->FilterRevision++;
+    QString theFilter = M_PREFS->getFilter(aFilter).filter;
+    if (theFilter.isEmpty()) {
+        if (p->tagFilter)
+            SAFE_DELETE(p->tagFilter);
+        return true;
+    }
+    p->tagFilter = TagSelector::parse(M_PREFS->getFilter(aFilter).filter);
+    return (p->tagFilter != NULL);
+}
+
+TagSelector* Document::getTagFilter()
+{
+    return p->tagFilter;
+}
+
+int Document::filterRevision() const
+{
+    return p->FilterRevision;
+}
+
+QString Document::title() const
+{
+    return p->title;
+}
+
+void Document::setTitle(const QString aTitle)
+{
+    p->title = aTitle;
+}
+
+QString Document::toPropertiesHtml()
+{
+    QString h;
+
+    h += "<big><strong>" + tr("Document") + "</strong></big><hr/>";
+    for (int i=0; i<p->Layers.size(); ++i) {
+        h += p->Layers[i]->toPropertiesHtml() + "<br/>";
+    }
+    h += "";
+
+    return h;
+}
+
+QStringList Document::getCurrentSourceTags()
+{
+    QStringList theSrc;
+    for (LayerIterator<ImageMapLayer*> ImgIt(this); !ImgIt.isEnd(); ++ImgIt) {
+        if (ImgIt.get()->isVisible()) {
+            QString s = ImgIt.get()->getMapAdapter()->getSourceTag();
+            if (!s.isEmpty())
+                theSrc << ImgIt.get()->getMapAdapter()->getSourceTag();
+        }
+    }
+    return theSrc;
+}
+
+Document* Document::getDocumentFromXml(QDomDocument* theXmlDoc)
+{
+    QDomElement c;
+    c = theXmlDoc->documentElement();
+//    if (c.tagName().isNull())
+//        c = c.firstChildElement();
+//    while (!c.isNull() && c.tagName().isNull()) {
+//        c =c.nextSiblingElement();
+//    }
+//    if (c.isNull())
+//        return NULL;
+
+    if (c.tagName() == "osm") {
+        Document* NewDoc = new Document(NULL);
+        DrawingLayer* l = new DrawingLayer("Dummy");
+        NewDoc->add(l);
+
+        c = c.firstChildElement();
+        while(!c.isNull()) {
+            if (c.tagName() == "bound") {
+            } else
+            if (c.tagName() == "way") {
+                Way::fromXML(NewDoc, l, c);
+            } else
+            if (c.tagName() == "relation") {
+                Relation::fromXML(NewDoc, l, c);
+            } else
+            if (c.tagName() == "node") {
+                Node::fromXML(NewDoc, l, c);
+            }
+
+            c = c.nextSiblingElement();
+        }
+
+        delete theXmlDoc;
+        return NewDoc;
+    } else
+    if (c.tagName() == "kml") {
+        Document* NewDoc = new Document(NULL);
+        DrawingLayer* l = new DrawingLayer("Dummy");
+        NewDoc->add(l);
+
+        ImportExportKML imp(NewDoc);
+        QByteArray ba = theXmlDoc->toByteArray();
+        QBuffer kmlBuf(&ba);
+        kmlBuf.open(QIODevice::ReadOnly);
+        if (imp.setDevice(&kmlBuf))
+            imp.import(l);
+
+        delete theXmlDoc;
+        return NewDoc;
+    } else
+    if (c.tagName() == "gpx") {
+    }
+    return NULL;
+}
+
+Document* Document::getDocumentFromClipboard()
+{
+    QClipboard *clipboard = QApplication::clipboard();
+    QDomDocument* theXmlDoc = new QDomDocument();
+
+    if (clipboard->mimeData()->hasFormat("application/x-openstreetmap+xml")) {
+        if (!theXmlDoc->setContent(clipboard->mimeData()->data("application/x-openstreetmap+xml"))) {
+            delete theXmlDoc;
+            return NULL;
+        }
+    } else
+    if (clipboard->mimeData()->hasFormat("application/vnd.google-earth.kml+xml")) {
+        if (!theXmlDoc->setContent(clipboard->mimeData()->data("application/vnd.google-earth.kml+xml"))) {
+            delete theXmlDoc;
+            return NULL;
+        }
+    } else
+    if (clipboard->mimeData()->hasText()) {
+        if (!theXmlDoc->setContent(clipboard->text())) {
+            delete theXmlDoc;
+            return NULL;
+        }
+    } else {
+        delete theXmlDoc;
+        return NULL;
+    }
+    return Document::getDocumentFromXml(theXmlDoc);
 }
 
 /* FEATUREITERATOR */
@@ -793,10 +1070,14 @@ FeatureIterator::FeatureIterator(Document *aDoc)
 : theDocument(aDoc), curLayerIdx(0), curFeatureIdx(0), isAtEnd(false)
 {
     docSize = theDocument->layerSize();
-    curLayerSize = theDocument->getLayer(curLayerIdx)->size();
+    if (!docSize)
+        isAtEnd = true;
+    else  {
+        curLayerSize = theDocument->getLayer(curLayerIdx)->size();
 
-    if(!check() && !isAtEnd)
-        ++(*this);
+        if(!check() && !isAtEnd)
+            ++(*this);
+    }
 }
 
 FeatureIterator::~FeatureIterator()
@@ -883,14 +1164,11 @@ bool VisibleFeatureIterator::check()
 {
     if (!FeatureIterator::check())
         return false;
-    else {
-        if (theDocument->getLayer(curLayerIdx)->isVisible()) {
-            if (CAST_NODE(theDocument->getLayer(curLayerIdx)->get(curFeatureIdx))
-                    && !(theDocument->getLayer(curLayerIdx)->arePointsDrawable()))
+    else if (theDocument->getLayer(curLayerIdx)->get(curFeatureIdx)->isHidden())
+        return false;
+    else if (CAST_NODE(theDocument->getLayer(curLayerIdx)->get(curFeatureIdx))
+            && !(theDocument->getLayer(curLayerIdx)->arePointsDrawable()))
                 return false;
-        } else
-            return false;
-    }
 
     return true;
 }
