@@ -1,3 +1,5 @@
+#include "Global.h"
+
 #include "Features.h"
 #include "Command.h"
 #include "DocumentCommands.h"
@@ -8,16 +10,29 @@
 #include "Document.h"
 #include "Layer.h"
 #include "PaintStyle/MasPaintStyle.h"
-#include "PaintStyle/TagSelector.h"
+#include "Utils/TagSelector.h"
+#include "MapView.h"
+#include "PropertiesDock.h"
 
 #include "Utils/Utils.h"
 
 #include <QApplication>
 #include <QUuid>
 #include <QProgressDialog>
+#include <QPainter>
+#include <QPainterPath>
 
 #include <algorithm>
 
+qint64 g_feat_rndId = 0;
+QStringList TechnicalTags = QString(TECHNICAL_TAGS).split("#");
+
+IFeature::FId Feature::newId(IFeature::FeatureType type) const
+{
+    IFeature::FId id = IFeature::FId(type, --g_feat_rndId);
+    return id;
+}
+
 //static QString randomId()
 //{
 //	QUuid uuid = QUuid::createUuid();
@@ -89,26 +104,24 @@ class MapFeaturePrivate
 {
     public:
         MapFeaturePrivate()
-            :  TagsSize(0), LastActor(Feature::User),
+            :  LastActor(Feature::User),
                 PossiblePaintersUpToDate(false),
                 PixelPerMForPainter(-1), CurrentPainter(0), HasPainter(false),
                 theFeature(0), LastPartNotification(0),
-                Time(QDateTime::currentDateTime()), Deleted(false), Visible(true), Uploaded(false), LongId(0)
-                , Virtual(false), Special(false)
-                , VirtualsUpdatesBlocked(false)
-                , Width(0)
+                Time(QDateTime::currentDateTime()), Deleted(false), Visible(true), Uploaded(false), ReadOnly(false), FilterRevision(-1)
+                , Virtual(false), Special(false), DirtyLevel(0)
+                , parentLayer(0)
         {
             initVersionNumber();
         }
         MapFeaturePrivate(const MapFeaturePrivate& other)
-            : Tags(other.Tags), TagsSize(other.TagsSize), LastActor(other.LastActor),
+            : Tags(other.Tags), LastActor(other.LastActor),
                 PossiblePaintersUpToDate(false),
                 PixelPerMForPainter(-1), CurrentPainter(0), HasPainter(false),
                 theFeature(0), LastPartNotification(0),
-                Time(other.Time), Deleted(false), Visible(true), Uploaded(false), LongId(0)
-                , Virtual(other.Virtual), Special(other.Special)
-                , VirtualsUpdatesBlocked(other.VirtualsUpdatesBlocked)
-                , Width(other.Width)
+                Time(other.Time), Deleted(false), Visible(true), Uploaded(false), ReadOnly(false), FilterRevision(-1)
+                , Virtual(other.Virtual), Special(other.Special), DirtyLevel(0)
+                , parentLayer(0)
         {
             initVersionNumber();
         }
@@ -118,9 +131,8 @@ class MapFeaturePrivate
         void updatePainters(double PixelPerM);
         void initVersionNumber();
 
-        mutable QString Id;
-        QList<QPair<QString, QString> > Tags;
-        int TagsSize;
+        mutable IFeature::FId Id;
+        QList<QPair<quint32, quint32> > Tags;
         Feature::ActorType LastActor;
         QList<const FeaturePainter*> PossiblePainters;
         bool PossiblePaintersUpToDate;
@@ -136,19 +148,22 @@ class MapFeaturePrivate
         bool Deleted;
         bool Visible;
         bool Uploaded;
-        qint64 LongId;
+        bool ReadOnly;
+        int FilterRevision;
         RenderPriority theRenderPriority;
         bool Virtual;
         bool Special;
-        bool VirtualsUpdatesBlocked;
-        double Width;
+        int DirtyLevel;
+        QList<FilterLayer*> FilterLayers;
+        double Alpha;
+        Layer* parentLayer;
 };
 
 void MapFeaturePrivate::initVersionNumber()
 {
 //    static int VN = -1;
 //    VersionNumber = VN--;
-    VersionNumber = 9999;
+    VersionNumber = 0;
 }
 
 Feature::Feature()
@@ -156,13 +171,17 @@ Feature::Feature()
 {
      p = new MapFeaturePrivate;
      p->theFeature = this;
+     p->Id = IFeature::FId(IFeature::Uninitialized, 0);
+
+//     qDebug() << "Feature size: " << sizeof(Feature) << sizeof(MapFeaturePrivate);
 }
 
 Feature::Feature(const Feature& other)
-: QObject(), MetaUpToDate(false), m_references(0)
+: MetaUpToDate(false), m_references(0)
 {
     p = new MapFeaturePrivate(*other.p);
     p->theFeature = this;
+    p->Id = IFeature::FId(IFeature::Uninitialized, 0);
 }
 
 Feature::~Feature(void)
@@ -174,35 +193,6 @@ Feature::~Feature(void)
     delete p;
 }
 
-#define DEFAULTWIDTH 6
-#define LANEWIDTH 4
-
-double Feature::widthOf()
-{
-    if (p->Width)
-        return p->Width;
-
-    QString s(tagValue("width",QString()));
-    if (!s.isNull())
-        p->Width = s.toDouble();
-    QString h = tagValue("highway",QString());
-    if ( (h == "motorway") || (h=="motorway_link") )
-        p->Width =  4*LANEWIDTH; // 3 lanes plus emergency
-    else if ( (h == "trunk") || (h=="trunk_link") )
-        p->Width =  3*LANEWIDTH; // 2 lanes plus emergency
-    else if ( (h == "primary") || (h=="primary_link") )
-        p->Width =  2*LANEWIDTH; // 2 lanes
-    else if (h == "secondary")
-        p->Width =  2*LANEWIDTH; // 2 lanes
-    else if (h == "tertiary")
-        p->Width =  1.5*LANEWIDTH; // shared middle lane
-    else if (h == "cycleway")
-        p->Width =  1.5;
-    p->Width = DEFAULTWIDTH;
-
-    return p->Width;
-}
-
 void Feature::setVersionNumber(int vn)
 {
     p->VersionNumber = vn;
@@ -215,12 +205,12 @@ int Feature::versionNumber() const
 
 void Feature::setLayer(Layer* aLayer)
 {
-    setParent(aLayer);
+    p->parentLayer = aLayer;
 }
 
 Layer* Feature::layer() const
 {
-    return dynamic_cast<Layer*>(parent());
+    return p->parentLayer;
 }
 
 const RenderPriority& Feature::renderPriority()
@@ -242,63 +232,53 @@ void Feature::setLastUpdated(Feature::ActorType A)
 
 Feature::ActorType Feature::lastUpdated() const
 {
-    Layer* L = dynamic_cast<Layer*>(parent());
+    Layer* L = p->parentLayer;
     if (L && L->classType() == Layer::DirtyLayerType)
         return Feature::User;
     else
         return p->LastActor;
 }
 
-void Feature::setId(const QString& id)
+QString Feature::stripToOSMId(const IFeature::FId& id)
+{
+    return QString::number(id.numId);
+}
+
+void Feature::setId(const IFeature::FId& id)
 {
     if (id == p->Id)
         return;
 
-    if (parent())
+    if (p->parentLayer)
     {
-        dynamic_cast<Layer*>(parent())->notifyIdUpdate(p->Id,0);
-        dynamic_cast<Layer*>(parent())->notifyIdUpdate(id,this);
+        if (p->Id.type != IFeature::Uninitialized)
+            p->parentLayer->notifyIdUpdate(p->Id,0);
+        if (id.type != IFeature::Uninitialized)
+            p->parentLayer->notifyIdUpdate(id,this);
     }
     p->Id = id;
+
+    // Make sure any new id is unique
+    if (id.numId < -12000)
+        qDebug("numid");
+    if (id.numId < g_feat_rndId)
+        g_feat_rndId = id.numId;
 }
 
-const QString& Feature::resetId()
+const IFeature::FId& Feature::resetId() const
 {
-    p->Id = QString::number((llabs((qint64)this) * -1));
-    if (parent())
-        dynamic_cast<Layer*>(parent())->notifyIdUpdate(p->Id,const_cast<Feature*>(this));
+    p->Id = newId((IFeature::FeatureType)getType());
     return p->Id;
 }
 
-const QString& Feature::id() const
+const IFeature::FId& Feature::id() const
 {
-    if (p->Id.isEmpty())
-    {
-        p->Id = QString::number((llabs((qint64)this) * -1));
-        Layer* L = dynamic_cast<Layer*>(parent());
-        if (L)
-            L->notifyIdUpdate(p->Id,const_cast<Feature*>(this));
-    }
+    if (p->Id.type == IFeature::Uninitialized)
+        resetId();
+
     return p->Id;
 }
 
-qint64 Feature::idToLong() const
-{
-    if (p->LongId)
-        return p->LongId;
-
-    if (hasOSMId()) {
-        bool ok;
-        QString s = stripToOSMId(id());
-        p->LongId = s.toLongLong(&ok);
-        Q_ASSERT(ok);
-    } else {
-        p->LongId = (llabs((qint64)this) * -1);
-    }
-
-    return p->LongId;
-}
-
 QString Feature::xmlId() const
 {
     return stripToOSMId(id());
@@ -306,13 +286,7 @@ QString Feature::xmlId() const
 
 bool Feature::hasOSMId() const
 {
-    if (p->Id.left(5) == "node_")
-        return true;
-    if (p->Id.left(4) == "way_")
-        return true;
-    if (p->Id.left(4) == "rel_")
-        return true;
-    return false;
+    return (p->Id.numId >= 0);
 }
 
 const QDateTime& Feature::time() const
@@ -335,10 +309,40 @@ void Feature::setUser(const QString& user)
     p->User = user;
 }
 
+int Feature::incDirtyLevel(int inc)
+{
+    return p->DirtyLevel += inc;
+}
+
+int Feature::decDirtyLevel(int inc)
+{
+    return p->DirtyLevel -= inc;
+}
+
+int Feature::setDirtyLevel(int newLevel)
+{
+    return (p->DirtyLevel = newLevel);
+}
+
+int Feature::getDirtyLevel() const
+{
+    return p->DirtyLevel;
+}
+
+qreal Feature::getAlpha()
+{
+    if (!MetaUpToDate)
+        updateMeta();
+    return p->Alpha;
+}
+
 bool Feature::isDirty() const
 {
-    if (parent())
-        return (dynamic_cast<Layer*>(parent())->classType() == Layer::DirtyLayerType);
+    if (g_Merk_Frisius)
+        return (p->DirtyLevel > 0);
+
+    if (p->parentLayer)
+        return (p->parentLayer->classType() == Layer::DirtyLayerType);
     else
         return false;
 }
@@ -355,12 +359,24 @@ bool Feature::isUploaded() const
 
 bool Feature::isUploadable() const
 {
-    if (parent())
-        return (dynamic_cast<Layer*>(parent())->isUploadable());
+    if (p->parentLayer)
+        return (p->parentLayer->isUploadable());
     else
         return false;
 }
 
+void Feature::setReadonly(bool val)
+{
+    p->ReadOnly = val;
+}
+
+bool Feature::isReadonly()
+{
+    if (!MetaUpToDate)
+        updateMeta();
+    return p->ReadOnly;
+}
+
 void Feature::setDeleted(bool delState)
 {
     if (delState == p->Deleted)
@@ -368,7 +384,7 @@ void Feature::setDeleted(bool delState)
 
     if (layer()) {
         if (delState)
-            layer()->indexRemove(boundingBox(), this);
+            layer()->indexRemove(boundingBox(false), this);
         else
             layer()->indexAdd(boundingBox(), this);
     }
@@ -384,29 +400,31 @@ void Feature::setVisible(bool state)
 {
     if (state == p->Visible)
         return;
-
-    if (layer()) {
-        if (!state)
-            layer()->indexRemove(boundingBox(), this);
-        else
-            layer()->indexAdd(boundingBox(), this);
-    }
     p->Visible = state;
 }
 
-bool Feature::isVisible() const
+bool Feature::isVisible()
 {
+    if (!MetaUpToDate)
+        updateMeta();
     return p->Visible;
 }
 
-bool Feature::isHidden() const
+bool Feature::isHidden()
 {
+    if (!MetaUpToDate)
+        updateMeta();
     return !p->Visible;
 }
 
 void Feature::setVirtual(bool val)
 {
+    if (val == p->Virtual)
+        return;
+
     p->Virtual = val;
+    if (!p->Virtual)
+        resetId();
 }
 
 bool Feature::isVirtual() const
@@ -424,66 +442,118 @@ bool Feature::isSpecial() const
     return p->Special;
 }
 
-void Feature::setTag(int index, const QString& key, const QString& value, bool addToTagList)
+void Feature::setTag(int index, const QString& key, const QString& value)
 {
-    p->Width = 0;
+    if (key.toLower() == "created_by")
+        return;
+
+    QPair<quint32, quint32> pi = g_addToTagList(key, value);
 
     int i = 0;
     for (; i<p->Tags.size(); ++i)
-        if (p->Tags[i].first == key)
+        if (p->Tags[i].first == pi.first)
         {
-            if (p->Tags[i].second == value)
+            if (p->Tags[i].second == pi.second)
                 return;
-            p->Tags[i].second = value;
+            g_removeFromTagList(p->Tags[i].first, p->Tags[i].second);
+            p->Tags[i].second = pi.second;
             break;
         }
     if (i == p->Tags.size()) {
-        p->Tags.insert(p->Tags.begin() + index, qMakePair(key,value));
-        p->TagsSize++;
+        p->Tags.insert(p->Tags.begin() + index, pi);
     }
     invalidatePainter();
     invalidateMeta();
-
-    if (parent() && addToTagList)
-        if (dynamic_cast<Layer*>(parent())->getDocument())
-            dynamic_cast<Layer*>(parent())->getDocument()->addToTagList(key, value);
 }
 
-void Feature::setTag(const QString& key, const QString& value, bool addToTagList)
+void Feature::setTag(const QString& key, const QString& value)
 {
-    p->Width = 0;
+    if (key.toLower() == "created_by")
+        return;
+
+    QPair<quint32, quint32> pi = g_addToTagList(key, value);
 
     int i = 0;
     for (; i<p->Tags.size(); ++i)
-        if (p->Tags[i].first == key)
+        if (p->Tags[i].first == pi.first)
         {
-            if (p->Tags[i].second == value)
+            if (p->Tags[i].second == pi.second)
                 return;
-            p->Tags[i].second = value;
+            g_removeFromTagList(p->Tags[i].first, p->Tags[i].second);
+            p->Tags[i].second = pi.second;
             break;
         }
     if (i == p->Tags.size()) {
-        p->Tags.push_back(qMakePair(key,value));
-        p->TagsSize++;
+        p->Tags.push_back(pi);
     }
     invalidateMeta();
     invalidatePainter();
-
-    if (parent() && addToTagList)
-        if (dynamic_cast<Layer*>(parent())->getDocument())
-            dynamic_cast<Layer*>(parent())->getDocument()->addToTagList(key, value);
 }
 
 void Feature::clearTags()
 {
-    p->Width = 0;
-
-    p->Tags.clear();
-    p->TagsSize = 0;
+    while (p->Tags.size()) {
+        g_removeFromTagList(p->Tags[0].first, p->Tags[0].second);
+        p->Tags.erase(p->Tags.begin());
+    }
     invalidateMeta();
     invalidatePainter();
 }
 
+void Feature::clearTag(const QString& k)
+{
+    quint32 ik = g_getTagKeyIndex(k);
+
+    for (int i=0; i<p->Tags.size(); ++i)
+        if (p->Tags[i].first == ik)
+        {
+            g_removeFromTagList(p->Tags[i].first, p->Tags[i].second);
+            p->Tags.erase(p->Tags.begin()+i);
+            break;
+        }
+    invalidateMeta();
+    invalidatePainter();
+}
+
+void Feature::removeTag(int idx)
+{
+    g_removeFromTagList(p->Tags[idx].first, p->Tags[idx].second);
+    p->Tags.erase(p->Tags.begin()+idx);
+    invalidateMeta();
+    invalidatePainter();
+}
+
+int Feature::tagSize() const
+{
+    return p->Tags.size();
+}
+
+QString Feature::tagValue(int i) const
+{
+    return g_getTagValue(p->Tags[i].second);
+}
+
+QString Feature::tagKey(int i) const
+{
+    return g_getTagKey(p->Tags[i].first);
+}
+
+int Feature::findKey(const QString &k) const
+{
+    for (int i=0; i<p->Tags.size(); ++i)
+        if (tagKey(i) == k)
+            return i;
+    return -1;
+}
+
+QString Feature::tagValue(const QString& k, const QString& Default) const
+{
+    for (int i=0; i<p->Tags.size(); ++i)
+        if (tagKey(i) == k)
+            return tagValue(i);
+    return Default;
+}
+
 void Feature::invalidateMeta()
 {
     MetaUpToDate = false;
@@ -495,93 +565,35 @@ void Feature::invalidatePainter()
     p->PixelPerMForPainter = -1;
 }
 
-void Feature::clearTag(const QString& k)
+const QPainterPath& Feature::getPath() const
 {
-    p->Width = 0;
-
-    for (int i=0; i<p->Tags.size(); ++i)
-        if (p->Tags[i].first == k)
-        {
-            p->PixelPerMForPainter = -1;
-            p->Tags.erase(p->Tags.begin()+i);
-            p->TagsSize--;
-            break;
-        }
-    invalidateMeta();
-    invalidatePainter();
-}
-
-void Feature::removeTag(int idx)
-{
-    p->Width = 0;
-
-    p->Tags.erase(p->Tags.begin()+idx);
-    p->TagsSize--;
-    invalidateMeta();
-    invalidatePainter();
-}
-
-int Feature::tagSize() const
-{
-    return p->TagsSize;
-}
-
-QString Feature::tagValue(int i) const
-{
-    return p->Tags[i].second;
-}
-
-QString Feature::tagKey(int i) const
-{
-    return p->Tags[i].first;
-}
-
-int Feature::findKey(const QString &k) const
-{
-    for (int i=0; i<p->TagsSize; ++i)
-        if (p->Tags[i].first == k)
-            return i;
-    return p->Tags.size();
-}
-
-QString Feature::tagValue(const QString& k, const QString& Default) const
-{
-    for (int i=0; i<p->TagsSize; ++i)
-        if (p->Tags[i].first == k)
-            return p->Tags[i].second;
-    return Default;
+    return QPainterPath();
 }
 
 void MapFeaturePrivate::updatePossiblePainters()
 {
-    //if the object has no tags or only the created_by tag, we don't check for style
-    //search is about 15 times faster like that !!!
-    //However, still match features with no tags and no parent, i.e. "lost" trackpoints
-    if ( (theFeature->layer()->isTrack()) && MerkaartorPreferences::instance()->getDisableStyleForTracks() ) return blankPainters();
+    //still match features with no tags and no parent, i.e. "lost" trackpoints
+    if ( (theFeature->layer()->isTrack()) && M_PREFS->getDisableStyleForTracks() ) return blankPainters();
 
     if ( (theFeature->layer()->isTrack()) || theFeature->sizeParents() ) {
-        int i;
-        for (i=0; i<theFeature->tagSize(); ++i)
-            if ((theFeature->tagKey(i) != "created_by") && (theFeature->tagKey(i) != "ele"))
-                break;
-
-        if (i == theFeature->tagSize()) return blankPainters();
+        if (CHECK_NODE(theFeature) && !STATIC_CAST_NODE(theFeature)->isPOI()) return blankPainters();
+        if (!theFeature->tagSize()) return blankPainters();
     }
 
     PossiblePainters.clear();
     QList<const FeaturePainter*> DefaultPainters;
-    for (int i=0; i<M_STYLE->painterSize(); ++i)
+    for (int i=0; i<theFeature->layer()->getDocument()->getPaintersSize(); ++i)
     {
-        const FeaturePainter* Current = M_STYLE->getPainter(i);
-        switch (Current->matchesTag(theFeature)) {
-            case TagSelect_Match:
-                PossiblePainters.push_back(Current);
-                break;
-            case TagSelect_DefaultMatch:
-                DefaultPainters.push_back(Current);
-                break;
-            default:
-                break;
+        const FeaturePainter* Current = dynamic_cast<const FeaturePainter*>(theFeature->layer()->getDocument()->getPainter(i));
+        switch (Current->matchesTag(theFeature,NULL)) {
+        case TagSelect_Match:
+            PossiblePainters.push_back(Current);
+            break;
+        case TagSelect_DefaultMatch:
+            DefaultPainters.push_back(Current);
+            break;
+        default:
+            break;
         }
     }
     if (!PossiblePainters.size())
@@ -613,19 +625,19 @@ void MapFeaturePrivate::blankPainters()
     HasPainter = false;
 }
 
-const FeaturePainter* Feature::getEditPainter(double PixelPerM) const
+const FeaturePainter* Feature::getPainter(double PixelPerM) const
 {
     if (p->PixelPerMForPainter != PixelPerM)
         p->updatePainters(PixelPerM);
     return p->CurrentPainter;
 }
 
-const FeaturePainter* Feature::getCurrentEditPainter() const
+const FeaturePainter* Feature::getCurrentPainter() const
 {
     return p->CurrentPainter;
 }
 
-bool Feature::hasEditPainter() const
+bool Feature::hasPainter() const
 {
     if (!p->PossiblePaintersUpToDate)
         p->updatePossiblePainters();
@@ -633,6 +645,13 @@ bool Feature::hasEditPainter() const
     return p->HasPainter;
 }
 
+bool Feature::hasPainter(double PixelPerM) const
+{
+    if (p->PixelPerMForPainter != PixelPerM)
+        p->updatePainters(PixelPerM);
+    return (p->CurrentPainter != NULL);
+}
+
 void Feature::setParentFeature(Feature* F)
 {
     if (std::find(p->Parents.begin(),p->Parents.end(),F) == p->Parents.end())
@@ -655,9 +674,78 @@ void Feature::updateIndex()
         return;
 
     if (layer()) {
-        layer()->indexRemove(BBox, this);
-        CoordBox bb = boundingBox();
-        layer()->indexAdd(bb, this);
+        layer()->indexRemove(boundingBox(false), this);
+        layer()->indexAdd(boundingBox(true), this);
+    }
+}
+
+void Feature::updateFilters()
+{
+    p->FilterLayers.clear();
+
+    Layer* L = layer();
+    if (!L)
+        return;
+
+    Document* D = L->getDocument();
+    if (!D)
+        return;
+
+    for (int i=0; i<D->layerSize(); ++i) {
+        if (D->getLayer(i)->classType() == Layer::FilterLayerType) {
+            FilterLayer* Fl = dynamic_cast<FilterLayer*>(D->getLayer(i));
+            if (!Fl->isEnabled() || !Fl->selector())
+                continue;
+            if (Fl->selector()->matches(this, 0) != TagSelect_NoMatch)
+                p->FilterLayers << Fl;
+        }
+    }
+    invalidateMeta();
+}
+
+void Feature::updateMeta()
+{
+    updateFilters();
+
+    Layer* L = layer();
+    if (!L)
+        return;
+
+    if (!L->isVisible())
+        p->Visible = false;
+    else {
+        p->Visible = true;
+        foreach(FilterLayer* Fl, p->FilterLayers) {
+            if (!Fl->isVisible()) {
+                p->Visible = false;
+                break;
+            }
+        }
+    }
+
+    if (L->getAlpha() != 1.0)
+        p->Alpha = L->getAlpha();
+    else {
+        p->Alpha = 1.0;
+        foreach(FilterLayer* Fl, p->FilterLayers) {
+            if (Fl->getAlpha() != 1) {
+                p->Alpha = Fl->getAlpha();
+                break;
+            }
+        }
+    }
+
+    if (L->isReadonly())
+        setReadonly(true);
+    else {
+        setReadonly(false);
+        foreach(FilterLayer* Fl, p->FilterLayers) {
+            if (Fl->isReadonly()) {
+                setReadonly(true);
+                break;
+            }
+
+        }
     }
 }
 
@@ -666,12 +754,12 @@ int Feature::sizeParents() const
     return p->Parents.size();
 }
 
-Feature* Feature::getParent(int i)
+IFeature* Feature::getParent(int i)
 {
     return p->Parents[i];
 }
 
-const Feature* Feature::getParent(int i) const
+const IFeature* Feature::getParent(int i) const
 {
     return p->Parents[i];
 }
@@ -692,6 +780,55 @@ void Feature::notifyParents(int Id)
     }
 }
 
+
+void Feature::drawHover(QPainter& thePainter, MapView* theView)
+{
+    QPen TP(M_PREFS->getHoverColor(),M_PREFS->getHoverWidth(),Qt::SolidLine);
+    thePainter.setPen(TP);
+
+    drawSpecial(thePainter, TP, theView);
+
+    drawChildrenSpecial(thePainter, TP, theView, 1);
+
+    if (M_PREFS->getShowParents()) {
+        TP.setDashPattern(M_PREFS->getParentDashes());
+        thePainter.setPen(TP);
+        drawParentsSpecial(thePainter, TP, theView);
+    }
+}
+
+void Feature::drawFocus(QPainter& thePainter, MapView* theView)
+{
+    QPen TP(M_PREFS->getFocusColor(),M_PREFS->getFocusWidth(),Qt::SolidLine);
+
+    thePainter.setPen(TP);
+
+    drawSpecial(thePainter, TP, theView);
+
+    drawChildrenSpecial(thePainter, TP, theView, 1);
+
+    if (M_PREFS->getShowParents()) {
+        TP.setDashPattern(M_PREFS->getParentDashes());
+        thePainter.setPen(TP);
+        drawParentsSpecial(thePainter, TP, theView);
+    }
+}
+
+void Feature::drawHighlight(QPainter& thePainter, MapView* theView)
+{
+    QPen TP(M_PREFS->getHighlightColor(),M_PREFS->getHighlightWidth(),Qt::SolidLine);
+    thePainter.setPen(TP);
+    drawSpecial(thePainter, TP, theView);
+
+    drawChildrenSpecial(thePainter, TP, theView, 1);
+
+    if (M_PREFS->getShowParents()) {
+        TP.setDashPattern(M_PREFS->getParentDashes());
+        thePainter.setPen(TP);
+        drawParentsSpecial(thePainter, TP, theView);
+    }
+}
+
 QString Feature::toXML(int lvl, QProgressDialog * progress)
 {
     QDomDocument doc;
@@ -707,12 +844,69 @@ QString Feature::toXML(int lvl, QProgressDialog * progress)
         return "";
 }
 
-bool Feature::tagsToXML(QDomElement xParent)
+void Feature::fromXML(const QDomElement& e, Feature* F)
+{
+    bool Deleted = (e.attribute("deleted") == "true");
+    int Dirty = (e.hasAttribute("dirtylevel") ? e.attribute("dirtylevel").toInt() : 0);
+    bool Uploaded = (e.attribute("uploaded") == "true");
+    bool Special = (e.attribute("special") == "true");
+    bool Selected = (e.attribute("selected") == "true");
+
+    QDateTime time;
+    time = QDateTime::fromString(e.attribute("timestamp").left(19), Qt::ISODate);
+    QString user = e.attribute("user");
+    int Version = e.attribute("version").toInt();
+    if (Version < 1)
+        Version = 0;
+    Feature::ActorType A = (Feature::ActorType)(e.attribute("actor", "2").toInt());
+
+    F->setLastUpdated(A);
+    F->setDeleted(Deleted);
+    F->setDirtyLevel(Dirty);
+    F->setUploaded(Uploaded);
+    F->setSpecial(Special);
+    F->setTime(time);
+    F->setUser(user);
+    F->setVersionNumber(Version);
+
+    // TODO Manage selection at document level
+//    if(Selected)
+//        g_Merk_MainWindow->properties()->addSelection(F);
+}
+
+void Feature::toXML(QDomElement& e, bool strict)
+{
+    e.setAttribute("id", xmlId());
+    e.setAttribute("timestamp", time().toString(Qt::ISODate)+"Z");
+    e.setAttribute("version", versionNumber());
+    e.setAttribute("user", user());
+    if (!strict) {
+        e.setAttribute("actor", (int)lastUpdated());
+        if (isDeleted())
+            e.setAttribute("deleted","true");
+        if (getDirtyLevel())
+            e.setAttribute("dirtylevel", getDirtyLevel());
+        if (isUploaded())
+            e.setAttribute("uploaded","true");
+        if (isSpecial())
+            e.setAttribute("special","true");
+        // TODO Manage selection at document level
+        if (g_Merk_MainWindow->properties()->isSelected(this))
+            e.setAttribute("selected","true");
+    }
+}
+
+bool Feature::tagsToXML(QDomElement xParent, bool strict)
 {
     bool OK = true;
 
     for (int i=0; i<tagSize(); ++i)
     {
+        if (strict) {
+            if (tagKey(i).startsWith('_') && (tagKey(i).endsWith('_')))
+                continue;
+        }
+
         QDomElement e = xParent.ownerDocument().createElement("tag");
         xParent.appendChild(e);
 
@@ -735,14 +929,6 @@ void Feature::tagsFromXML(Document* d, Feature * f, QDomElement e)
     }
 }
 
-QString Feature::stripToOSMId(const QString& id)
-{
-    int f = id.lastIndexOf("_");
-    if (f>0)
-        return id.right(id.length()-(f+1));
-    return id;
-}
-
 Relation * Feature::GetSingleParentRelation(Feature * mapFeature)
 {
     int parents = mapFeature->sizeParents();
@@ -755,13 +941,8 @@ Relation * Feature::GetSingleParentRelation(Feature * mapFeature)
     int i;
     for (i=0; i<parents; i++)
     {
-        Feature * parent = mapFeature->getParent(i);
-        if (parent->isDeleted()) continue;
-
-        Relation * rel = dynamic_cast<Relation*>(parent);
-
-        if (rel == NULL)
-            continue;
+        Relation * rel = dynamic_cast<Relation*>(mapFeature->getParent(i));
+        if (!rel || rel->isDeleted()) continue;
 
         if (parentRelation)
             return NULL;
@@ -774,62 +955,48 @@ Relation * Feature::GetSingleParentRelation(Feature * mapFeature)
 }
 
 //Static
-Node* Feature::getTrackPointOrCreatePlaceHolder(Document *theDocument, Layer *theLayer, const QString& Id)
+Node* Feature::getTrackPointOrCreatePlaceHolder(Document *theDocument, Layer *theLayer, const IFeature::FId& Id)
 {
-    Node* Part = dynamic_cast<Node*>(theDocument->getFeature("node_"+Id));
+    Node* Part = CAST_NODE(theDocument->getFeature(Id));
     if (!Part)
     {
-        Part = dynamic_cast<Node*>(theDocument->getFeature(Id));
-        if (!Part)
-        {
-            Part = new Node(Coord(0,0));
-            if (Id.startsWith('{') || Id.startsWith('-'))
-                Part->setId(Id);
-            else
-                Part->setId("node_"+Id);
-            Part->setLastUpdated(Feature::NotYetDownloaded);
-            theLayer->add(Part);
-        }
+        Part = new Node(Coord(0,0));
+        Part->setId(Id);
+        Part->setLastUpdated(Feature::NotYetDownloaded);
+        if (!theLayer)
+            theLayer = theDocument->getDirtyOrOriginLayer();
+        theLayer->add(Part);
     }
     return Part;
 }
 
-Way* Feature::getWayOrCreatePlaceHolder(Document *theDocument, Layer *theLayer, const QString& Id)
+Way* Feature::getWayOrCreatePlaceHolder(Document *theDocument, Layer *theLayer, const IFeature::FId& Id)
 {
-    Way* Part = dynamic_cast<Way*>(theDocument->getFeature("way_"+Id));
+    Way* Part = CAST_WAY(theDocument->getFeature(Id));
     if (!Part)
     {
-        Part = dynamic_cast<Way*>(theDocument->getFeature(Id));
-        if (!Part)
-        {
-            Part = new Way;
-            if (Id.startsWith('{') || Id.startsWith('-'))
-                Part->setId(Id);
-            else
-                Part->setId("way_"+Id);
-            Part->setLastUpdated(Feature::NotYetDownloaded);
-            theLayer->add(Part);
-        }
+        Part = new Way;
+        Part->setId(Id);
+        Part->setLastUpdated(Feature::NotYetDownloaded);
+        if (!theLayer)
+            theLayer = theDocument->getDirtyOrOriginLayer();
+        theLayer->add(Part);
+
     }
     return Part;
 }
 
-Relation* Feature::getRelationOrCreatePlaceHolder(Document *theDocument, Layer *theLayer, const QString& Id)
+Relation* Feature::getRelationOrCreatePlaceHolder(Document *theDocument, Layer *theLayer, const IFeature::FId& Id)
 {
-    Relation* Part = dynamic_cast<Relation*>(theDocument->getFeature("rel_"+Id));
+    Relation* Part = CAST_RELATION(theDocument->getFeature(Id));
     if (!Part)
     {
-        Part = dynamic_cast<Relation*>(theDocument->getFeature(Id));
-        if (!Part)
-        {
-            Part = new Relation;
-            if (Id.startsWith('{') || Id.startsWith('-'))
-                Part->setId(Id);
-            else
-                Part->setId("rel_"+Id);
-            Part->setLastUpdated(Feature::NotYetDownloaded);
-            theLayer->add(Part);
-        }
+        Part = new Relation;
+        Part->setId(Id);
+        Part->setLastUpdated(Feature::NotYetDownloaded);
+        if (!theLayer)
+            theLayer = theDocument->getDirtyOrOriginLayer();
+        theLayer->add(Part);
     }
     return Part;
 }
@@ -841,12 +1008,12 @@ void Feature::mergeTags(Document* theDocument, CommandList* L, Feature* Dest, Fe
         QString k = Src->tagKey(i);
         QString v1 = Src->tagValue(i);
         int j = Dest->findKey(k);
-        if (j == Dest->tagSize())
+        if (j == -1)
             L->add(new SetTagCommand(Dest,k,v1, theDocument->getDirtyOrOriginLayer(Dest->layer())));
         else
         {
             QString v2 = Dest->tagValue(j);
-            if (v1 != v2 && k !="created_by")
+            if (v1 != v2)
             {
                 L->add(new SetTagCommand(Dest,k,QString("%1;%2").arg(v2).arg(v1), theDocument->getDirtyOrOriginLayer(Dest->layer())));
             }
@@ -860,9 +1027,9 @@ QString Feature::toMainHtml(QString type, QString systemtype)
     QString desc;
     QString name(tagValue("name",""));
     if (!name.isEmpty())
-        desc = QString("<big><b>%1</b></big><br/><small>(%2)</small>").arg(name).arg(id());
+        desc = QString("<big><b>%1</b></big><br/><small>(%2)</small>").arg(name).arg(id().numId);
     else
-        desc = QString("<big><b>%1</b></big>").arg(id());
+        desc = QString("<big><b>%1</b></big>").arg(id().numId);
 
     QString S =
     "<html><head/><body>"
@@ -874,26 +1041,46 @@ QString Feature::toMainHtml(QString type, QString systemtype)
     S += QApplication::translate("MapFeature", "<i>V: </i><b>%1</b> ").arg(QString::number(versionNumber()));
     if (!user().isEmpty())
         S += QApplication::translate("MapFeature", "<i>last: </i><b>%1</b> by <b>%2</b>").arg(time().toString(Qt::SystemLocaleDate)).arg(user());
-        else
+    else
         S += QApplication::translate("MapFeature", "<i>last: </i><b>%1</b>").arg(time().toString(Qt::SystemLocaleDate));
+
+    if (layer())
+        S += QApplication::translate("MapFeature", "<br/><i>layer: </i><b>%1</b> ").arg(layer()->name());
     S += "</small>"
     "<hr/>"
     "%1";
-    int f = id().lastIndexOf("_");
-    if (f>0) {
+
+    if (tagSize()) {
+        QStringList sTags;
+        int t=0;
+        for (int i=0; i<tagSize(); ++i) {
+            if (tagKey(i) != "_description_" && tagKey(i) != "_comment_") {
+                ++t;
+                sTags << tagKey(i) + "&nbsp;=&nbsp;" + "<b>" + tagValue(i) + "</b>";
+            }
+        }
+
+        if (t) {
+            S += "<hr/><small>";
+            S += sTags.join("<br/>");
+            S += "</small>";
+        }
+    }
+
+    if (hasOSMId()) {
         S += "<hr/>"
-        "<a href='/api/" + M_PREFS->apiVersion() + "/" + systemtype + "/" + xmlId() + "/history'>"+QApplication::translate("MapFeature", "History")+"</a>";
+        "<a href='/" + systemtype + "/" + xmlId() + "/history'>"+QApplication::translate("MapFeature", "History")+"</a>";
         if (systemtype == "node") {
             S += "<br/>"
-            "<a href='/api/" + M_PREFS->apiVersion() + "/" + systemtype + "/" + xmlId() + "/ways'>"+QApplication::translate("MapFeature", "Referenced by ways")+"</a>";
+            "<a href='/" + systemtype + "/" + xmlId() + "/ways'>"+QApplication::translate("MapFeature", "Referenced by ways")+"</a>";
         }
         S += "<br/>"
-        "<a href='/api/" + M_PREFS->apiVersion() + "/" + systemtype + "/" + xmlId() + "/relations'>"+QApplication::translate("MapFeature", "Referenced by relation")+"</a>";
+        "<a href='/" + systemtype + "/" + xmlId() + "/relations'>"+QApplication::translate("MapFeature", "Referenced by relation")+"</a>";
     }
 //	QString bbox("bbox=%1,%2,%3,%4");
 //	bbox = bbox.arg(intToAng(boundingBox().bottomLeft().lon()), 0, 'f').arg(intToAng(boundingBox().bottomLeft().lat()), 0, 'f').arg(intToAng(boundingBox().topRight().lon()), 0, 'f').arg(intToAng(boundingBox().topRight().lat()), 0, 'f');
 //	S += "<br/>"
-//		"<a href='/api/" + M_PREFS->apiVersion() + "/changesets?" + bbox + "'>"+QApplication::translate("MapFeature", "Changesets in feature bbox")+"</a>";
+//		"<a href='" + "/changesets?" + bbox + "'>"+QApplication::translate("MapFeature", "Changesets in feature bbox")+"</a>";
     S += "</body></html>";
 
     return S;
