@@ -7,33 +7,39 @@
 #include <QtGui/QPainter>
 #include <QProgressDialog>
 
+#define TEST_RFLAGS(x) theView->renderOptions().options.testFlag(x)
+
 class NodePrivate
 {
     public:
         NodePrivate()
-        : IsWaypoint(false), ProjectionRevision(0)
+        : IsWaypoint(false)
+        , ProjectionRevision(0)
         , HasPhoto(false)
+        , Photo(0)
         , photoLocationBR(true)
         {
         }
 
         bool IsWaypoint;
+        bool IsPOI;
 #ifndef _MOBILE
         int ProjectionRevision;
+        QPointF Projected;
 #endif
         bool HasPhoto;
-        QPixmap Photo;
+        QPixmap* Photo;
         bool photoLocationBR;
 };
 
 Node::Node(const Coord& aCoord)
-    : Position(aCoord)
+    : Position(CoordBox(aCoord, aCoord))
     , Elevation(0.0)
     , Speed(0.0)
     , p(new NodePrivate)
 {
-    BBox = CoordBox(Position,Position);
     setRenderPriority(RenderPriority(RenderPriority::IsSingular,0., 0));
+//    qDebug() << "Node size: " << sizeof(Node) << sizeof(NodePrivate);
 }
 
 Node::Node(const Node& other)
@@ -41,17 +47,17 @@ Node::Node(const Node& other)
     , Position(other.Position)
     , Elevation(other.Elevation)
     , Speed(other.Speed)
-    , Projected(other.Projected)
     , p(new NodePrivate)
 {
-    setTime(other.time());
-    BBox = other.boundingBox();
+    p->Projected = other.p->Projected;
     p->ProjectionRevision = other.projectionRevision();
     setRenderPriority(RenderPriority(RenderPriority::IsSingular,0., 0));
+//    qDebug() << "Node size: " << sizeof(Node) << sizeof(NodePrivate);
 }
 
 Node::~Node(void)
 {
+    SAFE_DELETE(p->Photo)
     delete p;
 }
 
@@ -91,12 +97,11 @@ bool Node::isNull() const
 bool Node::isInteresting() const
 {
     // does its id look like one from osm
-    if (id().left(5) == "node_")
+    if (hasOSMId())
         return true;
     // if the user has added special tags, that's fine also
-    for (int i=0; i<tagSize(); ++i)
-        if ((tagKey(i) != "created_by") && (tagKey(i) != "ele"))
-            return true;
+    if (tagSize())
+        return true;
     // if it is part of a road, then too
     if (sizeParents())
         return true;
@@ -104,14 +109,12 @@ bool Node::isInteresting() const
     return false;
 }
 
-bool Node::isPOI() const
+bool Node::isPOI()
 {
-    // if the user has added special tags, that's fine also
-    for (int i=0; i<tagSize(); ++i)
-        if ((tagKey(i) != "created_by") && (tagKey(i) != "ele"))
-            return true;
+    if (!MetaUpToDate)
+        updateMeta();
 
-    return false;
+    return p->IsPOI;
 }
 
 bool Node::isWaypoint()
@@ -122,23 +125,22 @@ bool Node::isWaypoint()
     return p->IsWaypoint;
 }
 
-bool Node::isSelectable(MapView* view) const
+bool Node::isSelectable(MapView* theView)
 {
     // If Node has non-default tags -> POI -> always selectable
-    for (int i=0; i<tagSize(); ++i)
-        if ((tagKey(i) != "created_by") && (tagKey(i) != "ele"))
-            return true;
+    if (isPOI())
+        return true;
 
     bool Draw = false;
-    if (M_PREFS->getTrackPointsVisible() || (lastUpdated() == Feature::Log && !M_PREFS->getTrackSegmentsVisible())) {
-        Draw = (view->nodeWidth() >= 1);
+    if (TEST_RFLAGS(RendererOptions::NodesVisible) || (lastUpdated() == Feature::Log && !TEST_RFLAGS(RendererOptions::TrackSegmentVisible))) {
+        Draw = (theView->nodeWidth() >= 1);
         // Do not draw GPX nodes when simple GPX track appearance is enabled
         if (M_PREFS->getSimpleGpxTrack() && layer()->isTrack())
             Draw = false;
         if (!Draw) {
             if (!sizeParents())
                 Draw = true;
-            else if (lastUpdated() == Feature::Log && !M_PREFS->getTrackSegmentsVisible())
+            else if (lastUpdated() == Feature::Log && !TEST_RFLAGS(RendererOptions::TrackSegmentVisible))
                 Draw = true;
         }
     }
@@ -148,30 +150,29 @@ bool Node::isSelectable(MapView* view) const
 
 const Coord& Node::position() const
 {
-    return Position;
+    return Position.topRight();
 }
 
 void Node::setPosition(const Coord& aCoord)
 {
     if (layer())
-        layer()->indexRemove(BBox, this);
-    Position = aCoord;
-    BBox = CoordBox(Position,Position);
+        layer()->indexRemove(Position, this);
+    Position = CoordBox(aCoord, aCoord);
     p->ProjectionRevision = 0;
-    if (layer() && !isDeleted() && isVisible()) {
-        layer()->indexAdd(BBox, this);
+    if (layer() && !isDeleted()) {
+        layer()->indexAdd(Position, this);
     }
     notifyChanges();
 }
 
 const QPointF& Node::projection() const
 {
-    return Projected;
+    return p->Projected;
 }
 
 void Node::setProjection(const QPointF& aProjection)
 {
-    Projected = aProjection;
+    p->Projected = aProjection;
 }
 
 #ifndef _MOBILE
@@ -213,11 +214,15 @@ bool Node::hasPhoto() const
 
 QPixmap Node::photo() const
 {
-    return p->Photo;
+    if (p->Photo)
+        return *(p->Photo);
+    else
+        return QPixmap();
 }
 void Node::setPhoto(QPixmap thePhoto)
 {
-    p->Photo = thePhoto.scaled(M_PREFS->getMaxGeoPicWidth(), M_PREFS->getMaxGeoPicWidth(), Qt::KeepAspectRatio);
+    SAFE_DELETE(p->Photo)
+    p->Photo = new QPixmap(thePhoto.scaled(M_PREFS->getMaxGeoPicWidth(), M_PREFS->getMaxGeoPicWidth(), Qt::KeepAspectRatio));
     p->HasPhoto = true;
 }
 
@@ -226,70 +231,58 @@ bool Node::notEverythingDownloaded()
     return lastUpdated() == Feature::NotYetDownloaded;
 }
 
-CoordBox Node::boundingBox() const
+const CoordBox& Node::boundingBox(bool) const
 {
-    return BBox;
+    return Position;
 }
 
-#ifdef GEOIMAGE
-void Node::draw(QPainter& thePainter , MapView* theView )
+void Node::draw(QPainter& thePainter , MapView* theView)
 {
+#ifdef GEOIMAGE
     if (p->HasPhoto) {
-        QPoint me = theView->toView(this);
-        thePainter.setPen(QPen(QColor(0, 0, 0), 2));
-        QRect box(me - QPoint(5, 3), QSize(10, 6));
-        thePainter.drawRect(box);
+         QPoint me = theView->toView(this);
+         thePainter.setPen(QPen(QColor(0, 0, 0), 2));
+         QRect box(me - QPoint(5, 3), QSize(10, 6));
+         thePainter.drawRect(box);
+         if (TEST_RFLAGS(RendererOptions::PhotosVisible) && theView->pixelPerM() > M_PREFS->getRegionalZoom()) {
+             qreal rt = qBound(0.2, theView->pixelPerM(), 1.0);
+             QPoint phPt;
 
-        if (M_PREFS->getPhotosVisible() && theView->pixelPerM() > M_PREFS->getRegionalZoom()) {
-            qreal rt = qBound(0.2, theView->pixelPerM(), 1.0);
-            QPoint phPt;
-            if (p->photoLocationBR) {
-                phPt = me + QPoint(10*rt, 10*rt);
-            } else {
-                qreal rt = qBound(0.2, theView->pixelPerM(), 1.0);
-                double phRt = 1. * p->Photo.width() / p->Photo.height();
-                phPt = me - QPoint(10*rt, 10*rt) - QPoint(M_PREFS->getMaxGeoPicWidth()*rt, M_PREFS->getMaxGeoPicWidth()*rt/phRt);
-            }
-            thePainter.drawPixmap(phPt, p->Photo.scaledToWidth(M_PREFS->getMaxGeoPicWidth()*rt));
+             if (p->photoLocationBR) {
+                 phPt = me + QPoint(10*rt, 10*rt);
+             } else {
+                 qreal rt = qBound(0.2, theView->pixelPerM(), 1.0);
+                 double phRt = 1. * p->Photo->width() / p->Photo->height();
+                 phPt = me - QPoint(10*rt, 10*rt) - QPoint(M_PREFS->getMaxGeoPicWidth()*rt, M_PREFS->getMaxGeoPicWidth()*rt/phRt);
+             }
+             thePainter.drawPixmap(phPt, p->Photo->scaledToWidth(M_PREFS->getMaxGeoPicWidth()*rt));
+         }
+     }
+#endif
+    if (isDirty() && isUploadable() && M_PREFS->getDirtyVisible()) {
+        QPoint P = theView->toView(this);
+        double theWidth = theView->nodeWidth();
+        if (theWidth >= 1) {
+            QRect R(P-QPoint(theWidth/2,theWidth/2),QSize(theWidth,theWidth));
+            thePainter.fillRect(R,M_PREFS->getDirtyColor());
         }
     }
-}
-#else
-void Node::draw(QPainter& /* thePainter */, MapView* /*theView*/ )
-{
-}
-#endif
 
-void Node::drawFocus(QPainter& thePainter, MapView* theView, bool solid)
-{
-    thePainter.setPen(MerkaartorPreferences::instance()->getFocusColor());
-    QPoint me(theView->toView(this));
-    QRect R(me-QPoint(3,3),QSize(6,6));
-    thePainter.drawRect(R);
-    R.adjust(-7, -7, 7, 7);
-    thePainter.drawEllipse(R);
-
-    if (M_PREFS->getShowParents() && solid) {
-        for (int i=0; i<sizeParents(); ++i)
-            if (!getParent(i)->isDeleted())
-                getParent(i)->drawFocus(thePainter, theView, false);
-    }
 }
 
-void Node::drawHover(QPainter& thePainter, MapView* theView, bool solid)
-{
-    thePainter.setPen(MerkaartorPreferences::instance()->getHoverColor());
-    QPoint me(theView->toView(this));
-    QRect R(me-QPoint(3,3),QSize(6,6));
-    thePainter.drawRect(R);
-    R.adjust(-7, -7, 7, 7);
-    thePainter.drawEllipse(R);
-
 #ifdef GEOIMAGE
+void Node::drawHover(QPainter& thePainter, MapView* theView)
+{
+    /* call the parent function */
+    Feature::drawHover(thePainter, theView);
+
+    /* and then the image */
     if (p->HasPhoto) {
-        if (M_PREFS->getPhotosVisible() && theView->pixelPerM() > M_PREFS->getRegionalZoom()) {
+        if (TEST_RFLAGS(RendererOptions::PhotosVisible) && theView->pixelPerM() > M_PREFS->getRegionalZoom()) {
+            QPoint me(theView->toView(this));
+
             qreal rt = qBound(0.2, theView->pixelPerM(), 1.0);
-            double phRt = 1. * p->Photo.width() / p->Photo.height();
+            double phRt = 1. * p->Photo->width() / p->Photo->height();
             QPoint phPt;
             if (p->photoLocationBR) {
                 phPt = me + QPoint(10*rt, 10*rt);
@@ -300,31 +293,45 @@ void Node::drawHover(QPainter& thePainter, MapView* theView, bool solid)
             thePainter.drawRect(box);
         }
     }
+}
 #endif
 
-    if (M_PREFS->getShowParents() && solid) {
-        for (int i=0; i<sizeParents(); ++i)
-            if (!getParent(i)->isDeleted())
-                getParent(i)->drawHover(thePainter, theView, false);
-    }
-}
-
-void Node::drawHighlight(QPainter& thePainter, MapView* theView, bool /*solid*/)
+void Node::drawSpecial(QPainter& thePainter, QPen& Pen, MapView* theView)
 {
-    thePainter.setPen(MerkaartorPreferences::instance()->getHighlightColor());
+
+    QPen TP(Pen);
+    TP.setWidth(TP.width() / 2);
+
     QPoint me(theView->toView(this));
     QRect R(me-QPoint(3,3),QSize(6,6));
+
+    thePainter.setPen(TP);
     thePainter.drawRect(R);
     R.adjust(-7, -7, 7, 7);
     thePainter.drawEllipse(R);
-
-//	if (M_PREFS->getShowParents() && solid) {
-//		for (int i=0; i<sizeParents(); ++i)
-//			if (!getParent(i)->isDeleted())
-//				getParent(i)->drawHover(thePainter, theView, false);
-//	}
 }
 
+void Node::drawParentsSpecial(QPainter& thePainter, QPen& Pen, MapView* theView)
+{
+    for (int i=0; i<sizeParents(); ++i) {
+        if (!getParent(i)->isDeleted()) {
+            Feature* f = CAST_FEATURE(getParent(i));
+            if (f)
+                f->drawSpecial(thePainter, Pen, theView);
+        }
+    }
+}
+
+void Node::drawChildrenSpecial(QPainter& thePainter, QPen& Pen, MapView* theView, int depth)
+{
+    Q_UNUSED(thePainter);
+    Q_UNUSED(Pen);
+    Q_UNUSED(theView);
+    Q_UNUSED(depth);
+    // Node has no children
+}
+
+
 double Node::pixelDistance(const QPointF& Target, double, bool, MapView* theView) const
 {
     double Best = 1000000;
@@ -332,11 +339,12 @@ double Node::pixelDistance(const QPointF& Target, double, bool, MapView* theView
     QPoint me = theView->toView(const_cast<Node*>(this));
 
     Best = distance(Target, me);
+
 #ifdef GEOIMAGE
     if (p->HasPhoto) {
-        if (M_PREFS->getPhotosVisible() && theView->pixelPerM() > M_PREFS->getRegionalZoom()) {
+        if (TEST_RFLAGS(RendererOptions::PhotosVisible) && theView->pixelPerM() > M_PREFS->getRegionalZoom()) {
             qreal rt = qBound(0.2, theView->pixelPerM(), 1.0);
-            double phRt = 1. * p->Photo.width() / p->Photo.height();
+            double phRt = 1. * p->Photo->width() / p->Photo->height();
             QPoint phPt;
             if (p->photoLocationBR) {
                 phPt = me + QPoint(10*rt, 10*rt);
@@ -363,9 +371,9 @@ QString Node::description() const
 {
     QString s(tagValue("name",""));
     if (!s.isEmpty())
-        return QString("%1 (%2)").arg(s).arg(id());
+        return QString("%1 (%2)").arg(s).arg(id().numId);
     return
-        QString("%1").arg(id());
+        QString("%1").arg(id().numId);
 }
 
 void Node::partChanged(Feature*, int)
@@ -374,8 +382,32 @@ void Node::partChanged(Feature*, int)
 
 void Node::updateMeta()
 {
-    p->IsWaypoint = (findKey("_waypoint_") != tagSize());
+    Feature::updateMeta();
     MetaUpToDate = true;
+
+    p->IsWaypoint = (findKey("_waypoint_") != -1);
+    p->IsPOI = false;
+    for (int i=0; i<tagSize(); ++i) {
+        if (!M_PREFS->getTechnicalTags().contains(tagKey(i))) {
+            p->IsPOI = true;
+            break;
+        }
+    }
+
+    if (!p->IsPOI && !p->IsWaypoint) {
+        int i=0;
+        int prtReadonly=0, prtWritable=0;
+        for (; i<sizeParents(); ++i) {
+            if (getParent(i)->isReadonly())
+                ++prtReadonly;
+            else
+                ++prtWritable;
+        }
+        if (!isReadonly()) {
+            if (prtReadonly && !prtWritable)
+                setReadonly(true);
+        }
+    }
 }
 
 bool Node::toXML(QDomElement xParent, QProgressDialog * progress, bool strict)
@@ -388,19 +420,11 @@ bool Node::toXML(QDomElement xParent, QProgressDialog * progress, bool strict)
     QDomElement e = xParent.ownerDocument().createElement("node");
     xParent.appendChild(e);
 
-    e.setAttribute("id", xmlId());
-    e.setAttribute("lon",QString::number(coordToAng(Position.lon()),'f',8));
-    e.setAttribute("lat", QString::number(coordToAng(Position.lat()),'f',8));
-    e.setAttribute("timestamp", time().toString(Qt::ISODate)+"Z");
-    e.setAttribute("version", versionNumber());
-    e.setAttribute("user", user());
-    if (!strict) {
-        e.setAttribute("actor", (int)lastUpdated());
-        if (isDeleted())
-            e.setAttribute("deleted","true");
-    }
+    Feature::toXML(e, strict);
+    e.setAttribute("lon",COORD2STRING(coordToAng(Position.topRight().lon())));
+    e.setAttribute("lat", COORD2STRING(coordToAng(Position.topRight().lat())));
 
-    tagsToXML(e);
+    tagsToXML(e, strict);
 
     if (progress)
         progress->setValue(progress->value()+1);
@@ -428,8 +452,8 @@ bool Node::toGPX(QDomElement xParent, QProgressDialog * progress, bool forExport
 
     if (!forExport)
         e.setAttribute("xml:id", xmlId());
-    e.setAttribute("lon",QString::number(coordToAng(Position.lon()),'f',8));
-    e.setAttribute("lat", QString::number(coordToAng(Position.lat()),'f',8));
+    e.setAttribute("lon",COORD2STRING(coordToAng(Position.topRight().lon())));
+    e.setAttribute("lat", COORD2STRING(coordToAng(Position.topRight().lat())));
 
     QDomElement c = xParent.ownerDocument().createElement("time");
     e.appendChild(c);
@@ -466,7 +490,7 @@ bool Node::toGPX(QDomElement xParent, QProgressDialog * progress, bool forExport
 
     // OpenStreetBug
     s = tagValue("_special_","");
-    if (!s.isEmpty() && id().startsWith("osb_")) {
+    if (!s.isEmpty() && id().type & IFeature::Special) {
         QDomElement c = xParent.ownerDocument().createElement("extensions");
         e.appendChild(c);
         QDomElement osbId = xParent.ownerDocument().createElement("id");
@@ -486,38 +510,23 @@ Node * Node::fromXML(Document* d, Layer* L, const QDomElement e)
 {
     double Lat = e.attribute("lat").toDouble();
     double Lon = e.attribute("lon").toDouble();
-    bool Deleted = (e.attribute("deleted") == "true");
 
-    QDateTime time;
-    time = QDateTime::fromString(e.attribute("timestamp").left(19), Qt::ISODate);
-    QString user = e.attribute("user");
-    int Version = e.attribute("version").toInt();
-    if (Version < 1)
-        Version = 9999;
-    Feature::ActorType A = (Feature::ActorType)(e.attribute("actor", "2").toInt());
-
-    QString id = (e.hasAttribute("id") ? e.attribute("id") : e.attribute("xml:id"));
-    if (!id.startsWith('{') && !id.startsWith('-'))
-        id = "node_" + id;
-    Node* Pt = dynamic_cast<Node*>(d->getFeature(id));
+    QString sid = (e.hasAttribute("id") ? e.attribute("id") : e.attribute("xml:id"));
+    IFeature::FId id(IFeature::Point, sid.toLongLong());
+    Node* Pt = CAST_NODE(d->getFeature(id));
     if (!Pt) {
         Pt = new Node(Coord(angToCoord(Lat),angToCoord(Lon)));
         Pt->setId(id);
-        Pt->setLastUpdated(A);
+        Feature::fromXML(e, Pt);
         L->add(Pt);
     } else {
+        Feature::fromXML(e, Pt);
         if (Pt->layer() != L) {
             Pt->layer()->remove(Pt);
             L->add(Pt);
         }
         Pt->setPosition(Coord(angToCoord(Lat), angToCoord(Lon)));
-        if (Pt->lastUpdated() == Feature::NotYetDownloaded)
-            Pt->setLastUpdated(A);
     }
-    Pt->setDeleted(Deleted);
-    Pt->setTime(time);
-    Pt->setUser(user);
-    Pt->setVersionNumber(Version);
 
     Feature::tagsFromXML(d, Pt, e);
 
@@ -529,11 +538,9 @@ Node * Node::fromGPX(Document* d, Layer* L, const QDomElement e)
     double Lat = e.attribute("lat").toDouble();
     double Lon = e.attribute("lon").toDouble();
 
-    QString id = (e.hasAttribute("id") ? e.attribute("id") : e.attribute("xml:id"));
-    if (!id.startsWith('{') && !id.startsWith('-'))
-        id = "node_" + id;
-
-    Node* Pt = dynamic_cast<Node*>(d->getFeature(id));
+    QString sid = (e.hasAttribute("id") ? e.attribute("id") : e.attribute("xml:id"));
+    IFeature::FId id(IFeature::Point, sid.toLongLong());
+    Node* Pt = CAST_NODE(d->getFeature(id));
     if (!Pt) {
         Pt = new Node(Coord(angToCoord(Lat),angToCoord(Lon)));
         Pt->setId(id);
@@ -559,24 +566,24 @@ Node * Node::fromGPX(Document* d, Layer* L, const QDomElement e)
             Pt->setElevation(c.text().toFloat());
         } else
         if (c.tagName() == "name") {
-            Pt->setTag("name", c.text(), false);
+            Pt->setTag("name", c.text());
         } else
         if (c.tagName() == "cmt") {
-            Pt->setTag("_comment_", c.text(), false);
+            Pt->setTag("_comment_", c.text());
         } else
         if (c.tagName() == "desc") {
-            Pt->setTag("_description_", c.text(), false);
+            Pt->setTag("_description_", c.text());
         }
         else if (c.tagName() == "cmt")
         {
-            Pt->setTag("_comment_", c.text(), false);
+            Pt->setTag("_comment_", c.text());
         }
         else if (c.tagName() == "extensions") // for OpenStreetBugs
         {
             QDomNodeList li = c.elementsByTagName("id");
             if (li.size()) {
                 QString id = li.at(0).toElement().text();
-                Pt->setId("osb_" + id);
+                Pt->setId(IFeature::FId(IFeature::Point | IFeature::Special, id.toLongLong()));
                 Pt->setTag("_special_", "yes"); // Assumed to be OpenstreetBugs as they don't use their own namesoace
                 Pt->setSpecial(true);
             }
@@ -595,32 +602,18 @@ QString Node::toHtml()
     int i;
 
 
-    D += "<i>"+QApplication::translate("MapFeature", "timestamp")+": </i>" + time().toString(Qt::ISODate) + "<br/>";
-    D += "<i>"+QApplication::translate("MapFeature", "coord")+": </i>" + QString::number(coordToAng(position().lat()), 'f', 4) + " / " + QString::number(coordToAng(position().lon()), 'f', 4) + "<br/>";
+    if ((i = findKey("_waypoint_")) != -1)
+        D += "<p><b>"+QApplication::translate("MapFeature", "Waypoint")+"</b><br/>";
+    D += "<i>"+QApplication::translate("MapFeature", "coord")+": </i>" + COORD2STRING(coordToAng(position().lat())) + " (" + Coord2Sexa(position().lat()) + ") / " + COORD2STRING(coordToAng(position().lon())) + " (" + Coord2Sexa(position().lon()) + ")";
 
     if (elevation())
-        D += "<i>"+QApplication::translate("MapFeature", "elevation")+": </i>" + QString::number(elevation(), 'f', 4) + "<br/>";
+        D += "<br/><i>"+QApplication::translate("MapFeature", "elevation")+": </i>" + QString::number(elevation(), 'f', 4);
     if (speed())
-        D += "<i>"+QApplication::translate("MapFeature", "speed")+": </i>" + QString::number(speed(), 'f', 4) + "<br/>";
-    if ((i = findKey("_description_")) < tagSize())
-        D += "<i>"+QApplication::translate("MapFeature", "description")+": </i>" + tagValue(i) + "<br/>";
-    if ((i = findKey("_comment_")) < tagSize())
-        D += "<i>"+QApplication::translate("MapFeature", "comment")+": </i>" + tagValue(i) + "<br/>";
-
-    if ((i = findKey("_waypoint_")) < tagSize()) {
-        D += "<p><b>"+QApplication::translate("MapFeature", "Waypoint")+"</b><br/>";
-
-        if ((i = findKey("_description_")) < tagSize())
-            D += "<i>"+QApplication::translate("MapFeature", "description")+": </i>" + tagValue(i) + "<br/>";
-
-        if ((i = findKey("_comment_")) < tagSize())
-            D += "<i>"+QApplication::translate("MapFeature", "comment")+": </i>" + tagValue(i) + "<br/>";
-    }
-
-    D += "<i>"+QApplication::translate("MapFeature", "layer")+": </i>";
-    if (layer())
-        D += layer()->name();
-    D += "<br/>";
+        D += "<br/><i>"+QApplication::translate("MapFeature", "speed")+": </i>" + QString::number(speed(), 'f', 4);
+    if ((i = findKey("_description_")) != -1)
+        D += "<br/><i>"+QApplication::translate("MapFeature", "description")+": </i>" + tagValue(i);
+    if ((i = findKey("_comment_")) != -1)
+        D += "<br/><i>"+QApplication::translate("MapFeature", "comment")+": </i>" + tagValue(i);
 
     return Feature::toMainHtml(QApplication::translate("MapFeature", "Node"), "node").arg(D);
 }
@@ -629,8 +622,8 @@ void Node::toBinary(QDataStream& ds, QHash <QString, quint64>& theIndex)
 {
     Q_UNUSED(theIndex);
 
-    theIndex["N" + QString::number(idToLong())] = ds.device()->pos();
-    ds << (qint8)'N' << idToLong() << (qint32)(Position.lon()) << (qint32)(Position.lat());
+    theIndex["N" + QString::number(id().numId)] = ds.device()->pos();
+    ds << (qint8)'N' << id().numId << (qint32)(Position.topRight().lon()/180.*INT_MAX) << (qint32)(Position.topRight().lat()/180.*INT_MAX);
 }
 
 Node* Node::fromBinary(Document* d, OsbLayer* L, QDataStream& ds, qint8 c, qint64 id)
@@ -639,7 +632,6 @@ Node* Node::fromBinary(Document* d, OsbLayer* L, QDataStream& ds, qint8 c, qint6
 
     qint32	lon;
     qint32	lat;
-    QString strId;
 
     ds >> lon;
     ds >> lat;
@@ -647,12 +639,9 @@ Node* Node::fromBinary(Document* d, OsbLayer* L, QDataStream& ds, qint8 c, qint6
     if (!L)
         return NULL;
 
-    Coord cd( lat, lon );
-    if (id < 1)
-        strId = QString::number(id);
-    else
-        strId = QString("node_%1").arg(QString::number(id));
+    Coord cd( (double)lat/INT_MAX*180., (double)lon/INT_MAX*180. );
 
+    IFeature::FId strId(IFeature::Point, id);
     Node* Pt = CAST_NODE(d->getFeature(strId));
     if (!Pt) {
         Pt = new Node(cd);
@@ -669,3 +658,4 @@ Node* Node::fromBinary(Document* d, OsbLayer* L, QDataStream& ds, qint8 c, qint6
 
     return Pt;
 }
+
