@@ -15,6 +15,8 @@
 #include <utility>
 #include <QList>
 
+#define TEST_RFLAGS(x) theView->renderOptions().options.testFlag(x)
+
 class RelationMemberModel : public QAbstractTableModel
 {
     public:
@@ -34,24 +36,63 @@ class RelationPrivate
 {
     public:
         RelationPrivate(Relation* R)
-            : theRelation(R), theModel(0), ModelReferences(0),
-                BBoxUpToDate(false)
+            : theRelation(R), theModel(0), ModelReferences(0)
+            , BBoxUpToDate(false)
+            , Width(0)
         {
         }
         ~RelationPrivate()
         {
             delete theModel;
         }
+        void CalculateWidth();
+
         Relation* theRelation;
         QList<QPair<QString, MapFeaturePtr> > Members;
         RelationMemberModel* theModel;
         int ModelReferences;
         QPainterPath thePath;
         QPainterPath theBoundingPath;
+
+        mutable CoordBox BBox;
         bool BBoxUpToDate;
 
         RenderPriority theRenderPriority;
-};
+
+        double Width;
+    };
+
+#define DEFAULTWIDTH 6
+#define LANEWIDTH 4
+
+void RelationPrivate::CalculateWidth()
+{
+    QString s(theRelation->tagValue("width",QString()));
+    if (!s.isNull()) {
+        Width = s.toDouble();
+        return;
+    }
+    QString h = theRelation->tagValue("highway",QString());
+    if (s.isNull()) {
+        Width = DEFAULTWIDTH;
+        return;
+    }
+
+    if ( (h == "motorway") || (h=="motorway_link") )
+        Width =  4*LANEWIDTH; // 3 lanes plus emergency
+    else if ( (h == "trunk") || (h=="trunk_link") )
+        Width =  3*LANEWIDTH; // 2 lanes plus emergency
+    else if ( (h == "primary") || (h=="primary_link") )
+        Width =  2*LANEWIDTH; // 2 lanes
+    else if (h == "secondary")
+        Width =  2*LANEWIDTH; // 2 lanes
+    else if (h == "tertiary")
+        Width =  1.5*LANEWIDTH; // shared middle lane
+    else if (h == "cycleway")
+        Width =  1.5;
+    Width = DEFAULTWIDTH;
+}
+
 
 Relation::Relation()
 {
@@ -102,8 +143,8 @@ QString Relation::description() const
 {
     QString s(tagValue("name",""));
     if (!s.isEmpty())
-        return QString("%1 (%2)").arg(s).arg(id());
-    return QString("%1").arg(id());
+        return QString("%1 (%2)").arg(s).arg(id().numId);
+    return QString("%1").arg(id().numId);
 }
 
 const RenderPriority& Relation::renderPriority()
@@ -112,12 +153,12 @@ const RenderPriority& Relation::renderPriority()
     return p->theRenderPriority;
 }
 
-CoordBox Relation::boundingBox() const
+const CoordBox& Relation::boundingBox(bool update) const
 {
-    if (!p->BBoxUpToDate)
+    if (!p->BBoxUpToDate && update)
     {
         if (p->Members.size() == 0)
-            BBox = CoordBox(Coord(0,0),Coord(0,0));
+            p->BBox = CoordBox(Coord(0,0),Coord(0,0));
         else
         {
             CoordBox Clip;
@@ -130,103 +171,73 @@ CoordBox Relation::boundingBox() const
                     } else
                         Clip.merge(p->Members[i].second->boundingBox());
                 }
-            BBox = Clip;
+            p->BBox = Clip;
             p->BBoxUpToDate = true;
         }
     }
-    return BBox;
+    return p->BBox;
 }
 
 void Relation::draw(QPainter& P, MapView* theView)
 {
-    if (!M_PREFS->getRelationsVisible())
+    if (!TEST_RFLAGS(RendererOptions::RelationsVisible))
         return;
 
     if (notEverythingDownloaded())
         P.setPen(QPen(Qt::red,M_PREFS->getRelationsWidth(),Qt::DashLine));
-    else
-        P.setPen(QPen(M_PREFS->getRelationsColor(),M_PREFS->getRelationsWidth(),Qt::DashLine));
+    else {
+        if (isDirty() && isUploadable() && M_PREFS->getDirtyVisible())
+            P.setPen(QPen(M_PREFS->getDirtyColor(),M_PREFS->getDirtyWidth()));
+        else
+            P.setPen(QPen(M_PREFS->getRelationsColor(),M_PREFS->getRelationsWidth(),Qt::DashLine));
+    }
     P.drawPath(theView->transform().map(p->theBoundingPath));
 }
 
-void Relation::drawFocus(QPainter& P, MapView* theView, bool solid)
+void Relation::drawSpecial(QPainter& thePainter, QPen& Pen, MapView* theView)
 {
-    if (!solid) {
-        QPen thePen(M_PREFS->getFocusColor(),M_PREFS->getFocusWidth());
-        thePen.setDashPattern(M_PREFS->getParentDashes());
-        P.setPen(thePen);
-        P.drawPath(theView->transform().map(p->theBoundingPath));
-    } else {
-        P.setPen(QPen(M_PREFS->getFocusColor(),M_PREFS->getFocusWidth(),Qt::DashLine));
-        P.drawPath(theView->transform().map(p->theBoundingPath));
+    QPen TP(Pen);
 
-        for (int i=0; i<p->Members.size(); ++i)
-            if (p->Members[i].second && !p->Members[i].second->isDeleted())
-                if (p->Members[i].second->boundingBox().intersects(theView->viewport()))
-                    p->Members[i].second->drawFocus(P,theView, solid);
+    /* draw relation itself now solid
+    if (Pen.style() == Qt::SolidLine)
+    {
+        TP.setStyle(Qt::DashLine);
+    } */
 
-        if (M_PREFS->getShowParents()) {
-            for (int i=0; i<sizeParents(); ++i)
-                if (!getParent(i)->isDeleted())
-                    if (getParent(i)->boundingBox().intersects(theView->viewport()))
-                        getParent(i)->drawFocus(P, theView, false);
+    thePainter.setPen(TP);
+    thePainter.drawPath(theView->transform().map(p->theBoundingPath));
+}
+
+void Relation::drawParentsSpecial(QPainter& thePainter, QPen& Pen, MapView* theView)
+{
+    for (int i=0; i<sizeParents(); ++i) {
+        if (!getParent(i)->isDeleted()) {
+            Feature* f = CAST_FEATURE(getParent(i));
+            if (f)
+                f->drawSpecial(thePainter, Pen, theView);
         }
     }
 }
 
-void Relation::drawHover(QPainter& P, MapView* theView, bool solid)
+void Relation::drawChildrenSpecial(QPainter& thePainter, QPen& Pen, MapView *theView, int depth)
 {
-    if (!solid) {
-        QPen thePen(M_PREFS->getHoverColor(),M_PREFS->getHoverWidth());
-        thePen.setDashPattern(M_PREFS->getParentDashes());
-        P.setPen(thePen);
-        P.drawPath(theView->transform().map(p->theBoundingPath));
-    } else {
-        P.setPen(QPen(M_PREFS->getHoverColor(),M_PREFS->getHoverWidth(),Qt::DashLine));
-        P.drawPath(theView->transform().map(p->theBoundingPath));
-
-        for (int i=0; i<p->Members.size(); ++i)
-            if (p->Members[i].second && !p->Members[i].second->isDeleted())
-                if (p->Members[i].second->boundingBox().intersects(theView->viewport()))
-                    p->Members[i].second->drawHover(P,theView, false);
-
-        if (M_PREFS->getShowParents()) {
-            for (int i=0; i<sizeParents(); ++i)
-                if (!getParent(i)->isDeleted())
-                    if (getParent(i)->boundingBox().intersects(theView->viewport()))
-                        getParent(i)->drawHover(P, theView, false);
-        }
-    }
+    QPen TP(Pen);
+    TP.setStyle(Qt::DashLine);
+    for (int i=0; i<p->Members.size(); ++i)
+        if (p->Members[i].second && !p->Members[i].second->isDeleted())
+            if (p->Members[i].second->boundingBox().intersects(theView->viewport()))
+            {
+                p->Members[i].second->drawSpecial(thePainter, TP, theView);
+                if (--depth > 0)
+                    p->Members[i].second->drawChildrenSpecial(thePainter, TP, theView, depth);
+            }
 }
 
-void Relation::drawHighlight(QPainter& P, MapView* theView, bool solid)
-{
-    if (!solid) {
-        QPen thePen(M_PREFS->getHoverColor(),M_PREFS->getHighlightWidth());
-        thePen.setDashPattern(M_PREFS->getParentDashes());
-        P.setPen(thePen);
-        P.drawPath(theView->transform().map(p->theBoundingPath));
-    } else {
-        P.setPen(QPen(M_PREFS->getHighlightColor(),M_PREFS->getHighlightWidth(),Qt::DashLine));
-        P.drawPath(theView->transform().map(p->theBoundingPath));
-
-        for (int i=0; i<p->Members.size(); ++i)
-            if (p->Members[i].second && !p->Members[i].second->isDeleted())
-                if (p->Members[i].second->boundingBox().intersects(theView->viewport()))
-                    p->Members[i].second->drawHighlight(P,theView, solid);
-
-//		if (M_PREFS->getShowParents()) {
-//			for (int i=0; i<sizeParents(); ++i)
-//				if (!getParent(i)->isDeleted())
-//					getParent(i)->drawHover(P, theView, false);
-//		}
-    }
-}
 
 double Relation::pixelDistance(const QPointF& Target, double ClearEndDistance, bool, MapView* theView) const
 {
     double Best = 1000000;
-    if (!M_PREFS->getRelationsVisible() && !M_PREFS->getRelationsSelectableWhenHidden())
+    if (!TEST_RFLAGS(RendererOptions::RelationsVisible) && !M_PREFS->getRelationsSelectableWhenHidden())
         return Best;
 
     //for (int i=0; i<p->Members.size(); ++i)
@@ -271,10 +282,12 @@ void Relation::cascadedRemoveIfUsing(Document* theDocument, Feature* aFeature, C
                 QString Role = p->Members[i].first;
                 theList->add(new RelationRemoveFeatureCommand(this, i, theDocument->getDirtyOrOriginLayer(layer())));
                 for (int j=0; j<Alternatives.size(); ++j)
-                    if (i < p->Members.size())
-                        if (p->Members[i+j].second != Alternatives[j])
-                            if (p->Members[i+j-1].second != Alternatives[j])
-                                theList->add(new RelationAddFeatureCommand(this, Role, Alternatives[j], i+j, theDocument->getDirtyOrOriginLayer(Alternatives[j]->layer())));
+                    if (p->Members[i+j].second != Alternatives[j]) {
+                        if ((i+j) == 0)
+                            theList->add(new RelationAddFeatureCommand(this, Role, Alternatives[j], 0, theDocument->getDirtyOrOriginLayer(Alternatives[j]->layer())));
+                        else if (p->Members[i+j-1].second != Alternatives[j])
+                            theList->add(new RelationAddFeatureCommand(this, Role, Alternatives[j], i+j, theDocument->getDirtyOrOriginLayer(Alternatives[j]->layer())));
+                    }
                 continue;
             }
         }
@@ -297,14 +310,14 @@ bool Relation::notEverythingDownloaded()
 void Relation::add(const QString& Role, Feature* F)
 {
     if (layer())
-        layer()->indexRemove(BBox, this);
+        layer()->indexRemove(p->BBox, this);
     p->Members.push_back(qMakePair(Role,F));
     F->setParentFeature(this);
     p->BBoxUpToDate = false;
     MetaUpToDate = false;
-    if (layer() && !isDeleted() && isVisible()) {
+    if (layer() && !isDeleted()) {
         CoordBox bb = boundingBox();
-        layer()->indexAdd(bb, this);
+        layer()->indexAdd(boundingBox(), this);
     }
 
     notifyChanges();
@@ -313,13 +326,13 @@ void Relation::add(const QString& Role, Feature* F)
 void Relation::add(const QString& Role, Feature* F, int Idx)
 {
     if (layer())
-        layer()->indexRemove(BBox, this);
+        layer()->indexRemove(p->BBox, this);
     p->Members.push_back(qMakePair(Role,F));
     std::rotate(p->Members.begin()+Idx,p->Members.end()-1,p->Members.end());
     F->setParentFeature(this);
     p->BBoxUpToDate = false;
     MetaUpToDate = false;
-    if (layer() && !isDeleted() && isVisible()) {
+    if (layer() && !isDeleted()) {
         CoordBox bb = boundingBox();
         layer()->indexAdd(bb, this);
     }
@@ -330,15 +343,15 @@ void Relation::add(const QString& Role, Feature* F, int Idx)
 void Relation::remove(int Idx)
 {
     if (layer())
-        layer()->indexRemove(BBox, this);
-    if (p->Members[Idx].second) {
-        Feature* F = p->Members[Idx].second;
-        F->unsetParentFeature(this);
-    }
+        layer()->indexRemove(p->BBox, this);
+    Feature* F = p->Members[Idx].second;
+    // only remove as parent if the feature is only a member once
     p->Members.erase(p->Members.begin()+Idx);
+    if (F && find(F) == p->Members.size())
+        F->unsetParentFeature(this);
     p->BBoxUpToDate = false;
     MetaUpToDate = false;
-    if (layer() && !isDeleted() && isVisible()) {
+    if (layer() && !isDeleted()) {
         CoordBox bb = boundingBox();
         layer()->indexAdd(bb, this);
     }
@@ -416,7 +429,7 @@ void Relation::buildPath(Projection const &theProjection, const QTransform& /*th
     if (!p->Members.size())
         return;
 
-    QVector<QPointF>theVector;
+    QPolygonF theVector;
     theVector.append(theProjection.project(boundingBox().bottomLeft()));
     theVector.append(theProjection.project(boundingBox().topLeft()));
     theVector.append(theProjection.project(boundingBox().topRight()));
@@ -426,12 +439,12 @@ void Relation::buildPath(Projection const &theProjection, const QTransform& /*th
     //QRectF bb = QPolygonF(theVector).boundingRect();
     //p->theBoundingPath.addRect(bb);
 
-    p->theBoundingPath.addPolygon(QPolygonF(theVector));
+    p->theBoundingPath.addPolygon(theVector);
 
     p->theBoundingPath = p->theBoundingPath.intersected(clipPath);
 }
 
-QPainterPath Relation::getPath()
+const QPainterPath& Relation::getPath() const
 {
     p->thePath = QPainterPath();
     for (int i=0; i<size(); ++i)
@@ -443,13 +456,16 @@ QPainterPath Relation::getPath()
 
 void Relation::updateMeta()
 {
+    Feature::updateMeta();
+    MetaUpToDate = true;
+
+    p->CalculateWidth();
+
     p->theRenderPriority = RenderPriority(RenderPriority::IsSingular, 0., 0);
     for (int i=0; i<p->Members.size(); ++i) {
         if (p->Members.at(i).second->renderPriority() < p->theRenderPriority)
             p->theRenderPriority = p->Members.at(i).second->renderPriority();
     }
-
-    MetaUpToDate = true;
 }
 
 bool Relation::toXML(QDomElement xParent, QProgressDialog * progress, bool strict)
@@ -459,15 +475,7 @@ bool Relation::toXML(QDomElement xParent, QProgressDialog * progress, bool stric
     QDomElement e = xParent.ownerDocument().createElement("relation");
     xParent.appendChild(e);
 
-    e.setAttribute("id", xmlId());
-    e.setAttribute("timestamp", time().toString(Qt::ISODate)+"Z");
-    e.setAttribute("user", user());
-    e.setAttribute("version", versionNumber());
-    if (!strict) {
-        e.setAttribute("actor", (int)lastUpdated());
-        if (isDeleted())
-            e.setAttribute("deleted","true");
-    }
+    Feature::toXML(e, strict);
 
     for (int i=0; i<size(); ++i) {
         QString Type("node");
@@ -484,7 +492,7 @@ bool Relation::toXML(QDomElement xParent, QProgressDialog * progress, bool stric
         n.setAttribute("role", getRole(i));
     }
 
-    tagsToXML(e);
+    tagsToXML(e, strict);
 
     if (progress)
         progress->setValue(progress->value()+1);
@@ -493,30 +501,22 @@ bool Relation::toXML(QDomElement xParent, QProgressDialog * progress, bool stric
 
 Relation * Relation::fromXML(Document * d, Layer * L, const QDomElement e)
 {
-    QString id = e.attribute("id");
-    if (!id.startsWith('{') && !id.startsWith('-'))
-        id = "rel_" + id;
-    QDateTime time = QDateTime::fromString(e.attribute("timestamp").left(19), Qt::ISODate);
-    QString user = e.attribute("user");
-    bool Deleted = (e.attribute("deleted") == "true");
-    int Version = e.attribute("version").toInt();
-    if (Version < 1)
-        Version = 9999;
+    QString sid = (e.hasAttribute("id") ? e.attribute("id") : e.attribute("xml:id"));
+    IFeature::FId id;
+    id.type = IFeature::OsmRelation;
+    id.numId = sid.toLongLong();
 
-    Relation* R = dynamic_cast<Relation*>(d->getFeature(id));
+    Relation* R = CAST_RELATION(d->getFeature(id));
     if (!R) {
         R = new Relation;
         R->setId(id);
-        R->setLastUpdated(Feature::OSMServer);
+        Feature::fromXML(e, R);
     } else {
+        Feature::fromXML(e, R);
         R->layer()->remove(R);
         while (R->p->Members.size())
             R->remove(0);
     }
-    R->setTime(time);
-    R->setUser(user);
-    R->setDeleted(Deleted);
-    R->setVersionNumber(Version);
 
     QDomElement c = e.firstChildElement();
     while(!c.isNull()) {
@@ -524,10 +524,9 @@ Relation * Relation::fromXML(Document * d, Layer * L, const QDomElement e)
             QString Type = c.attribute("type");
             Feature* F = 0;
             if (Type == "node") {
-                QString nId = c.attribute("ref");
-                if (!nId.startsWith('{') && !nId.startsWith('-'))
-                    nId = "node_" + nId;
-                Node* Part = dynamic_cast<Node*>(d->getFeature(nId));
+                QString sId = c.attribute("ref");
+                IFeature::FId nId(IFeature::Point, sId.toLongLong());
+                Node* Part = CAST_NODE(d->getFeature(nId));
                 if (!Part)
                 {
                     Part = new Node(Coord(0,0));
@@ -537,10 +536,9 @@ Relation * Relation::fromXML(Document * d, Layer * L, const QDomElement e)
                 }
                 F = Part;
             } else if (Type == "way") {
-                QString rId = c.attribute("ref");
-                if (!rId.startsWith('{') && !rId.startsWith('-'))
-                    rId = "way_" + rId;
-                Way* Part = dynamic_cast<Way*>(d->getFeature(rId));
+                QString sId = c.attribute("ref");
+                IFeature::FId rId(IFeature::LineString, sId.toLongLong());
+                Way* Part = CAST_WAY(d->getFeature(rId));
                 if (!Part)
                 {
                     Part = new Way;
@@ -550,9 +548,8 @@ Relation * Relation::fromXML(Document * d, Layer * L, const QDomElement e)
                 }
                 F = Part;
             } else if (Type == "relation") {
-                QString RId = c.attribute("ref");
-                if (!RId.startsWith('{') && !RId.startsWith('-'))
-                    RId = "rel_" + RId;
+                QString sId = c.attribute("ref");
+                IFeature::FId RId(IFeature::OsmRelation, sId.toLongLong());
                 Relation* Part = dynamic_cast<Relation*>(d->getFeature(RId));
                 if (!Part)
                 {
@@ -584,9 +581,9 @@ QString Relation::toHtml()
     D += "<i>"+QApplication::translate("MapFeature", "size")+": </i>" + QString::number(size()) + " " + QApplication::translate("MapFeature", "members");
     CoordBox bb = boundingBox();
     D += "<br/>";
-    D += "<i>"+QApplication::translate("MapFeature", "Topleft")+": </i>" + QString::number(coordToAng(bb.topLeft().lat()), 'f', 4) + " / " + QString::number(coordToAng(bb.topLeft().lon()), 'f', 4);
+    D += "<i>"+QApplication::translate("MapFeature", "Topleft")+": </i>" + COORD2STRING(coordToAng(bb.topLeft().lat())) + " / " + COORD2STRING(coordToAng(bb.topLeft().lon()));
     D += "<br/>";
-    D += "<i>"+QApplication::translate("MapFeature", "Botright")+": </i>" + QString::number(coordToAng(bb.bottomRight().lat()), 'f', 4) + " / " + QString::number(coordToAng(bb.bottomRight().lon()), 'f', 4);
+    D += "<i>"+QApplication::translate("MapFeature", "Botright")+": </i>" + COORD2STRING(coordToAng(bb.bottomRight().lat())) + " / " + COORD2STRING(coordToAng(bb.bottomRight().lon()));
 
     return Feature::toMainHtml(QApplication::translate("MapFeature", "Relation"),"relation").arg(D);
 }
@@ -596,25 +593,25 @@ void Relation::toBinary(QDataStream& ds, QHash <QString, quint64>& theIndex)
     quint8 Type = '\0';
     quint64 ref;
 
-    theIndex["L" + QString::number(idToLong())] = ds.device()->pos();
+    theIndex["L" + id().numId] = ds.device()->pos();
     ds << (qint8)'L';
-    ds << idToLong();
+    ds << id().numId;
     ds << size();
     for (int i=0; i<size(); ++i) {
         if (dynamic_cast<const Node*>(get(i))) {
             Type='N';
-            ref = get(i)->idToLong();
+            ref = get(i)->id().numId;
         }
         else if (dynamic_cast<const Way*>(get(i))) {
             Type='R';
-            ref = get(i)->idToLong();
+            ref = get(i)->id().numId;
         }
         else if (dynamic_cast<const Relation*>(get(i))) {
             Type='L';
-            ref = get(i)->idToLong();
+            ref = get(i)->id().numId;
         }
 //		ds << (qint8) Type << ref << getRole(i);
-        ds << Type << get(i)->idToLong() << getRole(i);
+        ds << Type << get(i)->id().numId << getRole(i);
     }
 }
 
@@ -623,7 +620,6 @@ Relation* Relation::fromBinary(Document* d, OsbLayer* L, QDataStream& ds, qint8
     Q_UNUSED(c);
 
     qint32	fSize;
-    QString strId;
     quint8 Type;
     qint64 refId;
     QString Role;
@@ -639,12 +635,8 @@ Relation* Relation::fromBinary(Document* d, OsbLayer* L, QDataStream& ds, qint8
         return NULL;
     }
 
-    if (id < 1)
-        strId = QString::number(id);
-    else
-        strId = QString("rel_%1").arg(QString::number(id));
-
-    Relation* R = dynamic_cast<Relation*>(d->getFeature(strId));
+    IFeature::FId strId(IFeature::OsmRelation, id);
+    Relation* R = CAST_RELATION(d->getFeature(strId));
     if (!R) {
         R = new Relation();
         R->setId(strId);
@@ -672,13 +664,13 @@ Relation* Relation::fromBinary(Document* d, OsbLayer* L, QDataStream& ds, qint8
         Feature* F;
         switch (Type) {
             case 'N':
-                F = d->getFeature(QString("node_%1").arg(refId));
+                F = d->getFeature(IFeature::FId(IFeature::Point, refId));
                 break;
             case 'R':
-                F = d->getFeature(QString("way_%1").arg(refId));
+                F = d->getFeature(IFeature::FId(IFeature::LineString, refId));
                 break;
             case 'L':
-                F = d->getFeature(QString("rel_%1").arg(refId));
+                F = d->getFeature(IFeature::FId(IFeature::OsmRelation, refId));
                 break;
         default:
                 F = NULL;
@@ -692,6 +684,14 @@ Relation* Relation::fromBinary(Document* d, OsbLayer* L, QDataStream& ds, qint8
     return R;
 }
 
+double Relation::widthOf()
+{
+    if (MetaUpToDate == false)
+        updateMeta();
+
+    return p->Width;
+}
+
 /* RELATIONMODEL */
 
 RelationMemberModel::RelationMemberModel(RelationPrivate *aParent, MainWindow* aMain)
@@ -764,7 +764,7 @@ bool RelationMemberModel::setData(const QModelIndex &index, const QVariant &valu
     if (index.isValid() && role == Qt::EditRole)
     {
         Feature* Tmp = Parent->Members[index.row()].second;
-        CommandList* L = new CommandList(MainWindow::tr("Relation Modified %1").arg(Parent->theRelation->id()), Parent->theRelation);
+        CommandList* L = new CommandList(MainWindow::tr("Relation Modified %1").arg(Parent->theRelation->id().numId), Parent->theRelation);
         L->add(new RelationRemoveFeatureCommand(Parent->theRelation, index.row(), Main->document()->getDirtyOrOriginLayer(Parent->theRelation->layer())));
         L->add(new RelationAddFeatureCommand(Parent->theRelation,value.toString(),Tmp,index.row(), Main->document()->getDirtyOrOriginLayer(Parent->theRelation->layer())));
         Main->document()->addHistory(L);
@@ -774,9 +774,3 @@ bool RelationMemberModel::setData(const QModelIndex &index, const QVariant &valu
     return false;
 }
 
-
-
-
-
-
-
