@@ -1,3 +1,5 @@
+#include "Global.h"
+
 #include "Way.h"
 #include "Node.h"
 
@@ -13,7 +15,6 @@
 #include <QApplication>
 #include <QtGui/QPainter>
 #include <QtGui/QPainterPath>
-#include <QMessageBox>
 #include <QProgressDialog>
 
 #include <algorithm>
@@ -26,6 +27,7 @@
 #include <ggl/algorithms/intersects.hpp>
 #endif
 
+#define TEST_RFLAGS(x) theView->renderOptions().options.testFlag(x)
 
 class WayPrivate
 {
@@ -35,6 +37,8 @@ class WayPrivate
             , IsCoastline(false), Area(0), Distance(0)
             , wasPathComplete(false), VirtualsUptodate(false)
             , ProjectionRevision(0)
+            , BestSegment(-1)
+            , Width(0)
         {
         }
         Way* theWay;
@@ -43,6 +47,8 @@ class WayPrivate
         std::vector<NodePtr> virtualNodes;
         QList<Coord> Smoothed;
         bool SmoothedUpToDate;
+
+        mutable CoordBox BBox;
         bool BBoxUpToDate;
 
         bool IsCoastline;
@@ -57,7 +63,10 @@ class WayPrivate
 #ifndef _MOBILE
         int ProjectionRevision;
 #endif
+        int BestSegment;
+        double Width;
 
+        void CalculateWidth();
         void updateSmoothed(bool DoSmooth);
         void addSmoothedBezier(int i, int j, int k, int l);
         void doUpdateVirtuals();
@@ -65,6 +74,37 @@ class WayPrivate
         void addVirtuals();
 };
 
+#define DEFAULTWIDTH 6
+#define LANEWIDTH 4
+
+void WayPrivate::CalculateWidth()
+{
+    QString s(theWay->tagValue("width",QString()));
+    if (!s.isNull()) {
+        Width = s.toDouble();
+        return;
+    }
+    QString h = theWay->tagValue("highway",QString());
+    if (s.isNull()) {
+        Width = DEFAULTWIDTH;
+        return;
+    }
+
+    if ( (h == "motorway") || (h=="motorway_link") )
+        Width =  4*LANEWIDTH; // 3 lanes plus emergency
+    else if ( (h == "trunk") || (h=="trunk_link") )
+        Width =  3*LANEWIDTH; // 2 lanes plus emergency
+    else if ( (h == "primary") || (h=="primary_link") )
+        Width =  2*LANEWIDTH; // 2 lanes
+    else if (h == "secondary")
+        Width =  2*LANEWIDTH; // 2 lanes
+    else if (h == "tertiary")
+        Width =  1.5*LANEWIDTH; // shared middle lane
+    else if (h == "cycleway")
+        Width =  1.5;
+    Width = DEFAULTWIDTH;
+}
+
 void WayPrivate::addSmoothedBezier(int i, int j, int k, int l)
 {
     Coord A(Nodes[i]->position());
@@ -127,7 +167,7 @@ void WayPrivate::doUpdateVirtuals()
         return;
 
     removeVirtuals();
-    if (M_PREFS->getUseVirtualNodes() && theWay->layer() && !(theWay->layer()->isReadonly()) && !(theWay->isDeleted()))
+    if (M_PREFS->getUseVirtualNodes() && theWay->layer() && !(theWay->isReadonly()) && !(theWay->isDeleted()))
         addVirtuals();
 
     VirtualsUptodate = true;
@@ -162,11 +202,21 @@ Way::~Way(void)
     delete p;
 }
 
+char Way::getType() const
+{
+    if (isClosed())
+        return (IFeature::LineString | IFeature::Polygon);
+    else
+        return IFeature::LineString;
+}
+
+
 void Way::setDeleted(bool delState)
 {
     Feature::setDeleted(delState);
     p->VirtualsUptodate = false;
-    p->doUpdateVirtuals();
+    if (p->Nodes.size())
+        p->doUpdateVirtuals();
 }
 
 void Way::setLayer(Layer* L)
@@ -205,8 +255,8 @@ QString Way::description() const
 {
     QString s(tagValue("name",""));
     if (!s.isEmpty())
-        return QString("%1 (%2)").arg(s).arg(id());
-    return QString("%1").arg(id());
+        return QString("%1 (%2)").arg(s).arg(id().numId);
+    return QString("%1").arg(id().numId);
 }
 
 void Way::add(Node* Pt)
@@ -217,7 +267,7 @@ void Way::add(Node* Pt)
 void Way::add(Node* Pt, int Idx)
 {
     if (layer())
-        layer()->indexRemove(BBox, this);
+        layer()->indexRemove(p->BBox, this);
     p->Nodes.insert(p->Nodes.begin() + Idx, Pt);
 //	p->Nodes.push_back(Pt);
 //	std::rotate(p->Nodes.begin()+Idx,p->Nodes.end()-1,p->Nodes.end());
@@ -228,7 +278,7 @@ void Way::add(Node* Pt, int Idx)
     p->SmoothedUpToDate = false;
     p->wasPathComplete = false;
     p->VirtualsUptodate = false;
-    if (layer() && !isDeleted() && isVisible()) {
+    if (layer() && !isDeleted()) {
         CoordBox bb = boundingBox();
         layer()->indexAdd(bb, this);
     }
@@ -255,20 +305,20 @@ int Way::findVirtual(Feature* Pt) const
 void Way::remove(int idx)
 {
     if (layer())
-        layer()->indexRemove(BBox, this);
+        layer()->indexRemove(p->BBox, this);
 
-    if (p->Nodes[idx]) {
-        Node* Pt = p->Nodes[idx];
-        Pt->unsetParentFeature(this);
-    }
+    Node* Pt = p->Nodes[idx];
+    // only remove as parent if the node is only included once
     p->Nodes.erase(p->Nodes.begin()+idx);
+    if (Pt && (find(Pt) >= size()))
+        Pt->unsetParentFeature(this);
     p->BBoxUpToDate = false;
     p->wasPathComplete = false;
     MetaUpToDate = false;
     p->SmoothedUpToDate = false;
     p->wasPathComplete = false;
     p->VirtualsUptodate = false;
-    if (layer() && !isDeleted() && isVisible()) {
+    if (layer() && !isDeleted()) {
         CoordBox bb = boundingBox();
         layer()->indexAdd(bb, this);
     }
@@ -332,27 +382,31 @@ bool Way::notEverythingDownloaded()
     return p->NotEverythingDownloaded;
 }
 
-CoordBox Way::boundingBox() const
+const CoordBox& Way::boundingBox(bool update) const
 {
-    if (!p->BBoxUpToDate)
+    if (!p->BBoxUpToDate && update)
     {
         if (p->Nodes.size())
         {
-            BBox = CoordBox(p->Nodes[0]->position(),p->Nodes[0]->position());
+            p->BBox = CoordBox(p->Nodes[0]->position(),p->Nodes[0]->position());
             for (unsigned int i=1; i<p->Nodes.size(); ++i)
-                BBox.merge(p->Nodes[i]->position());
+                p->BBox.merge(p->Nodes[i]->position());
         }
         else
-            BBox = CoordBox(Coord(0,0),Coord(0,0));
+            p->BBox = CoordBox(Coord(0,0),Coord(0,0));
         p->BBoxUpToDate = true;
     }
-    return BBox;
+    return p->BBox;
 }
 
 void Way::updateMeta()
 {
+    Feature::updateMeta();
+    MetaUpToDate = true;
+
     p->Area = 0;
     p->Distance = 0;
+    p->CalculateWidth();
 
     p->NotEverythingDownloaded = false;
     if (lastUpdated() == Feature::NotYetDownloaded)
@@ -365,12 +419,7 @@ void Way::updateMeta()
             }
 
     if (p->Nodes.size() == 0)
-    {
-        MetaUpToDate = true;
         return;
-    }
-
-    p->doUpdateVirtuals();
 
     bool isArea = false;
     if (tagValue("junction", "") != "roundabout")
@@ -402,7 +451,7 @@ void Way::updateMeta()
         setRenderPriority(RenderPriority(RenderPriority::IsLinear,Priority, layer));
     }
 
-    MetaUpToDate = true;
+    p->doUpdateVirtuals();
 }
 
 double Way::distance()
@@ -432,11 +481,25 @@ double Way::area()
     return p->Area;
 }
 
+double Way::widthOf()
+{
+    if (MetaUpToDate == false)
+        updateMeta();
+
+    return p->Width;
+}
+
 void Way::draw(QPainter& P, MapView* theView)
 {
+    if (isDirty() && isUploadable() && M_PREFS->getDirtyVisible()) {
+        QPen thePen(M_PREFS->getDirtyColor(),M_PREFS->getDirtyWidth());
+        P.setPen(thePen);
+        P.drawPath(theView->transform().map(getPath()));
+    }
+
     double theWidth = theView->nodeWidth();
     bool Draw = (theWidth >= 1);
-    if (!Draw || !M_PREFS->getVirtualNodesVisible() || !M_PREFS->getTrackPointsVisible())
+    if (!Draw || !TEST_RFLAGS(RendererOptions::VirtualNodesVisible) || !TEST_RFLAGS(RendererOptions::NodesVisible) || isReadonly())
         return;
 
     theWidth /= 2;
@@ -450,120 +513,52 @@ void Way::draw(QPainter& P, MapView* theView)
     }
 }
 
-void Way::drawHover(QPainter& thePainter, MapView* theView, bool solid)
+void Way::drawSpecial(QPainter& thePainter, QPen& Pen, MapView* theView)
 {
-    // FIXME Selected route
-    if (!size())
-        return;
+    thePainter.setPen(Pen);
+    if (p->BestSegment != -1)
+        thePainter.drawLine(theView->transform().map(theView->projection().project(getSegment(p->BestSegment))));
+    else
+        thePainter.drawPath(theView->transform().map(p->thePath));
+}
 
-    QFont F(thePainter.font());
-    F.setPointSize(10);
-    F.setBold(true);
-    F.setWeight(QFont::Black);
-    thePainter.setFont(F);
-    QPen TP(MerkaartorPreferences::instance()->getHoverColor());
-    TP.setWidth(MerkaartorPreferences::instance()->getHoverWidth());
-    if (!solid) {
-        TP.setDashPattern(M_PREFS->getParentDashes());
-    }
-    thePainter.setPen(TP);
-    thePainter.setBrush(Qt::NoBrush);
-    //QRect clipRect = thePainter.clipRegion().boundingRect().adjusted(int(-20), int(-20), int(20), int(20));
-    //buildPath(theProjection, clipRect);
-    thePainter.drawPath(theView->transform().map(p->thePath));
-    if (solid) {
-        TP.setWidth(MerkaartorPreferences::instance()->getHoverWidth()*3);
-        TP.setCapStyle(Qt::RoundCap);
-        thePainter.setPen(TP);
-        QPolygonF Pl;
-        buildPolygonFromRoad(this,theView->projection(),Pl);
-        thePainter.drawPoints(theView->transform().map(Pl));
-
-        if (M_PREFS->getShowParents()) {
-            for (int i=0; i<sizeParents(); ++i)
-                if (!getParent(i)->isDeleted())
-                    getParent(i)->drawHover(thePainter, theView, false);
+void Way::drawParentsSpecial(QPainter& thePainter, QPen& Pen, MapView* theView)
+{
+    for (int i=0; i<sizeParents(); ++i) {
+        if (!getParent(i)->isDeleted()) {
+            Feature* f = CAST_FEATURE(getParent(i));
+            if (f)
+                f->drawSpecial(thePainter, Pen, theView);
         }
     }
 }
 
-void Way::drawHighlight(QPainter& thePainter, MapView* theView, bool solid)
+void Way::drawChildrenSpecial(QPainter& thePainter, QPen& Pen, MapView *theView, int depth)
 {
-    // FIXME Selected route
-    if (!size())
-        return;
+    Q_UNUSED(depth);
 
-    QFont F(thePainter.font());
-    F.setPointSize(10);
-    F.setBold(true);
-    F.setWeight(QFont::Black);
-    thePainter.setFont(F);
-    QPen TP(MerkaartorPreferences::instance()->getHighlightColor());
-    TP.setWidth(MerkaartorPreferences::instance()->getHighlightWidth());
-    if (!solid) {
-        TP.setDashPattern(M_PREFS->getParentDashes());
-    }
+    QPen TP(Pen);
+    TP.setWidth(TP.width()*3);
+    TP.setCapStyle(Qt::RoundCap);
     thePainter.setPen(TP);
-    thePainter.setBrush(Qt::NoBrush);
-    //QRect clipRect = thePainter.clipRegion().boundingRect().adjusted(int(-20), int(-20), int(20), int(20));
-    //buildPath(theProjection, clipRect);
-    thePainter.drawPath(theView->transform().map(p->thePath));
-    if (solid) {
-        TP.setWidth(MerkaartorPreferences::instance()->getHighlightWidth()*3);
-        TP.setCapStyle(Qt::RoundCap);
-        thePainter.setPen(TP);
-        QPolygonF Pl;
-        buildPolygonFromRoad(this,theView->projection(),Pl);
-        thePainter.drawPoints(theView->transform().map(Pl));
 
-//		if (M_PREFS->getShowParents()) {
-//			for (int i=0; i<sizeParents(); ++i)
-//				if (!getParent(i)->isDeleted())
-//					getParent(i)->drawHover(thePainter, theView, false);
-//		}
-    }
+    QPolygonF Pl;
+    if (p->BestSegment != -1) {
+        for (int i=p->BestSegment; i<=p->BestSegment+1; ++i)
+            if (getNode(i)->isVisible() && !getNode(i)->isVirtual())
+                Pl.append(theView->projection().project(getNode(i)));
+    } else
+        buildPolygonFromRoad(this,theView->projection(),Pl);
+
+    thePainter.drawPoints(theView->transform().map(Pl));
 }
 
-void Way::drawFocus(QPainter& thePainter, MapView* theView, bool solid)
-{
-    // FIXME Selected route
-    if (!size())
-        return;
-
-    QFont F(thePainter.font());
-    F.setPointSize(10);
-    F.setBold(true);
-    F.setWeight(QFont::Black);
-    thePainter.setFont(F);
-    QPen TP(MerkaartorPreferences::instance()->getFocusColor());
-    TP.setWidth(MerkaartorPreferences::instance()->getFocusWidth());
-    if (!solid) {
-        TP.setDashPattern(M_PREFS->getParentDashes());
-    }
-    thePainter.setPen(TP);
-    thePainter.setBrush(Qt::NoBrush);
-    //QRect clipRect = thePainter.clipRegion().boundingRect().adjusted(int(-20), int(-20), int(20), int(20));
-    //buildPath(theProjection, clipRect);
-    thePainter.drawPath(theView->transform().map(p->thePath));
-    if (solid) {
-        TP.setWidth(MerkaartorPreferences::instance()->getFocusWidth()*3);
-        TP.setCapStyle(Qt::RoundCap);
-        thePainter.setPen(TP);
-        QPolygonF Pl;
-        buildPolygonFromRoad(this,theView->projection(),Pl);
-        thePainter.drawPoints(theView->transform().map(Pl));
-
-        if (M_PREFS->getShowParents()) {
-            for (int i=0; i<sizeParents(); ++i)
-                if (!getParent(i)->isDeleted())
-                    getParent(i)->drawFocus(thePainter, theView, false);
-        }
-    }
-}
 
 double Way::pixelDistance(const QPointF& Target, double ClearEndDistance, bool selectNodes, MapView* theView) const
 {
     double Best = 1000000;
+    p->BestSegment = -1;
+
     if (selectNodes) {
         for (unsigned int i=0; i<p->Nodes.size(); ++i)
         {
@@ -587,13 +582,16 @@ double Way::pixelDistance(const QPointF& Target, double ClearEndDistance, bool s
                 Best = D;
         }
     else
-        for (unsigned int i=1; i<p->Nodes.size(); ++i)
+        for (unsigned int i=0; i<p->Nodes.size()-1; ++i)
         {
-            if (p->Nodes.at(i) && p->Nodes.at(i-1)) {
-                LineF F(theView->toView(p->Nodes.at(i-1)),theView->toView(p->Nodes.at(i)));
+            if (p->Nodes.at(i) && p->Nodes.at(i+1)) {
+                LineF F(theView->toView(p->Nodes.at(i)),theView->toView(p->Nodes.at(i+1)));
                 double D = F.capDistance(Target);
-                if (D < ClearEndDistance)
+                if (D < ClearEndDistance && D < Best) {
                     Best = D;
+                    if (g_Merk_Segment_Mode)
+                        p->BestSegment = i;
+                }
             }
         }
     return Best;
@@ -626,15 +624,12 @@ void Way::cascadedRemoveIfUsing(Document* theDocument, Feature* aFeature, Comman
             }
             for (int j=0; j<Alternatives.size(); ++j)
             {
-                if (i < p->Nodes.size())
+                if (p->Nodes[i+j] != Alternatives[j])
                 {
-                    if (p->Nodes[i+j] != Alternatives[j])
-                    {
-                        if ((i+j) == 0)
-                            theList->add(new WayAddNodeCommand(this, Alternatives[j], i+j,theDocument->getDirtyOrOriginLayer(layer())));
-                        else if (p->Nodes[i+j-1] != Alternatives[j] && p->Nodes[i+j+1] != Alternatives[j])
-                            theList->add(new WayAddNodeCommand(this, Alternatives[j], i+j,theDocument->getDirtyOrOriginLayer(layer())));
-                    }
+                    if ((i+j) == 0)
+                        theList->add(new WayAddNodeCommand(this, Alternatives[j], 0,theDocument->getDirtyOrOriginLayer(layer())));
+                    else if (p->Nodes[i+j-1] != Alternatives[j] && p->Nodes[i+j+1] != Alternatives[j])
+                        theList->add(new WayAddNodeCommand(this, Alternatives[j], i+j,theDocument->getDirtyOrOriginLayer(layer())));
                 }
             }
             theList->add(new WayRemoveNodeCommand(this, (Node*)aFeature,theDocument->getDirtyOrOriginLayer(layer())));
@@ -647,15 +642,18 @@ void Way::cascadedRemoveIfUsing(Document* theDocument, Feature* aFeature, Comman
     }
 }
 
-QPainterPath Way::getPath()
+const QPainterPath& Way::getPath() const
 {
     return p->thePath;
 }
 
-void Way::buildPath(const Projection &theProjection, const QTransform& /*theTransform*/, const QRectF& cr)
+void Way::buildPath(const Projection &theProjection, const QTransform& theTransform, const QRectF& cr)
 {
     using namespace ggl;
 
+    Q_UNUSED(theTransform);
+    Q_UNUSED(cr);
+
     if (p->Nodes.size() < 2)
         return;
 
@@ -669,91 +667,94 @@ void Way::buildPath(const Projection &theProjection, const QTransform& /*theTran
         p->ProjectionRevision = theProjection.projectionRevision();
         p->wasPathComplete = true;
 
-        QPointF pbl = theProjection.project(BBox.bottomLeft());
-        QPointF ptr = theProjection.project(BBox.topRight());
+        QPointF pbl = theProjection.project(p->BBox.bottomLeft());
+        QPointF ptr = theProjection.project(p->BBox.topRight());
         p->roadRect = QRectF(pbl, ptr);
     }
-
-    bool toClip = !cr.contains(p->roadRect);
-//    bool toClip = false;
-    if (!toClip) {
-        p->thePath = QPainterPath(p->theFullPath);
+    if (!g_Merk_SelfClip) {
+        p->thePath = p->theFullPath;
     } else {
-        if (area() > 0) {
-            QPainterPath clipPath;
-            clipPath.addRect(cr);
-
-            p->thePath = clipPath.intersected(p->theFullPath);
+        bool toClip = !cr.contains(p->roadRect);
+    //    bool toClip = false;
+        if (!toClip) {
+            p->thePath = p->theFullPath;
         } else {
-            std::vector<linestring_2d> clipped;
+            if (area() > 0) {
+                QPainterPath clipPath;
+                clipPath.addRect(cr);
 
-            // Ensure nodes' projection is up-to-date
-            for (unsigned int i=0; i<p->Nodes.size(); ++i)
-                theProjection.project(p->Nodes[i]);
-            box_2d clipRect (make<point_2d>(cr.bottomLeft().x(), cr.topRight().y()), make<point_2d>(cr.topRight().x(), cr.bottomLeft().y()));
+                p->thePath = clipPath.intersected(p->theFullPath);
+            } else {
+                std::vector<linestring_2d> clipped;
 
-            intersection <linestring_2d, box_2d, /*linestring_2d*/ std::vector<NodePtr>, std::back_insert_iterator <std::vector<linestring_2d> > >
-                    (clipRect, /*in*/ p->Nodes, std::back_inserter(clipped));
+                // Ensure nodes' projection is up-to-date
+                for (unsigned int i=0; i<p->Nodes.size(); ++i)
+                    theProjection.project(p->Nodes[i]);
+                box_2d clipRect (make<point_2d>(cr.bottomLeft().x(), cr.topRight().y()), make<point_2d>(cr.topRight().x(), cr.bottomLeft().y()));
 
-            p->thePath = QPainterPath();
-            for (std::vector<linestring_2d>::const_iterator it = clipped.begin(); it != clipped.end(); it++)
-            {
-                if (!(*it).empty()) {
-                    p->thePath.moveTo(QPointF((*it)[0].x(), (*it)[0].y()));
-                }
-                for (linestring_2d::const_iterator itl = (*it).begin()+1; itl != (*it).end(); itl++)
+                intersection <linestring_2d, box_2d, /*linestring_2d*/ std::vector<NodePtr>, std::back_insert_iterator <std::vector<linestring_2d> > >
+                        (clipRect, /*in*/ p->Nodes, std::back_inserter(clipped));
+
+                p->thePath = QPainterPath();
+                for (std::vector<linestring_2d>::const_iterator it = clipped.begin(); it != clipped.end(); it++)
                 {
-                    p->thePath.lineTo(QPointF((*itl).x(), (*itl).y()));
+                    if (!(*it).empty()) {
+                        p->thePath.moveTo(QPointF((*it)[0].x(), (*it)[0].y()));
+                    }
+                    for (linestring_2d::const_iterator itl = (*it).begin()+1; itl != (*it).end(); itl++)
+                    {
+                        p->thePath.lineTo(QPointF((*itl).x(), (*itl).y()));
+                    }
                 }
-            }
 
-//            bool lastPointVisible = true;
-//            QPointF lastPoint = p->Nodes.at(0)->projection();
-//            QPointF aP = lastPoint;
-//            if (!cr.contains(aP)) {
-//                aP.setX(qMax(cr.left(), aP.x()));
-//                aP.setX(qMin(cr.right(), aP.x()));
-//                aP.setY(qMax(cr.top(), aP.y()));
-//                aP.setY(qMin(cr.bottom(), aP.y()));
-//                lastPointVisible = false;
-//            }
-//            p->thePath = QPainterPath();
-//            p->thePath.moveTo(aP);
-//            for (int j=1; j<size(); ++j) {
-//                aP = p->Nodes.at(j)->projection();
-//                QLineF l(lastPoint, aP);
-//                if (!cr.contains(aP)) {
-//                    if (!lastPointVisible) {
-//                        QPointF a, b;
-//                        if (Utils::QRectInterstects(cr, l, a, b)) {
-//                            p->thePath.lineTo(a);
-//                            lastPoint = aP;
-//                            aP = b;
-//                        } else {
-//                            lastPoint = aP;
-//                            aP.setX(qMax(cr.left(), aP.x()));
-//                            aP.setX(qMin(cr.right(), aP.x()));
-//                            aP.setY(qMax(cr.top(), aP.y()));
-//                            aP.setY(qMin(cr.bottom(), aP.y()));
-//                        }
-//                    } else {
-//                        QPointF a, b;
-//                        Utils::QRectInterstects(cr, l, a, b);
-//                        lastPoint = aP;
-//                        aP = a;
-//                    }
-//                    lastPointVisible = false;
-//                } else {
-//                    if (!lastPointVisible) {
-//                        QPointF a, b;
-//                        Utils::QRectInterstects(cr, l, a, b);
-//                        p->thePath.lineTo(a);
-//                    }
-//                    lastPoint = aP;
-//                    lastPointVisible = true;
-//                }
-//                p->thePath.lineTo(aP);
-//            }
+    //            bool lastPointVisible = true;
+    //            QPointF lastPoint = p->Nodes.at(0)->projection();
+    //            QPointF aP = lastPoint;
+    //            if (!cr.contains(aP)) {
+    //                aP.setX(qMax(cr.left(), aP.x()));
+    //                aP.setX(qMin(cr.right(), aP.x()));
+    //                aP.setY(qMax(cr.top(), aP.y()));
+    //                aP.setY(qMin(cr.bottom(), aP.y()));
+    //                lastPointVisible = false;
+    //            }
+    //            p->thePath = QPainterPath();
+    //            p->thePath.moveTo(aP);
+    //            for (int j=1; j<size(); ++j) {
+    //                aP = p->Nodes.at(j)->projection();
+    //                QLineF l(lastPoint, aP);
+    //                if (!cr.contains(aP)) {
+    //                    if (!lastPointVisible) {
+    //                        QPointF a, b;
+    //                        if (Utils::QRectInterstects(cr, l, a, b)) {
+    //                            p->thePath.lineTo(a);
+    //                            lastPoint = aP;
+    //                            aP = b;
+    //                        } else {
+    //                            lastPoint = aP;
+    //                            aP.setX(qMax(cr.left(), aP.x()));
+    //                            aP.setX(qMin(cr.right(), aP.x()));
+    //                            aP.setY(qMax(cr.top(), aP.y()));
+    //                            aP.setY(qMin(cr.bottom(), aP.y()));
+    //                        }
+    //                    } else {
+    //                        QPointF a, b;
+    //                        Utils::QRectInterstects(cr, l, a, b);
+    //                        lastPoint = aP;
+    //                        aP = a;
+    //                    }
+    //                    lastPointVisible = false;
+    //                } else {
+    //                    if (!lastPointVisible) {
+    //                        QPointF a, b;
+    //                        Utils::QRectInterstects(cr, l, a, b);
+    //                        p->thePath.lineTo(a);
+    //                    }
+    //                    lastPoint = aP;
+    //                    lastPointVisible = true;
+    //                }
+    //                p->thePath.lineTo(aP);
+    //            }
+            }
         }
     }
 }
@@ -763,34 +764,31 @@ bool Way::deleteChildren(Document* theDocument, CommandList* theList)
     if (lastUpdated() == Feature::OSMServerConflict)
         return true;
 
-    MDiscardableMessage dlg(NULL,
-        MainWindow::tr("Delete Children."),
-        MainWindow::tr("Do you want to delete the children nodes also?"));
-    if (dlg.check() == QDialog::Accepted) {
-        QList<Feature*> Alternatives;
-        QMap<Feature*, int> ToDelete;
-        for (int i=(int)p->Nodes.size()-1; i>=0; --i) {
-            Node* N = p->Nodes[i];
-            int sizeValidParents = 0;
-            for (int j=0; j<N->sizeParents(); ++j)
-                if (!N->getParent(j)->isDeleted())
-                    sizeValidParents++;
-            if (sizeValidParents == 1) {
-                ToDelete[N] = i;
-            }
-        }
-        QList<Feature*> ToDeleteKeys = ToDelete.uniqueKeys();
-        for (int i=0; i<ToDeleteKeys.size(); ++i) {
-            if (!ToDeleteKeys[i]->isDeleted())
-                theList->add(new RemoveFeatureCommand(theDocument, ToDeleteKeys[i], Alternatives));
+    QList<Feature*> Alternatives;
+    QMap<Feature*, int> ToDelete;
+    for (int i=(int)p->Nodes.size()-1; i>=0; --i) {
+        Node* N = p->Nodes[i];
+        if (!theDocument->isDownloadedSafe(N->boundingBox()) && N->hasOSMId())
+            continue;
+        int sizeValidParents = 0;
+        for (int j=0; j<N->sizeParents(); ++j)
+            if (!N->getParent(j)->isDeleted())
+                sizeValidParents++;
+        if (sizeValidParents == 1) {
+            ToDelete[N] = i;
         }
     }
+    QList<Feature*> ToDeleteKeys = ToDelete.uniqueKeys();
+    for (int i=0; i<ToDeleteKeys.size(); ++i) {
+        if (!ToDeleteKeys[i]->isDeleted())
+            theList->add(new RemoveFeatureCommand(theDocument, ToDeleteKeys[i], Alternatives));
+    }
     return true;
 }
 
-void Way::setTag(const QString& key, const QString& value, bool addToTagList)
+void Way::setTag(const QString& key, const QString& value)
 {
-    Feature::setTag(key, value, addToTagList);
+    Feature::setTag(key, value);
     MetaUpToDate = false;
     if (key == "natural") {
         if (value == "coastline")
@@ -800,9 +798,9 @@ void Way::setTag(const QString& key, const QString& value, bool addToTagList)
     }
 }
 
-void Way::setTag(int index, const QString& key, const QString& value, bool addToTagList)
+void Way::setTag(int index, const QString& key, const QString& value)
 {
-    Feature::setTag(index, key, value, addToTagList);
+    Feature::setTag(index, key, value);
     MetaUpToDate = false;
     if (key == "natural") {
         if (value == "coastline")
@@ -873,15 +871,7 @@ bool Way::toXML(QDomElement xParent, QProgressDialog * progress, bool strict)
     QDomElement e = xParent.ownerDocument().createElement("way");
     xParent.appendChild(e);
 
-    e.setAttribute("id", xmlId());
-    e.setAttribute("timestamp", time().toString(Qt::ISODate)+"Z");
-    e.setAttribute("user", user());
-    e.setAttribute("version", versionNumber());
-    if (!strict) {
-        e.setAttribute("actor", (int)lastUpdated());
-        if (isDeleted())
-            e.setAttribute("deleted","true");
-    }
+    Feature::toXML(e, strict);
 
     if (size()) {
         QDomElement n = xParent.ownerDocument().createElement("nd");
@@ -899,7 +889,7 @@ bool Way::toXML(QDomElement xParent, QProgressDialog * progress, bool strict)
         }
     }
 
-    tagsToXML(e);
+    tagsToXML(e, strict);
 
     if (progress)
         progress->setValue(progress->value()+1);
@@ -909,54 +899,31 @@ bool Way::toXML(QDomElement xParent, QProgressDialog * progress, bool strict)
 
 Way * Way::fromXML(Document* d, Layer * L, const QDomElement e)
 {
-    QString id = e.attribute("id");
-    if (!id.startsWith('{') && !id.startsWith('-'))
-        id = "way_" + id;
-    QDateTime time = QDateTime::fromString(e.attribute("timestamp").left(19), Qt::ISODate);
-    QString user = e.attribute("user");
-    bool Deleted = (e.attribute("deleted") == "true");
-    int Version = e.attribute("version").toInt();
-    if (Version < 1)
-        Version = 9999;
-    Feature::ActorType A;
-    if (e.hasAttribute("actor"))
-        A = (Feature::ActorType)(e.attribute("actor", "2").toInt());
-    else
-        if ((L == d->getDirtyOrOriginLayer()))
-            A = Feature::User;
-        else
-            A = Feature::OSMServer;
-
-    Way* R = dynamic_cast<Way*>(d->getFeature(id));
+    QString sid = (e.hasAttribute("id") ? e.attribute("id") : e.attribute("xml:id"));
+    IFeature::FId id(IFeature::LineString, sid.toLongLong());
+    Way* R = CAST_WAY(d->getFeature(id));
 
     if (!R) {
         R = new Way();
         R->setId(id);
-        R->setLastUpdated(A);
+        Feature::fromXML(e, R);
         L->add(R);
     } else {
+        Feature::fromXML(e, R);
         if (R->layer() != L) {
             R->layer()->remove(R);
             L->add(R);
         }
-        if (R->lastUpdated() == Feature::NotYetDownloaded)
-            R->setLastUpdated(A);
         while (R->p->Nodes.size())
             R->remove(0);
     }
-    R->setTime(time);
-    R->setUser(user);
-    R->setDeleted(Deleted);
-    R->setVersionNumber(Version);
 
     QDomElement c = e.firstChildElement();
     while(!c.isNull()) {
         if (c.tagName() == "nd") {
-            QString nId = c.attribute("ref");
-            if (!nId.startsWith('{') && !nId.startsWith('-'))
-                nId = "node_" + nId;
-            //TrackPoint* Part = MapFeature::getTrackPointOrCreatePlaceHolder(d, L, NULL, c.attribute("ref"));
-            Node* Part = dynamic_cast<Node*>(d->getFeature(nId));
+            QString sId = c.attribute("ref");
+            IFeature::FId nId(IFeature::Point, sId.toLongLong());
+            Node* Part = CAST_NODE(d->getFeature(nId));
             if (!Part)
             {
                 Part = new Node(Coord(0,0));
@@ -979,7 +946,7 @@ Feature::TrafficDirectionType trafficDirection(const Way* R)
     // TODO some duplication with Way trafficDirection
     QString d;
     int idx=R->findKey("oneway");
-    if (idx<R->tagSize())
+    if (idx != -1)
     {
         d = R->tagValue(idx);
         if ( (d == "yes") || (d == "1") || (d == "true")) return Feature::OneWay;
@@ -988,7 +955,7 @@ Feature::TrafficDirectionType trafficDirection(const Way* R)
     }
 
     idx=R->findKey("junction");
-    if (idx<R->tagSize())
+    if (idx != -1)
     {
         d = R->tagValue(idx);
         if(d=="roundabout") return Feature::OneWay;
@@ -1053,14 +1020,18 @@ QString Way::toHtml()
 
     QString D;
 
+    if (isClosed()) {
+        D += "<i>"+QApplication::translate("MapFeature", "Closed way")+"</i>";
+        D += "<br/>";
+    }
     D += "<i>"+QApplication::translate("MapFeature", "Length")+": </i>" + distanceLabel;
     D += "<br/>";
     D += "<i>"+QApplication::translate("MapFeature", "Size")+": </i>" + QApplication::translate("MapFeature", "%1 nodes").arg(size());
     CoordBox bb = boundingBox();
     D += "<br/>";
-    D += "<i>"+QApplication::translate("MapFeature", "Topleft")+": </i>" + QString::number(coordToAng(bb.topLeft().lat()), 'f', 4) + " / " + QString::number(coordToAng(bb.topLeft().lon()), 'f', 4);
+    D += "<i>"+QApplication::translate("MapFeature", "Topleft")+": </i>" + COORD2STRING(coordToAng(bb.topLeft().lat())) + " / " + COORD2STRING(coordToAng(bb.topLeft().lon()));
     D += "<br/>";
-    D += "<i>"+QApplication::translate("MapFeature", "Botright")+": </i>" + QString::number(coordToAng(bb.bottomRight().lat()), 'f', 4) + " / " + QString::number(coordToAng(bb.bottomRight().lon()), 'f', 4);
+    D += "<i>"+QApplication::translate("MapFeature", "Botright")+": </i>" + COORD2STRING(coordToAng(bb.bottomRight().lat())) + " / " + COORD2STRING(coordToAng(bb.bottomRight().lon()));
 
     QString type = isClosed() ? QApplication::translate("MapFeature", "Area") : QApplication::translate("MapFeature", "Way");
 
@@ -1075,16 +1046,16 @@ void Way::toBinary(QDataStream& ds, QHash <QString, quint64>& theIndex)
         type = 'A';
         sz--;
     }
-    theIndex[type  + QString::number(idToLong())] = ds.device()->pos();
+    theIndex[type  + QString::number(id().numId)] = ds.device()->pos();
     ds << (qint8)type;
-    ds << idToLong();
+    ds << id().numId;
     ds << (qint32)sz;
     for (int i=0; i < sz; ++i) {
         Node* N = CAST_NODE(get(i));
         if (N->sizeParents() > 1)
-            ds << (qint8)'N' << (qint64)(N->idToLong());
+            ds << (qint8)'N' << (qint64)(N->id().numId);
         else
-            ds << (qint8)'C' << N->position().lat() << N->position().lon();
+            ds << (qint8)'C' << (qint32)(N->position().lat()/180.*INT_MAX) << (qint32)(N->position().lon()/180.*INT_MAX);
     }
 }
 
@@ -1093,7 +1064,6 @@ Way* Way::fromBinary(Document* d, OsbLayer* L, QDataStream& ds, qint8 c, qint64
     char type = c;
 
     qint32	fSize;
-    QString strId;
     qint32 lat, lon;
 
     quint8 nodeType;
@@ -1109,12 +1079,8 @@ Way* Way::fromBinary(Document* d, OsbLayer* L, QDataStream& ds, qint8 c, qint64
         return NULL;
     }
 
-    if (id < 1)
-        strId = QString::number(id);
-    else
-        strId = QString("way_%1").arg(QString::number(id));
-
-    Way* R = dynamic_cast<Way*>(d->getFeature(strId));
+    IFeature::FId strId(IFeature::LineString, id);
+    Way* R = CAST_WAY(d->getFeature(strId));
     if (!R) {
         R = new Way();
         R->setId(strId);
@@ -1142,12 +1108,12 @@ Way* Way::fromBinary(Document* d, OsbLayer* L, QDataStream& ds, qint8 c, qint64
                 ds >> lat;
                 ds >> lon;
 
-                N = new Node(Coord(lat, lon));
-                N->setParent(R);
+                N = new Node(Coord( (double)lat/INT_MAX*180., (double)lon/INT_MAX*180. ));
+//                N->setParent(R);
                 break;
             case 'N':
                 ds >> refId;
-                N = CAST_NODE(d->getFeature(QString("node_%1").arg(refId)));
+                N = CAST_NODE(d->getFeature(IFeature::FId(IFeature::Point, refId)));
                 Q_ASSERT(N);
 
         }
@@ -1176,22 +1142,13 @@ bool Way::isExtrimity(Node* node)
 
 Way * Way::GetSingleParentRoad(Feature * mapFeature)
 {
-    int parents = mapFeature->sizeParents();
-
-    if (parents == 0)
-        return NULL;
-
     Way * parentRoad = NULL;
 
-    int i;
-    for (i=0; i<parents; i++)
+    for (int i=0; i<mapFeature->sizeParents(); i++)
     {
-        Feature * parent = mapFeature->getParent(i);
-        if (parent->isDeleted()) continue;
+        Way * road = CAST_WAY(mapFeature->getParent(i));
 
-        Way * road = CAST_WAY(parent);
-
-        if (road == NULL)
+        if (!road || road->isDeleted())
             continue;
 
         if (parentRoad && road != parentRoad)
@@ -1207,23 +1164,14 @@ Way * Way::GetSingleParentRoad(Feature * mapFeature)
 
 Way * Way::GetSingleParentRoadInner(Feature * mapFeature)
 {
-    int parents = mapFeature->sizeParents();
-
-    if (parents == 0)
-        return NULL;
-
     Way * parentRoad = NULL;
     Node* trackPoint = CAST_NODE(mapFeature);
 
-    int i;
-    for (i=0; i<parents; i++)
+    for (int i=0; i<mapFeature->sizeParents(); i++)
     {
-        Feature * parent = mapFeature->getParent(i);
-        if (parent->isDeleted()) continue;
+        Way * road = CAST_WAY(mapFeature->getParent(i));
 
-        Way * road = CAST_WAY(parent);
-
-        if (road == NULL)
+        if (!road || road->isDeleted())
             continue;
 
         if (road->isExtrimity(trackPoint) && !road->isClosed())
@@ -1277,3 +1225,18 @@ int Way::createJunction(Document* theDocument, CommandList* theList, Way* R1, Wa
 
     return numInter;
 }
+
+int Way::segmentCount()
+{
+    return p->Nodes.size()-1;
+}
+
+QLineF Way::getSegment(int i)
+{
+    return QLineF(p->Nodes[i]->position().toQPointF(), p->Nodes[i+1]->position().toQPointF());
+}
+
+int Way::bestSegment()
+{
+    return p->BestSegment;
+}
