@@ -10,6 +10,8 @@
 //
 //
 
+#include "Global.h"
+
 #include <QtGui>
 
 #include "../ImportExport/ImportExportOsmBin.h"
@@ -268,13 +270,9 @@ void ImportExportOsmBin::tagsToBinary(Feature* F, QDataStream& ds)
 {
     qint64 k, v;
     quint8 tagSize = (quint8)qMin(F->tagSize(), (int) 255);
-    if (F->tagValue("created_by", "dummy") != "dummy")
-        tagSize--;
 
     ds << tagSize;
     for (int i=0; i<F->tagSize(); ++i) {
-        if (F->tagKey(i) == "created_by")
-            continue;
         k = theTagKeysIndex[F->tagKey(i)];
         v = theTagValuesIndex[F->tagValue(i)];
         ds << k;
@@ -331,7 +329,7 @@ void ImportExportOsmBin::tagsPopularity(Feature * F)
 bool ImportExportOsmBin::prepare(qint32 selRegion)
 {
     for (int j=0; j< theFeatures.size(); ++j) {
-        qint64 idx = theFeatures[j]->idToLong();
+        qint64 idx = theFeatures[j]->id().numId;
         if (Node* N = CAST_NODE(theFeatures[j])) {
             if (!N->isPOI())
                 continue;
@@ -453,7 +451,7 @@ bool ImportExportOsmBin::writeTagLists(QDataStream& ds)
     }
 
     tagKeysPos = Device->pos();
-    ds << (qint32) theDoc->getTagKeys().size();
+    ds << (qint32) g_getTagKeys().size();
 
     QMapIterator<qint32, QString> ki(popularityKey);
     ki.toBack();
@@ -471,7 +469,7 @@ bool ImportExportOsmBin::writeTagLists(QDataStream& ds)
     }
 
     tagValuesPos = Device->pos();
-    ds << (qint32) theDoc->getTagValues().size();
+    ds << (qint32) g_getTagValues().size();
 
     QMapIterator<qint32, QString> vi(popularityValue);
     vi.toBack();
@@ -623,14 +621,14 @@ Feature* ImportExportOsmBin::getFeature(OsbRegion* osr, Document* d, OsbLayer* t
     if (d) {
         switch (c) {
             case 'N':
-                F = d->getFeature(QString("node_%1").arg(QString::number(id)));
+            F = d->getFeature(IFeature::FId(IFeature::Point, id));
                 break;
             case 'A':
             case 'R':
-                F = d->getFeature(QString("way_%1").arg(QString::number(id)));
+                F = d->getFeature(IFeature::FId(IFeature::LineString, id));
                 break;
             case 'L':
-                F = d->getFeature(QString("rel_%1").arg(QString::number(id)));
+                F = d->getFeature(IFeature::FId(IFeature::OsmRelation, id));
                 break;
             default:
                 F = NULL;
@@ -681,22 +679,22 @@ Feature* ImportExportOsmBin::getFeature(OsbRegion* osr, Document* d, OsbLayer* t
     switch (c) {
         case 'N':
             if (d)
-                oF = d->getFeature(QString("node_%1").arg(QString::number(id)));
+                oF = d->getFeature(IFeature::FId(IFeature::Point, id));
             else
-                oF = ((Layer*)theLayer)->get(QString("node_%1").arg(QString::number(id)));
+                oF = ((Layer*)theLayer)->get(IFeature::FId(IFeature::Point, id));
             break;
         case 'A':
         case 'R':
             if (d)
-                oF = d->getFeature(QString("way_%1").arg(QString::number(id)));
+                oF = d->getFeature(IFeature::FId(IFeature::LineString, id));
             else
-                oF = ((Layer*)theLayer)->get(QString("way_%1").arg(QString::number(id)));
+                oF = ((Layer*)theLayer)->get(IFeature::FId(IFeature::LineString, id));
             break;
         case 'L':
             if (d)
-                oF = d->getFeature(QString("rel_%1").arg(QString::number(id)));
+                oF = d->getFeature(IFeature::FId(IFeature::OsmRelation, id));
             else
-                oF = ((Layer*)theLayer)->get(QString("rel_%1").arg(QString::number(id)));
+                oF = ((Layer*)theLayer)->get(IFeature::FId(IFeature::OsmRelation, id));
             break;
         default:
             oF = NULL;
