@@ -37,6 +37,8 @@ void parseStandardAttributes(const QXmlAttributes& atts, Feature* F)
 {
     QString ts = atts.value("timestamp");
     QDateTime time = QDateTime::fromString(ts.left(19), Qt::ISODate);
+    if (!time.isValid())
+        time = QDateTime::currentDateTime();
     QString user = atts.value("user");
     F->setTime(time);
     F->setUser(user);
@@ -50,14 +52,12 @@ void OSMHandler::parseNode(const QXmlAttributes& atts)
     double Lat = atts.value("lat").toDouble();
     double Lon = atts.value("lon").toDouble();
     QString id = atts.value("id");
-    if (id[0] != '-' && id[0] != '{')
-        id = "node_"+id;
-    Node* Pt = dynamic_cast<Node*>(theDocument->getFeature(id));
+    Node* Pt = CAST_NODE(theDocument->getFeature(IFeature::FId(IFeature::Point, id.toLongLong())));
     if (Pt)
     {
         Node* userPt = Pt;
         Pt = new Node(Coord(angToCoord(Lat),angToCoord(Lon)));
-        Pt->setId("conflict_"+id);
+        Pt->setId(IFeature::FId(IFeature::Point | IFeature::Conflict, id.toLongLong()));
         Pt->setLastUpdated(Feature::OSMServerConflict);
         parseStandardAttributes(atts,Pt);
 
@@ -98,10 +98,10 @@ void OSMHandler::parseNode(const QXmlAttributes& atts)
     else
     {
         Pt = new Node(Coord(angToCoord(Lat),angToCoord(Lon)));
+        Pt->setId(IFeature::FId(IFeature::Point, id.toLongLong()));
+        Pt->setLastUpdated(Feature::OSMServer);
         theLayer->add(Pt);
         NewFeature = true;
-        Pt->setId(id);
-        Pt->setLastUpdated(Feature::OSMServer);
     }
 
     if (NewFeature) {
@@ -120,7 +120,7 @@ void OSMHandler::parseNd(const QXmlAttributes& atts)
 {
     Way* R = dynamic_cast<Way*>(Current);
     if (!R) return;
-    Node *Part = Feature::getTrackPointOrCreatePlaceHolder(theDocument, theLayer, atts.value("ref"));
+    Node *Part = Feature::getTrackPointOrCreatePlaceHolder(theDocument, theLayer, IFeature::FId(IFeature::Point, atts.value("ref").toLongLong()));
     if (NewFeature)
         R->add(Part);
 }
@@ -128,15 +128,13 @@ void OSMHandler::parseNd(const QXmlAttributes& atts)
 void OSMHandler::parseWay(const QXmlAttributes& atts)
 {
     QString id = atts.value("id");
-    if (id[0] != '-' && id[0] != '{')
-        id = "way_"+id;
     QString ts = atts.value("timestamp"); ts.truncate(19);
-    Way* R = dynamic_cast<Way*>(theDocument->getFeature(id));
+    Way* R = CAST_WAY(theDocument->getFeature(IFeature::FId(IFeature::LineString, id.toLongLong())));
     if (R)
     {
         Way* userRd = R;
         R = new Way();
-        R->setId("conflict_"+id);
+        R->setId(IFeature::FId(IFeature::LineString | IFeature::Conflict, id.toLongLong()));
         R->setLastUpdated(Feature::OSMServerConflict);
         parseStandardAttributes(atts,R);
 
@@ -179,10 +177,10 @@ void OSMHandler::parseWay(const QXmlAttributes& atts)
     else
     {
         R = new Way;
+        R->setId(IFeature::FId(IFeature::LineString, id.toLongLong()));
+        R->setLastUpdated(Feature::OSMServer);
         theLayer->add(R);
         NewFeature = true;
-        R->setId(id);
-        R->setLastUpdated(Feature::OSMServer);
     }
 
     if (NewFeature) {
@@ -201,11 +199,11 @@ void OSMHandler::parseMember(const QXmlAttributes& atts)
     QString Type = atts.value("type");
     Feature* F = 0;
     if (Type == "node")
-        F = Feature::getTrackPointOrCreatePlaceHolder(theDocument, theLayer, atts.value("ref"));
+        F = Feature::getTrackPointOrCreatePlaceHolder(theDocument, theLayer, IFeature::FId(IFeature::Point, atts.value("ref").toLongLong()));
     else if (Type == "way")
-        F = Feature::getWayOrCreatePlaceHolder(theDocument, theLayer, atts.value("ref"));
+        F = Feature::getWayOrCreatePlaceHolder(theDocument, theLayer, IFeature::FId(IFeature::LineString, atts.value("ref").toLongLong()));
     else if (Type == "relation")
-        F = Feature::getRelationOrCreatePlaceHolder(theDocument, theLayer, atts.value("ref"));
+        F = Feature::getRelationOrCreatePlaceHolder(theDocument, theLayer, IFeature::FId(IFeature::OsmRelation, atts.value("ref").toLongLong()));
 
     if (F && F != R)
         R->add(atts.value("role"),F);
@@ -214,15 +212,13 @@ void OSMHandler::parseMember(const QXmlAttributes& atts)
 void OSMHandler::parseRelation(const QXmlAttributes& atts)
 {
     QString id = atts.value("id");
-    if (id[0] != '-' && id[0] != '{')
-        id = "rel_"+id;
     QString ts = atts.value("timestamp"); ts.truncate(19);
-    Relation* R = dynamic_cast<Relation*>(theDocument->getFeature(id));
+    Relation* R = CAST_RELATION(theDocument->getFeature(IFeature::FId(IFeature::OsmRelation, id.toLongLong())));
     if (R)
     {
         Relation* userR = R;
         R = new Relation();
-        R->setId("conflict_"+id);
+        R->setId(IFeature::FId(IFeature::OsmRelation | IFeature::Conflict, id.toLongLong()));
         R->setLastUpdated(Feature::OSMServerConflict);
         parseStandardAttributes(atts,R);
 
@@ -265,10 +261,10 @@ void OSMHandler::parseRelation(const QXmlAttributes& atts)
     else
     {
         R = new Relation;
-        NewFeature = true;
-        R->setId(id);
-        theLayer->add(R);
+        R->setId(IFeature::FId(IFeature::OsmRelation, id.toLongLong()));
         R->setLastUpdated(Feature::OSMServer);
+        NewFeature = true;
+        theLayer->add(R);
     }
 
     if (NewFeature) {
@@ -316,7 +312,7 @@ static bool downloadToResolve(const QList<Feature*>& Resolution, QWidget* aParen
 
     for (int i=0; i<Resolution.size(); i++ )
     {
-        QString URL = theDownloader->getURLToFetchFull(Resolution[i]);
+        QUrl URL = M_PREFS->getOsmWebsite()+theDownloader->getURLToFetchFull(Resolution[i]);
         Lbl->setText(QApplication::translate("Downloader","Downloading unresolved %1 of %2").arg(i+1).arg(Resolution.size()));
         if (theDownloader->go(URL))
         {
@@ -362,7 +358,7 @@ static bool downloadToResolve(const QList<Feature*>& Resolution, QWidget* aParen
 static void recurseDelete (Feature* F, QList<Feature*>& MustDelete)
 {
     for (int i=0; i<F->sizeParents(); i++) {
-        recurseDelete(F->getParent(i), MustDelete);
+        recurseDelete(CAST_FEATURE(F->getParent(i)), MustDelete);
     }
     if (!MustDelete.contains(F))
         MustDelete.push_back(F);
