@@ -5,6 +5,7 @@
 #include "Maps/Projection.h"
 #include "Node.h"
 #include "PropertiesDock.h"
+#include "Utils/Utils.h"
 
 #include <QtGui/QMouseEvent>
 #include <QtGui/QPainter>
@@ -12,7 +13,11 @@
 #include <math.h>
 
 Interaction::Interaction(MapView* aView)
-: theView(aView), Panning(false), Dragging(false), StartDrag(0,0), EndDrag(0,0)
+    : theView(aView), Panning(false)
+    , LastSnap(0), SnapActive(true)
+    , NoSelectPoints(false), NoSelectRoads(false), NoSelectVirtuals(true)
+    , Dragging(false)
+    , StartDrag(0,0), EndDrag(0,0)
 {
     connect(this, SIGNAL(requestCustomContextMenu(const QPoint &)), theView, SLOT(on_customContextMenuRequested(const QPoint &)));
 }
@@ -41,16 +46,6 @@ Document* Interaction::document()
     return theView->document();
 }
 
-const Projection& Interaction::projection() const
-{
-    return theView->projection();
-}
-
-const QTransform& Interaction::transform() const
-{
-    return theView->transform();
-}
-
 void Interaction::mousePressEvent(QMouseEvent * anEvent)
 {
     if (anEvent->buttons() & Qt::MidButton) {
@@ -152,6 +147,10 @@ void Interaction::mouseMoveEvent(QMouseEvent* anEvent)
     }
 }
 
+void Interaction::mouseDoubleClickEvent(QMouseEvent* /*anEvent*/)
+{
+}
+
 void Interaction::wheelEvent(QWheelEvent* ev)
 {
     double finalZoom = 1.;
@@ -187,18 +186,134 @@ void Interaction::paintEvent(QPaintEvent*, QPainter& thePainter)
     }
 }
 
+void Interaction::updateSnap(QMouseEvent* event)
+{
+    if (panning())
+    {
+        LastSnap = 0;
+        return;
+    }
+    bool NoRoads =
+        ( (QApplication::keyboardModifiers() & Qt::AltModifier) &&  (QApplication::keyboardModifiers() &Qt::ControlModifier) );
+    Feature* Prev = LastSnap;
+    LastSnap = 0;
+    Feature* ReadOnlySnap = 0;
+    if (!SnapActive) return;
+    //QTime Start(QTime::currentTime());
+    CoordBox HotZone(XY_TO_COORD(event->pos()-QPoint(M_PREFS->getMaxGeoPicWidth()+5,M_PREFS->getMaxGeoPicWidth()+5)),XY_TO_COORD(event->pos()+QPoint(M_PREFS->getMaxGeoPicWidth()+5,M_PREFS->getMaxGeoPicWidth()+5)));
+    CoordBox HotZoneSnap(XY_TO_COORD(event->pos()-QPoint(15,15)),XY_TO_COORD(event->pos()+QPoint(15,15)));
+    SnapList.clear();
+    double BestDistance = 5;
+    double BestReadonlyDistance = 5;
+    bool areNodesSelectable = (theView->nodeWidth() >= 1 && M_PREFS->getTrackPointsVisible());
+
+    Way* R;
+    Node* N;
+    for (int j=0; j<document()->layerSize(); ++j) {
+        QList < MapFeaturePtr > ret = document()->getLayer(j)->indexFind(HotZone);
+        foreach(MapFeaturePtr F, ret) {
+            if (F)
+            {
+                if (F->isHidden())
+                    continue;
+                if (std::find(NoSnap.begin(),NoSnap.end(),F) != NoSnap.end())
+                    continue;
+                if (F->notEverythingDownloaded())
+                    continue;
+                if ((R = CAST_WAY(F))) {
+                    if ( NoRoads || NoSelectRoads)
+                        continue;
+
+                    if (HotZoneSnap.contains(R->boundingBox()))
+                        SnapList.push_back(F);
+                    else {
+                        QPointF lastPoint = R->getNode(0)->position().toQPointF();
+                        QPointF aP;
+                        for (int j=1; j<R->size(); ++j) {
+                            aP = R->getNode(j)->position().toQPointF();
+                            QLineF l(lastPoint, aP);
+                            QPointF a, b;
+                            if (Utils::QRectInterstects(HotZoneSnap.toQRectF(), l, a, b)) {
+                                SnapList.push_back(F);
+                                break;
+                            }
+                            lastPoint = aP;
+                        }
+                    }
+                }
+                if ((N = CAST_NODE(F))) {
+                    if (NoSelectPoints)
+                        continue;
+                    if (!N->isSelectable(theView))
+                        continue;
+                    if (HotZoneSnap.contains(N->boundingBox()))
+                        SnapList.push_back(F);
+                }
+
+                double Distance = F->pixelDistance(event->pos(), 5.01, areNodesSelectable, view());
+                if (Distance < BestDistance && !F->isReadonly())
+                {
+                    BestDistance = Distance;
+                    LastSnap = F;
+                } else if (Distance < BestReadonlyDistance && F->isReadonly())
+                {
+                    BestReadonlyDistance = Distance;
+                    ReadOnlySnap = F;
+                }
+            }
+        }
+    }
+    if (!NoSelectVirtuals && areNodesSelectable && M_PREFS->getVirtualNodesVisible()) {
+        R = CAST_WAY(LastSnap);
+        if (R) {
+            Node* N = R->pixelDistanceVirtual(event->pos(), 5.01, view());
+            if (N)
+                LastSnap = N;
+        }
+    }
+
+    if (Prev != LastSnap) {
+        curStackSnap = SnapList.indexOf(LastSnap);
+        view()->update();
+    }
+
+    if (M_PREFS->getMapTooltip()) {
+        if (LastSnap)
+            view()->setToolTip(LastSnap->toHtml());
+        else
+            view()->setToolTip("");
+    }
+    if (M_PREFS->getInfoOnHover() && main()->info()->isVisible()) {
+        if (LastSnap) {
+            main()->info()->setHoverHtml(LastSnap->toHtml());
+        } else
+            if (ReadOnlySnap)
+                main()->info()->setHoverHtml(ReadOnlySnap->toHtml());
+            else
+                main()->info()->unsetHoverHtml();
+    }
+}
+
+Feature* Interaction::lastSnap()
+{
+    return LastSnap;
+}
+
+
 /***************/
 
 FeatureSnapInteraction::FeatureSnapInteraction(MapView* theView)
-        : Interaction(theView), SnapActive(true),
-      NoSelectPoints(false), NoSelectRoads(false)
-          , NoSelectVirtuals(true), LastSnap(0)
+        : Interaction(theView)
 {
 //    handCursor = QCursor(QPixmap(":/Icons/grab.png"));
 //    grabCursor = QCursor(QPixmap(":/Icons/grabbing.png"));
     handCursor = QCursor(Qt::OpenHandCursor);
     grabCursor = QCursor(Qt::ClosedHandCursor);
     defaultCursor = QCursor(Qt::ArrowCursor);
+//    warningCursor = QCursor(Qt::ForbiddenCursor);
+    warningCursor = QCursor(QPixmap(":/Icons/cursor-warning"), 16, 5);
+
+    theView->setCursor(cursor());
 }
 
 void FeatureSnapInteraction::paintEvent(QPaintEvent* anEvent, QPainter& thePainter)
@@ -207,13 +322,13 @@ void FeatureSnapInteraction::paintEvent(QPaintEvent* anEvent, QPainter& thePaint
 
     for (int i=0; i<main()->features()->highlightedSize(); ++i)
         if (document()->exists(main()->features()->highlighted(i)))
-            main()->features()->highlighted(i)->drawHighlight(thePainter, view(), true);
+            main()->features()->highlighted(i)->drawHighlight(thePainter, view());
     for (int i=0; i<main()->properties()->size(); ++i)
         if (document()->exists(main()->properties()->selection(i)))
             main()->properties()->selection(i)->drawFocus(thePainter, view());
     for (int i=0; i<main()->properties()->highlightedSize(); ++i)
         if (document()->exists(main()->properties()->highlighted(i)))
-            main()->properties()->highlighted(i)->drawHighlight(thePainter, view(), true);
+            main()->properties()->highlighted(i)->drawHighlight(thePainter, view());
 
 #ifndef _MOBILE
     if (LastSnap && document()->exists(LastSnap)) {
@@ -227,7 +342,6 @@ void FeatureSnapInteraction::paintEvent(QPaintEvent* anEvent, QPainter& thePaint
 
 void FeatureSnapInteraction::mousePressEvent(QMouseEvent * event)
 {
-    updateSnap(event);
     if (event->button() == Qt::LeftButton)
         snapMousePressEvent(event,LastSnap);
     if (!(M_PREFS->getMouseSingleButton() && LastSnap))
@@ -239,7 +353,6 @@ void FeatureSnapInteraction::mouseReleaseEvent(QMouseEvent * event)
     if (event->button() == Qt::RightButton && !Panning && !Dragging)
         emit(requestCustomContextMenu(event->pos()));
 
-    updateSnap(event);
     snapMouseReleaseEvent(event,LastSnap);
     if (!(M_PREFS->getMouseSingleButton() && LastSnap))
         Interaction::mouseReleaseEvent(event);
@@ -247,13 +360,26 @@ void FeatureSnapInteraction::mouseReleaseEvent(QMouseEvent * event)
 
 void FeatureSnapInteraction::mouseMoveEvent(QMouseEvent* event)
 {
-    updateSnap(event);
-    view()->setCursor(cursor());
+//    if (!document()->isDownloadedSafe(theView->fromView(event->pos())))
+//        view()->setCursor(warningCursor);
+//    else
+        view()->setCursor(cursor());
     snapMouseMoveEvent(event, LastSnap);
     if (!(M_PREFS->getMouseSingleButton() && LastSnap))
         Interaction::mouseMoveEvent(event);
 }
 
+void FeatureSnapInteraction::mouseDoubleClickEvent(QMouseEvent* event)
+{
+//    if (!document()->isDownloadedSafe(theView->fromView(event->pos())))
+//        view()->setCursor(warningCursor);
+//    else
+        view()->setCursor(cursor());
+    snapMouseDoubleClickEvent(event, LastSnap);
+    if (!(M_PREFS->getMouseSingleButton() && LastSnap))
+        Interaction::mouseDoubleClickEvent(event);
+}
+
 void FeatureSnapInteraction::snapMousePressEvent(QMouseEvent * , Feature*)
 {
 }
@@ -266,6 +392,10 @@ void FeatureSnapInteraction::snapMouseMoveEvent(QMouseEvent* , Feature*)
 {
 }
 
+void FeatureSnapInteraction::snapMouseDoubleClickEvent(QMouseEvent* , Feature*)
+{
+}
+
 void FeatureSnapInteraction::activateSnap(bool b)
 {
     SnapActive = b;
@@ -309,6 +439,8 @@ void FeatureSnapInteraction::setSnap(QList<Feature*> aSnapList)
 
 void FeatureSnapInteraction::nextSnap()
 {
+    if (!StackSnap.size())
+        return;
     curStackSnap++;
     if (curStackSnap > StackSnap.size() -1)
         curStackSnap = 0;
@@ -318,6 +450,8 @@ void FeatureSnapInteraction::nextSnap()
 
 void FeatureSnapInteraction::previousSnap()
 {
+    if (!StackSnap.size())
+        return;
     curStackSnap--;
     if (curStackSnap < 0)
         curStackSnap = StackSnap.size() -1;
@@ -340,97 +474,6 @@ void FeatureSnapInteraction::setDontSelectVirtual(bool b)
     NoSelectVirtuals = b;
 }
 
-void FeatureSnapInteraction::updateSnap(QMouseEvent* event)
-{
-    if (panning())
-    {
-        LastSnap = 0;
-        return;
-    }
-    bool NoRoads =
-        ( (QApplication::keyboardModifiers() & Qt::AltModifier) &&  (QApplication::keyboardModifiers() &Qt::ControlModifier) );
-    Feature* Prev = LastSnap;
-    LastSnap = 0;
-    Feature* ReadOnlySnap = 0;
-    if (!SnapActive) return;
-    //QTime Start(QTime::currentTime());
-//    CoordBox HotZone(XY_TO_COORD(event->pos()-QPointF(15,15)),XY_TO_COORD(event->pos()+QPointF(15,15)));
-    CoordBox HotZone(XY_TO_COORD(event->pos()-QPointF(M_PREFS->getMaxGeoPicWidth()+5,M_PREFS->getMaxGeoPicWidth()+5)),XY_TO_COORD(event->pos()+QPointF(M_PREFS->getMaxGeoPicWidth()+5,M_PREFS->getMaxGeoPicWidth()+5)));
-    SnapList.clear();
-    double BestDistance = 5;
-    double BestReadonlyDistance = 5;
-    bool areNodesSelectable = (theView->nodeWidth() >= 1 && M_PREFS->getTrackPointsVisible());
-
-    Way* R;
-    Node* N;
-    for (int j=0; j<document()->layerSize(); ++j) {
-        if (!document()->getLayer(j)->isVisible())
-            continue;
-
-        QList < MapFeaturePtr > ret = document()->getLayer(j)->indexFind(HotZone);
-        foreach(MapFeaturePtr F, ret) {
-            if (F)
-            {
-                if (F->notEverythingDownloaded())
-                    continue;
-                if ((R = CAST_WAY(F))) {
-                    if ( NoRoads || NoSelectRoads)
-                        continue;
-                }
-                if ((N = CAST_NODE(F))) {
-                    if (NoSelectPoints)
-                        continue;
-                    if (!N->isSelectable(theView))
-                        continue;
-                }
-                if (std::find(NoSnap.begin(),NoSnap.end(),F) != NoSnap.end())
-                    continue;
-
-                double Distance = F->pixelDistance(event->pos(), 5.01, areNodesSelectable, view());
-                SnapList.push_back(F);
-                if (Distance < BestDistance && !document()->getLayer(j)->isReadonly())
-                {
-                    BestDistance = Distance;
-                    LastSnap = F;
-                } else if (Distance < BestReadonlyDistance && document()->getLayer(j)->isReadonly())
-                {
-                    BestReadonlyDistance = Distance;
-                    ReadOnlySnap = F;
-                }
-            }
-        }
-    }
-    if (!NoSelectVirtuals && areNodesSelectable) {
-        R = CAST_WAY(LastSnap);
-        if (R) {
-            Node* N = R->pixelDistanceVirtual(event->pos(), 5.01, view());
-            if (N)
-                LastSnap = N;
-        }
-    }
-
-    if (Prev != LastSnap) {
-        curStackSnap = SnapList.indexOf(LastSnap);
-        view()->update();
-    }
-
-    if (M_PREFS->getMapTooltip()) {
-        if (LastSnap)
-            view()->setToolTip(LastSnap->toHtml());
-        else
-            view()->setToolTip("");
-    }
-    if (M_PREFS->getInfoOnHover() && main()->info()->isVisible()) {
-        if (LastSnap) {
-            main()->info()->setHoverHtml(LastSnap->toHtml());
-        } else
-            if (ReadOnlySnap)
-                main()->info()->setHoverHtml(ReadOnlySnap->toHtml());
-            else
-                main()->info()->unsetHoverHtml();
-    }
-}
-
 #ifndef Q_OS_SYMBIAN
 QCursor FeatureSnapInteraction::cursor() const
 {
