@@ -1,3 +1,4 @@
+#include "Global.h"
 #include "MoveNodeInteraction.h"
 
 #include "MapView.h"
@@ -61,8 +62,12 @@ QString MoveNodeInteraction::toHtml()
 #ifndef Q_OS_SYMBIAN
 QCursor MoveNodeInteraction::cursor() const
 {
+#ifdef Q_OS_MAC
     QPixmap pm(":/Icons/move.xpm");
     return QCursor(pm);
+#else
+    return QCursor(Qt::SizeAllCursor);
+#endif
 }
 #endif
 
@@ -96,10 +101,17 @@ void MoveNodeInteraction::snapMousePressEvent(QMouseEvent * event, Feature* aLas
                 StartDragPosition = Pt->position();
         }
         else if (Way* R = CAST_WAY(sel[i])) {
-            for (int j=0; j<R->size(); ++j)
-                if (std::find(Moving.begin(),Moving.end(),R->get(j)) == Moving.end()) {
-                    Moving.push_back(R->getNode(j));
-                }
+            if (!g_Merk_Segment_Mode) {
+                for (int j=0; j<R->size(); ++j)
+                    if (std::find(Moving.begin(),Moving.end(),R->get(j)) == Moving.end()) {
+                        Moving.push_back(R->getNode(j));
+                    }
+            } else {
+                for (int j=R->bestSegment(); j<=R->bestSegment()+1; ++j)
+                    if (std::find(Moving.begin(),Moving.end(),R->get(j)) == Moving.end()) {
+                        Moving.push_back(R->getNode(j));
+                    }
+            }
             addToNoSnap(R);
         }
         else if (Relation* L = CAST_RELATION(sel[i])) {
@@ -140,7 +152,7 @@ void MoveNodeInteraction::snapMouseReleaseEvent(QMouseEvent * event, Feature* Cl
             theList->setFeature(Moving[0]);
         } else {
             if (!Virtual) {
-                theList->setDescription(MainWindow::tr("Move Node %1").arg(Moving[0]->id()));
+                theList->setDescription(MainWindow::tr("Move Node %1").arg(Moving[0]->id().numId));
                 theList->setFeature(Moving[0]);
             }
         }
@@ -155,8 +167,11 @@ void MoveNodeInteraction::snapMouseReleaseEvent(QMouseEvent * event, Feature* Cl
             for (int j=0; j<Moving[i]->sizeParents(); ++j) {
                 if (Way* aRoad = CAST_WAY(Moving[i]->getParent(j)))
                     WaysToUpdate << aRoad;
-                else
-                    Moving[i]->getParent(j)->updateIndex();
+                else {
+                    Feature* f = CAST_FEATURE(Moving[i]->getParent(j));
+                    if (f)
+                        f->updateIndex();
+                }
             }
         }
         foreach (Way* w, WaysToUpdate) {
@@ -200,7 +215,7 @@ void MoveNodeInteraction::snapMouseReleaseEvent(QMouseEvent * event, Feature* Cl
                     // Merge all nodes into the first node that has been found (not the node being moved)
                     Node* merged = samePosPts[0];
                     // Change the command description to reflect the merge
-                    theList->setDescription(MainWindow::tr("Merge Nodes into %1").arg(merged->id()));
+                    theList->setDescription(MainWindow::tr("Merge Nodes into %1").arg(merged->id().numId));
                     theList->setFeature(merged);
 
                     // from mergeNodes(theDocument, theList, theProperties);
@@ -220,7 +235,7 @@ void MoveNodeInteraction::snapMouseReleaseEvent(QMouseEvent * event, Feature* Cl
         view()->invalidate(true, false);
     } else
         if (theList)
-            delete theList;
+            SAFE_DELETE(theList);
     Moving.clear();
     OriginalPosition.clear();
     clearNoSnap();
@@ -237,14 +252,12 @@ void MoveNodeInteraction::snapMouseMoveEvent(QMouseEvent* event, Feature* Closer
                 Virtual = true;
                 Node* v = Moving[i];
                 Way* aRoad = CAST_WAY(v->getParent(0));
-                theList->setDescription(MainWindow::tr("Create node in Road: %1").arg(aRoad->id()));
+                theList->setDescription(MainWindow::tr("Create node in Road: %1").arg(aRoad->id().numId));
                 theList->setFeature(aRoad);
                 int SnapIdx = aRoad->findVirtual(v)+1;
                 Node* N = new Node(*v);
                 N->setVirtual(false);
                 N->setPosition(OriginalPosition[i]+Diff);
-                if (M_PREFS->apiVersionNum() < 0.6)
-                    N->setTag("created_by", QString("Merkaartor v%1%2").arg(STRINGIFY(VERSION)).arg(STRINGIFY(REVISION)));
 
                 if (view()->properties()->isSelected(v)) {
                     view()->properties()->toggleSelection(v);
@@ -292,3 +305,11 @@ Coord MoveNodeInteraction::calculateNewPosition(QMouseEvent *event, Feature *aLa
     }
     return TargetC;
 }
+
+bool MoveNodeInteraction::isIdle()
+{
+    if (Moving.size() && !panning())
+        return false;
+
+    return true;
+}
