@@ -1,3 +1,5 @@
+#include "Global.h"
+
 #include "LayerWidget.h"
 #include "LayerDock.h"
 
@@ -6,23 +8,49 @@
 #include "Layer.h"
 #include "Preferences/MerkaartorPreferences.h"
 
+#include "IMapAdapterFactory.h"
 #include "IMapAdapter.h"
 
 #include <QApplication>
 #include <QMouseEvent>
-#include <QPainter>
+#include <QStylePainter>
+#include <QInputDialog>
+#include <QMessageBox>
+
+#include "Utils/SelectionDialog.h"
 
 #define LINEHEIGHT 25
 
+class OsbLayer;
+
 LayerWidget::LayerWidget(Layer* aLayer, QWidget* aParent)
-: QAbstractButton(aParent), theLayer(aLayer), ctxMenu(0), closeAction(0), actZoom(0), associatedMenu(0)
+: QPushButton(aParent), theLayer(aLayer), ctxMenu(0), closeAction(0), actZoom(0), associatedMenu(0)
 {
+    ui.setupUi(this);
+
     setCheckable(true);
-    //setAutoExclusive(true) ;
+#ifdef Q_OS_MAC
+    setFlat(true);
+#endif
     setFocusPolicy(Qt::NoFocus);
     setContextMenuPolicy(Qt::NoContextMenu);
-    visibleIcon = QPixmap(":Icons/eye.xpm");
-    hiddenIcon = QPixmap(":Icons/empty.xpm");
+
+    ui.cbVisible->blockSignals(true);
+    ui.cbVisible->setChecked(theLayer->isVisible());
+    ui.cbVisible->blockSignals(false);
+#ifdef Q_OS_MAC
+        ui.cbVisible->setMinimumWidth(30);
+#endif
+
+    ui.edName->setText(theLayer->name());
+    ui.edName->setReadOnly(true);
+    ui.edName->setAttribute(Qt::WA_TransparentForMouseEvents);
+
+    ui.edName->setFrame(false);
+    ui.edName->setCursorPosition(0);
+    ui.edName->setStyleSheet(" background: transparent; ");
+
+    connect(this, SIGNAL(toggled(bool)), SLOT(checkedStatusToggled(bool)));
 }
 
 LayerWidget::~LayerWidget()
@@ -30,6 +58,19 @@ LayerWidget::~LayerWidget()
     delete associatedMenu;
 }
 
+void LayerWidget::paintEvent(QPaintEvent *)
+{
+    QStylePainter p(this);
+    QStyleOptionButton option;
+    initStyleOption(&option);
+    p.drawControl(QStyle::CE_PushButton, option);
+
+    if (theLayer && !theLayer->isUploadable()) {
+        p.fillRect(rect().adjusted(20,0,0,-1),QBrush(Qt::red, Qt::BDiagPattern));
+    }
+
+}
+
 void LayerWidget::mousePressEvent(QMouseEvent *event)
 {
     if (event->button() == Qt::LeftButton)
@@ -60,69 +101,47 @@ void LayerWidget::mouseMoveEvent(QMouseEvent *event)
 
 void LayerWidget::mouseReleaseEvent(QMouseEvent* anEvent)
 {
-    if (anEvent->pos().x()<20)
-    {
-        setLayerVisible(!theLayer->isVisible());
-        anEvent->accept();
-    }
+    anEvent->ignore();
+}
+
+void LayerWidget::on_cbVisible_stateChanged ( int state )
+{
+    setLayerVisible((state > Qt::Unchecked));
+}
+
+void LayerWidget::mouseDoubleClickEvent(QMouseEvent */*event*/)
+{
+    ui.edName->setReadOnly(false);
+    ui.edName->setAttribute(Qt::WA_TransparentForMouseEvents, false);
+    ui.edName->setFrame(true);
+    ui.edName->setStyleSheet("");
+    ui.edName->setFocus();
+}
+
+void LayerWidget::on_edName_editingFinished()
+{
+    ui.edName->setReadOnly(true);
+    ui.edName->setAttribute(Qt::WA_TransparentForMouseEvents, true);
+    ui.edName->setFrame(false);
+    ui.edName->setStyleSheet(" background: transparent; ");
+
+    if (!ui.edName->text().isEmpty())
+        theLayer->setName(ui.edName->text());
     else
-    {
-        anEvent->ignore();
-    }
+        ui.edName->setText(theLayer->name());
 }
 
-QSize LayerWidget::minimumSizeHint () const
+void LayerWidget::checkedStatusToggled(bool newVal)
 {
-    return QSize(100, LINEHEIGHT);
+    theLayer->setSelected(newVal);
 }
 
-QSize LayerWidget::sizeHint () const
+void LayerWidget::setName(const QString& s)
 {
-    return QSize(100, LINEHEIGHT);
+    ui.edName->setText(s);
 }
 
-void LayerWidget::paintEvent(QPaintEvent*)
-{
-    if (!theLayer || !theLayer->isEnabled())
-        return;
-
-    QPainter P(this);
-
-    P.drawLine(rect().bottomLeft(), rect().bottomRight());
-    if (isChecked()) {
-        P.fillRect(rect().adjusted(20,0,0,-1),QBrush(palette().highlight()));
-//		P.fillRect(20, 1, width()-19, rect().height()-1, QBrush(palette().highlight()));
-        P.setPen(palette().highlightedText().color());
-        P.drawText(rect().adjusted(23,0,0,-1), Qt::AlignLeft | Qt::AlignVCenter , theLayer->name());
-    } else {
-        P.fillRect(rect().adjusted(0,0,0,-1),backColor);
-        P.setPen(QColor(0,0,0));
-        P.drawText(rect().adjusted(23,0,0,-1), Qt::AlignLeft | Qt::AlignVCenter , theLayer->name());
-    }
-    if (!getMapLayer()->isUploadable()) {
-        P.fillRect(rect().adjusted(20,0,0,-1),QBrush(Qt::red, Qt::BDiagPattern));
-    }
-
-    if (theLayer->isVisible())
-        P.drawPixmap(QPoint(2, rect().center().y()-visibleIcon.height()/2), visibleIcon);
-    else
-        P.drawPixmap(QPoint(2, rect().center().y()-hiddenIcon.height()/2), hiddenIcon);
-}
-
-void LayerWidget::checkStateSet()
-{
-    QAbstractButton::checkStateSet();
-    //emit (layerSelected(this));
-}
-
-void LayerWidget::nextCheckState()
-{
-    QAbstractButton::nextCheckState();
-    theLayer->setSelected(isChecked());
-    //emit (layerSelected(this));
-}
-
-Layer* LayerWidget::getMapLayer()
+Layer* LayerWidget::getLayer()
 {
     return theLayer;
 }
@@ -131,20 +150,15 @@ void LayerWidget::showContextMenu(QContextMenuEvent* anEvent)
 {
     //initActions();
 
-    if (ctxMenu) {
-        if (actZoom) {
-            actZoom->setVisible(true);
-            ImageMapLayer* il = dynamic_cast<ImageMapLayer *>(theLayer.data());
-            if (il) {
-                if (!il->getMapAdapter() || il->getMapAdapter()->getBoundingbox().isNull())
-                    actZoom->setVisible(false);
-            } else
-                actZoom->setEnabled(theLayer->size());
-        }
-        if (closeAction)
-            closeAction->setEnabled(theLayer->canDelete());
-        ctxMenu->exec(anEvent->globalPos());
+    if (!ctxMenu)
+        return;
+
+    if (actZoom) {
+        actZoom->setEnabled(theLayer->size());
     }
+    if (closeAction)
+        closeAction->setEnabled(theLayer->canDelete());
+    ctxMenu->exec(anEvent->globalPos());
 }
 
 #define NUMOP 3
@@ -188,11 +202,6 @@ void LayerWidget::initActions()
     ctxMenu->addMenu(alphaMenu);
     connect(alphaMenu, SIGNAL(triggered(QAction*)), this, SLOT(setOpacity(QAction*)));
     associatedMenu->addMenu(alphaMenu);
-
-    actZoom = new QAction(tr("Zoom"), ctxMenu);
-    ctxMenu->addAction(actZoom);
-    associatedMenu->addAction(actZoom);
-    connect(actZoom, SIGNAL(triggered(bool)), this, SLOT(zoomLayer(bool)));
 }
 
 void LayerWidget::setOpacity(QAction *act)
@@ -203,6 +212,18 @@ void LayerWidget::setOpacity(QAction *act)
 
 void LayerWidget::close()
 {
+    if (theLayer.data()->getDirtyLevel()) {
+        if (QMessageBox::question(this, tr("Layer CLose: Dirty objects present"),
+                                     tr("There are dirty features on this layer.\n"
+                                        "Are you sure you want to close it? (no Undo possible)"),
+                                     QMessageBox::Ok | QMessageBox::Cancel, QMessageBox::Cancel) == QMessageBox::Cancel)
+            return;
+    } else if (theLayer.data()->size())
+        if (QMessageBox::question(this, tr("Layer CLose: Not empty"),
+                                     tr("Are you sure you want to close this layer? (no Undo possible)"),
+                                     QMessageBox::Ok | QMessageBox::Cancel, QMessageBox::Cancel) == QMessageBox::Cancel)
+            return;
+
     emit(layerClosed(theLayer));
 }
 
@@ -211,7 +232,7 @@ void LayerWidget::clear()
     emit(layerCleared(theLayer));
 }
 
-void LayerWidget::zoomLayer(bool)
+void LayerWidget::zoomLayer()
 {
     emit (layerZoom(theLayer));
 }
@@ -223,7 +244,7 @@ void LayerWidget::visibleLayer(bool)
 
 void LayerWidget::readonlyLayer(bool)
 {
-    theLayer->setReadonly(actReadonly->isChecked());
+    setLayerReadonly(actReadonly->isChecked());
 }
 
 QMenu* LayerWidget::getAssociatedMenu()
@@ -231,12 +252,22 @@ QMenu* LayerWidget::getAssociatedMenu()
     return associatedMenu;
 }
 
-void LayerWidget::setLayerVisible(bool b)
+void LayerWidget::setLayerVisible(bool b, bool updateLayer)
 {
-    theLayer->setVisible(b);
+    actVisible->blockSignals(true);
     actVisible->setChecked(b);
+    actVisible->blockSignals(false);
+
+    ui.cbVisible->blockSignals(true);
+    ui.cbVisible->setChecked(b);
+    ui.cbVisible->blockSignals(false);
+
     update();
-    emit(layerChanged(this, false));
+
+    if (updateLayer) {
+        theLayer->setVisible(b);
+        emit(layerChanged(this, false));
+    }
 }
 
 void LayerWidget::setLayerReadonly(bool b)
@@ -258,7 +289,6 @@ void LayerWidget::associatedAboutToShow()
 DrawingLayerWidget::DrawingLayerWidget(DrawingLayer* aLayer, QWidget* aParent)
     : LayerWidget(aLayer, aParent)
 {
-    backColor = QColor(165,209,255);
     initActions();
 }
 
@@ -269,6 +299,11 @@ void DrawingLayerWidget::initActions()
     ctxMenu->addSeparator();
     associatedMenu->addSeparator();
 
+    actZoom = new QAction(tr("Zoom"), ctxMenu);
+    ctxMenu->addAction(actZoom);
+    associatedMenu->addAction(actZoom);
+    connect(actZoom, SIGNAL(triggered(bool)), this, SLOT(zoomLayer()));
+
     closeAction = new QAction(tr("Close"), this);
     connect(closeAction, SIGNAL(triggered()), this, SLOT(close()));
     ctxMenu->addAction(closeAction);
@@ -281,18 +316,16 @@ void DrawingLayerWidget::initActions()
 ImageLayerWidget::ImageLayerWidget(ImageMapLayer* aLayer, QWidget* aParent)
 : LayerWidget(aLayer, aParent), wmsMenu(0) //, actgrWms(0)
 {
-    backColor = QColor(255,255,204);
-    //actgrAdapter = new QActionGroup(this);
-
     actNone = new QAction(tr("None"), this);
     //actNone->setCheckable(true);
-    actNone->setChecked((MerkaartorPreferences::instance()->getBackgroundPlugin() == NONE_ADAPTER_UUID));
+    actNone->setChecked((M_PREFS->getBackgroundPlugin() == NONE_ADAPTER_UUID));
     actNone->setData(QVariant::fromValue(NONE_ADAPTER_UUID));
+    connect(actNone, SIGNAL(triggered()), this, SLOT(setNone()));
 
     if (M_PREFS->getUseShapefileForBackground()) {
         actShape = new QAction(tr("Shape adapter"), this);
         //actShape->setCheckable(true);
-        actShape->setChecked((MerkaartorPreferences::instance()->getBackgroundPlugin() == SHAPE_ADAPTER_UUID));
+        actShape->setChecked((M_PREFS->getBackgroundPlugin() == SHAPE_ADAPTER_UUID));
         actShape->setData(QVariant::fromValue(SHAPE_ADAPTER_UUID));
     }
 
@@ -303,6 +336,14 @@ ImageLayerWidget::~ImageLayerWidget()
 {
 }
 
+void ImageLayerWidget::setNone()
+{
+    ((ImageMapLayer *)theLayer.data())->setMapAdapter(NONE_ADAPTER_UUID);
+
+    this->update(rect());
+    emit (layerChanged(this, true));
+}
+
 void ImageLayerWidget::setWms(QAction* act)
 {
     WmsServerList* L = M_PREFS->getWmsServers();
@@ -329,29 +370,33 @@ void ImageLayerWidget::setTms(QAction* act)
     emit (layerChanged(this, true));
 }
 
-void ImageLayerWidget::setOther(QAction* act)
-{
-    QUuid u(act->data().toString());
-
-    ((ImageMapLayer *)theLayer.data())->setMapAdapter(u);
-    theLayer->setVisible(true);
-
-    this->update(rect());
-    emit (layerChanged(this, true));
-}
-
-void ImageLayerWidget::setBackground(QAction* act)
+void ImageLayerWidget::setPlugin(QAction* act)
 {
     QUuid aUuid = act->data().value<QUuid>();
     if (aUuid.isNull())
         return;
 
     ((ImageMapLayer *)theLayer.data())->setMapAdapter(aUuid);
+    theLayer->setVisible(true);
 
     this->update(rect());
     emit (layerChanged(this, true));
 }
 
+void ImageLayerWidget::setProjection()
+{
+    IMapAdapter* ma = ((ImageMapLayer*)theLayer.data())->getMapAdapter();
+    if (ma) {
+        emit (layerProjection(ma->projection()));
+    }
+}
+
+void ImageLayerWidget::resetAlign()
+{
+    ((ImageMapLayer *)theLayer.data())->resetAlign();
+    emit (layerChanged(this, true));
+}
+
 void ImageLayerWidget::initActions()
 {
     //if (actgrWms)
@@ -359,12 +404,30 @@ void ImageLayerWidget::initActions()
     //actgrWms = new QActionGroup(this);
 
     LayerWidget::initActions();
-    ImageMapLayer* il = ((ImageMapLayer *)theLayer.data());
-    if (il && il->boundingBox().isNull())
-        actZoom->setVisible(false);
+//    ImageMapLayer* il = ((ImageMapLayer *)theLayer.data());
+
+    actZoom = new QAction(tr("Zoom"), ctxMenu);
+    ctxMenu->addAction(actZoom);
+    associatedMenu->addAction(actZoom);
+    connect(actZoom, SIGNAL(triggered(bool)), this, SLOT(zoomLayer()));
 
     actReadonly->setVisible(false);
 
+    actProjection = new QAction(tr("Set view projection to layer's"), this);
+    connect(actProjection, SIGNAL(triggered()), this, SLOT(setProjection()));
+    ctxMenu->addAction(actProjection);
+    associatedMenu->addAction(actProjection);
+
+    actResetAlign = new QAction(tr("Reset alignment adjustment"), this);
+    connect(actResetAlign, SIGNAL(triggered()), this, SLOT(resetAlign()));
+    ctxMenu->addAction(actResetAlign);
+    associatedMenu->addAction(actResetAlign);
+
+    closeAction = new QAction(tr("Close"), this);
+    connect(closeAction, SIGNAL(triggered()), this, SLOT(close()));
+    ctxMenu->addAction(closeAction);
+    associatedMenu->addAction(closeAction);
+
     ctxMenu->addSeparator();
     associatedMenu->addSeparator();
 
@@ -385,7 +448,7 @@ void ImageLayerWidget::initActions()
     }
 
     tmsMenu = new QMenu(tr("TMS adapter"), this);
-    TmsServerList* TmsServers = MerkaartorPreferences::instance()->getTmsServers();
+    TmsServerList* TmsServers = M_PREFS->getTmsServers();
     TmsServerListIterator ti(*TmsServers);
     while (ti.hasNext()) {
         ti.next();
@@ -400,9 +463,28 @@ void ImageLayerWidget::initActions()
         }
     }
 
-    actNone->setChecked((MerkaartorPreferences::instance()->getBackgroundPlugin() == NONE_ADAPTER_UUID));
+    pluginsMenu = new QMenu(tr("Plugins"), this);
+    QMapIterator <QUuid, IMapAdapterFactory *> it(M_PREFS->getBackgroundPlugins());
+    QMap<QString, QUuid> plugins;
+    while (it.hasNext()) {
+        it.next();
+
+        plugins[it.value()->getName()] = it.key();
+    }
+    QMapIterator <QString, QUuid> pluginsIt(plugins);
+    while (pluginsIt.hasNext()) {
+        pluginsIt.next();
+
+        QAction* actBackPlug = new QAction(pluginsIt.key(), this);
+        actBackPlug->setChecked((M_PREFS->getBackgroundPlugin() == pluginsIt.value()));
+        actBackPlug->setData(QVariant::fromValue(pluginsIt.value()));
+
+        pluginsMenu->addAction(actBackPlug);
+    }
+
+    actNone->setChecked((M_PREFS->getBackgroundPlugin() == NONE_ADAPTER_UUID));
     if (M_PREFS->getUseShapefileForBackground())
-        actShape->setChecked((MerkaartorPreferences::instance()->getBackgroundPlugin() == SHAPE_ADAPTER_UUID));
+        actShape->setChecked((M_PREFS->getBackgroundPlugin() == SHAPE_ADAPTER_UUID));
 
     ctxMenu->addAction(actNone);
     associatedMenu->addAction(actNone);
@@ -415,29 +497,54 @@ void ImageLayerWidget::initActions()
     associatedMenu->addMenu(tmsMenu);
     connect(tmsMenu, SIGNAL(triggered(QAction*)), this, SLOT(setTms(QAction*)));
 
+    ctxMenu->addMenu(pluginsMenu);
+    associatedMenu->addMenu(pluginsMenu);
+    connect(pluginsMenu, SIGNAL(triggered(QAction*)), this, SLOT(setPlugin(QAction*)));
+
     if (M_PREFS->getUseShapefileForBackground()) {
         ctxMenu->addAction(actShape);
         associatedMenu->addAction(actShape);
     }
+}
 
-    QMapIterator <QUuid, IMapAdapter *> it(M_PREFS->getBackgroundPlugins());
-    while (it.hasNext()) {
-        it.next();
+void ImageLayerWidget::showContextMenu(QContextMenuEvent* anEvent)
+{
+    if (!ctxMenu)
+        return;
 
-        QAction* actBackPlug = new QAction(it.value()->getName(), this);
-        actBackPlug->setChecked((M_PREFS->getBackgroundPlugin() == it.key()));
-        actBackPlug->setData(QVariant::fromValue(it.key()));
+    ImageMapLayer* theMapLayer = qobject_cast<ImageMapLayer*>(theLayer);
+    QList<QAction*> plugActions;
 
-        if (it.value()->getMenu()) {
-            actBackPlug->setMenu(it.value()->getMenu());
-            disconnect(it.value()->getMenu(), 0, 0, 0);
-            connect(it.value()->getMenu(), SIGNAL(triggered(QAction*)), SLOT(setOther(QAction*)));
+    QAction* sep = new QAction(this);
+    sep->setSeparator(true);
+
+    actZoom->setEnabled(false);
+    if (theMapLayer && theMapLayer->getMapAdapter()) {
+        if (!theMapLayer->getMapAdapter()->getBoundingbox().isNull())
+            actZoom->setEnabled(true);
+
+        if (theMapLayer->getMapAdapter()->getMenu()) {
+            ctxMenu->addAction(sep);
+            associatedMenu->addAction(sep);
+            plugActions << sep;
+            foreach (QAction* a, theMapLayer->getMapAdapter()->getMenu()->actions()) {
+                ctxMenu->addAction(a);
+                associatedMenu->addAction(a);
+                plugActions << a;
+            }
         }
-
-        ctxMenu->addAction(actBackPlug);
-        associatedMenu->addAction(actBackPlug);
     }
-    connect(ctxMenu, SIGNAL(triggered(QAction*)), this, SLOT(setBackground(QAction*)));
+    closeAction->setEnabled(theLayer->canDelete());
+
+    ctxMenu->exec(anEvent->globalPos());
+
+    foreach (QAction* a, plugActions) {
+        if (ctxMenu->actions().contains(a))
+            ctxMenu->removeAction(a);
+        if (associatedMenu->actions().contains(a))
+            associatedMenu->removeAction(a);
+    }
+    delete sep;
 }
 
 // TrackLayerWidget
@@ -445,7 +552,6 @@ void ImageLayerWidget::initActions()
 TrackLayerWidget::TrackLayerWidget(TrackLayer* aLayer, QWidget* aParent)
     : LayerWidget(aLayer, aParent)
 {
-    backColor = QColor(122,204,166);
     initActions();
 }
 
@@ -463,7 +569,7 @@ void TrackLayerWidget::initActions()
     actZoom = new QAction(tr("Zoom"), ctxMenu);
     ctxMenu->addAction(actZoom);
     associatedMenu->addAction(actZoom);
-    connect(actZoom, SIGNAL(triggered(bool)), this, SLOT(zoomLayer(bool)));
+    connect(actZoom, SIGNAL(triggered(bool)), this, SLOT(zoomLayer()));
 
     ctxMenu->addSeparator();
     associatedMenu->addSeparator();
@@ -490,7 +596,6 @@ void TrackLayerWidget::extractLayer(bool)
 DirtyLayerWidget::DirtyLayerWidget(DirtyLayer* aLayer, QWidget* aParent)
     : LayerWidget(aLayer, aParent)
 {
-    backColor = QColor(200,200,200);
     initActions();
 }
 
@@ -503,7 +608,7 @@ void DirtyLayerWidget::initActions()
     actZoom = new QAction(tr("Zoom"), ctxMenu);
     ctxMenu->addAction(actZoom);
     associatedMenu->addAction(actZoom);
-    connect(actZoom, SIGNAL(triggered(bool)), this, SLOT(zoomLayer(bool)));
+    connect(actZoom, SIGNAL(triggered(bool)), this, SLOT(zoomLayer()));
 }
 
 // UploadLayerWidget
@@ -511,7 +616,6 @@ void DirtyLayerWidget::initActions()
 UploadedLayerWidget::UploadedLayerWidget(UploadedLayer* aLayer, QWidget* aParent)
     : LayerWidget(aLayer, aParent)
 {
-    backColor = QColor(200,200,200);
     initActions();
 }
 
@@ -524,7 +628,7 @@ void UploadedLayerWidget::initActions()
     actZoom = new QAction(tr("Zoom"), ctxMenu);
     ctxMenu->addAction(actZoom);
     associatedMenu->addAction(actZoom);
-    connect(actZoom, SIGNAL(triggered(bool)), this, SLOT(zoomLayer(bool)));
+    connect(actZoom, SIGNAL(triggered(bool)), this, SLOT(zoomLayer()));
 
     closeAction = new QAction(tr("Clear"), this);
     connect(closeAction, SIGNAL(triggered()), this, SLOT(clear()));
@@ -538,7 +642,6 @@ void UploadedLayerWidget::initActions()
 OsbLayerWidget::OsbLayerWidget(OsbLayer* aLayer, QWidget* aParent)
     : LayerWidget(aLayer, aParent)
 {
-    backColor = QColor(165,209,192);
     initActions();
 }
 
@@ -556,3 +659,55 @@ void OsbLayerWidget::initActions()
     closeAction->setEnabled(theLayer->canDelete());
 }
 
+// FilterLayerWidget
+
+FilterLayerWidget::FilterLayerWidget(FilterLayer* aLayer, QWidget* aParent)
+    : LayerWidget(aLayer, aParent)
+{
+    initActions();
+}
+
+void FilterLayerWidget::initActions()
+{
+    LayerWidget::initActions();
+
+    ctxMenu->addSeparator();
+    associatedMenu->addSeparator();
+
+    closeAction = new QAction(tr("Close"), this);
+    connect(closeAction, SIGNAL(triggered()), this, SLOT(close()));
+    ctxMenu->addAction(closeAction);
+    associatedMenu->addAction(closeAction);
+    closeAction->setEnabled(theLayer->canDelete());
+}
+
+void FilterLayerWidget::mouseDoubleClickEvent(QMouseEvent */*event*/)
+{
+    FilterLayer* Fl = dynamic_cast<FilterLayer*>(theLayer.data());
+
+    QDialog* dlg = new QDialog(this);
+    ui = new Ui::FilterEditDialog;
+    ui->setupUi(dlg);
+    ui->edName->setText(Fl->name());
+    ui->edFilter->setText(Fl->filter());
+    connect(ui->btFilterHelper, SIGNAL(clicked()), SLOT(on_filterHelperClicked()));
+    if (dlg->exec() == QDialog::Accepted) {
+        setName(ui->edName->text());
+        Fl->setName(ui->edName->text());
+        Fl->setFilter(ui->edFilter->text());
+    }
+    delete ui;
+}
+
+void FilterLayerWidget::on_filterHelperClicked()
+{
+    SelectionDialog* Sel = new SelectionDialog(g_Merk_MainWindow, false);
+    if (!Sel)
+        return;
+
+    Sel->edTagQuery->setText(ui->edFilter->text());
+    if (Sel->exec() == QDialog::Accepted) {
+        ui->edFilter->setText(Sel->edTagQuery->text());
+    }
+}
+
