@@ -1,14 +1,10 @@
 #include "Maps/Projection.h"
-#include "Node.h"
 
 #include <QRect>
 #include <QRectF>
 
 #include <math.h>
 
-#include <ggl/extensions/gis/projections/parameters.hpp>
-#include <ggl/extensions/gis/projections/factory.hpp>
-
 // from wikipedia
 #define EQUATORIALRADIUS 6378137.0
 #define POLARRADIUS      6356752.0
@@ -16,15 +12,30 @@
 #define EQUATORIALMETERHALFCIRCUMFERENCE  20037508.34
 #define EQUATORIALMETERPERDEGREE    222638.981555556
 
+#ifndef _MOBILE
+
+#include "Node.h"
+
+#ifndef USE_PROJ
+#include <ggl/extensions/gis/projections/parameters.hpp>
+#include <ggl/extensions/gis/projections/factory.hpp>
+#include "ggl/extensions/gis/projections/impl/pj_transform.hpp"
+
 using namespace ggl;
+#endif
+#endif
+
 
 // ProjectionPrivate
 
 class ProjectionPrivate
 {
 public:
-    ProjProjection *theWGS84Proj;
-    ProjectionType projType;
+#ifndef _MOBILE
+    ProjProjection theWGS84Proj;
+#endif
+    QString projType;
+    QString projProj4;
     QRectF ProjectedViewport;
     int ProjectionRevision;
     bool IsMercator;
@@ -40,8 +51,13 @@ public:
 
     QPointF mercatorProject(const Coord& c) const
     {
-        double x = coordToAng(c.lon()) / 180. * EQUATORIALMETERHALFCIRCUMFERENCE;
-        double y = log(tan(coordToRad(c.lat())) + 1/cos(coordToRad(c.lat()))) / M_PI * (EQUATORIALMETERHALFCIRCUMFERENCE);
+        return mercatorProject(QPointF(coordToAng(c.lon()), coordToAng(c.lat())));
+    }
+
+    QPointF mercatorProject(const QPointF& c) const
+    {
+        double x = coordToAng(c.x()) / 180. * EQUATORIALMETERHALFCIRCUMFERENCE;
+        double y = log(tan(coordToRad(c.y())) + 1/cos(coordToRad(c.y()))) / M_PI * (EQUATORIALMETERHALFCIRCUMFERENCE);
 
         return QPointF(x, y);
     }
@@ -56,57 +72,96 @@ public:
 
     inline QPointF latlonProject(const Coord& c) const
     {
-        return QPointF(coordToAng(c.lon())*EQUATORIALMETERPERDEGREE, coordToAng(c.lat())*EQUATORIALMETERPERDEGREE);
+        return latlonProject(QPointF(coordToAng(c.lon()), coordToAng(c.lat())));
+    }
+
+    inline QPointF latlonProject(const QPointF& c) const
+    {
+        return QPointF(coordToAng(c.x())/**EQUATORIALMETERPERDEGREE*/, coordToAng(c.y())/**EQUATORIALMETERPERDEGREE*/);
     }
 
     inline Coord latlonInverse(const QPointF& point) const
     {
-        return Coord(angToCoord(point.y()/EQUATORIALMETERPERDEGREE), angToCoord(point.x()/EQUATORIALMETERPERDEGREE));
+        return Coord(angToCoord(point.y()/*/EQUATORIALMETERPERDEGREE*/), angToCoord(point.x()/*/EQUATORIALMETERPERDEGREE*/));
     }
 };
 
 //Projection
 
 Projection::Projection(void)
-: theProj(0), p(new ProjectionPrivate)
+: p(new ProjectionPrivate)
 {
+#ifdef USE_PROJ
+#ifdef Q_OS_WIN
+    QString pdir(QDir::toNativeSeparators(qApp->applicationDirPath() + "/" STRINGIFY(SHARE_DIR) "/proj"));
+    const char* proj_dir = pdir.toUtf8().constData();
+//    const char* proj_dir = "E:\\cbro\\src\\merkaartor-devel\\binaries\\bin\\share\\proj";
+    pj_set_searchpath(1, &proj_dir);
+#endif // Q_OS_WIN
+#endif // USE_PROJ
+
 #ifndef _MOBILE
+    theProj = NULL;
     p->theWGS84Proj = Projection::getProjection("+proj=longlat +ellps=WGS84 +datum=WGS84");
-    p->projType = "";
     setProjectionType(M_PREFS->getProjectionType());
 #endif
 }
 
 Projection::~Projection(void)
 {
+#ifndef _MOBILE
+#ifdef USE_PROJ
+    pj_free(theProj);
+#else
+    SAFE_DELETE(theProj)
+#endif
+#endif // _MOBILE
     delete p;
 }
 
 QPointF Projection::project(const Coord & Map) const
 {
-#ifndef _MOBILE
     if (p->IsMercator)
         return p->mercatorProject(Map);
     else
     if (p->IsLatLong)
         return p->latlonProject(Map);
+#ifndef _MOBILE
     else
         return projProject(Map);
-#else
-    int numberOfTiles, tilesize;
-    numberOfTiles = tilesize = 1;
-    QPointF coordinate(intToAng(Map.lon()), intToAng(Map.lat()));
-    double x = (coordinate.x()+180.) * (numberOfTiles*tilesize) /360.;                // coord to pixel!
-    double y = (1.-(log(tan(M_PI/4+angToRad(coordinate.y())/2)) /M_PI)) * (numberOfTiles*tilesize) /2;
-
-    return QPointF(x, y);
 #endif
 }
 
+QPointF Projection::inverse2Point(const QPointF & Map) const
+{
+    if (p->IsLatLong)
+        return p->latlonInverse(Map).toQPointF();
+    else
+    if (p->IsMercator)
+        return p->mercatorInverse(Map).toQPointF();
+#ifndef _MOBILE
+    else
+        return projInverse(Map).toQPointF();
+#endif
+}
+
+QPointF Projection::project(const QPointF & Map) const
+{
+    if (p->IsMercator)
+        return p->mercatorProject(Map);
+    else
+    if (p->IsLatLong)
+        return p->latlonProject(Map);
+#ifndef _MOBILE
+    else
+        return projProject(Map);
+#endif
+}
+
+#ifndef _MOBILE
 QPointF Projection::project(Node* aNode) const
 {
-#ifndef _MOBILE
-    if (aNode && aNode->projectionRevision() == p->ProjectionRevision)
+    if (aNode && !aNode->projection().isNull() && aNode->projectionRevision() == p->ProjectionRevision)
         return aNode->projection();
 
     QPointF pt;
@@ -122,63 +177,92 @@ QPointF Projection::project(Node* aNode) const
     aNode->setProjection(pt);
 
     return pt;
-#else
-    if (aNode && aNode->projectionRevision() == p->ProjectionRevision)
-        return aNode->projection();
+}
+#endif
 
-    QPointF pt = project(aNode->position());
-    aNode->setProjectionRevision(p->ProjectionRevision);
-    aNode->setProjection(pt);
-
-    return pt;
+QLineF Projection::project(const QLineF & Map) const
+{
+    if (p->IsMercator)
+        return QLineF(p->mercatorProject(Map.p1()), p->mercatorProject(Map.p2()));
+    else
+    if (p->IsLatLong)
+        return QLineF(p->latlonProject(Map.p1()), p->latlonProject(Map.p2()));
+#ifndef _MOBILE
+    else
+        return QLineF(projProject(Map.p1()), projProject(Map.p2()));
 #endif
 }
 
-Coord Projection::inverse(const QPointF & Screen) const
+
+Coord Projection::inverse2Coord(const QPointF & Screen) const
 {
-#ifndef _MOBILE
     if (p->IsLatLong)
         return p->latlonInverse(Screen);
     else
     if (p->IsMercator)
         return p->mercatorInverse(Screen);
+#ifndef _MOBILE
     else
         return projInverse(Screen);
-#else
-    int numberOfTiles, tilesize;
-    numberOfTiles = tilesize = 1;
-    double longitude = (Screen.x() * (360.0/(numberOfTiles*(double)tilesize)) ) -180.0;
-    double latitude = radToAng(atan(sinh((1.0-Screen.y() * (2.0/(numberOfTiles*(double)tilesize)) ) *M_PI)));
-
-    return Coord(angToInt(latitude), angToInt(longitude));
 #endif
 }
 
 #ifndef _MOBILE
 
-#include "ggl/extensions/gis/projections/impl/pj_transform.hpp"
-void Projection::projTransform(ProjProjection *srcdefn,
-                           ProjProjection *dstdefn,
+void Projection::projTransform(ProjProjection srcdefn,
+                           ProjProjection dstdefn,
                            long point_count, int point_offset, double *x, double *y, double *z )
 {
+#ifdef USE_PROJ
+    pj_transform(srcdefn, dstdefn, point_count, point_offset, x, y, z);
+#else
     ggl::projection::detail::pj_transform(srcdefn, dstdefn, point_count, point_offset, x, y, z);
+#endif
 }
 
 void Projection::projTransformFromWGS84(long point_count, int point_offset, double *x, double *y, double *z ) const
 {
+#ifdef USE_PROJ
+    pj_transform(p->theWGS84Proj, theProj, point_count, point_offset, x, y, z);
+#else
     ggl::projection::detail::pj_transform(p->theWGS84Proj, theProj, point_count, point_offset, x, y, z);
+#endif
 }
 
 void Projection::projTransformToWGS84(long point_count, int point_offset, double *x, double *y, double *z ) const
 {
+#ifdef USE_PROJ
+    pj_transform(theProj, p->theWGS84Proj, point_count, point_offset, x, y, z);
+#else
     ggl::projection::detail::pj_transform(theProj, p->theWGS84Proj, point_count, point_offset, x, y, z);
-
+#endif
 }
 
 QPointF Projection::projProject(const Coord & Map) const
 {
+    return projProject(QPointF(coordToAng(Map.lon()), coordToAng(Map.lat())));
+}
+
+QPointF Projection::projProject(const QPointF & Map) const
+{
+#ifdef USE_PROJ
+//    projUV in;
+
+//    in.u = angToRad(Map.x());
+//    in.v = angToRad(Map.y());
+
+//    projUV out = pj_fwd(in, theProj);
+
+//    return QPointF(out.u, out.v);
+    double x = angToRad(Map.x());
+    double y = angToRad(Map.y());
+
+    projTransformFromWGS84(1, 0, &x, &y, NULL);
+
+    return QPointF(x, y);
+#else
     try {
-        point_ll_deg in(longitude<>(coordToAng(Map.lon())), latitude<>(coordToAng(Map.lat())));
+        point_ll_deg in(longitude<>(Map.x()), latitude<>(Map.y()));
         point_2d out;
 
         theProj->forward(in, out);
@@ -187,10 +271,27 @@ QPointF Projection::projProject(const Coord & Map) const
     } catch (...) {
         return QPointF(0., 0.);
     }
+#endif
 }
 
 Coord Projection::projInverse(const QPointF & pProj) const
 {
+#ifdef USE_PROJ
+//    projUV in;
+//    in.u = pProj.x();
+//    in.v = pProj.y();
+
+//    projUV out = pj_inv(in, theProj);
+
+//    return Coord(radToCoord(out.v), radToCoord(out.u));
+
+    double x = pProj.x();
+    double y = pProj.y();
+
+    projTransformToWGS84(1, 0, &x, &y, NULL);
+
+    return Coord(radToCoord(y), radToCoord(x));
+#else
     try {
         point_2d in(pProj.x(), pProj.y());
         point_ll_deg out;
@@ -201,47 +302,17 @@ Coord Projection::projInverse(const QPointF & pProj) const
     } catch (...) {
         return Coord(0, 0);
     }
+#endif
 }
 
-bool Projection::projIsLatLong() const
-{
-    return p->IsLatLong;
-}
-
-//bool Projection::projIsMercator()
-//{
-//    return p->IsMercator;
-//}
-
 QRectF Projection::getProjectedViewport(const CoordBox& Viewport, const QRect& screen) const
 {
     QPointF bl, tr;
     QRectF pViewport;
-    double x, y;
 
-    if (p->IsLatLong)
-        pViewport = Viewport.toQRectF();
-    else {
-        if (p->IsMercator)
-            tr = project(Viewport.topRight());
-        else {
-            x = coordToRad(Viewport.topRight().lon());
-            y = coordToRad(Viewport.topRight().lat());
-            projTransformFromWGS84(1, 0, &x, &y, NULL);
-            tr = QPointF(x, y);
-        }
-
-        if (p->IsMercator)
-            bl = project(Viewport.bottomLeft());
-        else {
-            x = coordToRad(Viewport.bottomLeft().lon());
-            y = coordToRad(Viewport.bottomLeft().lat());
-            projTransformFromWGS84(1, 0, &x, &y, NULL);
-            bl = QPointF(x, y);
-        }
-
-        pViewport = QRectF(bl.x(), tr.y(), tr.x() - bl.x(), bl.y() - tr.y());
-    }
+    tr = project(Viewport.topRight());
+    bl = project(Viewport.bottomLeft());
+    pViewport = QRectF(bl.x(), tr.y(), tr.x() - bl.x(), bl.y() - tr.y());
 
     QPointF pCenter(pViewport.center());
 
@@ -264,42 +335,65 @@ QRectF Projection::getProjectedViewport(const CoordBox& Viewport, const QRect& s
 
     return pViewport;
 }
+#endif // _MOBILE
+
+bool Projection::projIsLatLong() const
+{
+    return p->IsLatLong;
+}
+
+//bool Projection::projIsMercator()
+//{
+//    return p->IsMercator;
+//}
 
-#endif
 
 #ifndef _MOBILE
-
-ProjProjection * Projection::getProjection(QString projString)
+ProjProjection Projection::getProjection(QString projString)
 {
-//    qDebug() << "setProjection: " << projString;
-
+#ifdef USE_PROJ
+    ProjProjection theProj = pj_init_plus(QString("%1 +over").arg(projString).toLatin1());
+#else
     ggl::projection::factory<ggl::point_ll_deg, ggl::point_2d> fac;
     ggl::projection::parameters par;
-    ggl::projection::projection<ggl::point_ll_deg, ggl::point_2d> *theProj;
+    ProjProjection theProj;
 
     try {
         par = ggl::projection::init(std::string(QString("%1 +over").arg(projString).toLatin1().data()));
         theProj = fac.create_new(par);
     } catch (...) {
-        par = ggl::projection::init(std::string(QString("%1 +over").arg(M_PREFS->getProjection("mercator").projection).toLatin1().data()));
-        theProj = fac.create_new(par);
+        theProj = NULL;
     }
+#endif
     return theProj;
 }
+#endif // _MOBILE
 
-bool Projection::setProjectionType(ProjectionType aProjectionType)
+bool Projection::setProjectionType(QString aProjectionType)
 {
     if (aProjectionType == p->projType)
         return true;
 
+#ifndef _MOBILE
+#ifdef USE_PROJ
+    if (theProj) {
+        pj_free(theProj);
+        theProj = NULL;
+    }
+#else
     SAFE_DELETE(theProj)
+#endif
+#endif // _MOBILE
+
     p->ProjectionRevision++;
     p->projType = aProjectionType;
+    p->projProj4 = QString();
     p->IsLatLong = false;
     p->IsMercator = false;
 
     // Hardcode "Google " projection
     if (
+            p->projType.isEmpty() ||
             p->projType.toUpper().contains("OSGEO:41001") ||
             p->projType.toUpper().contains("EPSG:3785") ||
             p->projType.toUpper().contains("EPSG:900913") ||
@@ -307,6 +401,7 @@ bool Projection::setProjectionType(ProjectionType aProjectionType)
             )
     {
         p->IsMercator = true;
+        p->projType = "EPSG:3857";
         return true;
     }
     // Hardcode "lat/long " projection
@@ -315,20 +410,60 @@ bool Projection::setProjectionType(ProjectionType aProjectionType)
             )
     {
         p->IsLatLong = true;
+        p->projType = "EPSG:4326";
         return true;
     }
 
+#ifndef _MOBILE
     try {
-        theProj = getProjection(M_PREFS->getProjection(aProjectionType).projection);
-        if (!theProj)
+        p->projProj4 = M_PREFS->getProjection(aProjectionType).projection;
+        theProj = getProjection(p->projProj4);
+        if (!theProj) {
+            p->projType = "EPSG:3857";
             p->IsMercator = true;
-        else
-            if (theProj->params().is_latlong)
-                p->IsLatLong = true;
+            return false;
+        }
+//        else {
+//#ifdef USE_PROJ
+//            if (pj_is_latlong(theProj))
+//#else
+//            if (theProj->params().is_latlong)
+//#endif
+//                p->projType = "EPSG:4326";
+//                p->IsLatLong = true;
+//        }
     } catch (...) {
         return false;
     }
     return (theProj != NULL || p->IsLatLong || p->IsMercator);
+#else
+    return false;
+#endif // _MOBILE
+}
+
+#ifndef _MOBILE
+QString Projection::getProjectionType() const
+{
+    return p->projType;
+}
+
+QString Projection::getProjectionProj4() const
+{
+    if (p->IsLatLong)
+        return "+init=EPSG:4326";
+    else if (p->IsMercator)
+        return "+init=EPSG:3857";
+    else
+#ifdef USE_PROJ
+        return QString(pj_get_def(theProj, 0));
+#else
+        return p->projProj4;
+#endif
+}
+
+int Projection::projectionRevision() const
+{
+    return p->ProjectionRevision;
 }
 #endif
 
@@ -347,8 +482,36 @@ double Projection::lonAnglePerM(double Lat) const
     return 1 / LengthOfOneDegreeLon;
 }
 
-int Projection::projectionRevision() const
+bool Projection::toXML(QDomElement xParent)
 {
-    return p->ProjectionRevision;
+    bool OK = true;
+
+    QDomElement e = xParent.namedItem("Projection").toElement();
+    if (!e.isNull()) {
+        xParent.removeChild(e);
+    }
+    e = xParent.ownerDocument().createElement("Projection");
+    xParent.appendChild(e);
+
+    e.setAttribute("type", p->projType);
+    if (!p->IsLatLong && !p->IsMercator && !p->projProj4.isEmpty()) {
+        e.appendChild(xParent.ownerDocument().createTextNode(p->projProj4));
+    }
+
+
+    return OK;
 }
 
+void Projection::fromXML(const QDomElement e)
+{
+    if (e.tagName() == "Projection") {
+        if (e.hasChildNodes()) {
+            setProjectionType(e.firstChild().toText().nodeValue());
+            if (e.hasAttribute("type"))
+                p->projType = e.attribute("type");
+            else
+                p->projType = QCoreApplication::translate("Projection", "Document");
+        } else
+            setProjectionType(e.attribute("type"));
+    }
+}
