@@ -1,8 +1,12 @@
+#include "Global.h"
+
 #include "PaintStyle/PaintStyleEditor.h"
-#include "PaintStyle/PaintStyle.h"
+#include "PaintStyle/Painter.h"
 #include "MainWindow.h"
 #include "Document.h"
 
+#include "Utils/SelectionDialog.h"
+
 #include <QtGui/QCheckBox>
 #include <QtGui/QColorDialog>
 #include <QtGui/QDoubleSpinBox>
@@ -23,8 +27,8 @@ static void makeBoundaryIcon(QToolButton* bt, QColor C)
     bt->setIcon(pm);
 }
 
-PaintStyleEditor::PaintStyleEditor(QWidget *aParent, const GlobalPainter& aGlobalPainter, const QList<FeaturePainter>& aPainters)
-        : QDialog(aParent), theGlobalPainter(aGlobalPainter), thePainters(aPainters), FreezeUpdate(true)
+PaintStyleEditor::PaintStyleEditor(QWidget *aParent, const GlobalPainter& aGlobalPainter, const QList<Painter>& aPainters)
+    : QDialog(aParent), theGlobalPainter(aGlobalPainter), thePainters(aPainters), FreezeUpdate(true)
 {
     setupUi(this);
 
@@ -39,9 +43,8 @@ PaintStyleEditor::PaintStyleEditor(QWidget *aParent, const GlobalPainter& aGloba
     LabelBackgroundlColor->setIconSize(QSize(36, 18));
     GlobalBackgroundColor->setIconSize(QSize(36, 18));
 
-    for (int i = 0; i < thePainters.size(); ++i)
-        PaintList->addItem(thePainters[i].userName());
-    PaintList->setCurrentRow(0);
+    updatePaintList();
+
     LowerZoomBoundary->setSpecialValueText(tr("Always"));
     UpperZoomBoundary->setSpecialValueText(tr("Always"));
 
@@ -54,31 +57,64 @@ PaintStyleEditor::PaintStyleEditor(QWidget *aParent, const GlobalPainter& aGloba
     resize(1, 1);
 }
 
+void PaintStyleEditor::updatePaintList()
+{
+    QListWidgetItem* it;
+    QString curName;
+    if (PaintList->currentItem())
+        curName = PaintList->currentItem()->text();
+    PaintList->clear();
+    for (int i = 0; i < thePainters.size(); ++i) {
+        it = new QListWidgetItem(thePainters[i].userName());
+        it->setData(Qt::UserRole, i);
+        if (edFilter->text().isEmpty()) {
+            PaintList->addItem(it);
+        } else
+            if (thePainters[i].userName().contains(edFilter->text(), Qt::CaseInsensitive)) {
+            it = new QListWidgetItem(thePainters[i].userName());
+            it->setData(Qt::UserRole, i);
+            PaintList->addItem(it);
+        }
+        if (!curName.isEmpty() && thePainters[i].userName().contains(curName, Qt::CaseInsensitive))
+            PaintList->setCurrentItem(it);
+    }
+    if (curName.isEmpty())
+        PaintList->setCurrentRow(0);
+}
+
 void PaintStyleEditor::on_AddButton_clicked()
 {
     thePainters.push_back(FeaturePainter());
-    PaintList->addItem(thePainters[thePainters.size()-1].userName());
-    PaintList->setCurrentRow(thePainters.size() - 1);
+    QListWidgetItem* it = new QListWidgetItem(thePainters[thePainters.size()-1].userName());
+    it->setData(Qt::UserRole, thePainters.size()-1);
+    PaintList->addItem(it);
+    PaintList->setCurrentItem(it);
     on_PaintList_itemSelectionChanged();
 }
 
 void PaintStyleEditor::on_DuplicateButton_clicked()
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx < 0 || idx >= thePainters.size())
         return;
     //QList<FeaturePainter>::iterator theIterator = thePainters.begin();
-    thePainters.insert(thePainters.begin() + idx, FeaturePainter(thePainters[idx]));
+    thePainters.insert(thePainters.begin() + idx, Painter(thePainters[idx]));
     idx++;
-    PaintList->insertItem(idx, thePainters[idx].userName());
-    PaintList->setCurrentRow(idx);
+    it = new QListWidgetItem(thePainters[idx].userName());
+    it->setData(Qt::UserRole, idx);
+    PaintList->insertItem(PaintList->currentRow()+1, it);
+    PaintList->setCurrentItem(it);
     on_PaintList_itemSelectionChanged();
 }
 
 void PaintStyleEditor::on_RemoveButton_clicked()
 {
     FreezeUpdate = true;
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx < 0 || idx >= thePainters.size())
         return;
     thePainters.erase(thePainters.begin() + idx);
@@ -91,35 +127,47 @@ void PaintStyleEditor::on_RemoveButton_clicked()
 
 void PaintStyleEditor::on_btUp_clicked()
 {
-    int idx = static_cast<int>(PaintList->currentRow());
-    if (idx <= 0)
+    if (PaintList->currentRow() <= 0)
         return;
-    FeaturePainter fp = thePainters[idx-1];
-    thePainters[idx-1] = thePainters[idx];
+
+    int idx = PaintList->item(PaintList->currentRow())->data(Qt::UserRole).toInt();
+    int idxup = PaintList->item(PaintList->currentRow()-1)->data(Qt::UserRole).toInt();
+
+    Painter fp = thePainters[idxup];
+    thePainters[idxup] = thePainters[idx];
     thePainters[idx] = fp;
-    PaintList->item(idx-1)->setText(thePainters[idx-1].userName());
-    PaintList->item(idx)->setText(thePainters[idx].userName());
-    PaintList->setCurrentRow(idx-1);
+    PaintList->item(PaintList->currentRow()-1)->setText(thePainters[idxup].userName());
+    PaintList->item(PaintList->currentRow())->setText(thePainters[idx].userName());
+    PaintList->setCurrentRow(PaintList->currentRow()-1);
 }
 
 void PaintStyleEditor::on_btDown_clicked()
 {
-    int idx = PaintList->currentRow();
-    if (idx >= thePainters.size()-1)
+    if (PaintList->currentRow() >= PaintList->count()-1)
         return;
-    FeaturePainter fp = thePainters[idx+1];
-    thePainters[idx+1] = thePainters[idx];
+
+    int idx = PaintList->item(PaintList->currentRow())->data(Qt::UserRole).toInt();
+    int idxdn = PaintList->item(PaintList->currentRow()+1)->data(Qt::UserRole).toInt();
+
+    Painter fp = thePainters[idxdn];
+    thePainters[idxdn] = thePainters[idx];
     thePainters[idx] = fp;
-    PaintList->item(idx+1)->setText(thePainters[idx+1].userName());
-    PaintList->item(idx)->setText(thePainters[idx].userName());
-    PaintList->setCurrentRow(idx+1);
+    PaintList->item(PaintList->currentRow()+1)->setText(thePainters[idxdn].userName());
+    PaintList->item(PaintList->currentRow())->setText(thePainters[idx].userName());
+    PaintList->setCurrentRow(PaintList->currentRow()+1);
 }
 
 void PaintStyleEditor::on_PaintList_itemSelectionChanged()
 {
-    FreezeUpdate = true;
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+
+    int idx = -1;
+    if (it)
+        idx = it->data(Qt::UserRole).toInt();
+
     if (idx < 0) {
+        FreezeUpdate = false;
+
         editArea->setEnabled(false);
         DuplicateButton->setEnabled(false);
         RemoveButton->setEnabled(false);
@@ -131,7 +179,9 @@ void PaintStyleEditor::on_PaintList_itemSelectionChanged()
     }
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+
+    FreezeUpdate = true;
+    Painter& FP(thePainters[idx]);
     TagSelection->setText(FP.userName());
     if (FP.zoomBoundaries().first == 0)
         LowerZoomBoundary->setValue(0);
@@ -161,6 +211,7 @@ void PaintStyleEditor::on_PaintList_itemSelectionChanged()
     TouchupDashOff->setValue(FP.touchupBoundary().DashOff);
     DrawFill->setChecked(FP.fillColor().isValid());
     makeBoundaryIcon(FillColor, FP.fillColor());
+    DrawFillIcon->setChecked(FP.ForegroundFillUseIcon);;
     DrawIcon->setChecked(FP.icon().Draw);
     IconName->setText(FP.icon().Name);
     ProportionalIcon->setValue(FP.icon().Proportional);
@@ -207,17 +258,19 @@ void PaintStyleEditor::updatePagesIcons()
 
 void PaintStyleEditor::on_TagSelection_editingFinished()
 {
-    updatePaintList();
+    refreshPainter();
 }
 
 void PaintStyleEditor::on_LowerZoomBoundary_valueChanged()
 {
     if (FreezeUpdate)
         return;
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     QPair<double, double> Result(0, 0);
     if (LowerZoomBoundary->value() > 10e-6)
         Result.first = 1 / LowerZoomBoundary->value();
@@ -242,9 +295,9 @@ void PaintStyleEditor::on_GlobalBackgroundColor_clicked()
 {
     QColor rgb = QColorDialog::getColor(theGlobalPainter.getBackgroundColor(), this
 #if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
-                                            , tr("Select Color"), QColorDialog::ShowAlphaChannel
+                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
 #endif
-                                           );
+                                        );
     if (rgb.isValid()) {
         makeBoundaryIcon(GlobalBackgroundColor, rgb);
         theGlobalPainter.background(rgb);
@@ -253,7 +306,9 @@ void PaintStyleEditor::on_GlobalBackgroundColor_clicked()
 
 void PaintStyleEditor::on_DrawBackground_clicked(bool b)
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
     thePainters[idx].backgroundActive(b);
@@ -263,15 +318,17 @@ void PaintStyleEditor::on_DrawBackground_clicked(bool b)
 
 void PaintStyleEditor::on_BackgroundColor_clicked()
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     QColor rgb = QColorDialog::getColor(FP.backgroundBoundary().Color, this
 #if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
-                                            , tr("Select Color"), QColorDialog::ShowAlphaChannel
+                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
 #endif
-                                           );
+                                        );
     if (rgb.isValid()) {
         makeBoundaryIcon(BackgroundColor, rgb);
         FP.background(rgb, ProportionalBackground->value(), FixedBackground->value());
@@ -282,10 +339,12 @@ void PaintStyleEditor::on_ProportionalBackground_valueChanged()
 {
     if (FreezeUpdate)
         return;
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     FP.background(FP.backgroundBoundary().Color, ProportionalBackground->value(), FixedBackground->value());
 }
 
@@ -296,7 +355,9 @@ void PaintStyleEditor::on_FixedBackground_valueChanged()
 
 void PaintStyleEditor::on_DrawForeground_clicked(bool b)
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
     thePainters[idx].foregroundActive(b);
@@ -307,29 +368,45 @@ void PaintStyleEditor::on_DrawForeground_clicked(bool b)
 
 void PaintStyleEditor::on_ForegroundColor_clicked()
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     QColor rgb = QColorDialog::getColor(FP.foregroundBoundary().Color, this
 #if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
-                                            , tr("Select Color"), QColorDialog::ShowAlphaChannel
+                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
 #endif
-                                           );
+                                        );
     if (rgb.isValid()) {
         makeBoundaryIcon(ForegroundColor, rgb);
         FP.foreground(rgb, ProportionalForeground->value(), FixedForeground->value());
     }
 }
 
+void PaintStyleEditor::on_DrawFillIcon_clicked(bool b)
+{
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
+    if (idx >= thePainters.size())
+        return;
+    Painter& FP(thePainters[idx]);
+
+    FP.ForegroundFillUseIcon = b;
+}
+
 void PaintStyleEditor::on_ProportionalForeground_valueChanged()
 {
     if (FreezeUpdate)
         return;
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     FP.foreground(FP.foregroundBoundary().Color, ProportionalForeground->value(), FixedForeground->value());
 }
 
@@ -343,10 +420,12 @@ void PaintStyleEditor::on_ForegroundDashed_clicked()
 {
     if (FreezeUpdate)
         return;
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     if (ForegroundDashed->isChecked())
         FP.foregroundDash(ForegroundDashOn->value(), ForegroundDashOff->value());
     else
@@ -365,7 +444,9 @@ void PaintStyleEditor::on_ForegroundDashOn_valueChanged()
 
 void PaintStyleEditor::on_DrawTouchup_clicked(bool b)
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
     thePainters[idx].touchupActive(b);
@@ -375,15 +456,17 @@ void PaintStyleEditor::on_DrawTouchup_clicked(bool b)
 
 void PaintStyleEditor::on_TouchupColor_clicked()
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     QColor rgb = QColorDialog::getColor(FP.touchupBoundary().Color, this
 #if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
-                                            , tr("Select Color"), QColorDialog::ShowAlphaChannel
+                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
 #endif
-                                           );
+                                        );
     if (rgb.isValid()) {
         makeBoundaryIcon(TouchupColor, rgb);
         FP.touchup(rgb, ProportionalTouchup->value(), FixedTouchup->value());
@@ -394,10 +477,12 @@ void PaintStyleEditor::on_ProportionalTouchup_valueChanged()
 {
     if (FreezeUpdate)
         return;
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     FP.touchup(FP.touchupBoundary().Color, ProportionalTouchup->value(), FixedTouchup->value());
 }
 
@@ -411,10 +496,12 @@ void PaintStyleEditor::on_TouchupDashed_clicked()
 {
     if (FreezeUpdate)
         return;
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     if (TouchupDashed->isChecked())
         FP.touchupDash(TouchupDashOn->value(), TouchupDashOff->value());
     else
@@ -433,7 +520,9 @@ void PaintStyleEditor::on_TouchupDashOn_valueChanged()
 
 void PaintStyleEditor::on_DrawFill_clicked(bool b)
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
     thePainters[idx].fillActive(b);
@@ -443,15 +532,17 @@ void PaintStyleEditor::on_DrawFill_clicked(bool b)
 
 void PaintStyleEditor::on_FillColor_clicked()
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     QColor rgb = QColorDialog::getColor(FP.fillColor(), this
 #if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
-                                            , tr("Select Color"), QColorDialog::ShowAlphaChannel
+                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
 #endif
-                                           );
+                                        );
     if (rgb.isValid()) {
         makeBoundaryIcon(FillColor, rgb);
         FP.foregroundFill(rgb);
@@ -460,7 +551,9 @@ void PaintStyleEditor::on_FillColor_clicked()
 
 void PaintStyleEditor::on_DrawIcon_clicked(bool b)
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
     thePainters[idx].iconActive(b);
@@ -472,10 +565,12 @@ void PaintStyleEditor::on_IconName_textEdited()
 {
     if (FreezeUpdate)
         return;
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     FP.setIcon(IconName->text(), ProportionalIcon->value(), FixedIcon->value());
 }
 
@@ -491,7 +586,9 @@ void PaintStyleEditor::on_FixedIcon_valueChanged()
 
 void PaintStyleEditor::on_DrawLabel_clicked(bool b)
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
     thePainters[idx].labelActive(b);
@@ -501,7 +598,9 @@ void PaintStyleEditor::on_DrawLabel_clicked(bool b)
 
 void PaintStyleEditor::on_LabelHalo_clicked(bool b)
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
     thePainters[idx].labelHalo(b);
@@ -509,7 +608,9 @@ void PaintStyleEditor::on_LabelHalo_clicked(bool b)
 
 void PaintStyleEditor::on_LabelArea_clicked(bool b)
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
     thePainters[idx].labelArea(b);
@@ -517,7 +618,9 @@ void PaintStyleEditor::on_LabelArea_clicked(bool b)
 
 void PaintStyleEditor::on_LabelTag_textEdited()
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
     thePainters[idx].labelTag(LabelTag->text());
@@ -525,7 +628,9 @@ void PaintStyleEditor::on_LabelTag_textEdited()
 
 void PaintStyleEditor::on_LabelBackgroundTag_textEdited()
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
     thePainters[idx].labelBackgroundTag(LabelBackgroundTag->text());
@@ -533,15 +638,17 @@ void PaintStyleEditor::on_LabelBackgroundTag_textEdited()
 
 void PaintStyleEditor::on_LabelColor_clicked()
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     QColor rgb = QColorDialog::getColor(FP.labelBoundary().Color, this
 #if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
-                                            , tr("Select Color"), QColorDialog::ShowAlphaChannel
+                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
 #endif
-                                           );
+                                        );
     if (rgb.isValid()) {
         makeBoundaryIcon(LabelColor, rgb);
         FP.label(rgb, ProportionalLabel->value(), FixedLabel->value());
@@ -552,10 +659,12 @@ void PaintStyleEditor::on_ProportionalLabel_valueChanged()
 {
     if (FreezeUpdate)
         return;
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     FP.label(FP.labelBoundary().Color, ProportionalLabel->value(), FixedLabel->value());
 }
 
@@ -566,7 +675,9 @@ void PaintStyleEditor::on_FixedLabel_valueChanged()
 
 void PaintStyleEditor::on_DrawLabelBackground_clicked(bool b)
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
     thePainters[idx].labelBackgroundActive(b);
@@ -574,15 +685,17 @@ void PaintStyleEditor::on_DrawLabelBackground_clicked(bool b)
 
 void PaintStyleEditor::on_LabelBackgroundlColor_clicked()
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     QColor rgb = QColorDialog::getColor(FP.labelBackgroundColor(), this
 #if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
-                                            , tr("Select Color"), QColorDialog::ShowAlphaChannel
+                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
 #endif
-                                           );
+                                        );
     if (rgb.isValid()) {
         makeBoundaryIcon(LabelBackgroundlColor, rgb);
         FP.labelBackground(rgb);
@@ -591,18 +704,21 @@ void PaintStyleEditor::on_LabelBackgroundlColor_clicked()
 
 void PaintStyleEditor::on_LabelFont_currentFontChanged(const QFont & font)
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
     if (idx >= thePainters.size())
         return;
     thePainters[idx].setLabelFont(font.toString());
 }
 
-void PaintStyleEditor::updatePaintList()
+void PaintStyleEditor::refreshPainter()
 {
-    int idx = PaintList->currentRow();
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
     if (idx >= thePainters.size())
         return;
-    FeaturePainter& FP(thePainters[idx]);
+    Painter& FP(thePainters[idx]);
     FP.setSelector(TagSelection->text());
     PaintList->currentItem()->setText(FP.userName());
 }
@@ -614,3 +730,19 @@ void PaintStyleEditor::on_buttonBox_clicked(QAbstractButton * button)
     }
 }
 
+void PaintStyleEditor::on_edFilter_textChanged(const QString &/*text*/)
+{
+    updatePaintList();
+}
+
+void PaintStyleEditor::on_btSelectorHelper_clicked()
+{
+    SelectionDialog* Sel = new SelectionDialog(g_Merk_MainWindow, false);
+    if (!Sel)
+        return;
+
+    Sel->edTagQuery->setText(TagSelection->text());
+    if (Sel->exec() == QDialog::Accepted) {
+        TagSelection->setText(Sel->edTagQuery->text());
+    }
+}
