@@ -36,6 +36,67 @@ static void makeBoundaryIcon(QToolButton* bt, QColor C)
     bt->setIcon(pm);
 }
 
+OsmServerWidget::OsmServerWidget(QWidget * parent, Qt::WindowFlags f)
+    : QWidget(parent, f)
+{
+    setupUi(this);
+    setAttribute(Qt::WA_DeleteOnClose);
+}
+
+void OsmServerWidget::on_tbOsmServerAdd_clicked()
+{
+    QLayout* lay = parentWidget()->layout();
+    if (!lay)
+        return;
+
+    OsmServerWidget* w = new OsmServerWidget(parentWidget());
+    lay->addWidget(w);
+
+    for (int i=0; i<lay->count(); ++i) {
+        OsmServerWidget* w = dynamic_cast<OsmServerWidget*>(lay->itemAt(i)->widget());
+        if (w)
+            w->tbOsmServerDel->setEnabled(true);
+    }
+}
+
+void OsmServerWidget::on_tbOsmServerDel_clicked()
+{
+    QLayout* lay = parentWidget()->layout();
+    if (!lay)
+        return;
+
+    if (rbOsmServerSelected->isChecked()) {
+        OsmServerWidget* w = dynamic_cast<OsmServerWidget*>(lay->itemAt(0)->widget());
+        if (w)
+            w->rbOsmServerSelected->setChecked(true);
+    }
+    if (lay->count() > 2)
+        close();
+    else if (lay->count() == 2 ) {
+        for (int i=0; i<lay->count(); ++i) {
+            OsmServerWidget* w = dynamic_cast<OsmServerWidget*>(lay->itemAt(i)->widget());
+            if (w)
+                w->tbOsmServerDel->setEnabled(false);
+            close();
+        }
+    }
+}
+
+void OsmServerWidget::on_rbOsmServerSelected_clicked()
+{
+    QLayout* lay = parentWidget()->layout();
+    if (!lay)
+        return;
+
+    for (int i=0; i<lay->count(); ++i) {
+        OsmServerWidget* w = dynamic_cast<OsmServerWidget*>(lay->itemAt(i)->widget());
+        if (w)
+            w->rbOsmServerSelected->setChecked(false);
+    }
+
+    rbOsmServerSelected->setChecked(true);
+}
+
 PreferencesDialog::PreferencesDialog(QWidget* parent)
     : QDialog(parent)
 {
@@ -52,13 +113,6 @@ PreferencesDialog::PreferencesDialog(QWidget* parent)
         cbTemplates->addItem(intTemplates.entryList().at(i));
     }
 
-    QString qVer = QString(qVersion()).replace(".", "");
-    int iQVer = qVer.toInt();
-    if (iQVer < 451) {
-        cbCustomStyle->hide();
-        comboCustomStyle->hide();
-    }
-
     resize(1,1);
     QApplication::processEvents();
 
@@ -94,11 +148,11 @@ void PreferencesDialog::on_buttonBox_clicked(QAbstractButton * button)
 {
     if ((button == buttonBox->button(QDialogButtonBox::Apply))) {
         savePrefs();
-        emit(preferencesChanged());
+        emit(preferencesChanged(this));
     } else
         if ((button == buttonBox->button(QDialogButtonBox::Ok))) {
             savePrefs();
-            emit(preferencesChanged());
+            emit(preferencesChanged(this));
             this->accept();
         }
 }
@@ -107,12 +161,14 @@ void PreferencesDialog::initLanguages(QComboBox* aBox)
 {
     aBox->addItem(tr("English"),"-");
     aBox->addItem(tr("Arabic"),"ar");
+    aBox->addItem(tr("Croatian"),"hr");
     aBox->addItem(tr("Czech"),"cs");
     aBox->addItem(tr("Dutch"),"nl");
     aBox->addItem(tr("German"),"de");
     aBox->addItem(tr("French"),"fr");
+    aBox->addItem(tr("Hungarian"),"hu");
     aBox->addItem(tr("Italian"),"it");
-    aBox->addItem(tr("Japanase"),"ja");
+    aBox->addItem(tr("Japanese"),"ja");
     aBox->addItem(tr("Polish"),"pl");
     aBox->addItem(tr("Portuguese"),"pt");
     aBox->addItem(tr("Brazilian Portuguese"),"pt_BR");
@@ -126,7 +182,7 @@ void PreferencesDialog::initLanguages(QComboBox* aBox)
 void PreferencesDialog::loadPrefs()
 {
     initLanguages(Language);
-    QString CurrentLanguage(getDefaultLanguage());
+    QString CurrentLanguage(getDefaultLanguage(false));
     int l;
     for (l = 0; l < Language->count(); ++l)
         if (CurrentLanguage == Language->itemData(l))
@@ -136,9 +192,33 @@ void PreferencesDialog::loadPrefs()
     if (l < Language->count())
         Language->setCurrentIndex(l);
     TranslateTags->setChecked(M_PREFS->getTranslateTags());
-    edOsmUrl->setText(M_PREFS->getOsmWebsite());
-    edOsmUser->setText(M_PREFS->getOsmUser());
-    edOsmPwd->setText(M_PREFS->getOsmPassword());
+
+    OsmServerList* theOsmServers = M_PREFS->getOsmServers();
+    if (!theOsmServers->size()) {
+        OsmServerWidget* wOSmServer = new OsmServerWidget(grpOSM);
+
+        wOSmServer->edOsmServerUrl->setText(M_PREFS->getOsmWebsite());
+        wOSmServer->edOsmServerUser->setText(M_PREFS->getOsmUser());
+        wOSmServer->edOsmServerPwd->setText(M_PREFS->getOsmPassword());
+        wOSmServer->rbOsmServerSelected->setChecked(true);
+        wOSmServer->tbOsmServerDel->setEnabled(false);
+
+        OsmServersLayout->addWidget(wOSmServer);
+    } else {
+        foreach(OsmServer srv, *theOsmServers) {
+            OsmServerWidget* wOSmServer = new OsmServerWidget(grpOSM);
+
+            wOSmServer->edOsmServerUrl->setText(srv.Url);
+            wOSmServer->edOsmServerUser->setText(srv.User);
+            wOSmServer->edOsmServerPwd->setText(srv.Password);
+            wOSmServer->rbOsmServerSelected->setChecked(srv.Selected);
+
+            OsmServersLayout->addWidget(wOSmServer);
+        }
+    }
+
+    edXapiUrl->setText(M_PREFS->getXapiUrl());
+    edNominatimUrl->setText(M_PREFS->getNominatimUrl());
 
     edGpsPort->setText(M_PREFS->getGpsPort());
     edGpsdHost->setText(M_PREFS->getGpsdHost());
@@ -165,8 +245,7 @@ void PreferencesDialog::loadPrefs()
     edProxyPassword->setText(M_PREFS->getProxyPassword());
 
     cbLocalServer->setChecked(M_PREFS->getLocalServer());
-
-    //bbUse06Api->setChecked((M_PREFS->apiVersionNum() > 0.5));
+    sbNetworkTimeout->setValue(int(M_PREFS->getNetworkTimeout()/1000));
 
     edCacheDir->setText(M_PREFS->getCacheDir());
     sbCacheSize->setValue(M_PREFS->getCacheSize());
@@ -204,21 +283,27 @@ void PreferencesDialog::loadPrefs()
     FocusColor = M_PREFS->getFocusColor();
     HoverColor = M_PREFS->getHoverColor();
     HighlightColor = M_PREFS->getHighlightColor();
+    DirtyColor = M_PREFS->getDirtyColor();
     RelationsColor = M_PREFS->getRelationsColor();
     GpxTrackColor = M_PREFS->getGpxTrackColor();
     makeBoundaryIcon(btBgColor, BgColor);
     makeBoundaryIcon(btHoverColor, HoverColor);
     makeBoundaryIcon(btHighlightColor, HighlightColor);
+    makeBoundaryIcon(btDirtyColor, DirtyColor);
     makeBoundaryIcon(btFocusColor, FocusColor);
     makeBoundaryIcon(btRelationsColor, RelationsColor);
     makeBoundaryIcon(btGpxTrackColor, GpxTrackColor);
     HoverWidth->setValue(M_PREFS->getHoverWidth());
     HighlightWidth->setValue(M_PREFS->getHighlightWidth());
+    DirtyWidth->setValue(M_PREFS->getDirtyWidth());
     FocusWidth->setValue(M_PREFS->getFocusWidth());
     RelationsWidth->setValue(M_PREFS->getRelationsWidth());
     GpxTrackWidth->setValue(M_PREFS->getGpxTrackWidth());
     cbSimpleGpxTrack->setChecked(M_PREFS->getSimpleGpxTrack());
 
+    cbAutoLoadDoc->setChecked(M_PREFS->getHasAutoLoadDocument());
+    edAutoLoadDoc->setText(M_PREFS->getAutoLoadDocumentFilename());
+    edAutoLoadDoc->setEnabled(cbAutoLoadDoc->isChecked());
     cbAutoSaveDoc->setChecked(M_PREFS->getAutoSaveDoc());
     cbAutoExtractTracks->setChecked(M_PREFS->getAutoExtractTracks());
     cbReadonlyTracksDefault->setChecked(M_PREFS->getReadonlyTracksDefault());
@@ -234,6 +319,7 @@ void PreferencesDialog::loadPrefs()
 
     cbMouseSingleButton->setChecked(M_PREFS->getMouseSingleButton());
     cbSeparateMoveMode->setChecked(M_PREFS->getSeparateMoveMode());
+    cbSelectModeCreation->setChecked(M_PREFS->getSelectModeCreation());
     cbVirtualNodes->setChecked(M_PREFS->getUseVirtualNodes());
     cbRelationsHiddenSelectable->setChecked(M_PREFS->getRelationsSelectableWhenHidden());
 
@@ -242,6 +328,7 @@ void PreferencesDialog::loadPrefs()
     comboCustomStyle->setCurrentIndex(comboCustomStyle->findText(M_PREFS->getMerkaartorStyleString()));
 
     cbUseShapefileForBackground->setChecked(M_PREFS->getUseShapefileForBackground());
+    cbAutoSourceTag->setChecked(M_PREFS->getAutoSourceTag());
 }
 
 void PreferencesDialog::savePrefs()
@@ -258,16 +345,32 @@ void PreferencesDialog::savePrefs()
     //M_PREFS->setUse06Api(bbUse06Api->isChecked());
 
     bool OsmDataChanged = false;
-    if (edOsmUrl->text() != M_PREFS->getOsmWebsite() ||
-        edOsmUser->text() != M_PREFS->getOsmUser() ||
-        edOsmPwd->text() != M_PREFS->getOsmPassword())
-        OsmDataChanged = true;
 
-    M_PREFS->setOsmWebsite(edOsmUrl->text());
-    M_PREFS->setOsmUser(edOsmUser->text());
-    M_PREFS->setOsmPassword(edOsmPwd->text());
+    OsmServerList* theServerList = M_PREFS->getOsmServers();
+    theServerList->clear();
+    for (int i=0; i< OsmServersLayout->count(); ++i) {
+        OsmServerWidget* wOsmServer = dynamic_cast<OsmServerWidget*>(OsmServersLayout->itemAt(i)->widget());
+        if (!wOsmServer)
+            continue;
 
-    M_PREFS->setXapiWebSite(M_PREFS->getXapiWebSite());
+        OsmServer srv;
+        srv.Url = wOsmServer->edOsmServerUrl->text();
+        srv.User = wOsmServer->edOsmServerUser->text();
+        srv.Password = wOsmServer->edOsmServerPwd->text();
+        srv.Selected = wOsmServer->rbOsmServerSelected->isChecked();
+
+        if (srv.Selected && (srv.Url != M_PREFS->getOsmWebsite() || srv.User != M_PREFS->getOsmUser() || srv.Password != M_PREFS->getOsmPassword())) {
+            M_PREFS->setOsmWebsite(srv.Url);
+            M_PREFS->setOsmUser(srv.User);
+            M_PREFS->setOsmPassword(srv.Password);
+            OsmDataChanged = true;
+        }
+
+        theServerList->append(srv);
+    }
+
+    M_PREFS->setXapiUrl(edXapiUrl->text());
+    M_PREFS->setNominatimUrl(edNominatimUrl->text());
 
     M_PREFS->setGpsPort(edGpsPort->text());
     M_PREFS->setGpsdHost(edGpsdHost->text());
@@ -290,21 +393,14 @@ void PreferencesDialog::savePrefs()
     M_PREFS->setProxyPassword(edProxyPassword->text());
 
     M_PREFS->setLocalServer(cbLocalServer->isChecked());
+    M_PREFS->setNetworkTimeout(sbNetworkTimeout->value()*1000);
 
     M_PREFS->setCacheDir(edCacheDir->text());
     M_PREFS->setCacheSize(sbCacheSize->value());
 
     M_PREFS->setCustomStyle(CustomStylesDir->text());
-    QString NewStyle;
-    NewStyle = cbStyles->itemData(cbStyles->currentIndex()).toString();
 
     bool PainterToInvalidate = false;
-    if (NewStyle != M_PREFS->getDefaultStyle())
-    {
-        M_PREFS->setDefaultStyle(NewStyle);
-        M_STYLE->loadPainters(M_PREFS->getDefaultStyle());
-        PainterToInvalidate = true;
-    }
     if (cbDisableStyleForTracks->isChecked() != M_PREFS->getDisableStyleForTracks()) {
         M_PREFS->setDisableStyleForTracks(cbDisableStyleForTracks->isChecked());
         PainterToInvalidate = true;
@@ -343,12 +439,21 @@ void PreferencesDialog::savePrefs()
 
     M_PREFS->setBgColor(BgColor);
     M_PREFS->setBackgroundOverwriteStyle(cbBackgroundOverwriteStyle->isChecked());
-    M_PREFS->setFocusColor(FocusColor,FocusWidth->value());
-    M_PREFS->setHoverColor(HoverColor,HoverWidth->value());
-    M_PREFS->setHighlightColor(HighlightColor,HighlightWidth->value());
-    M_PREFS->setRelationsColor(RelationsColor,RelationsWidth->value());
-    M_PREFS->setGpxTrackColor(GpxTrackColor,GpxTrackWidth->value());
+    M_PREFS->setFocusColor(FocusColor);
+    M_PREFS->setFocusWidth(FocusWidth->value());
+    M_PREFS->setHoverColor(HoverColor);
+    M_PREFS->setHoverWidth(HoverWidth->value());
+    M_PREFS->setHighlightColor(HighlightColor);
+    M_PREFS->setHighlightWidth(HighlightWidth->value());
+    M_PREFS->setDirtyColor(DirtyColor);
+    M_PREFS->setDirtyWidth(DirtyWidth->value());
+    M_PREFS->setRelationsColor(RelationsColor);
+    M_PREFS->setRelationsWidth(RelationsWidth->value());
+    M_PREFS->setGpxTrackColor(GpxTrackColor);
+    M_PREFS->setGpxTrackWidth(GpxTrackWidth->value());
 
+    M_PREFS->setHasAutoLoadDocument(cbAutoLoadDoc->isChecked());
+    M_PREFS->setAutoLoadDocumentFilename((edAutoLoadDoc->text()));
     M_PREFS->setAutoSaveDoc(cbAutoSaveDoc->isChecked());
     M_PREFS->setAutoExtractTracks(cbAutoExtractTracks->isChecked());
     M_PREFS->setReadonlyTracksDefault(cbReadonlyTracksDefault->isChecked());
@@ -362,6 +467,7 @@ void PreferencesDialog::savePrefs()
 
     M_PREFS->setMouseSingleButton(cbMouseSingleButton->isChecked());
     M_PREFS->setSeparateMoveMode(cbSeparateMoveMode->isChecked());
+    M_PREFS->setSelectModeCreation(cbSelectModeCreation->isChecked());
     M_PREFS->setUseVirtualNodes(cbVirtualNodes->isChecked());
     M_PREFS->setRelationsSelectableWhenHidden(cbRelationsHiddenSelectable->isChecked());
 
@@ -369,6 +475,7 @@ void PreferencesDialog::savePrefs()
     M_PREFS->setMerkaartorStyleString(comboCustomStyle->currentText());
 
     M_PREFS->setUseShapefileForBackground(cbUseShapefileForBackground->isChecked());
+    M_PREFS->setAutoSourceTag(cbAutoSourceTag->isChecked());
 
     M_PREFS->save(OsmDataChanged);
 }
@@ -441,6 +548,19 @@ void PreferencesDialog::on_btHighlightColor_clicked()
     }
 }
 
+void PreferencesDialog::on_btDirtyColor_clicked()
+{
+    QColor rgb = QColorDialog::getColor(DirtyColor, this
+#if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
+                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
+#endif
+                                       );
+    if (rgb.isValid()) {
+        DirtyColor = rgb;
+        makeBoundaryIcon(btDirtyColor, DirtyColor);
+    }
+}
+
 void PreferencesDialog::on_btRelationsColor_clicked()
 {
     QColor rgb = QColorDialog::getColor(RelationsColor, this
@@ -543,6 +663,14 @@ void PreferencesDialog::on_btBrowse_clicked()
     }
 }
 
+void PreferencesDialog::on_btAutoloadBrowse_clicked()
+{
+    QString s = QFileDialog::getOpenFileName(this,tr("Select template document"), "", tr("Merkaartor document (*.mdc)"));
+    if (!s.isNull()) {
+        edAutoLoadDoc->setText(s);
+    }
+}
+
 void PreferencesDialog::on_btGpsLogDirBrowse_clicked()
 {
     QString s = QFileDialog::getExistingDirectory(this,tr("Select Log directory"));
@@ -558,7 +686,7 @@ void PreferencesDialog::changeEvent(QEvent * event)
         retranslateUi(this);
         Language->clear();
         initLanguages(Language);
-        QString CurrentLanguage(getDefaultLanguage());
+        QString CurrentLanguage(getDefaultLanguage(false));
         int l;
         for (l = 0; l < Language->count(); ++l)
             if (CurrentLanguage == Language->itemData(l))
