@@ -9,6 +9,7 @@
 // Copyright: See COPYING file that comes with this distribution
 //
 //
+#include "Global.h"
 
 #include "ProjectionsList.h"
 #include "MerkaartorPreferences.h"
@@ -113,6 +114,7 @@ ProjectionItem ProjectionsList::getProjection(QString name) const
                 return it.value();
         }
     }
+#ifndef USE_PROJ
     QString theProj;
     if (name.startsWith("epsg:", Qt::CaseInsensitive)) {
         theProj = searchEPSG(name, ":/proj/epsg");
@@ -122,6 +124,7 @@ ProjectionItem ProjectionsList::getProjection(QString name) const
         if (!theProj.isEmpty())
             return ProjectionItem(name, theProj);
     }
+#endif
     return ProjectionItem(name, QString("+init=%1").arg(name));
 }
 
