@@ -16,6 +16,7 @@
 #include <QUrl>
 #include <QTextEdit>
 #include <QComboBox>
+#include <QNetworkReply>
 
 // from wikipedia
 #define EQUATORIALRADIUS 6378137.0
@@ -25,7 +26,7 @@
 #define EQUATORIALMETERPERDEGREE    222638.981555556
 
 WMSPreferencesDialog::WMSPreferencesDialog(QWidget* parent)
-    : QDialog(parent)
+    : QDialog(parent), curReply(0)
 {
     setupUi(this);
 
@@ -34,18 +35,38 @@ WMSPreferencesDialog::WMSPreferencesDialog(QWidget* parent)
     loadPrefs();
 
     edWmsLayers->setVisible(false);
-    frWmsSettings->setVisible(false);
     lblWMSC->setVisible(false);
+    frWmsSettings->setVisible(true);
     isTiled = 0;
     frTileIt->setEnabled(false);
+    edWmsUrl->setValidator(&wmsValid);
 
     connect(tvWmsLayers, SIGNAL(itemChanged(QTreeWidgetItem *, int)), this, SLOT(on_tvWmsLayers_itemChanged(QTreeWidgetItem *, int)));
+    connect (&m_networkManager, SIGNAL(finished(QNetworkReply *)), this, SLOT(httpRequestFinished(QNetworkReply *)));
+
 }
 
 WMSPreferencesDialog::~WMSPreferencesDialog()
 {
 }
 
+void WMSPreferencesDialog::updateUrl()
+{
+    QUrl theUrl(edWmsUrl->text());
+    if (!theUrl.hasQueryItem("VERSION"))
+        theUrl.addQueryItem("VERSION", "1.1.1");
+    if (!theUrl.hasQueryItem("SERVICE"))
+        theUrl.addQueryItem("SERVICE", "WMS");
+
+    theUrl.removeQueryItem("TRANSPARENT"); theUrl.addQueryItem("TRANSPARENT", "TRUE");
+    theUrl.removeQueryItem("LAYERS"); theUrl.addQueryItem("LAYERS", edWmsLayers->text());
+    theUrl.removeQueryItem("SRS"); theUrl.addQueryItem("SRS", cbWmsProj->currentText());
+    theUrl.removeQueryItem("STYLES"); theUrl.addQueryItem("STYLES", cbWmsStyle->currentText());
+    theUrl.removeQueryItem("FORMAT"); theUrl.addQueryItem("FORMAT", cbWmsImgFormat->currentText());
+
+    edWmsUrl->setText(theUrl.toString());
+}
+
 void WMSPreferencesDialog::addServer(const WmsServer & srv)
 {
     theWmsServers.push_back(srv);
@@ -60,7 +81,7 @@ void WMSPreferencesDialog::generateWmscLayer()
 {
     selWmscLayer.LayerName = edWmsLayers->text();
     selWmscLayer.Projection = cbWmsProj->currentText();
-    selWmscLayer.Styles = edWmsStyles->text();
+    selWmscLayer.Styles = cbWmsStyle->currentText();
     selWmscLayer.ImgFormat = cbWmsImgFormat->currentText();
     selWmscLayer.TileHeight = 256;
     selWmscLayer.TileWidth = 256;
@@ -80,6 +101,28 @@ void WMSPreferencesDialog::generateWmscLayer()
     }
 }
 
+void WMSPreferencesDialog::on_edWmsUrl_textEdited(const QString &newText)
+{
+    QUrl u(newText);
+
+    if (u.hasEncodedQueryItem("LAYERS"))
+        edWmsLayers->setText(QUrl::fromPercentEncoding(u.queryItemValue("LAYERS").toLatin1()));
+    if (u.hasEncodedQueryItem("SRS"))
+        cbWmsProj->setEditText(QUrl::fromPercentEncoding(u.queryItemValue("SRS").toLatin1()));
+    if (u.hasEncodedQueryItem("STYLES"))
+        cbWmsStyle->setEditText(QUrl::fromPercentEncoding(u.queryItemValue("STYLES").toLatin1()));
+    if (u.hasEncodedQueryItem("FORMAT"))
+        cbWmsImgFormat->setEditText(QUrl::fromPercentEncoding(u.queryItemValue("FORMAT").toLatin1()));
+}
+
+void WMSPreferencesDialog::on_edWmsUrl_editingFinished()
+{
+    QUrl u(edWmsUrl->text());
+    if (u.isValid() && !u.host().isEmpty() && !u.path().isEmpty()) {
+        showCapabilities();
+    }
+}
+
 void WMSPreferencesDialog::on_btApplyWmsServer_clicked(void)
 {
     int idx = lvWmsServers->currentItem()->data(Qt::UserRole).toInt();
@@ -96,7 +139,7 @@ void WMSPreferencesDialog::on_btApplyWmsServer_clicked(void)
     WS.WmsPath = theUrl.toString(QUrl::RemoveScheme | QUrl::RemoveAuthority);
     WS.WmsLayers = edWmsLayers->text();
     WS.WmsProjections = cbWmsProj->currentText();
-    WS.WmsStyles = edWmsStyles->text();
+    WS.WmsStyles = cbWmsStyle->currentText();
     WS.WmsImgFormat = cbWmsImgFormat->currentText();
     if (cbTileIt->isChecked()) {
         isTiled = 2;
@@ -109,6 +152,8 @@ void WMSPreferencesDialog::on_btApplyWmsServer_clicked(void)
         WS.WmsStyles = selWmscLayer.Styles;
     }
     WS.WmsIsTiled = isTiled;
+    WS.WmsSourceTag = edSourceTag->text();
+    WS.WmsLicenseUrl = edLicenseUrl->text();
 
     lvWmsServers->currentItem()->setText(WS.WmsName);
     selectedServer = WS.WmsName;
@@ -126,13 +171,19 @@ void WMSPreferencesDialog::on_btAddWmsServer_clicked(void)
     }
     if (isTiled)
         addServer(WmsServer(edWmsName->text(), theAdress, theUrl.toString(QUrl::RemoveScheme | QUrl::RemoveAuthority),
-            edWmsLayers->text(), selWmscLayer.Projection, selWmscLayer.Styles, selWmscLayer.ImgFormat,
-            isTiled, selWmscLayer));
+            edWmsLayers->text()
+            , edSourceTag->text()
+            , edLicenseUrl->text()
+            , selWmscLayer.Projection, selWmscLayer.Styles, selWmscLayer.ImgFormat
+            , isTiled, selWmscLayer));
     else
         addServer(WmsServer(edWmsName->text(), theAdress, theUrl.toString(QUrl::RemoveScheme | QUrl::RemoveAuthority),
-            edWmsLayers->text(), cbWmsProj->currentText(), edWmsStyles->text(), cbWmsImgFormat->currentText()));
+            edWmsLayers->text()
+            , edSourceTag->text()
+            , edLicenseUrl->text()
+            , cbWmsProj->currentText(), cbWmsStyle->currentText(), cbWmsImgFormat->currentText()));
     lvWmsServers->setCurrentRow(lvWmsServers->count() - 1);
-    on_lvWmsServers_itemSelectionChanged();
+//    on_lvWmsServers_itemSelectionChanged();
 }
 
 void WMSPreferencesDialog::on_btDelWmsServer_clicked(void)
@@ -148,11 +199,11 @@ void WMSPreferencesDialog::on_btDelWmsServer_clicked(void)
 
 void WMSPreferencesDialog::on_lvWmsServers_itemSelectionChanged()
 {
-    frWmsSettings->setVisible(false);
     frTileIt->setEnabled(false);
     cbTileIt->setChecked(false);
     sbZoomLevels->setValue(0);
     lblWMSC->setVisible(false);
+    frWmsSettings->setVisible(true);
     isTiled = 0;
     selWmscLayer = WmscLayer();
 
@@ -167,9 +218,10 @@ void WMSPreferencesDialog::on_lvWmsServers_itemSelectionChanged()
     edWmsUrl->setText("http://" + WS.WmsAdress + WS.WmsPath);
     edWmsLayers->setText(WS.WmsLayers);
     cbWmsProj->setEditText(WS.WmsProjections);
-    on_cbWmsProj_currentIndexChanged(WS.WmsProjections);
-    edWmsStyles->setText(WS.WmsStyles);
+    cbWmsStyle->setEditText(WS.WmsStyles);
     cbWmsImgFormat->setEditText(WS.WmsImgFormat);
+    edSourceTag->setText(WS.WmsSourceTag);
+    edLicenseUrl->setText(WS.WmsLicenseUrl);
     isTiled = WS.WmsIsTiled;
     if (isTiled > 0)
         selWmscLayer = WS.WmsCLayer;
@@ -178,17 +230,20 @@ void WMSPreferencesDialog::on_lvWmsServers_itemSelectionChanged()
         cbTileIt->setChecked(true);
         sbZoomLevels->setValue(selWmscLayer.Resolutions.size());
     }
+    on_cbWmsProj_activated(WS.WmsProjections);
 
     selectedServer = WS.WmsName;
 
-    QUrl theUrl(edWmsUrl->text());
-    if ((theUrl.host() != "") && (theUrl.path() != "")) {
-        QUrl url(edWmsUrl->text() + "VERSION=1.1.1&SERVICE=WMS&request=GetCapabilities");
-        requestCapabilities(url);
+    if (curReply) {
+        curReply->abort();
+    }
+    QUrl u(edWmsUrl->text());
+    if (u.isValid() && !u.host().isEmpty() && !u.path().isEmpty()) {
+        showCapabilities();
     }
 }
 
-void WMSPreferencesDialog::on_cbWmsProj_currentIndexChanged(const QString &text)
+void WMSPreferencesDialog::on_cbWmsProj_activated(const QString &text)
 {
     frTileIt->setEnabled(false);
     if (
@@ -201,6 +256,37 @@ void WMSPreferencesDialog::on_cbWmsProj_currentIndexChanged(const QString &text)
     {
         frTileIt->setEnabled(true);
     }
+    updateUrl();
+}
+
+void WMSPreferencesDialog::refreshStyles()
+{
+    QStringList layers = edWmsLayers->text().split(',');
+    QStringList theStyles;
+    foreach(QString l, layers) {
+        theStyles << styles[l];
+    }
+    cbWmsStyle->setEditText(theStyles.join(","));
+
+    updateUrl();
+}
+
+void WMSPreferencesDialog::on_cbWmsStyle_activated(int index)
+{
+    QString style = cbWmsStyle->itemData(index).toString();
+    styles[curLayer] = style;
+
+    refreshStyles();
+}
+
+void WMSPreferencesDialog::on_cbWmsStyle_editTextChanged(const QString &/*newText*/)
+{
+    updateUrl();
+}
+
+void WMSPreferencesDialog::on_cbWmsImgFormat_activated(const QString &/*text*/)
+{
+    updateUrl();
 }
 
 QString WMSPreferencesDialog::getSelectedServer()
@@ -230,7 +316,7 @@ void WMSPreferencesDialog::on_buttonBox_clicked(QAbstractButton * button)
 
 void WMSPreferencesDialog::loadPrefs()
 {
-    WmsServerList* L = MerkaartorPreferences::instance()->getWmsServers();
+    WmsServerList* L = M_PREFS->getWmsServers();
     WmsServerListIterator i(*L);
     while (i.hasNext()) {
         i.next();
@@ -240,32 +326,45 @@ void WMSPreferencesDialog::loadPrefs()
 
 void WMSPreferencesDialog::savePrefs()
 {
-    WmsServerList* L = MerkaartorPreferences::instance()->getWmsServers();
+    WmsServerList* L = M_PREFS->getWmsServers();
     L->clear();
     for (int i = 0; i < theWmsServers.size(); ++i) {
         WmsServer S(theWmsServers[i]);
         L->insert(theWmsServers[i].WmsName, S);
     }
-    //MerkaartorPreferences::instance()->setSelectedWmsServer(getSelectedServer());
+    //M_PREFS->setSelectedWmsServer(getSelectedServer());
     M_PREFS->save();
 }
 
-void WMSPreferencesDialog::on_btShowCapabilities_clicked(void)
+void WMSPreferencesDialog::showCapabilities(void)
 {
+    if (curReply)
+        return;
+
     QUrl theUrl(edWmsUrl->text());
     if ((theUrl.host() == "") || (theUrl.path() == "")) {
         QMessageBox::critical(this, tr("Merkaartor: GetCapabilities"), tr("Address and Path cannot be blank."), QMessageBox::Ok);
     }
 
-    theUrl.addQueryItem("VERSION", "1.1.1");
-    theUrl.addQueryItem("SERVICE", "WMS");
-    theUrl.addQueryItem("request", "GetCapabilities");
+    if (!theUrl.hasQueryItem("VERSION"))
+        theUrl.addQueryItem("VERSION", "1.1.1");
+    if (!theUrl.hasQueryItem("SERVICE"))
+        theUrl.addQueryItem("SERVICE", "WMS");
+    theUrl.removeAllQueryItems("LAYERS");
+    theUrl.removeAllQueryItems("SRS");
+    theUrl.removeAllQueryItems("FORMAT");
+    theUrl.removeAllQueryItems("STYLES");
+    theUrl.addQueryItem("REQUEST", "GetCapabilities");
 //    QUrl url(edWmsUrl->text() + "VERSION=1.1.1&SERVICE=WMS&request=GetCapabilities");
+
     requestCapabilities(theUrl);
 }
 
 void WMSPreferencesDialog::requestCapabilities(QUrl url)
 {
+    if (curReply)
+        return;
+
     tvWmsLayers->clear();
 
     QString oldSrs = cbWmsProj->currentText();
@@ -276,104 +375,93 @@ void WMSPreferencesDialog::requestCapabilities(QUrl url)
     cbWmsImgFormat->clear();
     cbWmsImgFormat->setEditText(oldFormat);
 
-    http = new QHttp(this);
-    connect (http, SIGNAL(done(bool)), this, SLOT(httpRequestFinished(bool)));
-    connect(http, SIGNAL(responseHeaderReceived(const QHttpResponseHeader &)),
-        this, SLOT(readResponseHeader(const QHttpResponseHeader &)));
+    m_networkManager.setProxy(M_PREFS->getProxy(url));
+    QNetworkRequest req(url);
+    req.setRawHeader("Host", url.host().toLatin1());
+    req.setRawHeader("User-Agent", USER_AGENT.toLatin1());
 
-    QHttpRequestHeader header("GET", url.path() + "?" + url.encodedQuery());
-    qDebug() << header.toString();
-    const char *userAgent = "Mozilla/9.876 (X11; U; Linux 2.2.12-20 i686, en) Gecko/25250101 Netscape/5.432b1";
-
-    header.setValue("Host", url.host());
-    header.setValue("User-Agent", userAgent);
-
-    http->setHost(url.host(), url.port() == -1 ? 80 : url.port());
-    http->setProxy(M_PREFS->getProxy(url));
-
-    httpGetId = http->request(header);
+    curReply = m_networkManager.get(req);
 }
 
-void WMSPreferencesDialog::readResponseHeader(const QHttpResponseHeader &responseHeader)
+void WMSPreferencesDialog::httpRequestFinished(QNetworkReply * reply)
 {
-    qDebug() << responseHeader.toString();
-    switch (responseHeader.statusCode())
-    {
-        case 200:
-            break;
-
-        case 301:
-        case 302:
-        case 307:
-            http->abort();
-            requestCapabilities(QUrl(responseHeader.value("Location")));
-            break;
-
-        default:
-            http->abort();
-            QMessageBox::information(this, tr("Merkaartor: GetCapabilities"),
-                                  tr("Download failed: %1.")
-                                  .arg(responseHeader.reasonPhrase()));
-    }
-}
-
-void WMSPreferencesDialog::httpRequestFinished(bool error)
-{
-    //if (error) {
-    //	if (http->error() != QHttp::Aborted)
-    //		QMessageBox::critical(this, tr("Merkaartor: GetCapabilities"), tr("Error reading capabilities.\n") + http->errorString(), QMessageBox::Ok);
-    //} else {
-    //	QVBoxLayout *mainLayout = new QVBoxLayout;
-    //	QTextEdit* edit = new QTextEdit();
-    //	edit->setPlainText(QString(http->readAll()));
-    //	mainLayout->addWidget(edit);
-
-    //	QDialog* dlg = new QDialog(this);
-    //	dlg->setLayout(mainLayout);
-    //	dlg->show();
-    //	//delete dlg;
-    //}
-
-    if (error) {
-        if (http->error() != QHttp::Aborted)
-            QMessageBox::critical(this, tr("Merkaartor: GetCapabilities"), tr("Error reading capabilities.\n") + http->errorString(), QMessageBox::Ok);
+    if (reply != curReply)
         return;
+
+    int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+    if (reply->error() != QNetworkReply::NoError) {
+        if (reply->error() != QNetworkReply::OperationCanceledError)
+            QMessageBox::critical(this, tr("Merkaartor: GetCapabilities"), tr("Error reading capabilities.\n") + reply->errorString(), QMessageBox::Ok);
+    } else {
+        switch (statusCode) {
+            case 200:
+                break;
+            case 301:
+            case 302:
+            case 307:
+            // Redirected
+                requestCapabilities(reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl());
+                curReply->deleteLater();
+                curReply = NULL;
+                return;
+            default:
+                QMessageBox::information(this, tr("Merkaartor: GetCapabilities"),
+                                  tr("Download failed: %1.")
+                                  .arg(statusCode));
+                curReply->deleteLater();
+                curReply = NULL;
+                return;
+        }
     }
 
-    frWmsSettings->setVisible(true);
     lblWMSC->setVisible(false);
+    frWmsSettings->setVisible(true);
     isTiled = 0;
 
     QDomDocument theXmlDoc;
-    theXmlDoc.setContent(http->readAll());
+    QByteArray buf;
+    buf = reply->readAll();
+//    QString src(buf);
+//    qDebug() << src;
+    theXmlDoc.setContent(buf);
+    curReply->deleteLater();
+    curReply = NULL;
+
 
     QDomElement docElem = theXmlDoc.documentElement();
+
+    if (edWmsName->text().isEmpty()) {
+        QDomElement svcElem = docElem.firstChildElement("Service");
+        if (!svcElem.isNull()) {
+            QDomNodeList aNodeList = svcElem.elementsByTagName("Title");
+            if (aNodeList.size())
+                edWmsName->setText(aNodeList.item(0).firstChild().nodeValue());
+        }
+    }
+
     QDomElement capElem = docElem.firstChildElement("Capability");
     if (capElem.isNull()) {
         // No "Capability"
         return;
     }
 
-    QStringList srsList;
+    srsList.clear();
+    styleList.clear();
     QDomElement layElem = capElem.firstChildElement("Layer");
     while(!layElem.isNull()) {
         QTreeWidgetItem* it = parseLayers(layElem, NULL);
         tvWmsLayers->addTopLevelItem(it);
         tvWmsLayers->expandItem(it);
-        QDomNodeList aNodeList = layElem.elementsByTagName("SRS");
-        for (int i=0; i<aNodeList.size(); ++i) {
-            if (!srsList.contains(aNodeList.item(i).firstChild().toText().nodeValue()))
-                srsList.append(aNodeList.item(i).firstChild().toText().nodeValue());
-        }
 
         layElem = layElem.nextSiblingElement("Layer");
     }
-    QString oldSrs = cbWmsProj->currentText();
-    cbWmsProj->clear();
-    srsList.sort();
-    cbWmsProj->addItems(srsList);
-    cbWmsProj->setEditText(oldSrs);
-    on_cbWmsProj_currentIndexChanged(oldSrs);
+
+    QString oldStyle = cbWmsStyle->currentText();
+//    cbWmsProj->clear();
+//    srsList.sort();
+//    cbWmsProj->addItems(srsList);
+    cbWmsStyle->setEditText(oldStyle);
+//    on_cbWmsProj_currentIndexChanged(oldSrs);
 
     QDomElement reqElem = capElem.firstChildElement("Request");
     QDomElement GetMapElem = reqElem.firstChildElement("GetMap");
@@ -383,12 +471,13 @@ void WMSPreferencesDialog::httpRequestFinished(bool error)
     QString oldFormat = cbWmsImgFormat->currentText();
     cbWmsImgFormat->clear();
     for (int i=0; i<formatNodeList.size(); ++i) {
-        if (!formatList.contains(formatNodeList.item(i).firstChild().toText().nodeValue()))
-            formatList.append(formatNodeList.item(i).firstChild().toText().nodeValue());
+        if (!formatList.contains(formatNodeList.item(i).firstChild().nodeValue()))
+            formatList.append(formatNodeList.item(i).firstChild().nodeValue());
     }
     formatList.sort();
     cbWmsImgFormat->addItems(formatList);
     cbWmsImgFormat->setEditText(oldFormat);
+    on_cbWmsImgFormat_activated(oldFormat);
 
     QDomElement vendorElem = capElem.firstChildElement("VendorSpecificCapabilities");
     if (!vendorElem.isNull()) {
@@ -412,20 +501,20 @@ void WMSPreferencesDialog::parseVendorSpecific(QDomElement &vendorElem)
 
 void WMSPreferencesDialog::parseTileSet(QDomElement &tilesetElem, WmscLayer &aLayer)
 {
-    frWmsSettings->setVisible(false);
     lblWMSC->setVisible(true);
+    frWmsSettings->setVisible(false);
     isTiled = 1;
 
     QDomElement elem = tilesetElem.firstChildElement();
     while(!elem.isNull()) {
         if (elem.tagName() == "Layers") {
-            aLayer.LayerName = elem.firstChild().toText().nodeValue();
+            aLayer.LayerName = elem.firstChild().nodeValue();
         } else if (elem.tagName() == "SRS") {
-            aLayer.Projection = elem.firstChild().toText().nodeValue();
+            aLayer.Projection = elem.firstChild().nodeValue();
         } else if (elem.tagName() == "Format") {
-            aLayer.ImgFormat = elem.firstChild().toText().nodeValue();
+            aLayer.ImgFormat = elem.firstChild().nodeValue();
         } else if (elem.tagName() == "Styles") {
-            aLayer.Styles = elem.firstChild().toText().nodeValue();
+            aLayer.Styles = elem.firstChild().nodeValue();
         } else if (elem.tagName() == "BoundingBox") {
             QPointF tl, br;
             tl.setX(elem.attribute("minx").toDouble());
@@ -435,20 +524,21 @@ void WMSPreferencesDialog::parseTileSet(QDomElement &tilesetElem, WmscLayer &aLa
             aLayer.BoundingBox = QRectF(tl, br);
         } else if (elem.tagName() == "Resolutions") {
             QStringList resL;
-            resL = elem.firstChild().toText().nodeValue().split(" ", QString::SkipEmptyParts);
+            resL = elem.firstChild().nodeValue().split(" ", QString::SkipEmptyParts);
             foreach(QString res, resL)
                 aLayer.Resolutions << res.toDouble();
         } else if (elem.tagName() == "Width") {
-            aLayer.TileWidth = elem.firstChild().toText().nodeValue().toInt();
+            aLayer.TileWidth = elem.firstChild().nodeValue().toInt();
         } else if (elem.tagName() == "Height") {
-            aLayer.TileHeight = elem.firstChild().toText().nodeValue().toInt();
+            aLayer.TileHeight = elem.firstChild().nodeValue().toInt();
         }
 
         elem = elem.nextSiblingElement();
     }
 }
 
-QTreeWidgetItem * WMSPreferencesDialog::parseLayers(QDomElement& aLayerElem, QTreeWidgetItem* aLayerItem)
+QTreeWidgetItem * WMSPreferencesDialog::parseLayers(const QDomElement& aLayerElem,
+                              QTreeWidgetItem* aLayerItem)
 {
     if (aLayerElem.tagName() != "Layer")
         return NULL;
@@ -458,24 +548,62 @@ QTreeWidgetItem * WMSPreferencesDialog::parseLayers(QDomElement& aLayerElem, QTr
 
     QTreeWidgetItem *newItem = new QTreeWidgetItem;
     newItem->setFlags(Qt::NoItemFlags |Qt::ItemIsEnabled);
-    if (!title.isNull())
-        newItem->setText(0, title.firstChild().toText().nodeValue());
-    else
-        if (!name.isNull())
-            newItem->setText(0,name.firstChild().toText().nodeValue());
+    if (!name.isNull())
+        newItem->setText(0,name.firstChild().nodeValue());
+    else {
+        if (!title.isNull())
+            newItem->setText(0,title.firstChild().nodeValue());
+        else
+            newItem->setText(0,tr("Unnamed"));
+    }
 
+    if (!title.isNull())
+        newItem->setToolTip(0, title.firstChild().nodeValue());
+
+    QString theName;
     if (!name.isNull()) {
-        QString theName = name.firstChild().toText().nodeValue();
+        theName = name.firstChild().nodeValue();
         newItem->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsSelectable | Qt::ItemIsEnabled);
         newItem->setData(0, Qt::UserRole, theName);
-        if (edWmsLayers->text().contains(theName))
-            newItem->setCheckState(0, Qt::Checked);
-        else
-            newItem->setCheckState(0, Qt::Unchecked);
+        foreach (QString s, edWmsLayers->text().split(',')) {
+            if (theName == s)
+                newItem->setCheckState(0, Qt::Checked);
+            else
+                newItem->setCheckState(0, Qt::Unchecked);
+        }
+    } else {
+//        newItem->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsSelectable | Qt::ItemIsEnabled);
     }
+
     if (aLayerItem)
         aLayerItem->addChild(newItem);
 
+    QStringList upSrsList;
+    if (aLayerItem)
+        upSrsList = srsList.value(aLayerItem->text(0));
+    QDomElement srsNode = aLayerElem.firstChildElement("SRS");
+    while (!srsNode.isNull()) {
+        if (!upSrsList.contains(srsNode.firstChild().nodeValue()))
+            upSrsList.append(srsNode.firstChild().nodeValue());
+        srsNode = srsNode.nextSiblingElement("SRS");
+    }
+    srsList[newItem->text(0)] = upSrsList;
+
+    QList <QPair<QString, QString> > upStyleList;
+    if (aLayerItem)
+        upStyleList = styleList.value(aLayerItem->text(0));
+    QDomElement styleNode = aLayerElem.firstChildElement("Style");
+    while (!styleNode.isNull()) {
+        QDomElement title = styleNode.firstChildElement("Title");
+        QDomElement name = styleNode.firstChildElement("Name");
+
+        if (!name.isNull() && !upStyleList.contains(qMakePair(name.firstChild().nodeValue(), title.firstChild().nodeValue())))
+            upStyleList.append(qMakePair(name.firstChild().nodeValue(), title.firstChild().nodeValue()));
+
+        styleNode = styleNode.nextSiblingElement("Style");
+    }
+    styleList[newItem->text(0)] = upStyleList;
+
     QDomElement layElem = aLayerElem.firstChildElement("Layer");
     while(!layElem.isNull()) {
         parseLayers(layElem, newItem);
@@ -513,4 +641,31 @@ void WMSPreferencesDialog::on_tvWmsLayers_itemChanged(QTreeWidgetItem *twi, int)
         ++it;
     }
     edWmsLayers->setText(theLayers.join(","));
+    refreshStyles();
+}
+
+void WMSPreferencesDialog::on_tvWmsLayers_currentItemChanged(QTreeWidgetItem *current, QTreeWidgetItem *previous)
+{
+    if (current)
+        curLayer = current->text(0);
+    else
+        return;
+
+    QStringList theSRS = srsList[curLayer];
+    QString oldSrs = cbWmsProj->currentText();
+    cbWmsProj->clear();
+    theSRS.sort();
+    cbWmsProj->addItems(theSRS);
+    cbWmsProj->setEditText(oldSrs);
+    on_cbWmsProj_activated(oldSrs);
+
+    QList <QPair<QString, QString> > theStyles = styleList[curLayer];
+    QString oldStyle = cbWmsStyle->currentText();
+    cbWmsStyle->clear();
+    QPair<QString, QString> aStyle;
+    foreach(aStyle, theStyles) {
+        cbWmsStyle->addItem(aStyle.second, QVariant(aStyle.first));
+    }
+    cbWmsStyle->setEditText(oldStyle);
+    on_cbWmsProj_activated(oldSrs);
 }
