@@ -18,14 +18,16 @@
 #include "ImageMapLayer.h"
 #include "Utils/LineF.h"
 
+#define TEST_RFLAGS(x) theOptions.options.testFlag(x)
+#define TEST_RENDERER_RFLAGS(x) r->theOptions.options.testFlag(x)
+
 void BackgroundStyleLayer::draw(Way* R)
 {
-    const FeaturePainter* paintsel = R->getEditPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = R->getPainter(r->theView->pixelPerM());
     if (paintsel) {
         paintsel->drawBackground(R,r->thePainter,r->theView);
-        return;
-    }
-    if (/*!globalZoom(r->theProjection) && */!R->hasEditPainter()) //FIXME Untagged roads level of zoom?
+    } else if (!R->hasPainter() && !TEST_RENDERER_RFLAGS(RendererOptions::UnstyledHidden))
+//    if (/*!globalZoom(r->theProjection) && */!R->hasPainter()) //FIXME Untagged roads level of zoom?
     {
         QPen thePen(QColor(0,0,0),1);
 
@@ -50,44 +52,56 @@ void BackgroundStyleLayer::draw(Way* R)
 
 void BackgroundStyleLayer::draw(Relation* R)
 {
-    const FeaturePainter* paintsel = R->getEditPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = R->getPainter(r->theView->pixelPerM());
     if (paintsel)
         paintsel->drawBackground(R,r->thePainter,r->theView);
 }
 
 
-void BackgroundStyleLayer::draw(Node*)
+void BackgroundStyleLayer::draw(Node* N)
 {
+    if ((N->isReadonly() || !N->isSelectable(r->theView)) && (!N->isPOI() && !N->isWaypoint()))
+        return;
+
+    const FeaturePainter* paintsel = N->getPainter(r->theView->pixelPerM());
+    if (paintsel)
+        paintsel->drawBackground(N,r->thePainter,r->theView);
 }
 
 void ForegroundStyleLayer::draw(Way* R)
 {
-    const FeaturePainter* paintsel = R->getEditPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = R->getPainter(r->theView->pixelPerM());
     if (paintsel)
         paintsel->drawForeground(R,r->thePainter,r->theView);
 }
 
 void ForegroundStyleLayer::draw(Relation* R)
 {
-    const FeaturePainter* paintsel = R->getEditPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = R->getPainter(r->theView->pixelPerM());
     if (paintsel)
         paintsel->drawForeground(R,r->thePainter,r->theView);
 }
 
-void ForegroundStyleLayer::draw(Node*)
+void ForegroundStyleLayer::draw(Node* N)
 {
+    if ((N->isReadonly() || !N->isSelectable(r->theView)) && (!N->isPOI() && !N->isWaypoint()))
+        return;
+
+    const FeaturePainter* paintsel = N->getPainter(r->theView->pixelPerM());
+    if (paintsel)
+        paintsel->drawForeground(N,r->thePainter,r->theView);
 }
 
 void TouchupStyleLayer::draw(Way* R)
 {
-    const FeaturePainter* paintsel = R->getEditPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = R->getPainter(r->theView->pixelPerM());
     if (paintsel)
         paintsel->drawTouchup(R,r->thePainter,r->theView);
-    else {
-        if ( M_PREFS->getDirectionalArrowsVisible() != DirectionalArrows_Never )
+    else if (!R->hasPainter() && !TEST_RENDERER_RFLAGS(RendererOptions::UnstyledHidden)) {
+        if ( r->theOptions.arrowOptions != RendererOptions::ArrowsNever )
         {
             Feature::TrafficDirectionType TT = trafficDirection(R);
-            if ( (TT != Feature::UnknownDirection) || (M_PREFS->getDirectionalArrowsVisible() == DirectionalArrows_Always) )
+            if ( (TT != Feature::UnknownDirection) || (r->theOptions.arrowOptions == RendererOptions::ArrowsAlways) )
             {
                 double theWidth = r->theView->pixelPerM()*R->widthOf()-4;
                 if (theWidth > 8)
@@ -121,7 +135,7 @@ void TouchupStyleLayer::draw(Way* R)
                             }
                             else
                             {
-                                if ( M_PREFS->getDirectionalArrowsVisible() == DirectionalArrows_Always )
+                                if ( r->theOptions.arrowOptions == RendererOptions::ArrowsAlways )
                                 {
                                     r->thePainter->setPen(QPen(QColor(255,0,0), 2));
                                     r->thePainter->drawLine(H-T,H-T+V1);
@@ -142,21 +156,22 @@ void TouchupStyleLayer::draw(Relation* /* R */)
 
 void TouchupStyleLayer::draw(Node* Pt)
 {
-    const FeaturePainter* paintsel = Pt->getEditPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = Pt->getPainter(r->theView->pixelPerM());
     if (paintsel)
         paintsel->drawTouchup(Pt,r->thePainter,r->theView);
-    else if (!Pt->hasEditPainter()) {
-        if (Pt->isSelectable(r->theView))
+    else if (!Pt->hasPainter() && !TEST_RENDERER_RFLAGS(RendererOptions::UnstyledHidden)) {
+        if (! ((Pt->isReadonly() || !Pt->isSelectable(r->theView)) && (!Pt->isPOI() && !Pt->isWaypoint())))
+//        if (!Pt->isReadonly() && Pt->isSelectable(r->theView))
         {
-            QPoint P = r->theView->transform().map(r->theView->projection().project(Pt)).toPoint();
-            double theWidth = r->theView->nodeWidth();
-            if (theWidth >= 1) {
+            QPointF P(r->theView->toView(Pt));
+            double WW = r->theView->nodeWidth();
+            if (WW >= 1) {
                 if (Pt->isWaypoint()) {
-                    QRect R2(P-QPoint(theWidth*4/3/2,theWidth*4/3/2),QSize(theWidth*4/3,theWidth*4/3));
+                    QRect R2(P.x()-(WW+4)/2, P.y()-(WW+4)/2, WW+4, WW+4);
                     r->thePainter->fillRect(R2,QColor(255,0,0,128));
                 }
 
-                QRect R(P-QPoint(theWidth/2,theWidth/2),QSize(theWidth,theWidth));
+                QRect R(P.x()-WW/2, P.y()-WW/2, WW, WW);
                 r->thePainter->fillRect(R,QColor(0,0,0,128));
             }
         }
@@ -165,7 +180,7 @@ void TouchupStyleLayer::draw(Node* Pt)
 
 void LabelStyleLayer::draw(Way* R)
 {
-    const FeaturePainter* paintsel = R->getEditPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = R->getPainter(r->theView->pixelPerM());
     if (paintsel)
         paintsel->drawLabel(R,r->thePainter,r->theView);
 }
@@ -176,7 +191,7 @@ void LabelStyleLayer::draw(Relation* /* R */)
 
 void LabelStyleLayer::draw(Node* Pt)
 {
-    const FeaturePainter* paintsel = Pt->getEditPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = Pt->getPainter(r->theView->pixelPerM());
     if (paintsel)
         paintsel->drawLabel(Pt,r->thePainter,r->theView);
 }
@@ -191,119 +206,28 @@ MapRenderer::MapRenderer()
     lbllayer = LabelStyleLayer(this);
 }
 
+#if 0
 void MapRenderer::render(
         QPainter* P,
         QMap<RenderPriority, QSet <Feature*> > theFeatures,
+        const RendererOptions& options,
         MapView* aView
 )
 {
+    #ifndef NDEBUG
+        QTime Start(QTime::currentTime());
+    #endif
+
     theView = aView;
+    theOptions = options;
 
     QMap<RenderPriority, QSet<Feature*> >::const_iterator itm;
-    QMap<RenderPriority, QSet<Feature*> >::const_iterator itmCur;
     QSet<Feature*>::const_iterator it;
 
-    bool bgLayerVisible = M_PREFS->getBackgroundVisible();
-    bool fgLayerVisible = M_PREFS->getForegroundVisible();
-    bool tchpLayerVisible = M_PREFS->getTouchupVisible();
-    bool lblLayerVisible = M_PREFS->getNamesVisible();
-
-#if 1
-    P->setRenderHint(QPainter::Antialiasing);
-    thePainter = P;
-
-    itm = theFeatures.constBegin();
-    while (itm != theFeatures.constEnd())
-    {
-        int curLayer = (itm.key()).layer();
-        itmCur = itm;
-        while (itm != theFeatures.constEnd() && (itm.key()).layer() == curLayer)
-        {
-            if (bgLayerVisible)
-            {
-                for (it = itm.value().constBegin(); it != itm.value().constEnd(); ++it) {
-                    P->save();
-                    P->setOpacity((*it)->layer()->getAlpha());
-                    if (Way * R = CAST_WAY(*it)) {
-                        for (int i=0; i<R->sizeParents(); ++i)
-                            if (!R->getParent(i)->isDeleted() && R->getParent(i)->getEditPainter(theView->pixelPerM()))
-                                continue;
-                        bglayer.draw(R);
-                    } else if (Node * Pt = CAST_NODE(*it))
-                        bglayer.draw(Pt);
-                    else if (Relation * RR = CAST_RELATION(*it))
-                        bglayer.draw(RR);
-                    P->restore();
-                }
-            }
-            ++itm;
-        }
-        itm = itmCur;
-        while (itm != theFeatures.constEnd() && (itm.key()).layer() == curLayer)
-        {
-            if (fgLayerVisible)
-            {
-                for (it = itm.value().constBegin(); it != itm.value().constEnd(); ++it) {
-                    P->save();
-                    P->setOpacity((*it)->layer()->getAlpha());
-                    if (Way * R = CAST_WAY(*it)) {
-                        for (int i=0; i<R->sizeParents(); ++i)
-                            if (!R->getParent(i)->isDeleted() && R->getParent(i)->getEditPainter(theView->pixelPerM()))
-                                continue;
-                        fglayer.draw(R);
-                    } else if (Node * Pt = CAST_NODE(*it))
-                        fglayer.draw(Pt);
-                    else if (Relation * RR = CAST_RELATION(*it))
-                        fglayer.draw(RR);
-                    P->restore();
-                }
-            }
-            ++itm;
-        }
-    }
-
-    if (tchpLayerVisible)
-    {
-        P->save();
-
-        for (itm = theFeatures.constBegin() ;itm != theFeatures.constEnd(); ++itm)
-            for (it = itm.value().constBegin(); it != itm.value().constEnd(); ++it) {
-                P->setOpacity((*it)->layer()->getAlpha());
-                if (Way * R = dynamic_cast < Way * >(*it))
-                    tchuplayer.draw(R);
-                else if (Node * Pt = dynamic_cast < Node * >(*it))
-                    tchuplayer.draw(Pt);
-                else if (Relation * RR = dynamic_cast < Relation * >(*it))
-                    tchuplayer.draw(RR);
-            }
-        P->restore();
-    }
-    if (lblLayerVisible) {
-        P->save();
-
-        for (itm = theFeatures.constBegin() ;itm != theFeatures.constEnd(); ++itm)
-            for (it = itm.value().constBegin(); it != itm.value().constEnd(); ++it) {
-                P->setOpacity((*it)->layer()->getAlpha());
-                if (Way * R = dynamic_cast < Way * >(*it))
-                    lbllayer.draw(R);
-                else if (Node * Pt = dynamic_cast < Node * >(*it))
-                    lbllayer.draw(Pt);
-                else if (Relation * RR = dynamic_cast < Relation * >(*it))
-                    lbllayer.draw(RR);
-            }
-        P->restore();
-    }
-
-    for (itm = theFeatures.constBegin() ;itm != theFeatures.constEnd(); ++itm)
-    {
-        for (it = itm.value().constBegin() ;it != itm.value().constEnd(); ++it)
-        {
-            P->setOpacity((*it)->layer()->getAlpha());
-            (*it)->draw(*P, aView);
-        }
-    }
-
-#else
+    bool bgLayerVisible = TEST_RFLAGS(RendererOptions::BackgroundVisible);
+    bool fgLayerVisible = TEST_RFLAGS(RendererOptions::ForegroundVisible);
+    bool tchpLayerVisible = TEST_RFLAGS(RendererOptions::TouchupVisible);
+    bool lblLayerVisible = TEST_RFLAGS(RendererOptions::NamesVisible);
 
     Way * R = NULL;
     Node * Pt = NULL;
@@ -315,9 +239,6 @@ void MapRenderer::render(
     itm = theFeatures.constBegin();
     while (itm != theFeatures.constEnd())
     {
-//#ifndef NDEBUG
-//    QTime Start(QTime::currentTime());
-//#endif
         pix.fill(Qt::transparent);
         thePainter->begin(&pix);
         thePainter->setRenderHint(QPainter::Antialiasing);
@@ -326,7 +247,8 @@ void MapRenderer::render(
         {
             for (it = itm.value().constBegin(); it != itm.value().constEnd(); ++it)
             {
-                thePainter->setOpacity((*it)->layer()->getAlpha());
+                double alpha = (*it)->getAlpha();
+                thePainter->setOpacity(alpha);
 
                 R = NULL;
                 Pt = NULL;
@@ -340,7 +262,7 @@ void MapRenderer::render(
                     // If there is painter at the relation level, don't paint at the way level
                     bool draw = true;
                     for (int i=0; i<R->sizeParents(); ++i) {
-                        if (!R->getParent(i)->isDeleted() && R->getParent(i)->getEditPainter(theView->pixelPerM()))
+                        if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(theView->pixelPerM()))
                             draw = false;
                     }
                     if (!draw)
@@ -409,14 +331,176 @@ void MapRenderer::render(
         }
         thePainter->end();
         P->drawPixmap(0, 0, pix);
-//#ifndef NDEBUG
-//    QTime Stop(QTime::currentTime());
-//    qDebug() << "curLayer: " << curLayer << " " << Start.msecsTo(Stop) << "ms";
-//#endif
+#ifndef NDEBUG
+    QTime Stop(QTime::currentTime());
+    qDebug() << Start.msecsTo(Stop) << "ms";
+#endif
+    }
+}
+#else
+void MapRenderer::render(
+        QPainter* P,
+        const QMap<RenderPriority, QSet <Feature*> >& theFeatures,
+        const RendererOptions& options,
+        MapView* aView
+)
+{
+//    #ifndef NDEBUG
+//        QTime Start(QTime::currentTime());
+//    #endif
+
+    theView = aView;
+    theOptions = options;
+
+    bool bgLayerVisible = TEST_RFLAGS(RendererOptions::BackgroundVisible);
+    bool fgLayerVisible = TEST_RFLAGS(RendererOptions::ForegroundVisible);
+    bool tchpLayerVisible = TEST_RFLAGS(RendererOptions::TouchupVisible);
+    bool lblLayerVisible = TEST_RFLAGS(RendererOptions::NamesVisible);
+
+    QMap<RenderPriority, QSet<Feature*> >::const_iterator itm;
+    QMap<RenderPriority, QSet<Feature*> >::const_iterator itmCur;
+    QSet<Feature*>::const_iterator it;
+
+    P->setRenderHint(QPainter::Antialiasing);
+    thePainter = P;
+
+    itm = theFeatures.constBegin();
+    while (itm != theFeatures.constEnd())
+    {
+        int curLayer = (itm.key()).layer();
+        itmCur = itm;
+        while (itm != theFeatures.constEnd() && (itm.key()).layer() == curLayer)
+        {
+            if (bgLayerVisible)
+            {
+                for (it = itm.value().constBegin(); it != itm.value().constEnd(); ++it) {
+                    double alpha = (*it)->getAlpha();
+                    if ((*it)->isReadonly() && !TEST_RFLAGS(RendererOptions::ForPrinting))
+                        alpha /= 2.0;
+                    if (alpha != 1.) {
+                        P->save();
+                        P->setOpacity(alpha);
+                    }
+
+                    if (CHECK_WAY(*it)) {
+                        Way * R = STATIC_CAST_WAY(*it);
+                        for (int i=0; i<R->sizeParents(); ++i)
+                            if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(theView->pixelPerM()))
+                                continue;
+                        bglayer.draw(R);
+                    } else if (CHECK_NODE(*it))
+                        bglayer.draw(STATIC_CAST_NODE(*it));
+                    else if (CHECK_RELATION(*it))
+                        bglayer.draw(STATIC_CAST_RELATION(*it));
+                    if (alpha != 1.) {
+                        P->restore();
+                    }
+                }
+            }
+            ++itm;
+        }
+        itm = itmCur;
+        while (itm != theFeatures.constEnd() && (itm.key()).layer() == curLayer)
+        {
+            if (fgLayerVisible)
+            {
+                for (it = itm.value().constBegin(); it != itm.value().constEnd(); ++it) {
+                    double alpha = (*it)->getAlpha();
+                    if ((*it)->isReadonly() && !TEST_RFLAGS(RendererOptions::ForPrinting))
+                        alpha /= 2.0;
+                    if (alpha != 1.) {
+                        P->save();
+                        P->setOpacity(alpha);
+                    }
+
+                    if (CHECK_WAY(*it)) {
+                        Way * R = STATIC_CAST_WAY(*it);
+                        for (int i=0; i<R->sizeParents(); ++i)
+                            if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(theView->pixelPerM()))
+                                continue;
+                        fglayer.draw(R);
+                    } else if (CHECK_NODE(*it))
+                        fglayer.draw(STATIC_CAST_NODE(*it));
+                    else if (CHECK_RELATION(*it))
+                        fglayer.draw(STATIC_CAST_RELATION(*it));
+                    if (alpha != 1.) {
+                        P->restore();
+                    }
+                }
+            }
+            ++itm;
+        }
+    }
+    if (tchpLayerVisible)
+    {
+        for (itm = theFeatures.constBegin() ;itm != theFeatures.constEnd(); ++itm) {
+            for (it = itm.value().constBegin(); it != itm.value().constEnd(); ++it) {
+                double alpha = (*it)->getAlpha();
+                if ((*it)->isReadonly() && !TEST_RFLAGS(RendererOptions::ForPrinting))
+                    alpha /= 2.0;
+                if (alpha != 1.) {
+                    P->save();
+                    P->setOpacity(alpha);
+                }
+
+                if (CHECK_WAY(*it)) {
+                    Way * R = STATIC_CAST_WAY(*it);
+                    for (int i=0; i<R->sizeParents(); ++i)
+                        if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(theView->pixelPerM()))
+                            continue;
+                    tchuplayer.draw(R);
+                } else if (CHECK_NODE(*it))
+                    tchuplayer.draw(STATIC_CAST_NODE(*it));
+                else if (CHECK_RELATION(*it))
+                    tchuplayer.draw(STATIC_CAST_RELATION(*it));
+                if (alpha != 1.) {
+                    P->restore();
+                }
+            }
+        }
+    }
+    if (lblLayerVisible)
+    {
+        for (itm = theFeatures.constBegin() ;itm != theFeatures.constEnd(); ++itm) {
+            for (it = itm.value().constBegin(); it != itm.value().constEnd(); ++it) {
+                P->save();
+                double alpha = (*it)->getAlpha();
+                if ((*it)->isReadonly() && !TEST_RFLAGS(RendererOptions::ForPrinting))
+                    alpha /= 2.0;
+                P->setOpacity(alpha);
+
+                if (CHECK_WAY(*it)) {
+                    Way * R = STATIC_CAST_WAY(*it);
+                    for (int i=0; i<R->sizeParents(); ++i)
+                        if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(theView->pixelPerM()))
+                            continue;
+                    lbllayer.draw(R);
+                } else if (CHECK_NODE(*it))
+                    lbllayer.draw(STATIC_CAST_NODE(*it));
+                else if (CHECK_RELATION(*it))
+                    lbllayer.draw(STATIC_CAST_RELATION(*it));
+                P->restore();
+            }
+        }
     }
 
+    for (itm = theFeatures.constBegin() ;itm != theFeatures.constEnd(); ++itm)
+    {
+        for (it = itm.value().constBegin() ;it != itm.value().constEnd(); ++it)
+        {
+            double alpha = (*it)->getAlpha();
+            if ((*it)->isReadonly() && !TEST_RFLAGS(RendererOptions::ForPrinting))
+                alpha /= 2.0;
+            if (alpha != 1.)
+                P->setOpacity(alpha);
 
-#endif
+            (*it)->draw(*P, aView);
+        }
+    }
+//    #ifndef NDEBUG
+//        QTime Stop(QTime::currentTime());
+//        qDebug() << Start.msecsTo(Stop) << "ms";
+//    #endif
 }
-
+#endif
 
