@@ -1,3 +1,5 @@
+#include "Global.h"
+
 #include "DirtyList.h"
 #include "Command.h"
 #include "Maps/Coord.h"
@@ -13,7 +15,6 @@
 #include <QtGui/QListWidget>
 #include <QtGui/QMessageBox>
 #include <QtGui/QProgressDialog>
-#include <QtNetwork/QHttp>
 #include <QtNetwork/QTcpSocket>
 #include <QInputDialog>
 
@@ -22,12 +23,9 @@
 int glbAdded, glbUpdated, glbDeleted;
 QString glbChangeSetComment;
 
-QString stripToOSMId(const QString& id)
+QString stripToOSMId(const IFeature::FId& id)
 {
-    int f = id.lastIndexOf("_");
-    if (f>0)
-        return id.right(id.length()-(f+1));
-    return id;
+    return QString::number(id.numId);
 }
 
 QString userName(const Feature* F)
@@ -297,106 +295,100 @@ bool DirtyListDescriber::showChanges(QWidget* aParent)
 
     runVisit();
 
-    if (M_PREFS->apiVersionNum() < 0.6) {
-        Ui.lblChangesetComment->setVisible(false);
-        Ui.edChangesetComment->setVisible(false);
-    } else {
-        CoordBox bbox = theDocument->getDirtyOrOriginLayer()->boundingBox();
-        QString bboxComment = QString("BBOX:%1,%2,%3,%4")
+    CoordBox bbox = theDocument->getDirtyOrOriginLayer()->boundingBox();
+    QString bboxComment = QString("BBOX:%1,%2,%3,%4")
             .arg(QString::number(coordToAng(bbox.bottomLeft().lon()), 'f', 2))
             .arg(QString::number(coordToAng(bbox.bottomLeft().lat()), 'f', 2))
             .arg(QString::number(coordToAng(bbox.topRight().lon()), 'f', 2))
             .arg(QString::number(coordToAng(bbox.topRight().lat()), 'f', 2));
 
-        QString statComment = QString("ADD:%1 UPD:%2 DEL:%3").arg(glbAdded).arg(glbUpdated).arg(glbDeleted);
+    QString statComment = QString("ADD:%1 UPD:%2 DEL:%3").arg(glbAdded).arg(glbUpdated).arg(glbDeleted);
 
-        glbChangeSetComment = bboxComment + " " + statComment;
-        Ui.edChangesetComment->setText(glbChangeSetComment);
-    }
+    glbChangeSetComment = bboxComment + " " + statComment;
+    Ui.edChangesetComment->setText(glbChangeSetComment);
+    Ui.edChangesetComment->selectAll();
 
     bool ok = (dlg->exec() == QDialog::Accepted);
 
-    if (M_PREFS->apiVersionNum() > 0.5) {
-        if (!Ui.edChangesetComment->text().isEmpty())
-            glbChangeSetComment = Ui.edChangesetComment->text();
-        else
-            glbChangeSetComment = "-";
-    }
+    if (!Ui.edChangesetComment->text().isEmpty())
+        glbChangeSetComment = Ui.edChangesetComment->text();
+    else
+        glbChangeSetComment = "-";
 
     Task = Ui.ChangesList->count();
-    SAFE_DELETE(dlg);
+    SAFE_DELETE(dlg)
     return ok;
 }
 
 
 bool DirtyListDescriber::addRoad(Way* R)
 {
-    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","ADD road %1").arg(R->id()) + userName(R), theListWidget);
-    it->setData(Qt::UserRole, R->id());
+    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","ADD road %1").arg(R->id().numId) + userName(R), theListWidget);
+    it->setData(Qt::UserRole, QVariant::fromValue(R->id()));
     ++glbAdded;
     return false;
 }
 
 bool DirtyListDescriber::addPoint(Node* Pt)
 {
-    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","ADD trackpoint %1").arg(Pt->id()) + userName(Pt), theListWidget);
-    it->setData(Qt::UserRole, Pt->id());
+    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","ADD trackpoint %1").arg(Pt->id().numId) + userName(Pt), theListWidget);
+    it->setData(Qt::UserRole, QVariant::fromValue(Pt->id()));
     ++glbAdded;
     return false;
 }
 
 bool DirtyListDescriber::addRelation(Relation* R)
 {
-    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","ADD relation %1").arg(R->id()) + userName(R), theListWidget);
-    it->setData(Qt::UserRole, R->id());
+    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","ADD relation %1").arg(R->id().numId) + userName(R), theListWidget);
+    it->setData(Qt::UserRole, QVariant::fromValue(R->id()));
     ++glbAdded;
     return false;
 }
 
 bool DirtyListDescriber::updatePoint(Node* Pt)
 {
-    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","UPDATE trackpoint %1").arg(Pt->id()) + userName(Pt), theListWidget);
-    it->setData(Qt::UserRole, Pt->id());
+    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","UPDATE trackpoint %1").arg(Pt->id().numId) + userName(Pt), theListWidget);
+    it->setData(Qt::UserRole, QVariant::fromValue(Pt->id()));
     ++glbUpdated;
     return false;
 }
 
 bool DirtyListDescriber::updateRelation(Relation* R)
 {
-    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","UPDATE relation %1").arg(R->id()) + userName(R), theListWidget);
-    it->setData(Qt::UserRole, R->id());
+    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","UPDATE relation %1").arg(R->id().numId) + userName(R), theListWidget);
+    it->setData(Qt::UserRole, QVariant::fromValue(R->id()));
     ++glbUpdated;
     return false;
 }
 
 bool DirtyListDescriber::updateRoad(Way* R)
 {
-    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","UPDATE road %1").arg(R->id()) + userName(R), theListWidget);
-    it->setData(Qt::UserRole, R->id());
+    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","UPDATE road %1").arg(R->id().numId) + userName(R), theListWidget);
+    it->setData(Qt::UserRole, QVariant::fromValue(R->id()));
     ++glbUpdated;
     return false;
 }
 
 bool DirtyListDescriber::erasePoint(Node* Pt)
 {
-    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","REMOVE trackpoint %1").arg(Pt->id()) + userName(Pt), theListWidget);
-    it->setData(Qt::UserRole, Pt->id());
+    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","REMOVE trackpoint %1").arg(Pt->id().numId) + userName(Pt), theListWidget);
+    it->setData(Qt::UserRole, QVariant::fromValue(Pt->id()));
     ++glbDeleted;
     return false;
 }
 
 bool DirtyListDescriber::eraseRoad(Way* R)
 {
-    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","REMOVE road %1").arg(R->id()) + userName(R), theListWidget);
-    it->setData(Qt::UserRole, R->id());
+    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","REMOVE road %1").arg(R->id().numId) + userName(R), theListWidget);
+    it->setData(Qt::UserRole, QVariant::fromValue(R->id()));
     ++glbDeleted;
     return false;
 }
 
 bool DirtyListDescriber::eraseRelation(Relation* R)
 {
-    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","REMOVE relation %1").arg(R->id()) + userName(R), theListWidget);
-    it->setData(Qt::UserRole, R->id());
+    QListWidgetItem* it = new QListWidgetItem(QApplication::translate("DirtyListExecutor","REMOVE relation %1").arg(R->id().numId) + userName(R), theListWidget);
+    it->setData(Qt::UserRole, QVariant::fromValue(R->id()));
     ++glbDeleted;
     return false;
 }
@@ -407,7 +399,7 @@ bool DirtyListDescriber::eraseRelation(Relation* R)
 DirtyListExecutor::DirtyListExecutor(Document* aDoc, const DirtyListBuild& aFuture, const QString& aWeb, const QString& aUser, const QString& aPwd, int aTasks)
 : DirtyListVisit(aDoc, aFuture, false), Tasks(aTasks), Done(0), Web(aWeb), User(aUser), Pwd(aPwd), theDownloader(0)
 {
-    theDownloader = new Downloader(Web, User, Pwd);
+    theDownloader = new Downloader(User, Pwd);
 }
 
 DirtyListExecutor::~DirtyListExecutor()
@@ -423,7 +415,8 @@ bool DirtyListExecutor::sendRequest(const QString& Method, const QString& URL, c
 
     QMessageBox::StandardButton theChoice = QMessageBox::Retry;
     while (theChoice == QMessageBox::Retry) {
-        if (!theDownloader->request(Method,URL,Data))
+        QUrl theUrl(Web+URL);
+        if (!theDownloader->request(Method,theUrl,Data))
         {
             qDebug() << QString("Upload error: request (%1); Server message is '%2'").arg(theDownloader->resultCode()).arg(theDownloader->resultText());
             if (theDownloader->resultCode() == 401) {
@@ -491,7 +484,6 @@ bool DirtyListExecutor::start()
 {
     ChangeSetId = "";
     Progress->setValue(++Done);
-    if (!(M_PREFS->apiVersionNum() > 0.5)) return true;
 
     qDebug() << QString("OPEN changeset");
 
@@ -519,7 +511,6 @@ bool DirtyListExecutor::start()
 bool DirtyListExecutor::stop()
 {
     Progress->setValue(++Done);
-    if (!(M_PREFS->apiVersionNum() > 0.5)) return true;
 
     qDebug() << QString("CLOSE changeset");
 
@@ -541,26 +532,30 @@ bool DirtyListExecutor::addRelation(Relation *R)
 {
     Progress->setValue(++Done);
 
-    qDebug() << QString("ADD relation %1").arg(R->id());
+    qDebug() << QString("ADD relation %1").arg(R->id().numId);
 
-    Progress->setLabelText(tr("ADD relation %1").arg(R->id()) + userName(R));
+    Progress->setLabelText(tr("ADD relation %1").arg(R->id().numId) + userName(R));
     QEventLoop L; L.processEvents(QEventLoop::ExcludeUserInputEvents);
 
-    QString DataIn, DataOut, OldId;
+    QString DataIn, DataOut;
+    IFeature::FId OldId;
     OldId = R->id();
-    R->setId("0");
+    R->setId(IFeature::FId(IFeature::OsmRelation, 0));
     DataIn = wrapOSM(exportOSM(*R, ChangeSetId), ChangeSetId);
     R->setId(OldId);
     QString URL=theDownloader->getURLToCreate("relation");
     if (sendRequest("PUT",URL,DataIn,DataOut))
     {
         // chop off extra spaces, newlines etc
-        R->setId("rel_"+QString::number(DataOut.toInt()));
+        R->setId(IFeature::FId(IFeature::OsmRelation, DataOut.toInt()));
         R->setLastUpdated(Feature::OSMServer);
         R->setVersionNumber(1);
-        R->layer()->remove(R);
-        document()->getUploadedLayer()->add(R);
+        if (!g_Merk_Frisius) {
+            R->layer()->remove(R);
+            document()->getUploadedLayer()->add(R);
+        }
         R->setUploaded(true);
+        R->setDirtyLevel(0);
         return EraseFromHistory;
     }
     return false;
@@ -570,26 +565,30 @@ bool DirtyListExecutor::addRoad(Way *R)
 {
     Progress->setValue(++Done);
 
-    qDebug() << QString("ADD road %1").arg(R->id());
+    qDebug() << QString("ADD road %1").arg(R->id().numId);
 
-    Progress->setLabelText(tr("ADD road %1").arg(R->id()) + userName(R));
+    Progress->setLabelText(tr("ADD road %1").arg(R->id().numId) + userName(R));
     QEventLoop L; L.processEvents(QEventLoop::ExcludeUserInputEvents);
 
-    QString DataIn, DataOut, OldId;
+    QString DataIn, DataOut;
+    IFeature::FId OldId;
     OldId = R->id();
-    R->setId("0");
+    R->setId(IFeature::FId(IFeature::LineString, 0));
     DataIn = wrapOSM(exportOSM(*R, ChangeSetId), ChangeSetId);
     R->setId(OldId);
     QString URL=theDownloader->getURLToCreate("way");
     if (sendRequest("PUT",URL,DataIn,DataOut))
     {
         // chop off extra spaces, newlines etc
-        R->setId("way_"+QString::number(DataOut.toInt()));
+        R->setId(IFeature::FId(IFeature::LineString, DataOut.toInt()));
         R->setLastUpdated(Feature::OSMServer);
         R->setVersionNumber(1);
-        R->layer()->remove(R);
-        document()->getUploadedLayer()->add(R);
+        if (!g_Merk_Frisius) {
+            R->layer()->remove(R);
+            document()->getUploadedLayer()->add(R);
+        }
         R->setUploaded(true);
+        R->setDirtyLevel(0);
         return EraseFromHistory;
     }
     return false;
@@ -600,26 +599,30 @@ bool DirtyListExecutor::addPoint(Node* Pt)
 {
     Progress->setValue(++Done);
 
-    qDebug() << QString("ADD trackpoint %1").arg(Pt->id());
+    qDebug() << QString("ADD trackpoint %1").arg(Pt->id().numId);
 
-    Progress->setLabelText(tr("ADD trackpoint %1").arg(Pt->id()) + userName(Pt));
+    Progress->setLabelText(tr("ADD trackpoint %1").arg(Pt->id().numId) + userName(Pt));
     QEventLoop L; L.processEvents(QEventLoop::ExcludeUserInputEvents);
 
-    QString DataIn, DataOut, OldId;
+    QString DataIn, DataOut;
+    IFeature::FId OldId;
     OldId = Pt->id();
-    Pt->setId("0");
+    Pt->setId(IFeature::FId(IFeature::Point, 0));
     DataIn = wrapOSM(exportOSM(*Pt, ChangeSetId), ChangeSetId);
     Pt->setId(OldId);
     QString URL=theDownloader->getURLToCreate("node");
     if (sendRequest("PUT",URL,DataIn,DataOut))
     {
         // chop off extra spaces, newlines etc
-        Pt->setId("node_"+QString::number(DataOut.toInt()));
+        Pt->setId(IFeature::FId(IFeature::Point, DataOut.toInt()));
         Pt->setLastUpdated(Feature::OSMServer);
         Pt->setVersionNumber(1);
-        Pt->layer()->remove(Pt);
-        document()->getUploadedLayer()->add(Pt);
+        if (!g_Merk_Frisius) {
+            Pt->layer()->remove(Pt);
+            document()->getUploadedLayer()->add(Pt);
+        }
         Pt->setUploaded(true);
+        Pt->setDirtyLevel(0);
         return EraseFromHistory;
     }
     return false;
@@ -631,9 +634,9 @@ bool DirtyListExecutor::updateRelation(Relation* R)
 {
     Progress->setValue(++Done);
 
-    qDebug() << QString("UPDATE relation %1").arg(R->id());
+    qDebug() << QString("UPDATE relation %1").arg(R->id().numId);
 
-    Progress->setLabelText(tr("UPDATE relation %1").arg(R->id()) + userName(R));
+    Progress->setLabelText(tr("UPDATE relation %1").arg(R->id().numId) + userName(R));
     QEventLoop L; L.processEvents(QEventLoop::ExcludeUserInputEvents);
     QString URL = theDownloader->getURLToUpdate("relation",stripToOSMId(R->id()));
     QString DataIn, DataOut;
@@ -641,16 +644,17 @@ bool DirtyListExecutor::updateRelation(Relation* R)
     if (sendRequest("PUT",URL,DataIn,DataOut))
     {
         R->setLastUpdated(Feature::OSMServer);
-        if (M_PREFS->apiVersionNum() > 0.5)
-        {
-            int NewVersion = DataOut.toInt();
-            if (NewVersion <= R->versionNumber())
-                NewVersion = R->versionNumber()+1;
-            R->setVersionNumber(NewVersion);
+        int NewVersion = DataOut.toInt();
+        if (NewVersion <= R->versionNumber())
+            NewVersion = R->versionNumber()+1;
+        R->setVersionNumber(NewVersion);
+
+        if (!g_Merk_Frisius) {
+            R->layer()->remove(R);
+            document()->getUploadedLayer()->add(R);
         }
-        R->layer()->remove(R);
-        document()->getUploadedLayer()->add(R);
         R->setUploaded(true);
+        R->setDirtyLevel(0);
         return EraseFromHistory;
     }
     return true;
@@ -661,9 +665,9 @@ bool DirtyListExecutor::updateRoad(Way* R)
 {
     Progress->setValue(++Done);
 
-    qDebug() << QString("UPDATE road %1").arg(R->id());
+    qDebug() << QString("UPDATE road %1").arg(R->id().numId);
 
-    Progress->setLabelText(tr("UPDATE road %1").arg(R->id()) + userName(R));
+    Progress->setLabelText(tr("UPDATE road %1").arg(R->id().numId) + userName(R));
     QEventLoop L; L.processEvents(QEventLoop::ExcludeUserInputEvents);
     QString URL = theDownloader->getURLToUpdate("way",stripToOSMId(R->id()));
     QString DataIn, DataOut;
@@ -671,16 +675,18 @@ bool DirtyListExecutor::updateRoad(Way* R)
     if (sendRequest("PUT",URL,DataIn,DataOut))
     {
         R->setLastUpdated(Feature::OSMServer);
-        if (M_PREFS->apiVersionNum() > 0.5)
-        {
-            int NewVersion = DataOut.toInt();
-            if (NewVersion <= R->versionNumber())
-                NewVersion = R->versionNumber()+1;
-            R->setVersionNumber(NewVersion);
+
+        int NewVersion = DataOut.toInt();
+        if (NewVersion <= R->versionNumber())
+            NewVersion = R->versionNumber()+1;
+        R->setVersionNumber(NewVersion);
+
+        if (!g_Merk_Frisius) {
+            R->layer()->remove(R);
+            document()->getUploadedLayer()->add(R);
         }
-        R->layer()->remove(R);
-        document()->getUploadedLayer()->add(R);
         R->setUploaded(true);
+        R->setDirtyLevel(0);
         return EraseFromHistory;
     }
     return true;
@@ -690,9 +696,9 @@ bool DirtyListExecutor::updatePoint(Node* Pt)
 {
     Progress->setValue(++Done);
 
-    qDebug() << QString("UPDATE trackpoint %1").arg(Pt->id());
+    qDebug() << QString("UPDATE trackpoint %1").arg(Pt->id().numId);
 
-    Progress->setLabelText(tr("UPDATE trackpoint %1").arg(Pt->id()) + userName(Pt));
+    Progress->setLabelText(tr("UPDATE trackpoint %1").arg(Pt->id().numId) + userName(Pt));
     QEventLoop L; L.processEvents(QEventLoop::ExcludeUserInputEvents);
 //	QString URL("/api/0.3/node/%1");
 //	URL = URL.arg(stripToOSMId(Pt->id()));
@@ -702,16 +708,18 @@ bool DirtyListExecutor::updatePoint(Node* Pt)
     if (sendRequest("PUT",URL,DataIn,DataOut))
     {
         Pt->setLastUpdated(Feature::OSMServer);
-        if (M_PREFS->apiVersionNum() > 0.5)
-        {
-            int NewVersion = DataOut.toInt();
-            if (NewVersion <= Pt->versionNumber())
-                NewVersion = Pt->versionNumber()+1;
-            Pt->setVersionNumber(NewVersion);
+
+        int NewVersion = DataOut.toInt();
+        if (NewVersion <= Pt->versionNumber())
+            NewVersion = Pt->versionNumber()+1;
+        Pt->setVersionNumber(NewVersion);
+
+        if (!g_Merk_Frisius) {
+            Pt->layer()->remove(Pt);
+            document()->getUploadedLayer()->add(Pt);
         }
-        Pt->layer()->remove(Pt);
-        document()->getUploadedLayer()->add(Pt);
         Pt->setUploaded(true);
+        Pt->setDirtyLevel(0);
         return EraseFromHistory;
     }
     return false;
@@ -721,22 +729,24 @@ bool DirtyListExecutor::erasePoint(Node *Pt)
 {
     Progress->setValue(++Done);
 
-    qDebug() << QString("REMOVE trackpoint %1").arg(Pt->id());
+    qDebug() << QString("REMOVE trackpoint %1").arg(Pt->id().numId);
 
-    Progress->setLabelText(tr("REMOVE trackpoint %1").arg(Pt->id()) + userName(Pt));
+    Progress->setLabelText(tr("REMOVE trackpoint %1").arg(Pt->id().numId) + userName(Pt));
     QEventLoop L; L.processEvents(QEventLoop::ExcludeUserInputEvents);
 //	QString URL("/api/0.3/node/%1");
 //	URL = URL.arg(stripToOSMId(Pt->id()));
     QString URL = theDownloader->getURLToDelete("node",stripToOSMId(Pt->id()));
     QString DataIn, DataOut;
-    if (M_PREFS->apiVersionNum() > 0.5)
-        DataIn = wrapOSM(exportOSM(*Pt, ChangeSetId), ChangeSetId);
+    DataIn = wrapOSM(exportOSM(*Pt, ChangeSetId), ChangeSetId);
     if (sendRequest("DELETE",URL,DataIn,DataOut))
     {
         Pt->setLastUpdated(Feature::OSMServer);
-        Pt->layer()->remove(Pt);
-        document()->getUploadedLayer()->add(Pt);
+        if (!g_Merk_Frisius) {
+            Pt->layer()->remove(Pt);
+            document()->getUploadedLayer()->add(Pt);
+        }
         Pt->setUploaded(true);
+        Pt->setDirtyLevel(0);
         return EraseFromHistory;
     }
     return false;
@@ -746,22 +756,24 @@ bool DirtyListExecutor::eraseRoad(Way *R)
 {
     Progress->setValue(++Done);
 
-    qDebug() << QString("REMOVE road %1").arg(R->id());
+    qDebug() << QString("REMOVE road %1").arg(R->id().numId);
 
-    Progress->setLabelText(tr("REMOVE road %1").arg(R->id()) + userName(R));
+    Progress->setLabelText(tr("REMOVE road %1").arg(R->id().numId) + userName(R));
     QEventLoop L; L.processEvents(QEventLoop::ExcludeUserInputEvents);
 //	QString URL("/api/0.3/way/%1");
 //	URL = URL.arg(stripToOSMId(R->id()));
     QString URL = theDownloader->getURLToDelete("way",stripToOSMId(R->id()));
     QString DataIn, DataOut;
-    if (M_PREFS->apiVersionNum() > 0.5)
-        DataIn = wrapOSM(exportOSM(*R, ChangeSetId), ChangeSetId);
+    DataIn = wrapOSM(exportOSM(*R, ChangeSetId), ChangeSetId);
     if (sendRequest("DELETE",URL,DataIn,DataOut))
     {
         R->setLastUpdated(Feature::OSMServer);
-        R->layer()->remove(R);
-        document()->getUploadedLayer()->add(R);
+        if (!g_Merk_Frisius) {
+            R->layer()->remove(R);
+            document()->getUploadedLayer()->add(R);
+        }
         R->setUploaded(true);
+        R->setDirtyLevel(0);
         return EraseFromHistory;
     }
     return false;
@@ -771,20 +783,22 @@ bool DirtyListExecutor::eraseRelation(Relation *R)
 {
     Progress->setValue(++Done);
 
-    qDebug() << QString("REMOVE relation %1").arg(R->id());
+    qDebug() << QString("REMOVE relation %1").arg(R->id().numId);
 
-    Progress->setLabelText(tr("REMOVE relation %1").arg(R->id()) + userName(R));
+    Progress->setLabelText(tr("REMOVE relation %1").arg(R->id().numId) + userName(R));
     QEventLoop L; L.processEvents(QEventLoop::ExcludeUserInputEvents);
     QString URL = theDownloader->getURLToDelete("relation",stripToOSMId(R->id()));
     QString DataIn, DataOut;
-    if (M_PREFS->apiVersionNum() > 0.5)
-        DataIn = wrapOSM(exportOSM(*R, ChangeSetId), ChangeSetId);
+    DataIn = wrapOSM(exportOSM(*R, ChangeSetId), ChangeSetId);
     if (sendRequest("DELETE",URL,DataIn,DataOut))
     {
         R->setLastUpdated(Feature::OSMServer);
-        R->layer()->remove(R);
-        document()->getUploadedLayer()->add(R);
+        if (!g_Merk_Frisius) {
+            R->layer()->remove(R);
+            document()->getUploadedLayer()->add(R);
+        }
         R->setUploaded(true);
+        R->setDirtyLevel(0);
         return EraseFromHistory;
     }
     return false;
