@@ -36,18 +36,10 @@
 
 /* DOWNLOADER */
 
-Downloader::Downloader(const QString& aWeb, const QString& aUser, const QString& aPwd)
-: Port(80), Web(aWeb), User(aUser), Password(aPwd),
+Downloader::Downloader(const QString& aUser, const QString& aPwd)
+: User(aUser), Password(aPwd),
   Id(0),Error(false), AnimatorLabel(0), AnimatorBar(0), AnimationTimer(0)
 {
-    int p = Web.lastIndexOf(':');
-    if (p != -1)
-    {
-        Port = Web.right(Web.length()-(p+1)).toUInt();
-        Web = Web.left(p);
-    }
-
-    Request.setHost(Web,Port);
     //IdAuth = Request.setUser(User.toUtf8(), Password.toUtf8());
 //	connect(&Request,SIGNAL(done(bool)), this,SLOT(allDone(bool)));
     connect(&Request,SIGNAL(requestFinished(int, bool)),this,SLOT(on_requestFinished(int, bool)));
@@ -64,12 +56,13 @@ void Downloader::animate()
 
 void Downloader::setAnimator(QProgressDialog *anAnimator, QLabel* anAnimatorLabel, QProgressBar* anAnimatorBar, bool anAnimate)
 {
+    if (AnimationTimer)
+        delete AnimationTimer;
+
     AnimatorLabel = anAnimatorLabel;
     AnimatorBar = anAnimatorBar;
     if (AnimatorBar && anAnimate)
     {
-        if (AnimationTimer)
-            delete AnimationTimer;
         AnimationTimer = new QTimer(this);
         connect(AnimationTimer,SIGNAL(timeout()),this,SLOT(animate()));
     }
@@ -175,25 +168,25 @@ QByteArray gzipDecode(const QByteArray& In)
 
 
 
-bool Downloader::go(const QString& url)
+bool Downloader::go(const QUrl& url)
 {
     if (Error) return false;
 
-    qDebug() << "Downloader::go: " << url;
+    Request.setProxy(M_PREFS->getProxy(url));
+    Request.setHost(url.host(),url.port(80));
 
     if (AnimationTimer)
         AnimationTimer->start(200);
     Content.clear();
     QBuffer ResponseBuffer(&Content);
     ResponseBuffer.open(QIODevice::WriteOnly);
-    QHttpRequestHeader Header("GET",url);
+    QString sReq = url.toString(QUrl::RemoveScheme | QUrl::RemoveAuthority);
+    qDebug() << "Downloader::go: " << url << sReq;
+    QHttpRequestHeader Header("GET",sReq);
     Header.setValue("Accept-Encoding", "gzip,deflate");
-    if (Port == 80)
-        Header.setValue("Host",Web);
-    else
-        Header.setValue("Host",Web+':'+QString::number(Port));
+    Header.setValue("Host",url.host()+':'+QString::number(url.port(80)));
+    Header.setValue("User-Agent", USER_AGENT);
     Content.clear();
-    Request.setProxy(M_PREFS->getProxy(QUrl(QString("http://" + Header.value("Host") + url))));
     Id = Request.request(Header,QByteArray(), &ResponseBuffer);
 
     if (Loop.exec() == QDialog::Rejected)
@@ -222,39 +215,56 @@ bool Downloader::go(const QString& url)
     }
 
 #ifdef DEBUG_EVERY_CALL
-    showDebug("GET", url, QByteArray() ,Content);
+    showDebug("GET", Web.path() + url, QByteArray() ,Content);
 #endif
-    Result = Request.lastResponse().statusCode();
+    SAFE_DELETE(AnimationTimer);
     LocationText = Request.lastResponse().value("Location");
+
+    Result = Request.lastResponse().statusCode();
+    switch (Result) {
+    case 301:
+    case 302:
+    case 307: {
+            qDebug() << "New location: " << LocationText;
+            if (!LocationText.isEmpty()) {
+                QUrl aURL(LocationText);
+                return go(aURL);
+            }
+            break;
+        }
+    }
+
     ResultText = Request.lastResponse().reasonPhrase();
     ErrorText = Request.lastResponse().value("Error");
-    SAFE_DELETE(AnimationTimer);
     return !Error;
 }
 
-bool Downloader::request(const QString& Method, const QString& URL, const QString& Data)
+bool Downloader::request(const QString& Method, const QUrl& url, const QString& Data, bool FireForget)
 {
     if (Error) return false;
 
-    qDebug() << "Downloader::request: " << URL;
+    Request.setProxy(M_PREFS->getProxy(url));
+    Request.setHost(url.host(),url.port(80));
+    qDebug() << "Downloader::request: " << url;
 
     QByteArray ba(Data.toUtf8());
     QBuffer Buf(&ba);
 
-    QHttpRequestHeader Header(Method,URL);
-    if (Port == 80)
-        Header.setValue("Host",Web);
-    else
-        Header.setValue("Host",Web+':'+QString::number(Port));
+    QString sReq = url.toString(QUrl::RemoveScheme | QUrl::RemoveAuthority);
+    QHttpRequestHeader Header(Method,sReq);
+    Header.setValue("Host",url.host()+':'+url.port(80));
+    Header.setValue("User-Agent", USER_AGENT);
 
     QString auth = QString("%1:%2").arg(User).arg(Password);
     QByteArray ba_auth = auth.toUtf8().toBase64();
     Header.setValue("Authorization", QString("Basic %1").arg(QString(ba_auth)));
 
     Content.clear();
-    Request.setProxy(M_PREFS->getProxy(QUrl(QString("http://" + Header.value("Host") + URL))));
     Id = Request.request(Header,ba);
 
+    if (FireForget)
+        return true;
+
     if (Loop.exec() == QDialog::Rejected)
     {
         Request.abort();
@@ -352,37 +362,37 @@ const QString &Downloader::locationText()
 
 QString Downloader::getURLToOpenChangeSet()
 {
-    return QString("/api/%1/changeset/create").arg(M_PREFS->apiVersion());
+    return QString("/changeset/create");
 }
 
 QString Downloader::getURLToCloseChangeSet(const QString& Id)
 {
-    return QString("/api/%1/changeset/%2/close").arg(M_PREFS->apiVersion()).arg(Id);
+    return QString("/changeset/%1/close").arg(Id);
 }
 
 QString Downloader::getURLToUploadDiff(QString changesetId)
 {
-    return QString("/api/%1/changeset/%2/upload").arg(M_PREFS->apiVersion()).arg(changesetId);
+    return QString("/changeset/%1/upload").arg(changesetId);
 }
 
 QString Downloader::getURLToFetch(const QString &What)
 {
-    QString URL = QString("/api/%1/%2?%3=");
-    return URL.arg(M_PREFS->apiVersion()).arg(What).arg(What);
+    QString URL = QString("/%1?%2=");
+    return URL.arg(What).arg(What);
 }
 
-QString Downloader::getURLToFetchFull(QString id)
+QString Downloader::getURLToFetchFull(IFeature::FId id)
 {
-    QRegExp What("(.*)_(\\d*)");
-    int pos = What.indexIn(id);
-    if (pos == -1)
-        return QString();
-
-    QString type = What.cap(1);
-    if (type == "rel")
+    QString type;
+    if (id.type & IFeature::Point)
+        type = "node";
+    else if (id.type & IFeature::LineString)
+        type = "way";
+    else if (id.type & IFeature::OsmRelation)
         type = "relation";
-    QString URL = QString("/api/%1/%2/%3/full");
-    return URL.arg(M_PREFS->apiVersion()).arg(type).arg(What.cap(2));
+
+    QString URL = QString("/%1/%2/full");
+    return URL.arg(type).arg(id.numId);
 }
 
 QString Downloader::getURLToFetchFull(Feature* aFeature)
@@ -392,47 +402,43 @@ QString Downloader::getURLToFetchFull(Feature* aFeature)
 
 QString Downloader::getURLToFetch(const QString &What, const QString& Id)
 {
-    QString URL = QString("/api/%1/%2/%3");
-    return URL.arg(M_PREFS->apiVersion()).arg(What).arg(Id);
+    QString URL = QString("/%1/%2");
+    return URL.arg(What).arg(Id);
 }
 
 QString Downloader::getURLToCreate(const QString &What)
 {
-    QString URL = QString("/api/%1/%2/create");
-    return URL.arg(M_PREFS->apiVersion()).arg(What);
+    QString URL = QString("/%1/create");
+    return URL.arg(What);
 }
 
 QString Downloader::getURLToUpdate(const QString &What, const QString& Id)
 {
-    QString URL = QString("/api/%1/%2/%3");
-    return URL.arg(M_PREFS->apiVersion()).arg(What).arg(Id);
+    QString URL = QString("/%1/%2");
+    return URL.arg(What).arg(Id);
 }
 
 QString Downloader::getURLToDelete(const QString &What, const QString& Id)
 {
-    QString URL = QString("/api/%1/%2/%3");
-    return URL.arg(M_PREFS->apiVersion()).arg(What).arg(Id);
+    QString URL = QString("/%1/%2");
+    return URL.arg(What).arg(Id);
 }
 
 QString Downloader::getURLToMap()
 {
-    QString apiURL = QString("/api/%1").arg(M_PREFS->apiVersion());
-    QString URL = apiURL + QString("/map?bbox=%1,%2,%3,%4");
+    QString URL("/map?bbox=%1,%2,%3,%4");
     return URL;
 }
 
 QString Downloader::getURLToTrackPoints()
 {
-    QString URL = QString("/api/%1").arg(M_PREFS->apiVersion()) + QString("/trackpoints?bbox=%1,%2,%3,%4&page=%5");
+    QString URL = QString("/trackpoints?bbox=%1,%2,%3,%4&page=%5");
     return URL;
 }
 
 bool downloadOSM(QWidget* aParent, const QUrl& theUrl, const QString& aUser, const QString& aPassword, Document* theDocument, Layer* theLayer)
 {
-    QString aWeb = theUrl.host();
-    //QString URL = theUrl.path();
-    QString URL = theUrl.toString(QUrl::RemoveScheme | QUrl::RemoveAuthority);
-    Downloader Rcv(aWeb, aUser, aPassword);
+    Downloader Rcv(aUser, aPassword);
 
     IProgressWindow* aProgressWindow = dynamic_cast<IProgressWindow*>(aParent);
     if (aProgressWindow) {
@@ -456,7 +462,7 @@ bool downloadOSM(QWidget* aParent, const QUrl& theUrl, const QString& aUser, con
 
         Rcv.setAnimator(dlg, Lbl, Bar, true);
     }
-    if (!Rcv.go(URL))
+    if (!Rcv.go(theUrl))
     {
 #ifndef Q_OS_SYMBIAN
         aParent->setCursor(QCursor(Qt::ArrowCursor));
@@ -473,11 +479,10 @@ bool downloadOSM(QWidget* aParent, const QUrl& theUrl, const QString& aUser, con
         break;
     case 301:
     case 302:
-    case 307:
-        aWeb = Rcv.locationText();
+    case 307: {
+        QString aWeb = Rcv.locationText();
         if (!aWeb.isEmpty()) {
-            QUrl aURL = theUrl;
-            aURL.setHost(aWeb);
+            QUrl aURL(aWeb);
             return downloadOSM(aParent, aURL, aUser, aPassword, theDocument, theLayer);
         } else {
             QString msg = QApplication::translate("Downloader","Unexpected http status code (%1)\nServer message is '%2'").arg(x).arg(Rcv.resultText());
@@ -487,6 +492,7 @@ bool downloadOSM(QWidget* aParent, const QUrl& theUrl, const QString& aUser, con
             return false;
         }
         break;
+    }
     case 401:
         QMessageBox::warning(aParent,QApplication::translate("Downloader","Download failed"),QApplication::translate("Downloader","Username/password invalid"));
         return false;
@@ -497,7 +503,7 @@ bool downloadOSM(QWidget* aParent, const QUrl& theUrl, const QString& aUser, con
         QMessageBox::warning(aParent,QApplication::translate("Downloader","Download failed"), msg);
         return false;
     }
-    Downloader Down(aWeb, aUser, aPassword);
+    Downloader Down(aUser, aPassword);
     bool OK = importOSM(aParent, Rcv.content(), theDocument, theLayer, &Down);
     return OK;
 }
@@ -509,15 +515,11 @@ bool downloadOSM(QWidget* aParent, const QString& aWeb, const QString& aUser, co
         QMessageBox::warning(aParent,QApplication::translate("Downloader","Unresolved conflicts"), QApplication::translate("Downloader","Please resolve existing conflicts first"));
         return false;
     }
-    Downloader Rcv(aWeb, aUser, aPassword);
+    Downloader Rcv(aUser, aPassword);
     QString URL = Rcv.getURLToMap();
     URL = URL.arg(coordToAng(aBox.bottomLeft().lon()), 0, 'f').arg(coordToAng(aBox.bottomLeft().lat()), 0, 'f').arg(coordToAng(aBox.topRight().lon()), 0, 'f').arg(coordToAng(aBox.topRight().lat()), 0, 'f');
 
-    QUrl theUrl;
-    theUrl.setHost(aWeb);
-    theUrl.setPath(URL);
-    theUrl.setScheme("http");
-
+    QUrl theUrl(aWeb+URL);
     return downloadOSM(aParent, theUrl, aUser, aPassword, theDocument, theLayer);
 }
 
@@ -542,7 +544,7 @@ bool downloadOSM(QWidget* Main, const QString& aUser, const QString& aPassword,
 
 bool downloadTracksFromOSM(QWidget* Main, const QString& aWeb, const QString& aUser, const QString& aPassword, const CoordBox& aBox , Document* theDocument)
 {
-    Downloader theDownloader(aWeb, aUser, aPassword);
+    Downloader theDownloader(aUser, aPassword);
     QList<TrackLayer*> theTracklayers;
     //TrackMapLayer* trackLayer = new TrackMapLayer(QApplication::translate("Downloader","Downloaded tracks"));
     //theDocument->add(trackLayer);
@@ -574,7 +576,8 @@ bool downloadTracksFromOSM(QWidget* Main, const QString& aWeb, const QString& aU
                 arg(coordToAng(aBox.topRight().lon())).
                 arg(coordToAng(aBox.topRight().lat())).
                 arg(Page);
-        if (!theDownloader.go(URL))
+        QUrl theUrl(aWeb+URL);
+        if (!theDownloader.go(theUrl))
             return false;
         if (theDownloader.resultCode() != 200)
             return false;
@@ -602,7 +605,7 @@ bool checkForConflicts(Document* theDocument)
 
 bool downloadFeatures(MainWindow* Main, const QList<Feature*>& aDownloadList , Document* theDocument)
 {
-    QList<QString> list;
+    QList<IFeature::FId> list;
     foreach (Feature* F, aDownloadList) {
         list << F->id();
     }
@@ -612,54 +615,50 @@ bool downloadFeatures(MainWindow* Main, const QList<Feature*>& aDownloadList , D
     return ok;
 }
 
-bool downloadFeature(MainWindow* Main, const QString& id, Document* theDocument, Layer* theLayer)
+bool downloadFeature(MainWindow* Main, const IFeature::FId& id, Document* theDocument, Layer* theLayer)
 {
-    QList<QString> list;
+    QList<IFeature::FId> list;
     list << id;
     bool ok = downloadFeatures(Main, list, theDocument, theLayer);
 
     return ok;
 }
 
-bool downloadFeatures(MainWindow* Main, const QList<QString>& idList , Document* theDocument, Layer* theLayer)
+bool downloadFeatures(MainWindow* Main, const QList<IFeature::FId>& idList , Document* theDocument, Layer* theLayer)
 {
     if (!theLayer) {
         if (!theDocument->getLastDownloadLayer()) {
             theLayer = new DrawingLayer(QApplication::translate("Downloader","%1 download").arg(QDateTime::currentDateTime().toString(Qt::ISODate)));
             theDocument->add(theLayer);
         } else
-            theLayer = theDocument->getLastDownloadLayer();
+            theLayer = (Layer*)theDocument->getLastDownloadLayer();
     }
     theLayer->blockIndexing(true);
 
     QString osmWebsite, osmUser, osmPwd;
 
-    osmWebsite = MerkaartorPreferences::instance()->getOsmWebsite();
-    osmUser = MerkaartorPreferences::instance()->getOsmUser();
-    osmPwd = MerkaartorPreferences::instance()->getOsmPassword();
+    osmWebsite = M_PREFS->getOsmWebsite();
+    osmUser = M_PREFS->getOsmUser();
+    osmPwd = M_PREFS->getOsmPassword();
 
     if (Main)
         Main->view()->setUpdatesEnabled(false);
 
     bool OK = true;
-    Downloader Rcv(osmWebsite, osmUser, osmPwd);
+    Downloader Rcv(osmUser, osmPwd);
 
     for (int i=0; i<idList.size(); ++i) {
         QString URL = Rcv.getURLToFetchFull(idList[i]);
-
-        QUrl theUrl;
-        theUrl.setHost(osmWebsite);
-        theUrl.setPath(URL);
-        theUrl.setScheme("http");
+        QUrl theUrl(osmWebsite+URL);
 
         downloadOSM(Main, theUrl, osmUser, osmPwd, theDocument, theLayer);
     }
 
     if (Main)
         Main->view()->setUpdatesEnabled(true);
+    theLayer->blockIndexing(false);
     if (OK)
     {
-        theLayer->blockIndexing(false);
         theLayer->reIndex();
         if (Main)
             Main->invalidateView();
@@ -683,7 +682,7 @@ bool downloadOpenstreetbugs(MainWindow* Main, const CoordBox& aBox, Document* th
     if (Main)
         Main->view()->setUpdatesEnabled(false);
 
-    Downloader theDownloader(osbUrl.host(), "", "");
+    Downloader theDownloader("", "");
     QList<TrackLayer*> theTracklayers;
     TrackLayer* trackLayer = new TrackLayer(QApplication::translate("Downloader","OpenStreetBugs"));
     trackLayer->setUploadable(false);
@@ -711,13 +710,13 @@ bool downloadOpenstreetbugs(MainWindow* Main, const CoordBox& aBox, Document* th
 
     theDownloader.setAnimator(dlg,Lbl,Bar,true);
     Lbl->setText(QApplication::translate("Downloader","Downloading points"));
-    osbUrl.addQueryItem("t", QString::number(coordToAng(aBox.topRight().lat())));
-    osbUrl.addQueryItem("l", QString::number(coordToAng(aBox.bottomLeft().lon())));
-    osbUrl.addQueryItem("b", QString::number(coordToAng(aBox.bottomLeft().lat())));
-    osbUrl.addQueryItem("r", QString::number(coordToAng(aBox.topRight().lon())));
+    osbUrl.addQueryItem("t", COORD2STRING(coordToAng(aBox.topRight().lat())));
+    osbUrl.addQueryItem("l", COORD2STRING(coordToAng(aBox.bottomLeft().lon())));
+    osbUrl.addQueryItem("b", COORD2STRING(coordToAng(aBox.bottomLeft().lat())));
+    osbUrl.addQueryItem("r", COORD2STRING(coordToAng(aBox.topRight().lon())));
     osbUrl.addQueryItem("open", "yes");
 
-    if (!theDownloader.go(osbUrl.toString()))
+    if (!theDownloader.go(osbUrl))
         return false;
     if (theDownloader.resultCode() != 200)
         return false;
@@ -726,12 +725,11 @@ bool downloadOpenstreetbugs(MainWindow* Main, const CoordBox& aBox, Document* th
 
     if (Main)
         Main->view()->setUpdatesEnabled(true);
-    if (OK) {
-        trackLayer->blockIndexing(false);
-        trackLayer->reIndex();
-        if (Main)
-            Main->invalidateView();
-    }
+    trackLayer->blockIndexing(false);
+    trackLayer->reIndex();
+    if (Main)
+        Main->invalidateView();
+
     return OK;
 }
 
@@ -742,23 +740,23 @@ bool downloadMoreOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocum
         theLayer = new DrawingLayer(QApplication::translate("Downloader","%1 download").arg(QDateTime::currentDateTime().toString(Qt::ISODate)));
         theDocument->add(theLayer);
     } else
-        theLayer = theDocument->getLastDownloadLayer();
+        theLayer = (Layer*)theDocument->getLastDownloadLayer();
     theLayer->blockIndexing(true);
 
     QString osmWebsite, osmUser, osmPwd;
 
-    osmWebsite = MerkaartorPreferences::instance()->getOsmWebsite();
-    osmUser = MerkaartorPreferences::instance()->getOsmUser();
-    osmPwd = MerkaartorPreferences::instance()->getOsmPassword();
+    osmWebsite = M_PREFS->getOsmWebsite();
+    osmUser = M_PREFS->getOsmUser();
+    osmPwd = M_PREFS->getOsmPassword();
 
     Main->view()->setUpdatesEnabled(false);
 
     bool OK = true;
     OK = downloadOSM(Main,osmWebsite,osmUser,osmPwd,aBox,theDocument,theLayer);
     Main->view()->setUpdatesEnabled(true);
+    theLayer->blockIndexing(false);
     if (OK)
     {
-        theLayer->blockIndexing(false);
         theLayer->reIndex();
         theDocument->setLastDownloadLayer(theLayer);
         theDocument->addDownloadBox(theLayer, aBox);
@@ -782,9 +780,9 @@ bool downloadOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocument)
 
     QDialog * dlg = new QDialog(Main);
 
-    osmWebsite = MerkaartorPreferences::instance()->getOsmWebsite();
-    osmUser = MerkaartorPreferences::instance()->getOsmUser();
-    osmPwd = MerkaartorPreferences::instance()->getOsmPassword();
+    osmWebsite = M_PREFS->getOsmWebsite();
+    osmUser = M_PREFS->getOsmUser();
+    osmPwd = M_PREFS->getOsmPassword();
 
     Ui::DownloadMapDialog ui;
     ui.setupUi(dlg);
@@ -794,7 +792,7 @@ bool downloadOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocument)
 #endif
     CoordBox Clip(aBox);
     SlippyMap->setViewportArea(Clip.toRectF());
-    ui.vboxLayout1->addWidget(SlippyMap);
+    ui.verticalLayout->addWidget(SlippyMap);
     QObject::connect(SlippyMap, SIGNAL(redraw()), ui.FromMap, SLOT(toggle()));
     BookmarkListIterator i(*(M_PREFS->getBookmarks()));
     while (i.hasNext()) {
@@ -802,9 +800,11 @@ bool downloadOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocument)
         if (i.value().deleted == false)
             ui.Bookmarks->addItem(i.key());
     }
+    ui.edXapiUrl->setText(QString("*[bbox=%1,%2,%3,%4]").arg(coordToAng(aBox.bottomLeft().lon()), 0, 'f').arg(coordToAng(aBox.bottomLeft().lat()), 0, 'f').arg(coordToAng(aBox.topRight().lon()), 0, 'f').arg(coordToAng(aBox.topRight().lat()), 0, 'f'));
     ui.IncludeTracks->setChecked(DownloadRaw);
     ui.ResolveRelations->setChecked(M_PREFS->getResolveRelations());
     bool OK = true, retry = true, directAPI = false, Regional=false;
+    QString directUrl;
     while (retry) {
         retry = false;
 #ifdef _MOBILE
@@ -826,6 +826,7 @@ bool downloadOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocument)
 
                 if (link.contains("/api/")) {
                     directAPI=true;
+                    directUrl = link;
                 } else {
                     link.toUInt(&Regional);
                     if (!Regional) {
@@ -836,6 +837,11 @@ bool downloadOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocument)
                     }
                 }
             }
+            else if (ui.FromXapi->isChecked())
+            {
+                directAPI = true;
+                directUrl = M_PREFS->getXapiUrl() + ui.edXapiUrl->text();
+            }
             else if (ui.FromMap->isChecked())
             {
                 QRectF R(SlippyMap->viewArea());
@@ -847,8 +853,11 @@ bool downloadOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocument)
             theDocument->add(theLayer);
             theLayer->blockIndexing(true);
             M_PREFS->setResolveRelations(ui.ResolveRelations->isChecked());
-            if (directAPI)
-                OK = downloadOSM(Main,QUrl(QUrl::fromEncoded(ui.Link->text().toAscii())),osmUser,osmPwd,theDocument,theLayer);
+            if (directAPI) {
+                if (ui.FromXapi->isChecked())
+                    theLayer->setUploadable(false);
+                OK = downloadOSM(Main,QUrl(QUrl::fromEncoded(directUrl.toAscii())),osmUser,osmPwd,theDocument,theLayer);
+            }
             else
             if (Regional)
                 OK = downloadOSM(Main,osmUser,osmPwd,ui.Link->text().toUInt(),theDocument,theLayer);
@@ -857,9 +866,9 @@ bool downloadOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocument)
             if (OK && ui.IncludeTracks->isChecked())
                 OK = downloadTracksFromOSM(Main,osmWebsite,osmUser,osmPwd, Clip,theDocument);
             Main->view()->setUpdatesEnabled(true);
+            theLayer->blockIndexing(false);
             if (OK)
             {
-                theLayer->blockIndexing(false);
                 theLayer->reIndex();
                 theDocument->setLastDownloadLayer(theLayer);
                 theDocument->addDownloadBox(theLayer, Clip);
