@@ -14,6 +14,7 @@
 
 #include "Features.h"
 #include "FeatureCommands.h"
+#include "MapView.h"
 
 #include <QComboBox>
 #include <QLabel>
@@ -153,9 +154,9 @@ TagTemplateWidgetCombo::~TagTemplateWidgetCombo()
 {
 }
 
-QWidget* TagTemplateWidgetCombo::getWidget(const Feature* F)
+QWidget* TagTemplateWidgetCombo::getWidget(const Feature* F, const MapView* V)
 {
-    if (theSelector && (theSelector->matches(F) != TagSelect_Match && theSelector->matches(F) != TagSelect_DefaultMatch))
+    if (theSelector && (theSelector->matches(F,V->pixelPerM()) != TagSelect_Match && theSelector->matches(F,V->pixelPerM()) != TagSelect_DefaultMatch))
         return NULL;
 
     QString lang = getDefaultLanguage();
@@ -188,6 +189,8 @@ QWidget* TagTemplateWidgetCombo::getWidget(const Feature* F)
     QComboBox* aCombo = new QComboBox();
     aCombo->setEditable(true);
 //	aCombo->setSizeAdjustPolicy(QComboBox::AdjustToMinimumContentsLengthWithIcon);
+    aCombo->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
+    aCombo->setMinimumWidth(100);
     aLayout->addWidget(aCombo);
 
     aCombo->addItem(tr("Undefined"), qVariantFromValue(new TagTemplateWidgetValue("__NULL__")));
@@ -303,9 +306,9 @@ void TagTemplateWidgetCombo::apply(const Feature*)
 
 /** TagTemplateWidgetYesno **/
 
-QWidget* TagTemplateWidgetYesno::getWidget(const Feature* F)
+QWidget* TagTemplateWidgetYesno::getWidget(const Feature* F, const MapView* V)
 {
-    if (theSelector && (theSelector->matches(F) != TagSelect_Match && theSelector->matches(F) != TagSelect_DefaultMatch))
+    if (theSelector && (theSelector->matches(F,V->pixelPerM()) != TagSelect_Match && theSelector->matches(F,V->pixelPerM()) != TagSelect_DefaultMatch))
         return NULL;
 
     QString lang = getDefaultLanguage();
@@ -415,9 +418,9 @@ TagTemplateWidgetConstant::~TagTemplateWidgetConstant()
 {
 }
 
-QWidget* TagTemplateWidgetConstant::getWidget(const Feature* F)
+QWidget* TagTemplateWidgetConstant::getWidget(const Feature* F, const MapView* V)
 {
-    if (theSelector && (theSelector->matches(F) != TagSelect_Match && theSelector->matches(F) != TagSelect_DefaultMatch))
+    if (theSelector && (theSelector->matches(F,V->pixelPerM()) != TagSelect_Match && theSelector->matches(F,V->pixelPerM()) != TagSelect_DefaultMatch))
         return NULL;
 
     QString lang = getDefaultLanguage();
@@ -547,9 +550,9 @@ bool TagTemplateWidgetConstant::toXML(QDomElement& xParent, bool header)
 
 /** TagTemplateWidgetEdit **/
 
-QWidget* TagTemplateWidgetEdit::getWidget(const Feature* F)
+QWidget* TagTemplateWidgetEdit::getWidget(const Feature* F, const MapView* V)
 {
-    if (theSelector && (theSelector->matches(F) != TagSelect_Match && theSelector->matches(F) != TagSelect_DefaultMatch))
+    if (theSelector && (theSelector->matches(F,V->pixelPerM()) != TagSelect_Match && theSelector->matches(F,V->pixelPerM()) != TagSelect_DefaultMatch))
         return NULL;
 
     QString lang = getDefaultLanguage();
@@ -664,13 +667,13 @@ TagTemplate::TagTemplate()
 TagTemplate::TagTemplate(QString aName)
     : theWidget(0), theSelector(0)
 {
-    Q_UNUSED(aName)
+    theDescriptions[getDefaultLanguage()] = aName;
 }
 
 TagTemplate::TagTemplate(QString aName, QString aSelector)
     : theWidget(0)
 {
-    Q_UNUSED(aName)
+    theDescriptions[getDefaultLanguage()] = aName;
     theSelector = TagSelector::parse(aSelector);
 }
 
@@ -684,39 +687,39 @@ TagTemplate::~TagTemplate()
     delete theSelector;
 }
 
-TagSelectorMatchResult TagTemplate::matchesTag(const Feature* F)
+TagSelectorMatchResult TagTemplate::matchesTag(const Feature* F, const MapView* V)
 {
     TagSelectorMatchResult res;
 
     if (!theSelector) return TagSelect_NoMatch;
     // Special casing for multipolygon roads
-    if (const Way* R = qobject_cast<const Way*>(F))
+    if (const Way* R = dynamic_cast<const Way*>(F))
     {
         // TODO create a isPartOfMultiPolygon(R) function for this
         for (int i=0; i<R->sizeParents(); ++i)
         {
-            if (const Relation* Parent = qobject_cast<const Relation*>(R->getParent(i)))
+            if (const Relation* Parent = dynamic_cast<const Relation*>(R->getParent(i)))
                 if (!Parent->isDeleted())
                     if (Parent->tagValue("type","") == "multipolygon")
                         return TagSelect_NoMatch;
         }
     }
-    if ((res = theSelector->matches(F)))
+    if ((res = theSelector->matches(F,V->pixelPerM())))
         return res;
     // Special casing for multipolygon relations
-    if (const Relation* R = qobject_cast<const Relation*>(F))
+    if (const Relation* R = dynamic_cast<const Relation*>(F))
     {
         if (R->tagValue("type","") == "multipolygon") {
             for (int i=0; i<R->size(); ++i)
                 if (!R->get(i)->isDeleted())
-                    if ((res = theSelector->matches(R->get(i))))
+                    if ((res = theSelector->matches(R->get(i),V->pixelPerM())))
                         return res;
         }
     }
     return TagSelect_NoMatch;
 }
 
-QWidget* TagTemplate::getWidget(const Feature* F)
+QWidget* TagTemplate::getWidget(const Feature* F, const MapView* V)
 {
     QString lang = getDefaultLanguage();
     QString defLang = "en";
@@ -732,7 +735,7 @@ QWidget* TagTemplate::getWidget(const Feature* F)
     aLayout->setContentsMargins(2, 2, 2, 2);
 
     for (int i=0; i<theFields.size(); ++i) {
-        QWidget* W = theFields[i]->getWidget(F);
+        QWidget* W = theFields[i]->getWidget(F,V);
         if (W)
             aLayout->addWidget(W);
     }
@@ -860,7 +863,7 @@ TagTemplates::~TagTemplates()
         delete items[i];
 }
 
-QWidget* TagTemplates::getWidget(const Feature* F)
+QWidget* TagTemplates::getWidget(const Feature* F, const MapView* V)
 {
     QString lang = getDefaultLanguage();
     QString defLang = "en";
@@ -896,6 +899,9 @@ QWidget* TagTemplates::getWidget(const Feature* F)
 //	f.setPointSize(6);
 //	theCombo->setFont(f);
 //	theCombo->setMaximumHeight(16);
+    theCombo->setMinimumWidth(100);
+    theCombo->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);
+
     headLayout->addWidget(theCombo);
 
     QFrame* line = new QFrame();
@@ -919,7 +925,7 @@ QWidget* TagTemplates::getWidget(const Feature* F)
     }
     if (!forcedTemplate) {
         for (int i=0; i<items.size(); ++i) {
-            if (idx == -1 && items[i]->matchesTag(F) == TagSelect_Match) {
+            if (idx == -1 && items[i]->matchesTag(F,V) == TagSelect_Match) {
                 curTemplate = items[i];
                 idx = i+1;
             }
@@ -927,7 +933,7 @@ QWidget* TagTemplates::getWidget(const Feature* F)
     }
     if (!forcedTemplate && idx == -1) {
         for (int i=0; i<items.size(); ++i) {
-            if (idx == -1 && items[i]->matchesTag(F) == TagSelect_DefaultMatch) {
+            if (idx == -1 && items[i]->matchesTag(F,V) == TagSelect_DefaultMatch) {
                 curTemplate = items[i];
                 idx = i+1;
             }
@@ -940,7 +946,7 @@ QWidget* TagTemplates::getWidget(const Feature* F)
         theCombo->setCurrentIndex(idx);
 
     if (curTemplate) {
-        aLayout->addWidget(curTemplate->getWidget(F));
+        aLayout->addWidget(curTemplate->getWidget(F,V));
         connect(curTemplate, SIGNAL(tagChanged(QString, QString)), this, SLOT(on_tag_changed(QString, QString)));
         connect(curTemplate, SIGNAL(tagCleared(QString)), this, SLOT(on_tag_cleared(QString)));
     }
@@ -960,8 +966,10 @@ void TagTemplates::on_combo_activated(int idx)
         TagTemplate* newTmpl = new TagTemplate(theCombo->currentText());
         items.append(newTmpl);
 
-        for (int i=0; i<curTemplate->theFields.count(); ++i) {
-            curTemplate->theFields[i]->getCurrentValue();
+        if (curTemplate) {
+            for (int i=0; i<curTemplate->theFields.count(); ++i) {
+                curTemplate->theFields[i]->getCurrentValue();
+            }
         }
     } else {
         forcedTemplate = theCombo->itemData(idx).value<TagTemplate*>();
@@ -1044,14 +1052,14 @@ bool TagTemplates::mergeXml(const QDomElement& e)
     return true;
 }
 
-TagTemplate* TagTemplates::match(Feature* F)
+TagTemplate* TagTemplates::match(const Feature* F, const MapView* V)
 {
     for (int i=0; i<items.size(); ++i) {
-        if (items[i]->matchesTag(F) == TagSelect_Match)
+        if (items[i]->matchesTag(F,V) == TagSelect_Match)
             return items[i];
     }
     for (int i=0; i<items.size(); ++i) {
-        if (items[i]->matchesTag(F) == TagSelect_DefaultMatch)
+        if (items[i]->matchesTag(F,V) == TagSelect_DefaultMatch)
             return items[i];
     }
     return NULL;
